#ifndef GBMSAPI_DEFINES_H
#define GBMSAPI_DEFINES_H

/////////////////////////////////////////////////////////////////////
///////// INTRODUCTION //////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////
/********************************************************************
*		Conventions													*
********************************************************************/
// ------------------------------------------------------------------
// 1) Functions names: 
//		Each function is identified in this way:
//
//		GBMSAPI_<FunctionName>
//
//		where <FunctionName> is the function name.

//
//	NOTE:
//		Please refer to GBMSAPI_Library.h file for a complete list
//		of all the functions in GBMSAPI SDK
// ------------------------------------------------------------------
// 2) Defines: 
//		Each constant defined in this file is identified in this way:
//
//		GBMSAPI_<DefineID>_<ConstantID>
//
//		where <DefineID> is an acronym of the group the constant
//		belongs to (for example "DN" means "DEVICE NAME" and
//		<ConstantID> is the identifier of the constant.
// ------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////
// VER 1100
// 				VERY IMPORTANT NOTE
//
// For applications that do not use windows event management, for
// example purely console applications or applications that do not
// use .NET framework, in the main acquisition loop a calling to
// windows event manager has to be made, so that timers and tasks
// internal to the GBMSAPI library can be informed. Otherwise timers
// do not elapse. For example:
//
// while (1) { // application main cycle
//	....
//	MSG msg;
//
//	if (PeekMessage(&msg, 0, 0, 0, PM_REMOVE))
//	{
//		TranslateMessage (&msg);
//		DispatchMessage (&msg);
//	}
// } // end of application main cycle
//
/////////////////////////////////////////////////////////////////////

/****************************************************************************
*																			*
*					PART 1													*
*																			*
*				ERROR CODES													*
*																			*
****************************************************************************/

//////////////////////////////////////////////////////////////////////
//////// ERROR CODES /////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//	In this section a list of the error codes than can be returned by
//	the SDK functions is given.

// Low level errors
//	GBMSAPI SDK methods wrap the corresponding low level
//	functions. The following error codes are generally returned by the
//	low level functions, but they could be returned also by the high
//	level functions (for example GBMSAPI_ERROR_CODE_NO_ERROR): in such a case
//	this is explicitly said into the functions comment.
#define GBMSAPI_ERROR_CODE_NO_ERROR								0
#define GBMSAPI_ERROR_CODE_USB_DRIVER							1
#define GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND						2
#define GBMSAPI_ERROR_CODE_USB_THREAD							3
#define GBMSAPI_ERROR_CODE_PARAMETER							4
#define GBMSAPI_ERROR_CODE_SCANNER_NOT_CONFIGURED				5
#define GBMSAPI_ERROR_CODE_DEVICE_NOT_RESPONDING				6
#define GBMSAPI_ERROR_CODE_SCANNER_COMMUNICATION				7
#define GBMSAPI_ERROR_CODE_UNAVAILABLE_OPTION					8
#define GBMSAPI_ERROR_CODE_INTERNAL								9
// VER 2.7.1.0
#define GBMSAPI_ERROR_CODE_USB_FULLSPEED_NOT_SUPPORTED			10
// GB_MULTISCAN_SDK errors
#define GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED				30
#define GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED					31 // it should be treated as a warning
#define GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED			32
#define GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED				33 // it should be treated as a warning
#define GBMSAPI_ERROR_CODE_ACQUISITION_THREAD					34
#define GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED			35
#define GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED				36 // it should be treated as a warning
#define GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET					37
#define GBMSAPI_ERROR_CODE_MEMORY_ALLOCATION					38
#define GBMSAPI_ERROR_CODE_GENERIC								39

/****************************************************************************
*																			*
*					PART 2													*
*																			*
*				DEVICE CHARACTERISTICS										*
*																			*
****************************************************************************/

//////////////////////////////////////////////////////////////////////
//////// DEVICE NAME /////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//	In this section a list of the supported devices by this SDK is 
//	given. For more detailed info about these devices, please check 
//	MULTISCAN_SDK_Overview.pdf file in the documentation.

#define GBMSAPI_DN_DS84		1	// DactyScan84
#define GBMSAPI_DN_MS500	2	// MultiScan500
#define GBMSAPI_DN_MS1000	3	// MultiScan1000
#define GBMSAPI_DN_VS3		4	// VisaScan3
#define GBMSAPI_DN_PS2		5	// PoliScan2
#define GBMSAPI_DN_DS40		6	// DactyScan40
#define GBMSAPI_DN_DS26		7	// DactyScan26 and DactyScan26i
// VER 2400
// Add MC500 and MSC500 device
#define GBMSAPI_DN_MC500	8	// MultiScan500, China market versione
#define GBMSAPI_DN_MSC500	9	// MultiScan500, China market version, FULL version
// VER 2500:
#define GBMSAPI_DN_DS40I	10	// DactyScan40i
// VER 2700: add DS84C device
#define GBMSAPI_DN_DS84C	11 // DactyScan40c
// VER 2.7.1.0: add MC517 device
#define GBMSAPI_DN_MC517	12 // MC517
// VER 2.8.0.0: add MSC517 device
#define GBMSAPI_DN_MSC517	13 // MSC517
// VER 2.8.0.0: add DactyScan32 device
#define GBMSAPI_DN_DS32		14 // DactyScan32

///////////////////////////////////////////////////////////////////////
//////// DEVICE INFO //////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_MAX_PLUGGED_DEVICE_NUM	127 // max number of devices
								// that can be plugged
#define GBMSAPI_MAX_SN_LENGHT				20	// max number of chars
								// for the device serial number

typedef struct GBMSAPI_DEVICE_INFO_STRUCT
{
	unsigned char DeviceID;		// can be one of the above-listed 
	char DeviceSerialNumber[GBMSAPI_MAX_SN_LENGHT];	// device serial number
}GBMSAPI_DeviceInfoStruct;

///////////////////////////////////////////////////////////////////////
//////// FW INFO //////////////////////////////////////////////////////
//	See GBMSAPI_GetDeviceNameAndVersion(...) function for further
//	details
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_MAX_CHARS_IN_DEVICE_NAME_AND_INFO	512 // max wchars




//////////////////////////////////////////////////////////////////////
//////// SCANNABLE BIOMETRIC TYPES ///////////////////////////////////
//////////////////////////////////////////////////////////////////////
//	In this section a list of the object types that can be scanned by
//	using GreenBit devices is given. They represent the types
//	listed in the EBTS document (Ver 8.0.0.2, April 1st 2008, freely
//	downloadable from 
//	http://www.fbibiospecs.org/fbibiometric/ebts.html
//	web site) in "Table F-1. Preferred Capture Sizes".
//	The values defined in the following list are intentionally given
//	as bit mask values (in the sense that they are choosen so
//	that for each of them only a bit of an unsigned int -4 bytes-
//	field is set to 1).
//	In order to know what types are supported by each Green Bit
//	device, use GBMSAPI_GetScannableTypes(...) function.
//
//	NOTE1:
//		There's a type, named GBMSAPI_SBT_FLAT_SINGLE_FINGER, that is not
//		specifically reported in the EBTS document, but is often used,
//		expecially for identification purposes.
//
//	NOTE1.1:
//		There's a type, named GBMSAPI_SBT_FLAT_SLAP_2, that is not
//		specifically reported in the EBTS document, but can be useful
//		for	application	purposes.
//
//	NOTE2:
//		In this library the "half palm" is represented by two types,
//		"lower half palm" (GBMSAPI_SBT_FLAT_LOWER_HALF_PALM, from the 
//		wrist bracelet to the top of the interdigital area - third
//		finger joint -) and "upper half palm" 
//		(GBMSAPI_SBT_FLAT_UPPER_HALF_PALM, from the top of the 
//		interdigital area to the fingers tip: do not mix up with
//		the "plain 4-fingers" type), as described in the
//		ANSI/NIST ITL 1-2007 document (freely downloadable from
//		http://fingerprint.nist.gov/standard/ web site), in chapter
//		"20 Type-15 variable-resolution palmprint image record".
//
//	NOTE3:
//		There's a type, named GBMSAPI_SBT_PHOTO, that is not included in the
//		above-mentioned F-1 table, but it is defined for backward 
//		compatibility with VisaScan3 device.
//
//	NOTE4:
//		The types listed below can be divided in two cathegories:
//		
//		- ROLLED:
//			this cathegory is represented by 
//			GBMSAPI_SBT_ROLL_SINGLE_FINGER type
//
//		- FLAT:
//			this cathegory is represented by all the other types.

///////////////////////////////////////////////////////////////////////
// GBMSAPI_SBT_NO_OBJECT_TYPE
//
// Undefined object type.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_SBT_NO_OBJECT_TYPE	((unsigned int)0)

///////////////////////////////////////////////////////////////////////
// GBMSAPI_SBT_ROLL_SINGLE_FINGER
//
// Meaning:	
//	It represents a rolled fingerprint. It must be scanned on the 
//	device roll area (see "FLAT FINGER ON ROLL SCAN AREA" paragraph).
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_SBT_ROLL_SINGLE_FINGER		((unsigned int)0x00000001)

///////////////////////////////////////////////////////////////////////
// GBMSAPI_SBT_FLAT_SINGLE_FINGER
//
// Meaning:
//	It represents a single flat fingerprint. It's not reported as a 
//	specific type in the EBTS document, but it can be useful for 
//	application	purposes. Normally it should be scanned on the 
//	device flat area (see "FLAT FINGER ON ROLL SCAN AREA" paragraph).
//
//	NOTE:
//		For some devices (like DS84, VS3 and PS2), the flat fingerprint
//		can be got also on the device roll area.
//		For other devices (like DS40), it can be taken ONLY on the
//		roll area. Therefore, for GBMSAPI_SBT_FLAT_SINGLE_FINGER
//		type it is left the option to choose the area where the image
//		will be taken. If this option is not available (like for MS500 or
//		MS1000 devices) it will be not considered and the flat
//		fingerprint will be taken on the flat area.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_SBT_FLAT_SINGLE_FINGER		((unsigned int)0x00000002)

///////////////////////////////////////////////////////////////////////
// GBMSAPI_SBT_FLAT_SLAP_4
//
// Meaning:
//	It represents a flat plain 4-fingers. It must be scanned on the 
//	device flat area (see "FLAT FINGER ON ROLL SCAN AREA" section).
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_SBT_FLAT_SLAP_4			((unsigned int)0x00000004)

///////////////////////////////////////////////////////////////////////
// GBMSAPI_SBT_FLAT_SLAP_2
//
// Meaning:
//	It represents a flat plain 2-fingers (index and middle). It's not
//	reported as a specific type in the EBTS document, but it can be useful
//	for application	purposes. It must be scanned on the 
//	device roll area (see "FLAT FINGER ON ROLL SCAN AREA" paragraph).
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_SBT_FLAT_SLAP_2			((unsigned int)0x00000008)

///////////////////////////////////////////////////////////////////////
// GBMSAPI_SBT_FLAT_THUMBS_2
//
// Meaning:
//	It represents a flat plain 2-thumbs type. It must be scanned on the 
//	deviceflat area (see "FLAT FINGER ON ROLL SCAN AREA" section).
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_SBT_FLAT_THUMBS_2			((unsigned int)0x00000010)

///////////////////////////////////////////////////////////////////////
// GBMSAPI_SBT_FLAT_LOWER_HALF_PALM
//
// Meaning:
//	It represents the flat lower half palm type. It must be scanned on 
//	the device flat area (see "FLAT FINGER ON ROLL SCAN AREA" section).
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_SBT_FLAT_LOWER_HALF_PALM	((unsigned int)0x00000020)

///////////////////////////////////////////////////////////////////////
// GBMSAPI_SBT_FLAT_UPPER_HALF_PALM
//
// Meaning:
//	It represents the flat upper half palm type. It must be scanned on 
//	the device flat area (see "FLAT FINGER ON ROLL SCAN AREA" section).
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_SBT_FLAT_UPPER_HALF_PALM	((unsigned int)0x00000040)

///////////////////////////////////////////////////////////////////////
// GBMSAPI_SBT_FLAT_WRITER_PALM
//
// Meaning:
//	It represents the flat writer's palm type. It must be scanned on the 
//	device flat area (see "FLAT FINGER ON ROLL SCAN AREA" section).
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_SBT_FLAT_WRITER_PALM		((unsigned int)0x00000080)

// VER 2000
///////////////////////////////////////////////////////////////////////
// GBMSAPI_SBT_FLAT_INDEXES_2
//
// Meaning:
//	It represents a flat plain 2-indexes type. It's not
//	reported as a specific type in the EBTS document, but it can be useful
//	for application	purposes. It must be scanned on the 
//	device roll area (see "FLAT FINGER ON ROLL SCAN AREA" paragraph).
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_SBT_FLAT_INDEXES_2			((unsigned int)0x00000100)

///////////////////////////////////////////////////////////////////////
// GBMSAPI_SBT_PHOTO
//
// Meaning:
//	This has been defined only for backward compatibility with
//	Visascan3 device, that has the possibility of getting pictures of 
//	documents on its own photo scanner. It must be scanned on the device
//	flat area (see "FLAT FINGER ON ROLL SCAN AREA" section).
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_SBT_PHOTO					((unsigned int)0x80000000)

//////////////////////////////////////////////////////////////////////
///// SCANNABLE OBJECTS //////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
// GBMSAPI_SBT_NO_OBJECT
//
// Undefined object.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_SBT_NO_OBJECT	((unsigned int)0)
/********************************************************************
*	FLAT FINGERS													*
********************************************************************/
// These objects belong to GBMSAPI_SBT_FLAT_SINGLE_FINGER type
#define GBMSAPI_SO_FLAT_RIGHT_THUMB			((unsigned int)1)
#define GBMSAPI_SO_FLAT_RIGHT_INDEX			((unsigned int)2)
#define GBMSAPI_SO_FLAT_RIGHT_MIDDLE			((unsigned int)3)
#define GBMSAPI_SO_FLAT_RIGHT_RING			((unsigned int)4)
#define GBMSAPI_SO_FLAT_RIGHT_LITTLE			((unsigned int)5)
#define GBMSAPI_SO_FLAT_LEFT_THUMB			((unsigned int)6)
#define GBMSAPI_SO_FLAT_LEFT_INDEX			((unsigned int)7)
#define GBMSAPI_SO_FLAT_LEFT_MIDDLE			((unsigned int)8)
#define GBMSAPI_SO_FLAT_LEFT_RING				((unsigned int)9)
#define GBMSAPI_SO_FLAT_LEFT_LITTLE			((unsigned int)10)
/********************************************************************
*	ROLL FINGERS													*
********************************************************************/
// These objects belong to GBMSAPI_SBT_ROLL_SINGLE_FINGER type
#define GBMSAPI_SO_ROLL_RIGHT_THUMB			((unsigned int)11)
#define GBMSAPI_SO_ROLL_RIGHT_INDEX			((unsigned int)12)
#define GBMSAPI_SO_ROLL_RIGHT_MIDDLE			((unsigned int)13)
#define GBMSAPI_SO_ROLL_RIGHT_RING			((unsigned int)14)
#define GBMSAPI_SO_ROLL_RIGHT_LITTLE			((unsigned int)15)
#define GBMSAPI_SO_ROLL_LEFT_THUMB			((unsigned int)16)
#define GBMSAPI_SO_ROLL_LEFT_INDEX			((unsigned int)17)
#define GBMSAPI_SO_ROLL_LEFT_MIDDLE			((unsigned int)18)
#define GBMSAPI_SO_ROLL_LEFT_RING				((unsigned int)19)
#define GBMSAPI_SO_ROLL_LEFT_LITTLE			((unsigned int)20)
/********************************************************************
*	SLAPS															*
********************************************************************/
// This object belongs to GBMSAPI_SBT_FLAT_THUMBS_2 type
#define GBMSAPI_SO_SLAP_2_THUMBS				((unsigned int)21)
// This object belongs to GBMSAPI_SBT_FLAT_SLAP_4 type
#define GBMSAPI_SO_SLAP_4_LEFT				((unsigned int)22)
#define GBMSAPI_SO_SLAP_4_RIGHT				((unsigned int)23)
// This object belongs to GBMSAPI_SBT_FLAT_SLAP_2 type
#define GBMSAPI_SO_SLAP_2_LEFT				((unsigned int)24)
#define GBMSAPI_SO_SLAP_2_RIGHT				((unsigned int)25)
// VER 2000
// This object belongs to GBMSAPI_SBT_FLAT_THUMBS_2 type
#define GBMSAPI_SO_SLAP_2_INDEXES				((unsigned int)33)
// end VER 2000
/********************************************************************
*	PALM/WRITER														*
********************************************************************/
// These objects belong to GBMSAPI_SBT_FLAT_UPPER_HALF_PALM type
#define GBMSAPI_SO_UPPER_HALF_PALM_LEFT		((unsigned int)26)
#define GBMSAPI_SO_UPPER_HALF_PALM_RIGHT		((unsigned int)27)
// These objects belong to GBMSAPI_SBT_FLAT_LOWER_HALF_PALM type
#define GBMSAPI_SO_LOWER_HALF_PALM_LEFT		((unsigned int)28)
#define GBMSAPI_SO_LOWER_HALF_PALM_RIGHT		((unsigned int)29)
// These objects belong to GBMSAPI_SBT_FLAT_WRITER_PALM type
#define GBMSAPI_SO_WRITER_PALM_LEFT			((unsigned int)30)
#define GBMSAPI_SO_WRITER_PALM_RIGHT			((unsigned int)31)
/********************************************************************
*	PHOTO															*
********************************************************************/
// This object belongs to GBMSAPI_SBT_PHOTO type
#define GBMSAPI_SO_PHOTO						((unsigned int)32)

//////////////////////////////////////////////////////////////////////
////// DEVICE FEATURES ///////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//	In this section the scanners software features are described.
//	They depend on the being used device.
//
//	The values defined in the following list are intentionally given
//	as bit mask values (in the sense that they are choosen so that
//	for each of them only a bit of an unsigned int -4 bytes-
//	field is set to 1).
//	In order to know what features are supported by each Green Bit
//	device, use GBMSAPI_GetDeviceFeatures(...) function.

///////////////////////////////////////////////////////////////////////
//	MULTIPLE USB CONNECTIONS ALLOWED
//
//	Some GreenBit devices (like DactyScan84) give the possibility to plug
//	at the same time more than one device of the same type to the USB
//	bus. The application can switch from a device to the other simply
//	by using GBMSAPI_SelectProductSerialNumber(...) function.
//
//	NOTE:
//		just one device at a time can access to the USB bus even if more
//		than one can be plugged, otherwise USB conflicts will be
//		generated. See GBMSAPI_GetUsbErrorCode(...) function for more
//		details.
//
//	In order to know if this feature is supported, the application
//	should check the <DeviceFeatures> parameter of
//	GBMSAPI_GetDeviceFeatures(...) function and see whether
//	GBMSAPI_DF_MULTIPLE_USB_CONNECTIONS_ALLOWED bit is set.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_DF_MULTIPLE_USB_CONNECTIONS_ALLOWED	((unsigned int)0x00000001)

///////////////////////////////////////////////////////////////////////
//	AUTO-CAPTURE BLOCKING
//
//	This feature allows to block temporarily the auto-capture system in 
//	order to prevent the automatic preview stopping in case 
//	the application itself decides to prohibit the current image selection
//	by analyzing the incoming fingerprint images or the corresponding 
//	diagnostic messages. For the practical use of this feature (when
//	and how to use it) see GBMSAPI_SetAutoCaptureBlocking(...)
//	function
//
//	In order to know if this feature is supported, the application
//	should check the <DeviceFeatures> parameter of
//	GBMSAPI_GetDeviceFeatures(...) function and see whether
//	GBMSAPI_DF_AUTO_CAPTURE_BLOCKING bit is set.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_DF_AUTO_CAPTURE_BLOCKING		((unsigned int)0x00000002)

///////////////////////////////////////////////////////////////////////
//	1000 DPI RESOLUTION
//
//	By default, Green Bit scanners acquire 500 DPI
//	resolution images.
//	Scanners having this feature are capable to get 1000
//	DPI resolution images as well.
//
//	In order to know if this feature is supported, the application
//	should check the <DeviceFeatures> parameter of
//	GBMSAPI_GetDeviceFeatures(...) function and see whether
//	GBMSAPI_DF_1000DPI_RESOLUTION bit is set.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_DF_1000DPI_RESOLUTION			((unsigned int)0x00000004)

// Ver 2500
///////////////////////////////////////////////////////////////////////
//	IMAGE ROTATION
//
//	Devices having this feature support rotation of the image. It 
//	could be useful to provide proper fingerprint image orientation in
//	case when the scanner is mounted inside a device box rotated respect
//	to its normal orientation. Possible orientations are:
#define GBMSAPI_IMG_ROTATE_NO		0	// No image rotation
#define GBMSAPI_IMG_ROTATE_90_CW	1	// Image rotation 90 degree
										// clockwise
#define GBMSAPI_IMG_ROTATE_90_CCW	2	// Image rotation 90 degree
										// counter-clockwise
#define GBMSAPI_IMG_ROTATE_180		3	// Image rotation 180 degree,
//
//	In order to know if this feature is supported, the application
//	should check the <DeviceFeatures> parameter of
//	GBMSAPI_GetDeviceFeatures(...) function and see whether
//	GBMSAPI_DF_IMAGE_ROTATION bit is set.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_DF_IMAGE_ROTATION			((unsigned int)0x00000008) 

// VER2700
///////////////////////////////////////////////////////////////////////
//	USER DATA STORAGE INTO THE DEVICE
//
//	Some GreenBit scanners, such as the DS84(n), allow the user to
//	permanently store some data inside the device (within a EEPROM or a
//	FLASH memory), and to read written data.
//
//	In order to know if this feature is supported, the application
//	should check the <DeviceFeatures> parameter of
//	GBMSAPI_GetDeviceFeatures(...) function and see whether
//	GBMSAPI_DF_1000DPI_RESOLUTION bit is set.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_DF_PERMANENT_USER_DATA_STORAGE	((unsigned int)0x00000010)

// VER2700
///////////////////////////////////////////////////////////////////////
//	FRAME RATE SETTING CAPABILITY
//
//	Some GreenBit scanners, such as the DS84(n), allow the user to
//	set the frame rate for the best fitting to the PC resources. For
//	more details see the "GBMSAPI_SetFrameRateComplete(...)" function.
//
//	In order to know if this feature is supported, the application
//	should check the <DeviceFeatures> parameter of
//	GBMSAPI_GetDeviceFeatures(...) function and see whether
//	GBMSAPI_DF_FRAME_RATE_SETTING bit is set.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_DF_FRAME_RATE_SETTING		((unsigned int)0x00000020)

// VER 2700
// VER 2800: CONSIDER THE FOLLOWING DEFINES AS OBSOLETE: they will be
// commented, and Green Bit recommends to use the
// GBMSAPI_GetFrameRateRange_Global(...) function instead of them
//////////////////////////////////////////////////////////////////////
////// FRAME RATE RANGE FOR DEVICES //////////////////////////////////
//////////////////////////////////////////////////////////////////////
// The following defines represent the ranges and the default value
// of the <FrameRate> parameter of the "GBMSAPI_SetFrameRate(...)"
// and "GBMSAPI_SetFrameRateComplete(...)" functions (for further
// details see them in the GBMSAPI_Library.h file). They should be
// used to check the <FrameRate> parameter before calling the above
// functions.
// For every scanner every define is composed this way:
//
// GBMSAPI_<DEVICE_NAME>_<SCAN_AREA>_<ACQUISITION_MODE>_<FRAME_RESOLUTION>_<RANGE_LIMIT>_FRAME_RATE
//
// where:
//
// - <DEVICE_NAME> represents the device
//
// - <SCAN_AREA> represents the scan area (see the 
//		GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA option)
//
// - <ACQUISITION_MODE> represents the class of the object to be
//		acquired: can be ROLL or FLAT
//
// - <FRAME_RESOLUTION> represents the class of the object to be
//		acquired: can be PREVIEW or FULLRES. 
//		NOTE: if this item is PREVIEW, then the corresponding define
//		should be used with	the "GBMSAPI_SetFrameRateComplete(...)"
//		function, otherwise with the "GBMSAPI_SetFrameRate(...)" one
//
// - <RANGE_LIMIT> can be MAX, MIN or DEF (default value)


/////////////////////////////////////////////////////////////////////
// DactyScan 84
//
// Recommendations:
//
// - Use the "GBMSAPI_DS84_FLAT_FLAT_..." defines before acquiring
//		flat objects on flat scan area
// - Use the "GBMSAPI_DS84_ROLL_ROLL_..." defines before acquiring
//		rolled objects
// - Use the "GBMSAPI_DS84_FLAT_FLAT_..." defines before acquiring
//		flat objects on roll scan area
/////////////////////////////////////////////////////////////////////
//// Flat Area - Flat Mode - High Resolution Frame
//#define GBMSAPI_DS84_FLAT_FLAT_FULLRES_MAX_FRAME_RATE ((double)5.0)
//#define GBMSAPI_DS84_FLAT_FLAT_FULLRES_MIN_FRAME_RATE ((double)2.5)
//#define GBMSAPI_DS84_FLAT_FLAT_FULLRES_DEF_FRAME_RATE ((double)5.0)
//// Roll Area - Roll Mode - High Resolution Frame
//#define GBMSAPI_DS84_ROLL_ROLL_FULLRES_MAX_FRAME_RATE ((double)15.0)
//#define GBMSAPI_DS84_ROLL_ROLL_FULLRES_MIN_FRAME_RATE ((double)11.0)
//#define GBMSAPI_DS84_ROLL_ROLL_FULLRES_DEF_FRAME_RATE ((double)15.0)
//// Roll Area - Flat Mode - High Resolution Frame
//#define GBMSAPI_DS84_ROLL_FLAT_FULLRES_MAX_FRAME_RATE ((double)15.0)
//#define GBMSAPI_DS84_ROLL_FLAT_FULLRES_MIN_FRAME_RATE ((double)5.0)
//#define GBMSAPI_DS84_ROLL_FLAT_FULLRES_DEF_FRAME_RATE ((double)15.0)

///////////////////////
// DactyScan 26
///////////////////////
// Flat Area - Flat Mode - High Resolution Frame
//#define GBMSAPI_DS26_FLAT_FLAT_FULLRES_MAX_FRAME_RATE ((double)10.0)
//#define GBMSAPI_DS26_FLAT_FLAT_FULLRES_MIN_FRAME_RATE ((double)2.5)
//#define GBMSAPI_DS26_FLAT_FLAT_FULLRES_DEF_FRAME_RATE ((double)5.0)

/////////////////////////////////////////////////////////////////////
// DactyScan 40i
//
// Recommendations:
//
// - Use the "GBMSAPI_DS40I_ROLL_ROLL_..." defines before acquiring
//		rolled objects
// - Use the "GBMSAPI_DS40I_FLAT_FLAT_..." defines before acquiring
//		flat objects on roll (or flat, it's the same) scan area
/////////////////////////////////////////////////////////////////////
//// Roll Area - Roll Mode - High Resolution Frame
//#define GBMSAPI_DS40I_ROLL_ROLL_FULLRES_MAX_FRAME_RATE ((double)16.0)
//#define GBMSAPI_DS40I_ROLL_ROLL_FULLRES_MIN_FRAME_RATE ((double)2.5)
//#define GBMSAPI_DS40I_ROLL_ROLL_FULLRES_DEF_FRAME_RATE ((double)12.0)
//// Flat Area - Flat Mode - High Resolution Frame
//#define GBMSAPI_DS40I_FLAT_FLAT_FULLRES_MAX_FRAME_RATE ((double)16.0)
//#define GBMSAPI_DS40I_FLAT_FLAT_FULLRES_MIN_FRAME_RATE ((double)2.5)
//#define GBMSAPI_DS40I_FLAT_FLAT_FULLRES_DEF_FRAME_RATE ((double)5.0)

/////////////////////////////////////////////////////////////////////
// DactyScan 84C
//
// Recommendations:
//
// - Use the "GBMSAPI_DS84C_FLAT_FLAT_FULLRES_..."
//		and "GBMSAPI_DS84C_FLAT_FLAT_PREVIEW_..." defines before
//		acquiring flat objects on flat scan area
// - Use the "GBMSAPI_DS84CI_ROLL_ROLL_..." defines before acquiring
//		rolled objects
// - Use the "GBMSAPI_DS84C_ROLL_FLAT_..." defines before acquiring
//		flat objects on roll scan area
//
// Note1:
//	using a frame rate value higher than nominal will lead to
//	an increasing CPU resource consumption due to USB traffic and
//	frames elaboration.
/////////////////////////////////////////////////////////////////////
///////////////////////
//// Flat Area - Flat Mode - High Resolution Frame
//#define GBMSAPI_DS84C_FLAT_FLAT_FULLRES_MAX_FRAME_RATE ((double)8.6)
//#define GBMSAPI_DS84C_FLAT_FLAT_FULLRES_MIN_FRAME_RATE ((double)1.4)
//#define GBMSAPI_DS84C_FLAT_FLAT_FULLRES_DEF_FRAME_RATE ((double)4.3)
//// Flat Area - Flat Mode - Low Resolution Frame
//#define GBMSAPI_DS84C_FLAT_FLAT_PREVIEW_MAX_FRAME_RATE ((double)27.0)
//#define GBMSAPI_DS84C_FLAT_FLAT_PREVIEW_MIN_FRAME_RATE ((double)3.1)
//#define GBMSAPI_DS84C_FLAT_FLAT_PREVIEW_DEF_FRAME_RATE ((double)13.5)
//// Roll Area - Roll Mode - High Resolution Frame
//#define GBMSAPI_DS84C_ROLL_ROLL_FULLRES_MAX_FRAME_RATE ((double)25.0)
//#define GBMSAPI_DS84C_ROLL_ROLL_FULLRES_MIN_FRAME_RATE ((double)2.9)
//#define GBMSAPI_DS84C_ROLL_ROLL_FULLRES_DEF_FRAME_RATE ((double)12.5)
//// Roll Area - Flat Mode - High Resolution Frame
//#define GBMSAPI_DS84C_ROLL_FLAT_FULLRES_MAX_FRAME_RATE ((double)25.0)
//#define GBMSAPI_DS84C_ROLL_FLAT_FULLRES_MIN_FRAME_RATE ((double)2.9)
//#define GBMSAPI_DS84C_ROLL_FLAT_FULLRES_DEF_FRAME_RATE ((double)12.5)

// VER 2.8.0.0
/////////////////////////////////////////////////////////////////////
// MC517
//
// Recommendations:
//
// - Use the "GBMSAPI_MC517_FLAT_FLAT_FULLRES_..."
//		and "GBMSAPI_MC517_FLAT_FLAT_PREVIEW_..." defines before
//		acquiring flat objects on flat scan area
// - Use the "GBMSAPI_MC517I_ROLL_ROLL_..." defines before acquiring
//		rolled objects
// - Use the "GBMSAPI_MC517_ROLL_FLAT_..." defines before acquiring
//		flat objects on roll scan area
/////////////////////////////////////////////////////////////////////
///////////////////////

//// Flat Area - Flat Mode - High Resolution Frame
//#define GBMSAPI_MC517_FLAT_FLAT_FULLRES_MAX_FRAME_RATE ((double)4.2)
//#define GBMSAPI_MC517_FLAT_FLAT_FULLRES_MIN_FRAME_RATE ((double)0.2)
//#define GBMSAPI_MC517_FLAT_FLAT_FULLRES_DEF_FRAME_RATE ((double)2.1)
//// Flat Area - Flat Mode - Low Resolution Frame
//#define GBMSAPI_MC517_FLAT_FLAT_PREVIEW_MAX_FRAME_RATE ((double)9.4)
//#define GBMSAPI_MC517_FLAT_FLAT_PREVIEW_MIN_FRAME_RATE ((double)1.0)
//#define GBMSAPI_MC517_FLAT_FLAT_PREVIEW_DEF_FRAME_RATE ((double)4.7)
//// Roll Area - Roll Mode - High Resolution Frame - GA STANDARD
//#define GBMSAPI_MC517_ROLL_ROLL_FULLRES_MAX_FRAME_RATE_GA ((double)35.0)
//#define GBMSAPI_MC517_ROLL_ROLL_FULLRES_MIN_FRAME_RATE_GA ((double)1.0)
//#define GBMSAPI_MC517_ROLL_ROLL_FULLRES_DEF_FRAME_RATE_GA ((double)17.5)
//// Roll Area - Roll Mode - High Resolution Frame - IQS STANDARD
//#define GBMSAPI_MC517_ROLL_ROLL_FULLRES_MAX_FRAME_RATE_IQS ((double)25.4)
//#define GBMSAPI_MC517_ROLL_ROLL_FULLRES_MIN_FRAME_RATE_IQS ((double)1.0)
//#define GBMSAPI_MC517_ROLL_ROLL_FULLRES_DEF_FRAME_RATE_IQS ((double)12.7)
//// Roll Area - Flat Mode - High Resolution Frame - GA STANDARD
//#define GBMSAPI_MC517_ROLL_FLAT_FULLRES_MAX_FRAME_RATE_GA ((double)35.0)
//#define GBMSAPI_MC517_ROLL_FLAT_FULLRES_MIN_FRAME_RATE_GA ((double)1.0)
//#define GBMSAPI_MC517_ROLL_FLAT_FULLRES_DEF_FRAME_RATE_GA ((double)17.5)
//// Roll Area - Flat Mode - High Resolution Frame - IQS STANDARD
//#define GBMSAPI_MC517_ROLL_FLAT_FULLRES_MAX_FRAME_RATE_IQS ((double)25.4)
//#define GBMSAPI_MC517_ROLL_FLAT_FULLRES_MIN_FRAME_RATE_IQS ((double)1.0)
//#define GBMSAPI_MC517_ROLL_FLAT_FULLRES_DEF_FRAME_RATE_IQS ((double)12.7)

// VER2700
///////////////////////////////////////////////////////////////////////
//	FIRMWARE INFO RETRIEVING CAPABILITY
//
//	Some GreenBit scanners, such as the DS84(n), allow the user to
//	retrieve information about the device name and fw version.
//	See also GBMSAPI_GetDeviceNameAndVersion(...) function.
//
//	In order to know if this feature is supported, the application
//	should check the <DeviceFeatures> parameter of
//	GBMSAPI_GetDeviceFeatures(...) function and see whether
//	GBMSAPI_DF_FW_INFO_RETRIEVE bit is set.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_DF_FW_INFO_RETRIEVE		((unsigned int)0x00000040)

// VER 2.7.1.0
///////////////////////////////////////////////////////////////////////
//	IQS ROLL SUPPORTED
//
//	Device having this feature support rolled images acquisition with
//	IQS standard.
//
//	In order to know if this feature is supported, the application
//	should check the <DeviceFeatures> parameter of
//	GBMSAPI_GetDeviceFeatures(...) function and see whether
//	GBMSAPI_DF_FRAME_RATE_SETTING bit is set.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_DF_ROLL_AREA_IQS			((unsigned int)0x00000080)

// VER 2.7.1.0
///////////////////////////////////////////////////////////////////////
//	GA ROLL SUPPORTED
//
//	Device having this feature support rolled images acquisition with
//	GA (China) standard.
//
//	In order to know if this feature is supported, the application
//	should check the <DeviceFeatures> parameter of
//	GBMSAPI_GetDeviceFeatures(...) function and see whether
//	GBMSAPI_DF_FRAME_RATE_SETTING bit is set.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_DF_ROLL_AREA_GA				((unsigned int)0x00000100)

//////////////////////////////////////////////////////////////////////
////// AVAILABLE IMAGE INFO //////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//	Generally and depending on the object type to
//	be scanned, Green Bit devices provide some useful information about the image being 
//	captured, for example concernig the fingerprint size or contrast, or the lower half
//	palm Completeness.
//	This info can be useful for those applications, where it is important
//	to perform a rough estimation of the image being captured.
//
//	The values defined in the following list are intentionally given
//	as bit mask values (in the sense that they are choosen so that
//  for each of them only a bit of an unsigned int -4 bytes-
//	field is set to 1).
//	In order to know what info are supported for the specific type of the 
//	given object to be scanned, use 
//	GBMSAPI_GetAvailableImageInfo(...) function.

#define GBMSAPI_AII_FINGERPRINT_SIZE				((unsigned int)0x00000001)
#define GBMSAPI_AII_FINGERPRINT_CONTRAST			((unsigned int)0x00000002)
#define GBMSAPI_AII_LOWER_HALF_PALM_COMPLETENESS	((unsigned int)0x00000004)

//////////////////////////////////////////////////////////////////////
////// OPTIONAL EXTERNAL EQUIPMENT ///////////////////////////////////
//////////////////////////////////////////////////////////////////////
//	The values defined in the following list are intentionally given
//	as bit mask values (in the sense that they are choosen so that
//	for each of them only a bit of an unsigned int -4 bytes-
//	field is set to 1).
//	In order to know what external equipment is supported by each 
//	Green Bit device, use GBMSAPI_GetOptionalExternalEquipment(...) 
//	function.

///////////////////////////////////////////////////////////////////////
//	PEDAL
//
//	The pedal plugged to some GreenBit devices (like Poliscan2 or MS<X>)
//	allows:
//
//	1)	During the FLAT objects acquisition process:
//		To Stop the preview phase and to switch to the final image acquisition
//		(no need for the operator to press buttons on a monitor).
//
//	2)	During the ROOL objects acquisition process (valid only if
//		GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP is set and is supported
//		by the scanner):
//		To stop the preview phase and to switch to the rolled image
//		composition (no need for the operator to press buttons on a
//		monitor).
//
//	3)	Outside the acquisition process:
//		The application is able to know when the pedal has been
//		pressed (see GBMSAPI_GetPedalState(...) function) and
//		to carry out some actions as a consequence of this pressure.
//
//	In order to know if this feature is supported, the application
//	should check the <OptionalExternalEquipment> parameter of
//	GBMSAPI_GetOptionalExternalEquipment(...) function and see whether
//	GBMSAPI_OED_PEDAL bit is set.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_OED_PEDAL						((unsigned int)0x00000001)

///////////////////////////////////////////////////////////////////////
//	SOUND
//
//	Some GreenBit devices (like DactyScan84) are provided with an
//	internal beeper, controllable via software. It can be useful in
//	order to instruct the user (for example when putting the finger
//	on the scanner and so on).
//
//	In order to know if this feature is supported, the application
//	should check the <OptionalExternalEquipment> parameter of
//  GBMSAPI_GetOptionalExternalEquipment(...) function and see whether
//	GBMSAPI_OED_SOUND bit is set.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_OED_SOUND						((unsigned int)0x00000002)

///////////////////////////////////////////////////////////////////////
//	FINGER INDICATOR LED VISUAL USER INTERFACE
//
//	DactyScan84 and DactyScan40 devices are provided with 3-color 
//	(yellow/green/red) LED indicators on the front panel: 
//
//	-	10 fingers: it should be used to indicate which finger(s) is (are) 
//		expected to be currently acquired
//	-	1 roll (only on DactyScan84): it should be turned on to indicate 
//		the acquisition of ROLLED objects
//	-	1 status (only on DactyScan84): it should be turned by default 
//		on green at power-on and when necessary it can be reprogrammed  
//		by the application
//
//	For more information see GBMSAPI_SetFingerIndicatorVUIState(...)
//	function.
//
//	In order to know if this feature is supported, the application
//	should check the <OptionalExternalEquipment> parameter of
//	GBMSAPI_GetOptionalExternalEquipment(...) function and see whether
//	GBMSAPI_OED_VUI_LED bit is set.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_OED_VUI_LED					((unsigned int)0x00000004)

///////////////////////////////////////////////////////////////////////
//	LCD DISPLAY VISUAL USER INTERFACE
//
//	On some devices (like MS500 and MS1000) a graphical touch-screen
//	LCD display is provided, in order to offer a more complete user
//	interface.
//	Three screens can appear:
//
//	- LOGO SCREEN: it is the initial screen; the Green Bit logo
//		is shown
//
//	- ACQUISITION SCREEN: it is the screen that should be displayed
//		during the image acquisition process (by using this SDK, it
//		is automatically shown)
//
//	- STOP SCREEN: it is the screen that should be displayed
//		after the image acquisition process
//
//	On these screens, several operations are allowed via software.
//	For example (do not consider this list as complete, look at the
//	documentation):
//
//	- The application can check via a polling mechanism if any
//		button among those that are shown has been pressed
//
//	- A "Start" button on the LOGO SCREEN can be displayed
//
//	- An "OK" button on the STOP SCREEN can be displayed
//
//	- The application can decide whether LOGO SCREEN or STOP SCREEN
//		should be displayed after the acquisition stopping
//
//	- Some quality parameters (like contrast, lower half palm
//		completeness, rolling artefacts size and so on) can be
//		displayed both on the ACQUISITION and on the STOP SCREEN
//
//	- Several messages can be displayed (both predefined and
//		customized) on the ACQUISITION and on the STOP SCREEN
//
//	In order to know if this feature is supported, the application
//	must check the <OptionalExternalEquipment> parameter of
//	the GBMSAPI_GetOptionalExternalEquipment(...) function and see whether
//	the GBMSAPI_OED_VUI_LCD bit is set.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_OED_VUI_LCD					((unsigned int)0x00000008)

// VER 2700
//////////////////////////////////////////////////////////////////////
////// FRAME RATE OPTIONS ////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//	The values defined in the following list are intentionally given
//	as bit mask values (in the sense that they are choosen so that
//	for each of them only a bit of an unsigned int -4 bytes-
//	field is set to 1).
//	The following defines represent the options that should be
//	passed to the "GBMSAPI_SetFrameRateComplete(...)" and
//	"GBMSAPI_GetFrameRateRange(...)" functions (for
//	further details see them in the GBMSAPI_Library.h file).

///////////////////////////////////////////////////////////////////////
//	ROLL AREA
//
//	This bit is used to select the scanner area the frame rate is set
//	for. By setting this bit the ROLL AREA (see "FLAT FINGER ON ROLL
//	SCAN AREA" paragraph) for the frame rate setting is requested.
//
//	Bit Meaning:
//	-	1:	Set frame rate for ROLL AREA
//	-	0:	Set frame rate for FLAT AREA
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_FRO_ROLL_AREA					((unsigned int)0x00000001)

///////////////////////////////////////////////////////////////////////
//	FULL RESOLUTION MODE
//
//	This bit is used to select the scan mode (preview or full 
//	resolution - see ACQUISITION PROCESS DESCRIPTION paragraph) the
//	frame rate is set for. By setting this bit the FULL RESOLUTION mode
//	for the frame rate setting is requested.
//
//	NOTE: This option is taken in account only when FLAT AREA is
//		selected for the GBMSAPI_FRO_ROLL_AREA bit
//
//	Bit Meaning:
//	-	1:	Set frame rate for FULL RESOLUTION MODE
//	-	0:	Set frame rate for PREVIEW MODE
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_FRO_FULL_RESOLUTION_MODE		((unsigned int)0x00000002)

///////////////////////////////////////////////////////////////////////
//	FLAT OBJECT ON ROLL AREA
//
//	This bit is specifically referred to acquitions on the ROLL AREA 
//	(see "FLAT FINGER ON ROLL SCAN AREA" paragraph), and is used in the 
//	"GBMSAPI_GetFrameRateRange(...)" function, because for some devices
//	the default frame rate value depends on the object to be acquired
//	also. By setting this bit the default frame rate for FLAT objects
//	acquisition is requested.
//
//	NOTE: This option is taken in account only when ROLL AREA is
//		selected for the GBMSAPI_FRO_ROLL_AREA bit
//
//	Bit Meaning:
//	-	1:	Get frame rate range for FLAT OBJECT ON ROLL AREA
//	-	0:	Get frame rate range for ROLLED object
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_FRO_FLAT_OBJECT_ON_ROLL_AREA	((unsigned int)0x00000004)

/****************************************************************************
*																			*
*					PART 3													*
*																			*
*				ACQUISITION PROCESS											*
*																			*
****************************************************************************/

//////////////////////////////////////////////////////////////////////
///// ACQUISITION PROCESS DESCRIPTION ////////////////////////////////
//////////////////////////////////////////////////////////////////////
//	The acquisition process is normally divided into two logical
//	phases:
//
//	-	PREVIEW:
//		during this phase the user puts the object to be scanned
//		(finger, palm, etc) on the scanning window in a manner that 
//		ensures the correct position and a good image quality.
//		During this phase normally only a visual feedback is needed.		
//
//	-	FULL RESOLUTION:
//		Once the correct position and a good image quality have been
//		achieved, the real acquisition process begins. This phase
//		consists of a single image acquistion for FLAT objects, and in
//		the composition of multiple images coming from the scanner for
//		ROLLED ones.
//
//	Acquisition phase can be detected by the application by means of
//	the <EventInfo> parameter of the 
//	GBMSAPI_AcquisitionEventsManagerCallback(...)	function (see the
//	ACQUISITION EVENTS MANAGER CALLBACK and the EVENT INFO sections)
//
//	The acquisition process can be performed in two ways:
//
//	-	MANUAL:
//		The operator himself decides when the image is good enough to
//		switch from the PREVIEW phase to the FULL RESOLUTION one (see
//		also the GBMSAPI_StopAcquisition(...) function). See the FLAT 
//		OBJECTS MANUAL ACQUISITION chapter (for FLAT objects) and the
//		ROLLED FINGERPRINT ACQUISITION section (for ROLL objects)
//		for more detailed information.
//
//	-	AUTO-CAPTURE:
//		During the PREVIEW phase, images coming from the scanner are
//		analyzed by the software, that decides when phase switching
//		has to occur. Decision criteria are described below: see the
//		FLAT AUTO-CAPTURE PROCESS chapter (for FLAT objects) and the
//		ROLLED FINGERPRINT ACQUISITION section (for ROLL objects)
//		for more detailed information.
//
//	NOTE1:
//		FLAT AUTO-CAPTURE mode supporting depends on device and object type 
//		to be scanned. For more detailed info see device specific
//		documentation and the ACQUISITION OPTIONS section.
//
//	VER2500: added manual roll preview stop for DS84 device
//	NOTE2:
//		ROLLED objects acquisition is performed only in AUTO-CAPTURE
//		mode. Exceptions to this rule are those devices supporting
//		GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP acquisition option.

//////////////////////////////////////////////////////////////////////
//	FLAT OBJECTS MANUAL ACQUISITION
//
//	In this acquisition mode it's up to the operator, by means of a
//	visual feedback for example, or by means of information about
//	acquired image (see AVAILABLE IMAGE INFO section), to decide when
//	the switching from the PREVIEW phase to the FULL-RESOLUTION one
//	has to occur.
//	This can be made in the application simply by calling the
//	GBMSAPI_StopAcquisition(...) function. When the acquisition has been
//	stopped, the applicaion is advised by means of the 
//	GBMSAPI_EI_ACQUISITION_STOP bit of the <EventInfo> parameter of 
//	the GBMSAPI_AcquisitionEventsManagerCallback(...) function (see the
//	ACQUISITION EVENTS MANAGER CALLBACK and the EVENT INFO sections).
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
//	FLAT AUTO-CAPTURE PROCESS
//
//	For FLAT objects AUTO-CAPTURE mode consists in the evaluation of
//	the image flow from the scanner during the PREVIEW phase and in
//	stopping it when the best image has been acquired.
//	For those objects the PREVIEW phase is divided into two 
//	sub-phases (see "AUTO-CAPTURE PHASE" section for defined values
//	and see also the GBMSAPI_GetAutoCapturePhase(...) and the 
//	GBMSAPI_SetSelectImageTimeout(...) functions)
//
//	- WAIT-FINGER:
//		AUTO-CAPTURE process is in this sub-phase when no object has
//		been put on the scanning surface. When an object appears on
//		the scanning surface, process switches to the next sub-phase.
//
//	- SELECT-IMAGE:
//		AUTO-CAPTURE process is in this sub-phase when an object is
//		detected on the scanning surface.
//		When in this phase, the image flow is analyzed by the software,
//		in order to get the best image by selecting the images with 
//		the highest contrast and the biggest fingerprint size.
//		Switching from this sub-phase to the FULL-RESOLUTION phase
//		normally occurs in two cases (automatic stop: see STOP TYPE
//		chapter in EVENT INFO section):
//		
//		1)	when in a certain period of time there is no significant 
//			increase of fingerprint size and contrast (stabilization
//			condition has been reached) 
//
//		2)	when the SELECT-IMAGE phase lasts more than a given
//			time (timeout expiration condition): see also
//			GBMSAPI_SetSelectImageTimeout(...) function. 
//
//	NOTE1:
//		The application can stop the AUTO-CAPTURE process by means
//		of the GBMSAPI_StopAcquisition(...) function (manual stop:see STOP
//		TYPE chapter in EVENT INFO section), but in such a case
//		image evaluation is up to the operator.
//
//	NOTE2:
//		Depending on the device, AUTO-CAPTURE process can be
//		temporarily kept appended by means of the 
//		GBMSAPI_SetAutoCaptureBlocking(...) function (see also
//		DEVICE FEATURES section). When AUTO-CAPTURE 
//		process is blocked, the switching from the PREVIEW phase
//		to the FULL-RESOLUTION one is not performed (unless the
//		GBMSAPI_StopAcquisition(...) is called, see NOTE1).
//
//	NOTE3:
//		When acquiring GBMSAPI_SBT_FLAT_LOWER_HALF_PALM object, also the
//		pattern Completeness is taken in account in order to decide
//		whether switching from the PREVIEW phase to the 
//		FULL-RESOLUTION one: the pattern Completeness must go over a
//		certain threshold (90% by default) that can be adjusted by
//		the application (see also 
//		GBMSAPI_SetLowerHalfPalmCompletenessThreshold(...) function).
//		Pattern Completeness can be checked by the application (or
//		the operator) by means of the 
//		GBMSAPI_GetLowerHalfPalmCompleteness(...) function. If for some
//		reason the pattern Completeness threshold cannot be reached,
//		acquisition should be stopped manually.
//////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////
////// ROLLED FINGERPRINT ACQUISITION ////////////////////////////////
//////////////////////////////////////////////////////////////////////
//	For ROLLED objects acquisition, the process consists in the
//	composition of several consecutive FULL-RESOLUTION images
//	coming from the scanner in order to get a composite fingerprint
//	image.
//	Present software implementation of "rolled fingerprint image 
//	composition" has built-in capability to control correct rolling 
//	procedure flow.
//
//	It is intended that every rolling procedure is performed as 
//	follows in several phases (corresponding to the acquisition 
//	phases, see AUTO-CAPTURE PROCESS chapter)
//
//	- PHASE 1 (PREVIEW)
//		In the PREVIEW phase image flow is displayed without 
//		generating the composite image, thus allowing the operator to
//		put the finger in the correct position before starting the
//		roll (composition) procedure.
//		This phase can be stopped manually (by using pedal after the
//		correct position has been achieved, for those devices 
//		supporting this feature, see GBMSAPI_GetScanOptions())
//		or automatically. 
//		Automatic preview stop can be performed in two ways: CENTER and
//		SIDE.
//		For automatic mode, multiple finger positioning attempts are 
//		allowed.
//
//		1) PEDAL (Manual preview stop):
//			wait a little after scanning start, then place the finger 
//			in the desired position, press the pedal and hold there
//			until the preview end signal (see below)
//
//		2) CENTER (Automatic preview stop):
//			wait a little after scanning start, then place the finger 
//			core on the center of the scanning surface, then rotate
//			finger to either side and hold there until the preview end
//			signal (see below)
//
//		3) SIDE (Automatic preview stop):
//			wait a little after scanning start, then place the left or
//			right lateral side of the finger to the correspondingly
//			left or right part of scanning window and hold there until
//			the preview end signal (see below)
//
//	VER2500: added manual roll preview stop for DS84 device
//		4) MANUAL (Manual preview stop):
//			wait a little after scanning start, then place the finger 
//			in the desired position, make the application to call
//			the GBMSAPI_ROLL_StopPreview() function
//
//		NOTE1:
//			It is important to select properly the point on scanning
//			window where the finger should be placed to at the 
//			beginning of the acquisition, thus ensuring that there is
//			enough space to perform finger rolling "from nail to 
//			nail".
//			If rolled finger goes out of scanning window borders, the 
//			corresponding diagnostic message will appear to inform the
//			operator about improper rolling procedure.
//
//		For the options to be set in order to choose the roll PREVIEW
//		stop mode, see the ACQUISITION OPTIONS section.
//
//		For automatic mode, the PREVIEW phase ends when the finger stays
//		still for more than a specified amount of time (see 
//		GBMSAPI_ROLL_SetPreviewTimeout(...) function).
//		The preview end signal is defined by the application, that is
//		advised by the system when the PREVIEW phase ends (see 
//		ACQUISITION EVENTS MANAGER CALLBACK section, in particular the
//		GBMSAPI_AE_PREVIEW_PHASE_END event). 
//		Then, the procedure switches to PHASE 2 (ROLL-COMPOSITION).
//	
//		NOTE2:
//			It is important to ensure that the finger is not placed on 
//			the scanning window when acquisition is beginning. It 
//			guarantees that the first live-scanned image is always 
//			"empty" and can be used as a reference to detect actual
//			finger appearance. If a finger has been placed on the 
//			scanning window before scanning start or scanning window 
//			is too much dirty, the corresponding diagnostic message will 
//			appear to inform the operator about improper rolling 
//			procedure.
//
//		NOTE3:
//			If the acquisition is manually stopped by means of the 
//			GBMSAPI_StopAcquisition(...) function during the PREVIEW phase,
//			no composite image has been acquired. Therefore acquisition must
//			be repeated.
//
//		NOTE4:
//			Through the GBMSAPI_AO_NO_ROLL_PREVIEW option (see the
//			ACQUISITION OPTIONS section for further details), the
//			PREVIEW phase duration for ROLLED objects acquisition can
//			be put to 0. In this case just few moments after the acquisition
//			is started, the system goes to the ROLL-COMPOSITION phase
//			(see PHASE 2 -ROLL-COMPOSITION- paragraph)
//
//	PHASE 2 (ROLL-COMPOSITION). 
//		Roll the finger gently to the opposite lateral side, 
//		moderately pressing.
//
//		NOTE4: 
//			Rolling phase duration should be about 2 second. If
//			rolling is too much fast, the corresponding diagnostic
//			message will appear to inform the operator about improper
//			rolling procedure. 
//
//		NOTE5:
//			It is important to roll the finger widely in a "from nail
//			to nail" manner. 
//			If the finger has been rolled too narrow, the 
//			corresponding diagnostic message will appear to inform the
//			operator about improper rolling procedure.
//
//		NOTE6:
//			It is important to roll the finger only in one selected 
//			direction (to the left or to the right), thus avoiding at
//			the	passage from flat to roll phase even little roll to 
//			the opposite direction. Otherwise, visible "seam" in final 
//			composite image will appear and the corresponding 
//			diagnostic message will be generated to inform the 
//			operator about improper rolling procedure.	
//
//	PHASE 3 (REMOVE). 
//		Remove the finger from the scanning window when the whole 
//		required finger surface is rolled.
//	
//		NOTE6:
//			This phase is not mentioned into the AUTO-CAPTURE PROCESS
//			description, but actually corresponds to the acquisition
//			stop.

//////////////////////////////////////////////////////////////////////
////// ACQUISITION OPTIONS ///////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//	The acquisition options consist in a bit mask. Each bit of this
//	mask represents an option: if the bit is 1 then the option is
//	requested. This mask must be passed to the GBMSAPI_StartAcquisition(...)
//	function.
//	In order to choose a given option set the corresponding bit (ex:
//	
//	MyOptionsMask |= GBMSAPI_AO_<OptionVal>;
//
//	where <OptionVal> is the ID of the option)
//
//	If some of the options are not supported by the given device for 
//	the given type to be scanned, the option will be ignored but 
//	the acquisition will start anyway and the application will be 
//	warned: in the GBMSAPI_AcquisitionEventsManagerCallback(...) 
//	function a similar mask is returned as an output value, indicating
//	whether the option has been accepted (corresponding bit = 0) or
//	ignored (corresponding bit = 1).
//	It's responsibility of the application to check whether the option 
//	has been accepted and to take the appropriate measures (some
//	recommendations are given within the specific option comments). 
//	It's recommended to treat the no acceptance of the option as a
//	warning message.
//	

///////////////////////////////////////////////////////////////////////
//	FLAT AUTO-CAPTURE MODE
//
//	The auto-capture feature depends on the device and on the type to
//	be scanned. By setting this bit the AUTO-CAPTURE mode for FLAT
//	objects is requested.
//	
//
//	NOTE1:
//		If the AUTO-CAPTURE mode has been requested but it's not
//		available by the scanner for the current object type to be 
//		scanned, it's HIGHLY recommended to advise the operator (by 
//		means of a TextBox in the application window for example: don't 
//		use MessageBox or similar, otherwise application, and 
//		consequently the acquisition thread, will be kept appended) that
//		the acquisition has to be manually stopped. 
//
//	NOTE2:
//		For ROLLED objects this option will be ignored.
//
//	Bit Meaning:
//	-	1:	Use auto-capture
//	-	0:	Do not use auto-capture
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_AO_AUTOCAPTURE				((unsigned int)0x00000001)

///////////////////////////////////////////////////////////////////////
//	NO ROLL PREVIEW
//
//	By default, ROLLED objects acquisition process is composed by
//	PREVIEW and FULL RESOLUTION phases. If device supports this option,
//	if this bit is set PREVIEW duration is set to 0.
//
//	NOTE1:
//		For logical compatibility with the general acquisition process,
//		the GBMSAPI_AE_PREVIEW_PHASE_END event is raised anyway.
//
//	Bit Meaning:
//	-	1:	No roll preview
//	-	0:	Preview for ROLLED objects
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_AO_NO_ROLL_PREVIEW			((unsigned int)0x00000002)

///////////////////////////////////////////////////////////////////////
//	ROLL PREVIEW STOP MODE
//
//	VER2500: added manual roll preview stop for DS84 device
//
//	Roll preview can be stopped (for "stopped" in this case is
//	intended that acquisition phase switches from PREVIEW to
//	FULL-RESOLUTION) in two ways: AUTOMATIC or MANUAL (by
//	using the pedal or calling the GBMSAPI_ROLL_StopPreview).
//	This option will be taken in care only for ROLLED fingerprints
//	acquisition and if scanner supports the 
//	GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP feature, otherwise roll
//	preview stop will be considered always AUTOMATIC.
//	By default the AUTOMATIC preview is performed. Application can 
//	choose manual preview stop by setting this bit.
//
//	NOTE1:
//		For ROLLED objects and scanners supporting the 
//		GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP feature this option will 
//		be always accepted whatever it is (1 or 0), while for FLAT 
//		objects this option is ignored,	so that application doesn't 
//		need to check whether it has been accepted or not during
//		acquisition.
//
//	NOTE2:
//		if the GBMSAPI_AO_NO_ROLL_PREVIEW bit has been set, this
//		option is not taken in care.
//
//	Bit Meaning:
//	-	1:	MANUAL Roll Preview Stop
//	-	0:	AUTOMATIC Roll Preview Stop
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP		((unsigned int)0x00000004)

///////////////////////////////////////////////////////////////////////
//	AUTOMATIC ROLL PREVIEW TYPE
//
//	Automatic roll preview can be of two types: CENTER or SIDE (for 
//	more detailed information see ROLLED FINGERPRINT ACQUISITION section.
//	This option will be taken in care only for ROLLED fingerprints
//	acquisition.
//	Normally the SIDE preview is performed. Application can choose
//	CENTER preview by setting this bit.
//
//	NOTE1:
//		For ROLLED objects this option will be always accepted whatever
//		it is (1 or 0), while for FLAT objects this option is ignored,
//		so that application doesn't need to check whether it has been
//		accepted or not during acquisition.
//
//	NOTE2:
//		if the GBMSAPI_AO_NO_ROLL_PREVIEW bit has been set, this
//		option is not taken in care.
//
//	Bit Meaning:
//	-	1:	CENTER Roll Preview
//	-	0:	SIDE Roll Preview
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_AO_ROLL_PREVIEW_TYPE			((unsigned int)0x00000008)

///////////////////////////////////////////////////////////////////////
//	PREVIEW IMAGE RESOLUTION
//
//	In order to save USB trafic and image processing time in the
//	application (since only visual feedback is needed, as told above),
//	normally images coming from scanner during the PREVIEW phase are
//	provided in a lower resolution (see also EVENT INFO chapter),
//	while the FULL-RESOLUTION images are at full scanner resolution.
//	Depending on the selected object to scan type for the current 
//	device, application can choose to get full resolution images by 
//	setting this bit.
//
//	NOTE1:
//		It's responsibility of the application to check whether this
//		option is supported or not. In this latter case the size
//		of the acquired image can be different respect to the desired
//		one	and this has to be taken in account for visualization. 
//		In each case it's highly recommended to check the <FrameSizeX>
//		and <FrameSizeY> returned by the 
//		GBMSAPI_AcquisitionEventsManagerCallback(...)function. In
//		this case the application directly knows the image size for 
//		the current frame, without caring whether the option is
//		supported or not.
//
//	Bit Meaning:
//	-	1:	Full resolution images in PREVIEW phase
//	-	0:	Preview resolution images in PREVIEW phase
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_AO_FULL_RES_PREVIEW			((unsigned int)0x00000010)

//////////////////////////////////////////////////////////////////////
//	FLAT SINGLE FINGER ON ROLL SCAN AREA
//
//	The object types described in the "SCANNABLE BIOMETRIC TYPES" have
//	to be taken on a well defined area of the scanning window in order
//	to satisfy image size and other requirements: objects like
//	rolled fingerprint have to be taken on a smaller area (1.6'x1.6'),
//	while objects like the plain 4-fingers require the whole available
//	area of the scanning window. There are two supported scanning 
//	areas: flat area, that corresponds to the whole scanning window, 
//	and roll area, that is the area where rolled fingerprints must be
//	taken.
//	For DS84, MS500 and MS1000 devices roll area is marked on the 
//	scanning window (see specific documentation), while for the 
//	remaining devices different areas correspond to different scanners
//	(for example, for Visascan3 the roll area corresponds to the Roll
//	scanner and the flat area corresponds to the palm scanner - if the
//	object that has to be taken is the GBMSAPI_SBT_PHOTO, the whole area
//	of the photo scanner will be used):
//
//	Roll area:
//
//	-	for DS84, MS500 and MS1000 devices:
//		the roll area, marked on the scanning window. The finger will
//		be put on this region during the acquisition process.
//
//	-	for VS3,PS2 and DS40 devices:
//		the Roll scanner. The finger will be put on this scanner 
//		during the acquisition process.
//
//	Flat area:
//
//	-	for DS84, MS500 and MS1000 devices:
//		the whole scanning window. The finger(s) (or palm) will
//		be put on this region during the acquisition process.
//
//	-	for PS2 device:
//		the PALM scanner. The finger(s) (or palm) will be put on this
//		scanner during the acquisition process.
//
//	-	for VS3 device:
//		1)	for objects different from GBMSAPI_SBT_PHOTO the same as PS2
//			device
//		2)	for GBMSAPI_SBT_PHOTO object represents the PHOTO scanner. The
//			document to be scanned will be put on this scanner during
//			the acquisition process.
//
//	-	for DS40 device:
//		the Roll scanner, since no other scanner is mounted on this
//		device.
//
//	By setting this bit, the Roll area for FLAT SINGLE FINGER is
//	requested as scanning window.
//
//	NOTE1:
//		This option is taken in care only for FLAT SINGLE FINGER
//		objects acquisition. If device does not support this
//		option and the bit is set, the object will be scanned on the
//		whole scan area.
//
//	Bit Meaning:
//	-	1:	FLAT SINGLE FINGER on Roll area
//	-	0:	FLAT SINGLE FINGER on Flat area
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA	((unsigned int)0x00000020)

// VER 2600
///////////////////////////////////////////////////////////////////////
//	HIGH SPEED FOR FULL RESOLUTION PREVIEW
//
//	When the GBMSAPI_AO_FULL_RES_PREVIEW is set, this bit allows to
//	get a higher frame rate during the full resolution preview, but on
//	some scanners only (at moment, this MC500 and MSC500 devices)
//
//	NOTE1:
//		If scanner does not support this functionality, simply this
//		option is not taken in care
//
//	Bit Meaning:
//	-	1:	high speed for Full resolution images in PREVIEW phase
//	-	0:	normal speed 
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_AO_HIGH_SPEED_PREVIEW		((unsigned int)0x00000040)


//////////////////////////////////////////////////////////////////////
////// DISPLAY OPTIONS ///////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//	For those devices that support external visual user interface
//	equipments (Finger Indicator LEDs for DS84 and DS40 devices and
//	Graphic LCD for MS500 and MS1000 - see also OPTIONAL EXTERNAL 
//	EQUIPMENT section), during acquisition an automatic display 
//	control is provided.
//	This control consists in displaying messages and information
//	about acquired image - see also AVAILABLE IMAGE INFO section - 
//	or in switching on/off finger indicator LEDs.
//	Default behaviour is the following:
//
//	-	after scanner has been started (see ACQUISITION EVENTS
//		paragraph) the GBMSAPI_OED_VUI_LCD screen is changed
//		to the Acqusition Screen (see LCD DISPLAY VISUAL USER
//		INTERFACE), a
//		GBMSAPI_VILCD_MSG_DIAG_PROGRESS_DONT_PUT_FINGER message and
//		eventual diagnostic messages are displayed. 
//		LEDs on Finger Indicator equipment (GBMSAPI_OED_VUI_LED) are
//		switched off.
//
//	-	after the first frame has been acquired, on the
//		GBMSAPI_OED_VUI_LCD the image representing the object to be
//		scanned (see SCANNABLE OBJECTS section), the Contrast and 
//		completness limits, info about acquired image, a 
//		GBMSAPI_VILCD_MSG_PREVIEW message and eventual diagnostic 
//		messages are displayed. 
//		On the GBMSAPI_OED_VUI_LED equipment the LED(s) corresponding
//		to the object to be acquired are switched on with a yellow
//		light.
//
//	-	after each valid frame (both in PREVIEW and in 
//		FULL-RESOLUTION acquisition phases) on the GBMSAPI_OED_VUI_LCD
//		info about acquired image and eventual diagnostic messages 
//		are updated. 
//		On the GBMSAPI_OED_VUI_LED equipment nothing changes (unless
//		the GBMSAPI_VUI_LED_BlinkDuringAcquisition(...) is not called).
//
//	-	after the PREVIEW phase has come to end, on the 
//		GBMSAPI_OED_VUI_LCD the GBMSAPI_VILCD_MSG_PREVIEW message is
//		replaced with a GBMSAPI_VILCD_MSG_ACQUISITION_PROGRESS message. 
//		On the GBMSAPI_OED_VUI_LED equipment nothing changes (unless
//		the GBMSAPI_VUI_LED_BlinkDuringAcquisition(...) is not called).
//
//	-	after the acquisition has come to an end or an error has
//		been encountered (see ACQUISITION EVENTS paragraph), the
//		control is returned to the developer (finger indicator
//		LEDs are left in last set status).
//
//	This behaviour can be partially customized by means of the options
//	that will be explained in the next paragraphs.
//	The developer can also choose to make the Finger indicator LED(s)
//	blink by means of the 
//	GBMSAPI_VUI_LED_BlinkDuringAcquisition(...) function.
//
//	The values defined in the following list are intentionally given
//	as bit mask values (in the sense that they are choosen in order
//	that for each of them only a bit of an unsigned int -4 bytes-
//	field is set to 1).
//	Each bit of this mask represents an option: if the bit is 1 then
//	the option is requested. This mask must be passed to the 
//	GBMSAPI_StartAcquisition(...) function.
//	In order to choose a given option set the corresponding bit (ex:
//	
//	MyDisplayOptionsMask |= GBMSAPI_DO_<OptionVal>;
//
//	where <OptionVal> is the ID of the option)

///////////////////////////////////////////////////////////////////////
//	CUSTOMIZE CONTRAST
//
//	During FLAT objects acquisition, on the GBMSAPI_OED_VUI_LCD contrast
//	as it's returned by the GBMSAPI_GetFingerprintContrast(...) function
//	is shown. By means of the GBMSAPI_DO_CUSTOMIZED_CONTRAST option the
//	developer can choose to display an alternative contrast value by
//	means of the GBMSAPI_VUI_LCD_SetContrastOnAcqScreen(...) function.
//
//	Bit Meaning:
//	-	1:	Don't show GBMSAPI_GetFingerprintContrast(...) result
//	-	0:	Show GBMSAPI_GetFingerprintContrast(...) result
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_DO_CUSTOMIZED_CONTRAST		((unsigned int)0x00000001)

///////////////////////////////////////////////////////////////////////
//	CUSTOMIZE COMPLETENESS
//
//	During GBMSAPI_SBT_FLAT_LOWER_HALF_PALM objects acquisition, on the
//	GBMSAPI_OED_VUI_LCD palm completeness as it's returned by the 
//	GBMSAPI_GetLowerHalfPalmCompleteness(...) function is shown.
//	By means of the GBMSAPI_DO_CUSTOMIZED_COMPLETENESS option the 
//	developer can choose to display an alternative palm completeness
//	value can be displayed by means of the
//	GBMSAPI_VUI_LCD_SetHLPCompletenessOnAcqScreen(...) function.
//
//	Bit Meaning:
//	-	1:	Don't show GBMSAPI_GetLowerHalfPalmCompleteness(...) result
//	-	0:	Show GBMSAPI_GetLowerHalfPalmCompleteness(...) result
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_DO_CUSTOMIZED_COMPLETENESS	((unsigned int)0x00000002)

///////////////////////////////////////////////////////////////////////
//	FINAL SCREEN
//
//	This option allows the developer to choose which screen type
//	should be set on the LCD (for those devices supporting it) at the
//	end of acquisition, whether the STOP-SCREEN
//	or the LOGO-SCREEN (see the LCD DISPLAY VISUAL USER INTERFACE
//	paragraph)
//
//	Bit Meaning:
//	-	1:	Show STOP-SCREEN
//	-	0:	Show LOGO-SCREEN
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_DO_FINAL_SCREEN				((unsigned int)0x00000004)

//////////////////////////////////////////////////////////////////////
////// ACQUISITION EVENTS MANAGER CALLBACK ///////////////////////////
//////////////////////////////////////////////////////////////////////
//	When the GBMSAPI_StartAcquisition(...) function is called, a thread 
//	is started that takes care of getting images from the device and
//	passing them to the application. This thread lasts until:
//
//	1)	The GBMSAPI_StopAcquisition(...) is called by the application
//		(MANUAL stop - see STOP TYPE chapter in EVENT INFO section)
//
//	2)	An automatic stop condition occurs (see GBMSAPI_AO_AUTOCAPTURE
//		option) (AUTOMATIC stop - see STOP TYPE chapter in EVENT INFO
//		section)
//
//	3)	A heavy error occurs
//
//	4)	On some scanners (eg MS1000) a given timeout has expired
//
//	Whenever a particular acquisition event occurs (see ACQUISITION
//	EVENTS chapter) application is advised by means of a callback
//	function (see ACQUISITION EVENTS MANAGER CALLBACK chapter), that
//	must be written by application developer. In this callback all the
//	actions useful to manage the occured event can be performed.

///////////////////////////////////////////////////////////////////////
// ACQUISITION EVENTS MANAGER CALLBACK
//
// Description: 
//	Here the prototype of the callback that will be called whenever an
//	event from the ones listed below occurs. In order to process them
//	the corresponding functions can be used inside the callback
//	and a pointer to this callback function must be passed as
//	<AcquisitionEventsManagerCallbackPtr> parameter of the
//	GBMSAPI_StartAcquisition(...) function.
//
// Parameters:
//	<OccurredEventCode> (input):
//		Code of the event that caused the callback calling. For the
//		values this parameter can assume and their meaning see the 
//		ACQUISITION EVENTS chapter.
//
//	<AcquisitionErrorCode> (input):
//		If <OccurredEventCode> is equal to GBMSAPI_AE_ACQUISITION_ERROR
//		(see the ACQUISITION EVENTS chapter) in this parameter the
//		error code (see the ERROR CODES section) is reported.
//		Otherwise is always equal to GBMSAPI_ERROR_CODE_NO_ERROR and should not
//		be considered
//
//	<EventInfo> (input):
//		Valid only if <OccurredEventCode> is equal to 
//		GBMSAPI_AE_VALID_FRAME_ACQUIRED or GBMSAPI_AE_ACQUISITION_END.
//		See the EVENT INFO section for more detailed information
//
//	<FramePtr> (input):
//		Valid only if <OccurredEventCode> is equal to 
//		GBMSAPI_AE_VALID_FRAME_ACQUIRED or GBMSAPI_AE_ACQUISITION_END
//		and the GBMSAPI_EI_ACQUISITION_TIMEOUT bit of the
//		<EventInfo> parameter is not set.
//		Here the pixels of the acquired image are stored. Its size is
//		given into the <FrameSizeX> and <FrameSizeY> parameters. It
//		is valid until this function returns. If the image must be
//		used outside this callback, it's higly recommended to copy 
//		the image stored in the <FramePtr> buffer into another buffer
//		allocated by the application.
//
//	<FrameSizeX> (input):
//		Valid only if <OccurredEventCode> is equal to 
//		GBMSAPI_AE_VALID_FRAME_ACQUIRED or GBMSAPI_AE_ACQUISITION_END. It
//		represents the X size (width) of the <FramePtr> image.
//
//	<FrameSizeY> (input):
//		Valid only if <OccurredEventCode> is equal to 
//		GBMSAPI_AE_VALID_FRAME_ACQUIRED or GBMSAPI_AE_ACQUISITION_END. It
//		represents the Y size (height) of the <FramePtr> image.
//
//	<CurrentFrameRate> (input):
//		Valid only if <OccurredEventCode> is equal to 
//		GBMSAPI_AE_VALID_FRAME_ACQUIRED. It represents the actual frame
//		rate in "frames per second" terms.
//		It is recommended to check that during acquisition the actual
//		frame rate is not far from the nominal (see also 
//		<NominalFrameRate>).
//		The possible reason	of frame rate degradation is low PC 
//		resources (CPU, memory, USB) that can deteriorate significantly
//		scanner performance.
//
//	<NominalFrameRate> (input):
//		Valid only if <OccurredEventCode> is equal to 
//		GBMSAPI_AE_VALID_FRAME_ACQUIRED. It represents the nominal frame
//		rate for the active scanner in "frames per second" units.
//
//	<GB_Diagnostic> (input):
//		Valid only if <OccurredEventCode> is equal to 
//		GBMSAPI_AE_VALID_FRAME_ACQUIRED or GBMSAPI_AE_ACQUISITION_END. It
//		is a 32-bit diagnostic word:
//
//			bit value '1' - diagnostic message is present
//			bit value '0' - diagnostic message is absent
//
//		For more details see the DIAGNOSTIC MESSAGES section.
//
//	<UserDefinedParameters> (input):
//		Useful parameters that can be passed by the application to the
//		callback function (for example the main application window
//		handle). It is the same pointer passed as 
//		<AcquisitionEventsManagerParameters> to the 
//		GBMSAPI_StartAcquisition(...) function (see GBMSAPI_Library.h
//		file).
//		In order to leave the largest freedom in developing 
//		this function, <UserDefinedParameters> has been declared 
//		as a void*, so that it can be a simple byte as well as a complex
//		data structure.
//
// Return:
//	- TRUE:		success
//
//	- FALSE:	some errors occurred: acquisition will be
//				immediately stopped. 
//
//	NOTE1:
//		If this function returns FALSE, acquisition thread and scanner
//		will be immediately stopped, so it's strongly recommended to 
//		return FALSE ONLY if something really heavy happened.
//
//	NOTE2:
//		No further images can be got until this function returns. 
//		Therefore, in order to not have too much time between two 
//		consecutive images of the flow, it's recommended that this 
//		callback returns as soon as possible.
///////////////////////////////////////////////////////////////////////
typedef BOOL (__stdcall *GBMSAPI_AcquisitionEventsManagerCallback)
(
	unsigned int OccurredEventCode,
	int GetFrameErrorCode,
	unsigned int EventInfo,
	unsigned char *FramePtr,
	int FrameSizeX,
	int FrameSizeY,
	double CurrentFrameRate,
	double NominalFrameRate,
	unsigned int GB_Diagnostic,
	void *UserDefinedParameters
);


//////////////////////////////////////////////////////////////////////
////// ACQUISITION EVENTS ////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//	The values defined in the following list are intentionally given
//	as bit mask values (in the sense that they are choosen in order
//	that for each of them only a bit of an unsigned int -4 bytes-
//	field is set to 1).

///////////////////////////////////////////////////////////////////////
//	SCANNER STARTED
//
//	This event is generated after the scanner has been started and 
//	before the first frame acquisition.
//	In order to perform user defined operations like displaying messages
//	or to set some functionalities (listed in the
//	AFTER START FUNCTIONS section of the the GBMSAPI_Library.h file)
//	like clipping region size, roll preview timeout, etc. the 
//	corresponding methods can be called when this event occurs.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_AE_SCANNER_STARTED			((unsigned int)0x00000001)

///////////////////////////////////////////////////////////////////////
//	VALID FRAME ACQUIRED
//
//	This event is generated whenever a valid frame (both in PREVIEW and
//	in FULL-RESOLUTION acquisition phases - see ACQUISITION PROCESS 
//	DESCRIPTION section) has been acquired.
//	In order to process images coming from the scanner and to retrieve
//	information about them (see ACQUISITION CALLBACK FUNCTIONS FOR 
//	ACQUIRED FRAME section in the GBMSAPI_Library.h file for a list of 
//	them), the corresponding functions can be called when this event
//	occurs.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_AE_VALID_FRAME_ACQUIRED		((unsigned int)0x00000002)

///////////////////////////////////////////////////////////////////////
//	PREVIEW PHASE END
//
//	This event is generated when the PREVIEW phase is switched to the
//	FULL-RESOLUTION one (see ACQUISITION PROCESS DESCRIPTION section).
//	In order to signal to the operator (by means of a TextBox in the 
//	application window for example: don't use MessageBox or similar,
//	otherwise application, and consequently the acquisition thread,
//	will be kept appended) that the PREVIEW phase has come to an end,
//	the appropriate actions to be taken can be performed when this
//	event occurs.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_AE_PREVIEW_PHASE_END			((unsigned int)0x00000004)

///////////////////////////////////////////////////////////////////////
//	ACQUISITION END
//
//	This event is generated when a stop condition (manual or automatic
//	- see STOP TYPE chapter) occurs.
//
//	NOTE1:
//		If a ROLLED object acquisition is stopped during the PREVIEW
//		phase, acquisition must be repeated, since no composition has
//		been performed during this phase.
//
//	NOTE2:
//		If a manual stop has been performed in the following cases:
//
//		-	PREVIEW phase in FLAT AUTO-CAPTURE mode
//		-	FULL RESOLUTION phase (ROLL COMPOSITION) for ROLL objects
//
//		
//		the operator should be warned, since in such a case image 
//		evaluation is up to the operator himself (however it's
//		recommended to repeat the acquisition).
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_AE_ACQUISITION_END			((unsigned int)0x00000008)

///////////////////////////////////////////////////////////////////////
//	ACQUISITION ERROR
//
//	This event is generated when an error occurs in low level functions
//	calling or in acquisition thread (see also ERROR CODES section).
//	In such a case the acquisition is stopped and the callback must 
//	warn the operator and return immediately.
//	The error code is reported in the <AcquisitionErrorCode> parameter
//	of the callback function (see ACQUISITION EVENTS MANAGER CALLBACK
//	chapter).
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_AE_ACQUISITION_ERROR			((unsigned int)0x00000010)


//////////////////////////////////////////////////////////////////////
////// EVENT INFO //////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//	In this section the value and the meaning that the <EventInfo>
//	parameter of the GBMSAPI_AcquisitionEventsManagerCallback(...) 
//	function can assume are explained.
//	The values <EventInfo> can assume consist in a bit mask. Each 
//	bit of this mask represents a condition.
//	In order to know if a particular condition is set check the mask.
//	For example:
//	
//	BOOL IsAFullResFrame = ((EventInfo & GBMSAPI_EI_ACQUISITION_PHASE)
//			== GBMSAPI_EI_ACQUISITION_PHASE);
//	BOOL AutomaticStop = ((EventInfo & GBMSAPI_EI_STOP_TYPE)
//			== GBMSAPI_EI_STOP_TYPE);

///////////////////////////////////////////////////////////////////////
//	ACQUISITION PHASE
//
//	This condition says whether the last frame was acquired during
//	the PREVIEW acquisition phase or the FULL-RESOLUTION one.
//
//	NOTE:
//		This condition is valid only when a valid frame has been
//		acquired, that is when <OccurredEventCode> parameter of the
//		GBMSAPI_AcquisitionEventsManagerCallback(...) is equal to
//		GBMSAPI_AE_VALID_FRAME_ACQUIRED or GBMSAPI_AE_ACQUISITION_END.
//
//	Bit meaning:
//	-	1:	FULL-RESOLUTION phase
//	-	0:	PREVIEW phase
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_EI_ACQUISITION_PHASE			((unsigned int)0x00000001)

///////////////////////////////////////////////////////////////////////
//	STOP TYPE
//
//	This condition is useful when the auto-capture option has been set
//	(see ACQUISITION OPTIONS chapter). Infact in auto-capture mode
//	acquisition should always be automatical: if manual is up to the
//	application/operator to evaluate image quality in order to accept
//	it or not.
//
//	NOTE:
//		This condition is valid only if accompained to the
//		GBMSAPI_AE_ACQUISITION_END event (acquisition stoppped).
//
//	Bit meaning:
//	-	1:	Automatic stop
//	-	0:	Manual stop
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_EI_STOP_TYPE				((unsigned int)0x00000002)

///////////////////////////////////////////////////////////////////////
//	PREVIEW RESOLUTION
//
//	This condition informs the application whether the current preview
//	frame resolution is FULL or LOW. Normally in PREVIEW phase the
//	frame resolution is LOW, except when the 
//	GBMSAPI_AO_FULL_RES_PREVIEW option has been set (see the
//	ACQUISITION OPTIONS section).
//	It is valid only when the GBMSAPI_EI_ACQUISITION_PHASE bit is
//	valid
//
//	NOTE:
//		This condition is valid only when a valid frame has been
//		acquired (that is when <OccurredEventCode> parameter of the
//		GBMSAPI_AcquisitionEventsManagerCallback(...) is equal to
//		GBMSAPI_AE_VALID_FRAME_ACQUIRED or 
//		GBMSAPI_AE_ACQUISITION_END) and the 
//		GBMSAPI_EI_ACQUISITION_PHASE bit is 0)
//		
//
//	Bit meaning:
//	-	1:	FULL RESOLUTION IN PREVIEW
//	-	0:	LOW RESOLUTION IN PREVIEW
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_EI_PREVIEW_RES				((unsigned int)0x00000004)

///////////////////////////////////////////////////////////////////////
//	ACQUISITION TIMEOUT
//
//	In order to avoid device overheating, for some scanners, like
//	MultiScan1000, a timeout mechanism is provided. That means that,
//	if after 20 seconds (approximatively) no object has been put on the
//	scanning surface, acquisition is automatically stopped. In such
//	a case, this bit is raised up and the frame must be considered
//	as NOT valid.
//
//	Bit meaning:
//	-	1:	acquisition timeout
//	-	0:	normal condition
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_EI_ACQUISITION_TIMEOUT	((unsigned int)0x00000008)

//////////////////////////////////////////////////////////////////////
////// DIAGNOSTIC MESSAGES ///////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//	Each diagnostic message is characterized by a color that 
//	identifies its heaviness:
//
//	- RED color: very serious diagnostic condition.
//	- YELLOW color: just a warning.
//
//	It is recommended that when a diagnostic message appears 
//	(<GB_Diagnostic> parameter different from 0 in GBMSAPI_GetFrame(...)),
//	the corresponding recommended text be displayed in a box with the 
//	background color corresponding to its heaviness.

//////////////////////////////////////////////////////////////////////////////////
//	GBMSAPI_DM_SCANNER_SURFACE_NOT_NORMA
//
//	Status of importance and corresponding color : RED
//
//	Scanner surface diagnostic is executed at the beginning of every acquisition
//	basing of the first frame and, if a problem is detected, continues until the
//	problem has been resolved. 
//  If this diagnostic bit is set, application should continue scanning process
//	waiting for resolving the problem,in the meanwhile displaying text message
//	and image flow from scanner in order to provide visual feedback.
//	The sensitivity of this diagnostic procedure can be modified by using the
//	GBMSAPI_SetDiagnosticSensitivity(...) function (for the scanners supporting it).
//	
//	Recommended short diagnostic message to display: 
//		A finger has been placed before scanning process start, or scanner
//		surface is dirty.
//
//	Recommended complete diagnostic message to display: 
//		This message appears when a finger has been placed before scanning
//		process start, or if scanner surface is dirty. Check that there are no 
//		fingers placed on the scanner at the beginning of scanning and that 
//		scanning surface is clear.
//		Contact technical support, if this diagnostic message persists.
//
//	NOTE:	presence of this diagnostic message does not automatically block
//			auto-capture system operation. But in this case auto-capture 
//			system operates in not-normal conditions. It is recommended for 
//			application to block explicitly auto-capture system using the 
//			GBMSAPI_SetAutoCaptureBlocking(...) function, if device supports it,
//			or to repeat acquisition process otherwise.
///////////////////////////////////////////////////////////////////////////////////
#define GBMSAPI_DM_SCANNER_SURFACE_NOT_NORMA	(unsigned int)0x00000001


//////////////////////////////////////////////////////////////////////////////////
//	GBMSAPI_DM_SCANNER_FAILURE
//
//	Status of importance and corresponding color : RED
//
//	Scanner surface diagnostic is executed at the beginning of every acquisition
//	basing of the first frame and, if a problem is detected, continues until the
//	problem has been resolved. 
//  If this diagnostic bit is set, application should continue scanning process
//	waiting for resolving the problem,in the meanwhile displaying text message
//	and image flow from scanner in order to provide visual feedback.
//	It is strongly recommended to refuse image acceptance in case when this
//	diagnostic message appears.
//
//	Recommended short diagnostic message to display:
//		Scanner error. Parameters of acquired image do not conform to quality
//		standard.
//
//	Recommended complete diagnostic message to display: 
//		This message reports that some parameters of acquired image are not 
//		suitable. Check that there is no finger placed on scanner at the beginning
//		of scanning and that scanning surface is clear. Contact technical support,
//		if this diagnostic message persists.
///////////////////////////////////////////////////////////////////////////////////
#define GBMSAPI_DM_SCANNER_FAILURE	(unsigned int)0x00000002

//////////////////////////////////////////////////////////////////////////////////
//	GBMSAPI_DM_COMPOSITION_SLOW
//
//	Status of importance and corresponding color : RED
//
//	PC resource diagnostic is executed during acquisition process in rolled 
//	composition mode to ensure no performance degradation. 
//	If this diagnostic bit is set, continue scanning process meanwhile displaying
//	the message in order to notify the operator.
//
//	Recommended short diagnostic message to display: 
//		Frame rate is too slow to ensure proper composition image of rolled 
//		fingerprint.
//
//	Recommended complete diagnostic message to display: 
//		"Critical resources for the rolled image composition.
//		This message indicates that the Personal Computer is running low on system 
//		resources, so that the optimal image composition of the rolled fingerprint
//		is not guaranteed. 
//		The operator should carefully evaluate the acquisition result. To avoid 
//		this diagnostic message, all the unnecessary processes on the Personal 
//		Computer should be stopped, to free more resources. To complete the current
//		acquisition operator can reduce the finger rotation speed. If the message
//		appears frequently even when all the unnecessary processes are stopped, it
//		could be recommended to increase the Personal Computer resources".
///////////////////////////////////////////////////////////////////////////////////
#define GBMSAPI_DM_COMPOSITION_SLOW	(unsigned int)0x00000004

//////////////////////////////////////////////////////////////////////////////////
//	GBMSAPI_DM_FLAT_FINGER_SLIDING
//
//	Status of importance and corresponding color : RED
//
//	Automatic sliding check for flat finger(s) acquisition is executed on every
//	incoming frame and is independent of auto-capture sub-system.
//	If this diagnostic bit is set, it means that the user behaviour could be not 
//	correct during flat finger scanning. Principally, this check is pointed to 
//	detect "sliding with friction" of finger skin on scanning surface. Such sliding
//	can lead to significant skin distortion. As a consequence, distorted fingerprint
//	image will be generated that can results in wrong verification/identification 
//	based on the given fingerprint image. 
//	The sensitivity of this diagnostic procedure can be modified by using the
//	GBMSAPI_SetDiagnosticSensitivity(...) function (for the scanners supporting it).
//
//	Recommended short diagnostic message to display: 
//		Flat finger(s) sliding is detected.
//
//	Recommended complete diagnostic message to display: 
//		Flat finger(s) sliding is detected.
//		The person should put his/her finger(s) on scanning surface flatly, 
//		ensuring no-sliding of finger skin on the glass. 
//		It is recommended to repeat acquisition, if this message appears.
//
//	NOTE1:	devote particular attention for moist fingers, where sliding effect is
//			most critical, because moisture creates "slippery film" between finger
//			skin and glass that leads to more frequent sliding.
//	NOTE2:	the higher frame rate, the better sliding check precision.
///////////////////////////////////////////////////////////////////////////////////
#define GBMSAPI_DM_FLAT_FINGER_SLIDING	(unsigned int)0x00000008


//////////////////////////////////////////////////////////////////////////////////
//	GBMSAPI_DM_EXT_LIGHT_TOO_STRONG
//
//	Status of importance and corresponding color : RED
//
//	Automatic external light check is executed on every incoming frame. Very 
//	powerful external light sources (such as direct sun rays coming in certain 
//	angles to scanner) can deteriorate fingerprint image quality. Built-in
//	diagnostic helps to detect these influence and eliminate negative consequences.
//	If this diagnostic bit is set, excessive external light influences strongly on 
//	scanner.
//	The sensitivity of this diagnostic procedure can be modified by using the
//	GBMSAPI_SetDiagnosticSensitivity(...) function (for the scanners supporting it).
//
//	Recommended short diagnostic message to display: 
//		Too much strong external light influence.
//
//	Recommended complete diagnostic message to display: 
//		Too strong external light influence is detected.
//		Very powerful external light source (such as direct sun rays coming 
//		in certain angles to scanner) can deteriorates fingerprint image quality.
//		It is recommended to eliminate external light influence and repeat 
//		acquisition."
///////////////////////////////////////////////////////////////////////////////////
#define GBMSAPI_DM_EXT_LIGHT_TOO_STRONG	(unsigned int)0x00000010

//////////////////////////////////////////////////////////////////////////////////
//	GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_LEFT
//	GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_RIGHT
//	GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_TOP
//
//	Status of importance and corresponding color : YELLOW
//
//	This diagnostic is executed on every incoming frame just for flat acquisitions.  
//	If one or more these diagnostic bits are set, flat fingerprint(s) significantly
//	goes out of the corresponding (left, right, top) border of image frame 
//	(or clipping region).
//	The sensitivity of this diagnostic procedure can be modified by using the
//	GBMSAPI_SetDiagnosticSensitivity(...) function (for the scanners supporting it).
//	Sensitivity can be changed separately for "left-right" and "top" messages.
//
//	Recommended short diagnostic message to display: 
//		"Finger goes out on the left border."
//		"Finger goes out on the right border."
//		"Finger goes out on the top border."
//	
//	Recommended text of instruction to avoid this diagnostic message: 
//		"It is recommended to take finger(s) away and then retry selecting properly
//		the region on scanning window where to place finger(s)".
///////////////////////////////////////////////////////////////////////////////////
#define GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_LEFT		(unsigned int)0x00000020
#define GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_RIGHT	(unsigned int)0x00000040
#define GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_TOP		(unsigned int)0x00000080

//////////////////////////////////////////////////////////////////////////////////
//	GBMSAPI_DM_FLAT_FINGER_DISPLACED_DOWN
//
//	Status of importance and corresponding color : YELLOW
//
//	This diagnostic is executed on every incoming frame just for flat acquisitions.  
//	If this diagnostic bit is set, flat fingerprint(s) significantly displaced to
//	down border of image frame.  
//	The sensitivity of this diagnostic procedure can be modified by using the
//	GBMSAPI_SetDiagnosticSensitivity(...) function (for the scanners supporting it).
//
//	Recommended short diagnostic message to display: 
//		"Finger significantly displaced to the down border."
//
//	Recommended text of instruction to avoid this diagnostic message: 
//		"It is recommended to take finger(s) away and then retry selecting properly
//		the region on scanning window where to place finger(s)".
///////////////////////////////////////////////////////////////////////////////////
#define GBMSAPI_DM_FLAT_FINGER_DISPLACED_DOWN	(unsigned int)0x00000100

//////////////////////////////////////////////////////////////////////////////////
//	GBMSAPI_DM_VSROLL_IMPROPER_ROLL
//
//	Status of importance and corresponding color : RED
//
//	This diagnostic is executed during the whole rolling procedure and checks, if
//	there are some significant visible "seams" in the final composite image.
//	The sensitivity of this diagnostic procedure can be modified by using the
//	GBMSAPI_SetDiagnosticSensitivity(...) function (for the scanners supporting it).
//	It is strongly recommended to repeat rolling of the current finger if this 
//	message appears.
//
//	Recommended short diagnostic message to display: 
//		"Finger has been rolled improperly."
//
//	Recommended text of instruction to avoid this diagnostic message: 
//		"It is important to roll the finger only in one selected direction (to the
//		left or to the right) avoiding at the passage from flat to roll phase even
//		the most little roll to the opposite direction. Otherwise, visible "seam"
//		in the final composite image will appear."
///////////////////////////////////////////////////////////////////////////////////
#define GBMSAPI_DM_VSROLL_IMPROPER_ROLL	(unsigned int)0x00020000


//////////////////////////////////////////////////////////////////////////////////
//	GBMSAPI_DM_VSROLL_TOO_FAST_ROLL
//
//	Status of importance and corresponding color : RED
//
//	This diagnostic is executed during the whole rolling procedure and checks if
//	rolling is performed too fastly. Too much fast rolling can produce fragmented
//	composite image and downgrade composite fingerprint image quality.
//	The sensitivity of this diagnostic procedure can be modified by using the
//	GBMSAPI_SetDiagnosticSensitivity(...) function (for the scanners supporting it).
//	It is strongly recommended to repeat rolling of the current finger if this 
//	message appears.
//
//	Recommended short diagnostic message to display: 
//		"Finger has been rolled too fastly."
//
//	Recommended text of instruction to avoid this diagnostic message: 
//		"Duration of rolling phase should be about 2 second"
///////////////////////////////////////////////////////////////////////////////////
#define GBMSAPI_DM_VSROLL_TOO_FAST_ROLL	(unsigned int)0x00040000


//////////////////////////////////////////////////////////////////////////////////
//	GBMSAPI_DM_VSROLL_TOO_NARROW_ROLL
//
//	Status of importance and corresponding color : RED
//
//	This diagnostic is executed during the whole rolling procedure and checks if
//	rolling is performed too narrow. It is expected that the square of the final
//	composite image is at least 1.5 times greater than the square of the biggest 
//	flat (input) fingerprint image. 
//	The sensitivity of this diagnostic procedure can be modified by using the
//	GBMSAPI_SetDiagnosticSensitivity(...) function (for the scanners supporting it).
//	It is strongly recommended to repeat rolling of the current finger if this 
//	message appears.
//
//	Recommended short diagnostic message to display: 
//		"Finger has been rolled too narrow."
//
//	Recommended text of instruction to avoid this diagnostic message: 
//		"It is important to roll the finger widely, ideally in a "from nail to nail"
//		manner."
///////////////////////////////////////////////////////////////////////////////////
#define GBMSAPI_DM_VSROLL_TOO_NARROW_ROLL	(unsigned int)0x00080000


//////////////////////////////////////////////////////////////////////////////////
//	GBMSAPI_DM_VSROLL_OUTSIDE_BORDER_LEFT
//	GBMSAPI_DM_VSROLL_OUTSIDE_BORDER_RIGHT
//	GBMSAPI_DM_VSROLL_OUTSIDE_BORDER_TOP
//
//	Status of importance and corresponding color : YELLOW
//
//	This diagnostic is executed at the end of rolling procedure. If one or more of
//	these diagnostic bits are set, the final composite fingerprint image 
//	significantly goes out of the corresponding (left, right, top) border of original
//	image frame (NOTE: this diagnostc does not check if composite fingerprint image
//	goes out of programmed clipping region).
//	The sensitivity of this diagnostic procedure can be modified by using the
//	GBMSAPI_SetDiagnosticSensitivity(...) function (for the scanners supporting it).
//	It could be recommended to repeat rolling of the current finger if this message
//	is present.
//
//	Recommended short diagnostic message to display: 
//		"Finger significantly goes out on the left border."
//		"Finger significantly goes out on the right border."
//		"Finger significantly goes out on the top border."
//	
//	Recommended text of instruction to avoid this diagnostic message: 
//		"It is important to select properly the point on scanning window where to 
//		place initially the finger, ensuring that finger rolling will be performed
//		near the center of the scanning window and there is enough space to perform
//		finger rolling "from nail to nail"."
///////////////////////////////////////////////////////////////////////////////////
#define GBMSAPI_DM_VSROLL_OUTSIDE_BORDER_LEFT		(unsigned int)0x00100000
#define GBMSAPI_DM_VSROLL_OUTSIDE_BORDER_RIGHT	(unsigned int)0x00200000
#define GBMSAPI_DM_VSROLL_OUTSIDE_BORDER_TOP		(unsigned int)0x00400000

//////////////////////////////////////////////////////////////////////////////////
//	GBMSAPI_DM_VSROLL_DISPLACED_DOWN
//
//	Status of importance and corresponding color : YELLOW
//
//	This diagnostic is executed at the end of rolling procedure and checks if in
//	the final composite image the fingerprint significantly displaced to down 
//	border of original image frame.
//
//	NOTE:	this diagnostic does not check if composite fingerprint image displaced
//			to down border of programmed clipping region).  
//
//	The sensitivity of this diagnostic procedure can be modified by using the
//	GBMSAPI_SetDiagnosticSensitivity(...) function (for the scanners supporting it).
//	It could be recommended to repeat rolling of the current finger if this message
//	is present.
//
//	Recommended short diagnostic message to display: 
//		"Finger significantly displaced to the down border."
//
//	Recommended text of instruction to avoid this diagnostic message: 
//		"It is important to select properly the point on scanning window where to 
//		place initially the finger, ensuring that finger rolling will be performed
//		near the center of the scanning window and there is enough space to perform
//		finger rolling "from nail to nail"."
///////////////////////////////////////////////////////////////////////////////////
#define GBMSAPI_DM_VSROLL_DISPLACED_DOWN	(unsigned int)0x00800000

// Ver 2300: rolling direction messages
//////////////////////////////////////////////////////////////////////////////////
//	GBMSAPI_DM_VSROLL_ROLL_DIRECTION_RIGHT
//
//	Status of importance and corresponding color : NA
//
//	This bit is not a generic diagnostic message; instead, it is used to inform 
//	the user about the expected direction of finger rolling. This message is 
//	generated for applications where one of preview modes is activated (so, it is
//	not generated for PREVIEW_MODE_OFF). 
//	It appears to be set at the end of preview phase when a finger before the 
//	beginning of rolling procedure resides on the LEFT (opposite) half of the roll
//	scanning window.
//	It is recommended to inform the user about the expected direction of finger
//	rolling by some sort of graphical/textual message.
///////////////////////////////////////////////////////////////////////////////////
#define GBMSAPI_DM_VSROLL_ROLL_DIRECTION_RIGHT	(unsigned int)0x40000000

//////////////////////////////////////////////////////////////////////////////////
//	GBMSAPI_DM_VSROLL_ROLL_DIRECTION_LEFT
//
//	Status of importance and corresponding color : NA
//
//	This bit is not a generic diagnostic message; instead, it is used to inform 
//	the user about the expected direction of finger rolling. This message is 
//	generated for applications where one of preview modes is activated (so, it is
//	not generated for PREVIEW_MODE_OFF). 
//	It appears to be set at the end of preview phase when a finger before the 
//	beginning of rolling procedure resides on the RIGHT (opposite) half of the roll
//	scanning window.
//	It is recommended to inform the user about the expected direction of finger
//	rolling by some sort of graphical/textual message.
///////////////////////////////////////////////////////////////////////////////////
#define GBMSAPI_DM_VSROLL_ROLL_DIRECTION_LEFT	(unsigned int)0x80000000
// end Ver 2300: rolling direction messages

///////////////////////////////////////////////////
///////// AUTO-CAPTURE PHASE //////////////////////
///////////////////////////////////////////////////
#define GBMSAPI_ACP_OFF				0 
#define GBMSAPI_ACP_WAIT_FINGER		1 
#define GBMSAPI_ACP_SELECT_IMAGE		2
#define GBMSAPI_ACP_BEST_IMAGE_STOP	3	//	Corresponds to the GBMSAPI_AE_ACQUISITION_END event.

/****************************************************************************
*																			*
*					PART 4													*
*																			*
*				ROLL FUNCTIONALITY DEFINES									*
*																			*
****************************************************************************/
//----------------------------------------------------------------------------------------------------------
//										ROLLING ARTEFACTS.
//----------------------------------------------------------------------------------------------------------
//  In a live-scan rolling process a round-shaped finger is moving on a flat scanner surface. 
//   Frequently it leads to non-uniform skin deformation that causes displacement of some ridges on 
//   scanner surface during finger rolling. Typically this displacement appears in finger tip and neighbour 
//   regions, but can also appear deeply inside finger region. Another possible reason of displacement 
//   is intentional finger movement across scanner surface during rolling in case when a scanned person 
//   resists to normal rolling procedure (criminal applications).
//	Such a displacement results in appearance within composite fingerprint image of "rolling artefacts" 
//   regions. In these regions false (originally not-existing) ridge endings/bifurcations can appear and/or 
//   true (originally existing) ridge endings/bifurcations can be destructed/lost. Besides, in final composite 
//   image these false endings/bifurcations can look high-reliably confusing then as IAFIS as human-expert 
//   authentication.
//  Present software implementation of "rolled fingerprint image composition" has built-in capability to 
//   detect "rolling artefact" regions and to decrease as mush as possible their presence in composite image.
//  Nevertheless, it is difficult in practice to avoid completely artefact regions in composite image for 
//   every rolled finger. More practical approach in this case is to estimate amount of artefact regions
//   and control, that it is within acceptable range. 
//  Additionally, due to the fact that the most frequently artefact regions appear on upper fingerprint 
//   edge (finger tip), it is sensible to wipe erroneous ridges from these regions extending 
//   light outer space inside artefact region for programmable depth. 
//  Border of artefact regions that remain in final composite image (non-wiped) can be marked by dotted 
//   line that can be programmed to be quite transparent for IAFIS but easily discernable by human expert 
//   to prevent mistakes in manual authentication. A human expert should be informed that all regions 
//   marked by dotted-line can PROBABLY contain erroneous minutiae information.
//
//	NOTE: in case when a rolled finger permanently demonstrates a large amount of artefact regions, 
//        it could be recommended to roll it more narrow thus decreasing skin deformation 
//		  and consequent ridges displacement.
//----------------------------------------------------------------------------------------------------------
///////////////////////////////////////////////////
///////// CLEANING DEPTH //////////////////////////
///////////////////////////////////////////////////
#define GBMSAPI_ARTEFACT_CLEANING_DEPTH_ZERO			0   
#define GBMSAPI_ARTEFACT_CLEANING_DEPTH_MAX			3   
#define GBMSAPI_ARTEFACT_CLEANING_DEPTH_UNLIMITED		255

///////////////////////////////////////////////////
///////// ARTEFACT REGION MARKERS /////////////////
///////////////////////////////////////////////////
// Can be used to display rolling artefact regions
// of the final composite image
#define GBMSAPI_ARTEFACT_REGION_WIPED_BORDER		0x10 // marks border pixels of wiped
												// rolling artefact regions

#define GBMSAPI_ARTEFACT_REGION_MARKED_BORDER		0x20 // marks border pixels of all 
												// the rolling artefact regions

#define GBMSAPI_ARTEFACT_REGION_WIPED				0x40 // marks all the pixels of 
												// wiped rolling artefact regions

#define GBMSAPI_ARTEFACT_REGION_MARKED			0x80 // marks all the pixels of all
												// the rolling artefact regions


/****************************************************************************
*																			*
*					PART 5													*
*																			*
*				VISUAL INTERFACE LEDS										*
*																			*
****************************************************************************/
// For further info see the 
// GBMSAPI_SetFingerIndicatorVUIState(...) function.

///////////////////////////////////////////////////
///////// LED MASK VALUES /////////////////////////
///////////////////////////////////////////////////
#define GBMSAPI_VIL_THUMB_RIGHT		0x0001
#define GBMSAPI_VIL_INDEX_RIGHT		0x0002
#define GBMSAPI_VIL_MIDDLE_RIGHT		0x0004
#define GBMSAPI_VIL_RING_RIGHT		0x0008
#define GBMSAPI_VIL_LITTLE_RIGHT		0x0010
#define GBMSAPI_VIL_THUMB_LEFT		0x0020
#define GBMSAPI_VIL_INDEX_LEFT		0x0040
#define GBMSAPI_VIL_MIDDLE_LEFT		0x0080
#define GBMSAPI_VIL_RING_LEFT			0x0100
#define GBMSAPI_VIL_LITTLE_LEFT		0x0200
#define GBMSAPI_VIL_ROLL				0x0400
#define GBMSAPI_VIL_STATUS			0x0800

///////////////////////////////////////////////////
///////// LED COLORS //////////////////////////////
///////////////////////////////////////////////////
#define GBMSAPI_VIL_COLOR_OFF		    0
#define GBMSAPI_VIL_COLOR_GREEN	    1
#define GBMSAPI_VIL_COLOR_RED		    2
#define GBMSAPI_VIL_COLOR_YELLOW		3

/****************************************************************************
*																			*
*					PART 6													*
*																			*
*				VISUAL INTERFACE LCD										*
*																			*
****************************************************************************/

//---------------------------------------------------------------------------
// LCD MODELS
//
//	Greenbit scanners, depending on models, can support two kinds of LCD
//	displays:
//
//	- Monochrome display: for example MS500, MSC500, MS1000
//
//	- Colour display: for example MSC517
//
//	Displays, depending on models, can have different features, for example
//	capability of setting intermediate limits for showing parameters like
//	contrast and so on. These features are listed in the DISPLAY FEATURES
//	paragraph and can be obtained by calling the
//	GBMSAPI_VUI_LCD_GetLcdFeatures(...) function.
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
// LCD SCREENS
//
//	Independently from LCD model, display can show three screen types:
//
//	- LOGO-SCREEN: it's the default screen, shown when the scanner is
//		selected, and at the end of acquisition, unless STOP-SCREEN is
//		selected in the <DisplayOptionMask> parameter of the
//		GBMSAPI_StartAcquisition(...) function.
//
//	- ACQUISITION-SCREEN: it's the screen that is shown during image
//		acquisition.
//		
//		Within the ACQUISITION-SCREEN the following is shown automatically:
//
//		- A button that allows to stop the acquisition. This button has the
//			same functionalities as the pedal (see GBMSAPI_OED_PEDAL);
//			furthermore, for rolled objects and when the
//			GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP is NOT set, if the button
//			is pressed during the preview phase, the acquisition is stopped.
//
//		- A graphical symbol of the object to be acquired
//
//		- Instructions for the user (don't put finger, roll to <x> direction,
//			and so on)
//
//		- Acquisition phase (preview, acquisition)
//
//		- Diagnostic messages
//
//		- Contrast and palm completeness values in case 
//			GBMSAPI_DO_CUSTOMIZED_COMPLETENESS and/or 
//			GBMSAPI_DO_CUSTOMIZED_CONTRAST are NOT set in the
//			<DisplayOptionMask> parameter of the 
//			GBMSAPI_StartAcquisition(...) function.
//
//		The following information are instead controlled by the application
//		developer:
//
//		- Contrast and palm completeness upper limits, by setting the
//			corresponding parameters of the GBMSAPI_StartAcquisition(...)
//			function.
//
//		- For Colour displays only, contrast and palm completeness
//			intermediate limits (set as parameters, respectively, of the
//			GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimit(...) and
//			GBMSAPI_VUI_LCD_SetContrastIntermediateLimit(...) functions),
//			that allow to make visible in yellow values between upper and
//			intermediate limits.
//
//		- Only in case GBMSAPI_DO_CUSTOMIZED_COMPLETENESS and/or 
//			GBMSAPI_DO_CUSTOMIZED_CONTRAST are set in the <DisplayOptionMask>
//			parameter of the GBMSAPI_StartAcquisition(...) function,
//			customized palm completeness and customized contrast, by means
//			of the GBMSAPI_VUI_LCD_SetCompletenessValueOnAcquisitionScreen(..)
//			and of the GBMSAPI_VUI_LCD_SetContrastValueOnAcquisitionScreen(..)
//			functions. Otherwise they are controlled by the GBMSAPI library.
//
//		Indipendently from the way they are set (automatic or by the
//		developer), palm completeness and contrast values are displayed in the
//		way	described below.
//	
//			- On Monochrome displays:
//				in white character on a black background if it is worse	than
//				the limit value defined by the customer
//
//			- On Colour displays (example for contrast value, for 
//				palm completeness the schema is identical):
//				Given that:
//		
//				- L1 is the threshold set with the 
//					GBMSAPI_VUI_LCD_SetContrastIntermediateLimit(...)
//					function (intermediate limit)
//		
//				- L2 is the <ContrastLimitToDisplayOnLCD> parameter of the 
//					GBMSAPI_StartAcquisition(...) function (upper limit)
//
//				the way the contrast value is shown on the lcd is the
//				following:
//
//				Contrast value <= L1 : RED
//				L1 < Contrast value <= L2 : YELLOW
//				Contrast value > L2 : GREEN
//
//				NOTE: if intermediate limit is not set, L1 will be
//					automatically set equal to L2, so that yellow
//					colour will never appear on display
//
//	- STOP-SCREEN: it's the screen that it's shown at the end of the
//		acquisition if the GBMSAPI_DO_FINAL_SCREEN option is set in the
//		<DisplayOptionMask>	parameter of the GBMSAPI_StartAcquisition(...)
//		function.
//		It automatically shows the following:
//
//		- Two buttons that allow to accept or repeat the acquisition
//
//		- A graphical symbol of the object to be acquired
//
//		It's instead controlled by the application developer the
//		following:
//
//		- Diagnostic and/or user messages, controlled by means of the
//			GBMSAPI_VUI_LCD_SetCustomerMessageOnStopScreen(...) and
//			GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(...). Refer
//			to them in the GBMSAPI_Library.h file for more details.
//
//		- segmentation results (as OK-FAIL symbols for each finger,
//			depending on the object acquired) by means of the
//			GBMSAPI_VUI_LCD_SetSegmentationOnStopScreen(...) function. Refer
//			to it in the GBMSAPI_Library.h file for more details.
//
//		- contrast limits and value by means of the
//			GBMSAPI_VUI_LCD_SetContrastLimitOnStopScreen(...),
//			GBMSAPI_VUI_LCD_SetContrastIntermediateLimitOnStopScreen(...)
//			for colour display only and
//			GBMSAPI_VUI_LCD_SetContrastValueOnStopScreen(...) functions. 
//			Refer to them in the GBMSAPI_Library.h file for more details.
//
//		- artefacts limits and value by means of the
//			GBMSAPI_VUI_LCD_SetArtefactLimitOnStopScreen(...),
//			GBMSAPI_VUI_LCD_SetArtefactIntermediateLimitOnStopScreen(...)
//			for colour display only and
//			GBMSAPI_VUI_LCD_SetArtefactOnStopScreen(...) functions. 
//			Refer to them in the GBMSAPI_Library.h file for more details.
//
//		- quality limits and value by means of the
//			GBMSAPI_VUI_LCD_SetQualityLimitOnStopScreen(...),
//			GBMSAPI_VUI_LCD_SetQualityIntermediateLimitOnStopScreen(...)
//			for colour display only and
//			GBMSAPI_VUI_LCD_SetQualityOnStopScreen(...) functions. 
//			Refer to them in the GBMSAPI_Library.h file for more details.
//
//		- palm completeness limits and value by means of the
//			GBMSAPI_VUI_LCD_SetCompletenessLimitOnStopScreen(...),
//			GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen(...)
//			for colour display only and
//			GBMSAPI_VUI_LCD_SetCompletenessValueOnStopScreen(...) functions. 
//			Refer to them in the GBMSAPI_Library.h file for more details.
//
//		Regarding the way contrast, artefacts, quality and palm completeness
//		are shown, please see the schema in the ACQUISITION-SCREEN section.
//
//		NOTE: regarding artefacts and quality, also a "direction" parameter
//			is foreseen for the GBMSAPI_VUI_LCD_Set<QA>LimitOnStopScreen(...)
//			(where <QA> stands for "Quality" or "Artefacts", depending on
//			the feature) function. It can assume two values, defined in the
//			LIMITS DIRECTION paragraph. In case its value is
//			GBMSAPI_VILCD_GREATER_THAN_LIMIT_DIRECTION the way artefact (or
//			quality) is shown is the same as described above, otherwise
//			it is the opposite, that is:
//
//			Given L1 and L2 defined as above, and given we are speaking
//			about artefacts (for quality is the same):
//
//			Artefacts value > L2 : RED
//			L1 < Artefacts value <= L2 : YELLOW
//			Artefacts value <= L1 : GREEN
//---------------------------------------------------------------------------
// VER 2.0.0.8
///////////////////////////////////////////////////
///////// DISPLAY FEATURES ////////////////////////
///////////////////////////////////////////////////
//	The values defined in the following list are intentionally given
//	as bit mask values (in the sense that they are choosen so that
//	for each of them only a bit of an unsigned int -4 bytes-
//	field is set to 1).
//	In order to know what LCD feature is supported by each 
//	Green Bit device, use the GBMSAPI_VUI_LCD_GetLcdFeatures(...) 
//	function.

///////////////////////////////////////////////////////////////////////
//	INTERMEDIATE LIMIT SETTING CAPABILITY
//
//	Lcd displays supporting this feature can show in three different
//	ways contrast, palm completeness, quality and artefacts (see
//	LCD SCREENS paragraph for more details).
//
//	In order to know if this feature is supported, the application
//	should check the <LcdFeatures> parameter of
//	GBMSAPI_VUI_LCD_GetLcdFeatures(...) function and see whether
//	GBMSAPI_VILCD_LF_INTERMEDIATE_LIMIT bit is set.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_VILCD_LF_INTERMEDIATE_LIMIT	((unsigned int)0x00000001)

///////////////////////////////////////////////////////////////////////
//	INTERMEDIATE LIMIT SETTING CAPABILITY
//
//	Lcd displays supporting this feature has the capability to set
//	the interface language in idioms other than English.
//
//	In order to know if this feature is supported, the application
//	should check the <LcdFeatures> parameter of
//	GBMSAPI_VUI_LCD_GetLcdFeatures(...) function and see whether
//	GBMSAPI_VILCD_LF_LANGUAGE_SETTING bit is set.
///////////////////////////////////////////////////////////////////////
#define GBMSAPI_VILCD_LF_LANGUAGE_SETTING	((unsigned int)0x00000002)

///////////////////////////////////////////////////
///////// PRELOADED DIAGNOSTIC MESSAGES ///////////
///////////////////////////////////////////////////
#define GBMSAPI_VILCD_DIAGNOSTIC_SCANNER_SURFACE_NOT_NORMA	 5  
#define GBMSAPI_VILCD_DIAGNOSTIC_SCANNER_FAILURE			     6  
#define GBMSAPI_VILCD_DIAGNOSTIC_COMPOSITION_SLOW			     7  
#define GBMSAPI_VILCD_DIAGNOSTIC_FLAT_FINGER_SLIDING			 8 
#define GBMSAPI_VILCD_DIAGNOSTIC_EXT_LIGHT_TOO_STRONG          9
#define GBMSAPI_VILCD_DIAGNOSTIC_FINGER_OUT_OF_REGION_LEFT	 10   
#define GBMSAPI_VILCD_DIAGNOSTIC_FINGER_OUT_OF_REGION_RIGHT	 11  
#define GBMSAPI_VILCD_DIAGNOSTIC_FINGER_OUT_OF_REGION_TOP		 12   
#define GBMSAPI_VILCD_DIAGNOSTIC_FINGER_DISPLACED_DOWN		 13   
#define GBMSAPI_VILCD_DIAGNOSTIC_VSROLL_IMPROPER_ROLL			 14 
#define GBMSAPI_VILCD_DIAGNOSTIC_VSROLL_TOO_FAST_ROLL			 15
#define GBMSAPI_VILCD_DIAGNOSTIC_VSROLL_TOO_NARROW_ROLL	     16	

///////////////////////////////////////////////////
///////// PRELOADED GENERAL MESSAGES //////////////
///////////////////////////////////////////////////
#define GBMSAPI_VILCD_MSG_ACQUISITION_PROGRESS 	             17  
#define GBMSAPI_VILCD_MSG_ACQUISITION_STOPPED	                 18  
#define GBMSAPI_VILCD_MSG_ACQUISITION_SUCCESSFUL	             19  
#define GBMSAPI_VILCD_MSG_AUTOACEPT_CONDITION_NO_SATISFIED     20 
#define GBMSAPI_VILCD_MSG_AND_DIAG_EMPTY                       21 
#define GBMSAPI_VILCD_MSG_DIAG_PROGRESS_DONT_PUT_FINGER        22   
#define GBMSAPI_VILCD_MSG_FINGER_INCLINATION_DETECTED          23  
#define GBMSAPI_VILCD_MSG_INCOMPLETE_PATTERN_DETECTED          24   
#define GBMSAPI_VILCD_MSG_INVALID_PATTERN_DETECTED	         25   
#define GBMSAPI_VILCD_MSG_NO_FINGERPRINT_DETECTED			     26 
#define GBMSAPI_VILCD_MSG_RELEASE_PEDAL                        27
#define GBMSAPI_VILCD_MSG_ROTATION_NO_COMPLETED                28	
#define GBMSAPI_VILCD_MSG_SEGMENTATION_FAILED                  29
#define GBMSAPI_VILCD_MSG_SEGMENT_BOUNDING_PROBLEM             30
#define GBMSAPI_VILCD_MSG_SEQUENCE_CHECK_FAILED                31
#define GBMSAPI_VILCD_MSG_PREVIEW                              64
#define GBMSAPI_VILCD_MSG_PROCESSING                           65
#define GBMSAPI_VILCD_MSG_LEFT_RIGHT_MIXUP     	     66
#define GBMSAPI_VILCD_MSG_POSSIBLE_LEFT_RIGHT_MIXUP	 67
#define GBMSAPI_VILCD_MSG_EVALUATE_THE_RESULT	                 68
#define GBMSAPI_VILCD_MSG_REPEATING	                         69
#define GBMSAPI_VILCD_MSG_SAVING_IMAGE	                     70
#define GBMSAPI_VILCD_MSG_AMBIGUOUS_SLAP	                     71
#define GBMSAPI_VILCD_MSG_INCOMPLETE_SLAP	                     72
#define GBMSAPI_VILCD_MSG_POSSIBLE_INCOMPLETE_SLAP	         73
#define GBMSAPI_VILCD_MSG_ACQUISITION_TIME_OUT    	         74

///////////////////////////////////////////////////
///////// PRELOADED MESSAGES AREA /////////////////
///////////////////////////////////////////////////
#define GBMSAPI_VILCD_MSG_AREA  0
#define GBMSAPI_VILCD_DIAG_AREA 1

///////////////////////////////////////////////////
///////// CUSTOM MESSAGES DEFINES /////////////////
///////////////////////////////////////////////////
#define GBMSAPI_VILCD_MAX_CUSTOM_MSG_LEN			15
#define GBMSAPI_VILCD_CUSTOM_MSG_TYPE_INFO_MSG	0
#define GBMSAPI_VILCD_CUSTOM_MSG_TYPE_ERROR_MSG	1

///////////////////////////////////////////////////
///////// LIMITS DIRECTION ////////////////////////
///////////////////////////////////////////////////
#define GBMSAPI_VILCD_GREATER_THAN_LIMIT_DIRECTION	2
#define GBMSAPI_VILCD_SMALLER_THAN_LIMIT_DIRECTION	3

///////////////////////////////////////////////////
///////// SEGMENTATION RESULTS VALUE //////////////
///////////////////////////////////////////////////
#define GBMSAPI_VILCD_SEG_EMPTY			0 // should
//	be used when segment is not a part of the
//	whole slap (for example when acquiring the
//	GBMSAPI_SBT_FLAT_SLAP_2 object)
#define GBMSAPI_VILCD_SEG_OK			1
#define GBMSAPI_VILCD_SEG_NOT_OK		2
#define GBMSAPI_VILCD_SEG_UNAVAILABLE	3

///////////////////////////////////////////////////
///////// PRESSED BUTTON IDs //////////////////////
///////////////////////////////////////////////////
#define GBMSAPI_VILCD_TOUCHSCREEN_OK_BUTTON      0x18      
#define GBMSAPI_VILCD_TOUCHSCREEN_REPEAT_BUTTON  0x20
#define GBMSAPI_VILCD_TOUCHSCREEN_START_BUTTON   0x07


#endif GBMSAPI_DEFINES_H
