using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Resources;
using System.Globalization;
using System.Reflection;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using System.IO;
using GBMSAPI_NET;
using GBMSAPI_NET.GBMSAPI_NET_Defines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_DeviceCharacteristicsDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_ErrorCodesDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_RollFunctionalityDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_AcquisitionProcessDefines;
using GBMSAPI_NET.GBMSAPI_NET_LibraryFunctions;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_VisualInterfaceLCDDefines;
using LFS_NET_WRAPPER;
using GBFINIMG_NET_WRAPPER;

namespace GBMSGUI_NET
{
    // GBMSGUI: main class of the class library
    public partial class GBMSGUI
    {      
        // Return codes for functions
        public class ReturnCodes
        {
            public const int Ret_Success = 0;
            public const int Ret_Failure = 1;
            public const int Ret_SkipImage = 2;
            public const int Ret_BreakSequence = 3;
            public const int Ret_BusyAcquiring = 4;
            public const int Ret_GoBack = 5;
        }

        // returned by GetErrorCode when return code is Ret_Failure
        public class ErrorCodes : GBMSAPI_NET_ErrorCodes
        {
            // MSGUI specific errors
            public const int GBMSGUI_ERR_Invalid_Parameter = 101;
            public const int GBMSGUI_ERR_Session_Already_Open = 102;
            public const int GBMSGUI_ERR_Session_Not_Open = 103;
            public const int GBMSGUI_ERR_Segmentation_Required = 104;
            public const int GBMSGUI_ERR_AfterStartCallback_Failed = 105;
            public const int GBMSGUI_ERR_Full_Enhanced_SDK_Not_Supported = 106;
        }

        // Acquisition Modes for Session
        public class AcquisitionModes
        {
            public const int SingleAcquisition = 0;
            public const int MultipleAcquisition = 1;
        }

        // session options
        public class SessionOption
        {
            public const uint SequenceCheck = 0x00000001;
            public const uint AskUnavailabilityReason = 0x00000002;
            public const uint NoAdvancedView = 0x00000004;
            public const uint DryFingerImageEnhancement = 0x00000008; // 1.9.12.0
        }

        // acquisition options
        public class AcquisitionOption
        {
            public const uint Segmentation = 0x00000001;
            public const uint Autocapture = 0x00000002;
            public const uint Sound = 0x00000004;
            public const uint Autoaccept = 0x00000008;
            public const uint BlockAutocapture = 0x00000010;
            public const uint RemoveHaloLatent = 0x00000020;
            public const uint DetectInvalidPattern = 0x00000040;
            public const uint DetectIncompletePattern = 0x00000080;
            public const uint DetectInclination = 0x00000100;
            public const uint RotateFinger = 0x00000200;
            public const uint FullResPreview = 0x00000400;
            public const uint RollPreviewManualStop = 0x00000800;
            public const uint RollPreviewType = 0x00001000;
            public const uint PalmPrintQualityCalculation = 0x00002000;
            public const uint SoundOnRollPreviewEnd = 0x00004000;
            public const uint NoRollPreview = 0x00008000;
            public const uint NoArtefactsDisplay = 0x00010000;
            public const uint HighSpeedPreview = 0x00020000;
            public const uint AutoClearOutsideRoll = 0x00040000;
            public const uint SegmentsQualityEval = 0x00080000;
            public const uint AutoIncompleteSlapSegm = 0x00100000;
        }

        public class QualityAlgorithms
        {
            public const int NFIQAlgorithm = 1;	// use NFIQ algorithm as defined in NISTIR 7151
            public const int GBAlgorithm = 2;	// use Green Bit fingerprint quality algorithm
        }

        public class UnavailabilityReason
        {
            public const int Amputated = 1;     // finger is missing
            public const int Unprintable = 2;   // other reasons (e.g. bandaged)
        }

        // 1.9.12.0
        public class FingerContactEvaluationMode
        {
            public const int DryWetWarning = 1; // use dry/wet warning
            public const int Contrast = 2;      // use contrast value evaluation
        }

        // Afterstart callback
        public delegate bool AfterStartCallback(IntPtr UserDefinedParameter);

        // Scanned objects
        // see GBMSAPI_NET_ScannableObjects { }

        //----------------------------------------------------------------------------------------
        internal uint SessionOptions = 0;
        internal uint AcquisitionOptions = 0;
        internal bool Resolution1000Dpi = false;
        internal bool BlockAutocaptureSupported = false;
        internal bool ImageRotationSupported = false;
        internal bool DryFingerImageEnhancementSupported = false;  // 1.9.12.0
        internal int FingerContactEvalMode = FingerContactEvaluationMode.DryWetWarning; // 1.9.12.0
        internal AfterStartCallback AfterStartCallbackRef = null;
        internal IntPtr AfterStartCallbackParam;
        internal uint SupportedScanOptions;       // options supported by current scanner

        internal class ConfigurationData
        {
            public int ArtefactsThreshold1 = 7;
            public int ArtefactsThreshold2 = 14;
            public int QualityAlgorithm = GBMSGUI.QualityAlgorithms.NFIQAlgorithm;
            public int PatternValidityThreshold = 75;
            public int PatternCompletenessThreshold = 85;
            public int LowerPalmCompletenessThreshold1 = 80;
            public int LowerPalmCompletenessThreshold2 = 90;
            public uint BlockAutoCaptureMask = GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_SCANNER_SURFACE_NOT_NORMA
                | GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_SLIDING
                | GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_LEFT
                | GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_RIGHT
                | GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_OUT_OF_REGION_TOP
                | GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_FLAT_FINGER_DISPLACED_DOWN
                // 1.9.12.0
                | GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_DRY_FINGER
                | GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_WET_FINGER;
            public bool BlockAutoCaptureContrast = false;
            public int InclinationThreshold = 35;
            public int AcqWindowLeft = -1;
            public int AcqWindowTop = -1;
            public int AcqWindowWidth = -1;
            public int AcqWindowHeight = -1;
            public bool AcqWindowMaximized = true;
            public CultureInfo Culture = null;
            public uint IgnoreDiagnosticMask = 0;
            public bool FlatFingerOnRollArea = false;
        }

        internal ConfigurationData CfgData = new ConfigurationData();

        internal class SegmentsConfigurationData
        {
            public int ImageSizeX = 500;
            public int ImageSizeY = 500;
            public int UnavailabilityReason = -1;
            public int QualityThreshold1 = 2;
            public int QualityThreshold2 = 3;
        }
        internal SegmentsConfigurationData[] SegmCfgData = new SegmentsConfigurationData[4];

        internal class ScannedObject : Object 
        {
            public uint ScannedObjectID;
            public uint ImageSizeX;     // requested image size
            public uint ImageSizeY;
            public int ContrastThreshold1;
	        public int ContrastThreshold2;
	        public int QualityThreshold1;
	        public int QualityThreshold2;
            // output values (filled after acquisition)
            public bool Acquired;       
            public int NFIQQuality;
            public int GBQuality;
            public uint RolledArtefactSize;
            public uint Completeness;    // for lower palm
            public int UnavailabilityReason;

            // ScannedObject: object used for ScannedObjectsList
            public ScannedObject(uint ObjID)
            {
                uint ObjType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ObjID);
                ScannedObjectID = ObjID;

                // set default values according to ObjID

                // image size: use values defined in ANSI/NIST ITL 1-2007
                // NOTE: size is initialized at 500 dpi; after, in InitScannedObjectsList (called in StartSession),
                // is adapted to current scanner resolution
                if (GBMSGUI.IsSlap(ObjID))
                {
                    //V1.1 - for slap 2 (DS40), set 800x800
                    if ((ObjType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_SLAP_2) ||
                        //V1.2
                        (ObjType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_INDEXES_2))
                    {
                        ImageSizeX = 800;
                        ImageSizeY = 800;
                    }
                    else
                    {
                        ImageSizeX = 1600;
                        ImageSizeY = 1500;
                    }
                }
                else if (GBMSGUI.IsRolled(ObjID))
                {
                    ImageSizeX = 800;
                    ImageSizeY = 750;
                }
                else if ((ObjType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_LOWER_HALF_PALM) ||
                    (ObjType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_UPPER_HALF_PALM))
                {
                    ImageSizeX = 2500;
                    ImageSizeY = 2500;
                }
                else if (ObjType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_WRITER_PALM)
                {
                    ImageSizeX = 900;
                    ImageSizeY = 2500;
                }
                else if ((ObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_THUMB) ||
                    (ObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_THUMB))
                {
                    ImageSizeX = 500;
                    ImageSizeY = 1000;
                }
                else // remaining flat single fingers
                {
                    ImageSizeX = 500;
                    ImageSizeY = 500;
                }

                // Default contrast thresholds
                ContrastThreshold1 = 120;
                ContrastThreshold2 = 150;

                // Default Quality thresholds for NFIQ
                QualityThreshold1 = 2;
                QualityThreshold2 = 3;

                Acquired = false;
                NFIQQuality = 254;  // not calculated
            }

            public override bool Equals(Object obj)
            {
                // If parameter is null return false.
                if (obj == null)
                    return false;

                // If parameter cannot be cast to ScannedObject return false.
                ScannedObject s = obj as ScannedObject;
                if ((System.Object)s == null)
                {
                    return false;
                }

                if (s.ScannedObjectID == this.ScannedObjectID)
                    return true;

                return false;
            }

            public override int GetHashCode()
            {
                return (int)ScannedObjectID;
            }
        }
        // list of objects to scan, created in the session
        internal List<ScannedObject> ScannedObjectsList;

        // data for sequence check
        internal class SequenceCheckItem
        {
            public String FingerID;
            public List<NW_LFS_MINUTIA> Minutiae;
        }
        internal List<SequenceCheckItem> SequenceCheckData = new List<SequenceCheckItem>();

        // class used for UnavailabilityReasonForm
        internal class UnavailabilityReasonListItem
        {
            public UnavailabilityReasonListItem(String ObjectName, int Reason)
            {
                this.mObjectName = ObjectName;
                this.mReason = Reason;
            }
            private String mObjectName;
            private int mReason;
            public String ObjectName
            {
                get { return mObjectName; }
                set { mObjectName = value; }
            }
            public int Reason
            {
                get { return mReason; }
                set { mReason = value; }
            }
        }


        private ResourceManager LocRM;              // Resource Manager for Localization
        private AcquisitionForm frmAcquisition;     // Acquisition form
        private String ErrorMessage;                // last error message
        private int ErrorCode;                      // last error code

        private bool OnSession = false;
        private bool OnAcquisition = false;

        // link to the library for beeping using DirectSound in OS not supporting Beep
        [DllImport("DSBeep.dll")]
        private static extern bool DSInit(uint CoInit, bool ForceDS);

        public GBMSGUI()
        {
            for (int i = 0; i < 4; i++)
                SegmCfgData[i] = new SegmentsConfigurationData();

            // create resource manager
            try
            {
                LocRM = new ResourceManager("GBMSGUI_NET.MSGUIStrings", typeof(GBMSGUI).Assembly);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + " in GBMSGUI constructor");
                return;
            }
        }

        // public functions:
        // are the funcion wrapped and exported by C wrapper GBMSGUI.dll
        // GBMSGUI.h contains description of these functions
        public int SetLanguage(CultureInfo Culture)
        {
            if (Culture == null)
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return ReturnCodes.Ret_Failure;
            }

            CfgData.Culture = Culture;

            // if LCD supports multilanguage, set language to LCD too
            uint ExternalEquipment;
            int ret;
            ret = GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetOptionalExternalEquipment(out ExternalEquipment);
            if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                SetError(GetMSAPIErrorMessage("GBMSAPI_NET_GetOptionalExternalEquipment", ret), ret);
                return ReturnCodes.Ret_Failure;
            }
            if (GBMSGUI.CheckMask(ExternalEquipment, GBMSAPI_NET_OptionalExternalEquipment.GBMSAPI_NET_OED_VUI_LCD))
            {
                uint LCDFeatures;
                GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_GetLcdFeatures(out LCDFeatures);
                if (GBMSGUI.CheckMask(LCDFeatures, GBMSAPI_NET_DisplayFeatures.GBMSAPI_NET_VILCD_LF_LANGUAGE_SETTING))
                    GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetLanguage(Culture.TwoLetterISOLanguageName);
            }

            return ReturnCodes.Ret_Success;
        }

        public void GetVersion(out int Major, out int Minor, out int Build, out int Revision)
        {
            Assembly assembly = Assembly.GetExecutingAssembly();
            AssemblyName name = assembly.GetName();
            Version ver = name.Version;

            Major = ver.Major;
            Minor = ver.Minor;
            Build = ver.Build;
            Revision = ver.Revision;
        }

	    public int SetWindowSizeAndPosition(int Left, int Top, int Width, int Height, bool Maximized)
        {
            // set size anyway, error is like a warning
            if (Left < 0)
                CfgData.AcqWindowLeft = 0;
            else
                CfgData.AcqWindowLeft = Left;
            if (Top < 0)
                CfgData.AcqWindowTop = 0;
            else
                CfgData.AcqWindowTop = Top;
            CfgData.AcqWindowWidth = Width;
            CfgData.AcqWindowHeight = Height;
            CfgData.AcqWindowMaximized = Maximized;

            if (Width < 1024)
            {
                Width = 1024;
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return ReturnCodes.Ret_Failure;
            }

            if (Height < 768)
            {
                Height = 768;
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return ReturnCodes.Ret_Failure;
            }

            return ReturnCodes.Ret_Success;
        }

        public int GetWindowSizeAndPosition(out int Left, out int Top, out int Width, out int Height, out bool Maximized)
        {
            Left = CfgData.AcqWindowLeft;
            Top = CfgData.AcqWindowTop;
            Width = CfgData.AcqWindowWidth;
            Height = CfgData.AcqWindowHeight;
            Maximized = CfgData.AcqWindowMaximized;

            return ReturnCodes.Ret_Success;
        }

        public int SetArtefactsThresholds(int Threshold1, int Threshold2)
        {
            if ((Threshold1 < -1) || (Threshold1 > Threshold2))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return ReturnCodes.Ret_Failure;
            }

            CfgData.ArtefactsThreshold1 = Threshold1;
            CfgData.ArtefactsThreshold2 = Threshold2;

            return ReturnCodes.Ret_Success;
        }

        public int SetQualityAlgorithm(int AlgorithmID)
        {
            if ((AlgorithmID != GBMSGUI.QualityAlgorithms.GBAlgorithm)
                && (AlgorithmID != GBMSGUI.QualityAlgorithms.NFIQAlgorithm))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return ReturnCodes.Ret_Failure;
            }

            CfgData.QualityAlgorithm = AlgorithmID;

            return ReturnCodes.Ret_Success;
        }

        public int SetPatternValidityThreshold(int Threshold)
        {
            if ((Threshold < 0) || (Threshold > 100))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return ReturnCodes.Ret_Failure;
            }

            CfgData.PatternValidityThreshold = Threshold;

            return ReturnCodes.Ret_Success;
        }

        public int SetPatternCompletenessThreshold(int Threshold)
        {
            if ((Threshold < 0) || (Threshold > 100))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return ReturnCodes.Ret_Failure;
            }

            CfgData.PatternCompletenessThreshold = Threshold;

            return ReturnCodes.Ret_Success;
        }

        public int SetLowerPalmCompletenessThresholds(int Threshold1, int Threshold2)
        {
            if ((Threshold1 < 0) || (Threshold2 < 0) || (Threshold1 > 100) || (Threshold2 > 100)
                || (Threshold1 > Threshold2))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return ReturnCodes.Ret_Failure;
            }

            CfgData.LowerPalmCompletenessThreshold1 = Threshold1;
            CfgData.LowerPalmCompletenessThreshold2 = Threshold2;

            return ReturnCodes.Ret_Success;
        }

        public int SetInclinationThreshold(int Threshold)
        {
            if ((Threshold < 0) || (Threshold > 55))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return ReturnCodes.Ret_Failure;
            }

            CfgData.InclinationThreshold = Threshold;

            return ReturnCodes.Ret_Success;
        }

        public int SetBlockAutoCaptureMask(uint Mask)
        {
            CfgData.BlockAutoCaptureMask = Mask;

            return ReturnCodes.Ret_Success;
        }

        public int SetBlockAutoCaptureContrast(bool Flag)
        {
            CfgData.BlockAutoCaptureContrast = Flag;

            return ReturnCodes.Ret_Success;
        }

        public int SetIgnoreDiagnosticMask(uint Mask)
        {
            CfgData.IgnoreDiagnosticMask = Mask;

            // "scanner error" and "object on scanner" cannot be masked
            CfgData.IgnoreDiagnosticMask &= ~GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_SCANNER_FAILURE;
            CfgData.IgnoreDiagnosticMask &= ~GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_SCANNER_SURFACE_NOT_NORMA;

            return ReturnCodes.Ret_Success;
        }

        public int SetFlatFingerOnRollArea(bool FlatFingerOnRollArea)
        {
            CfgData.FlatFingerOnRollArea = FlatFingerOnRollArea;

            return ReturnCodes.Ret_Success;
        }

        public int StartSession(int AcquisitionMode, String PersonID, uint Options)
        {
            int ret;

            if (OnAcquisition)
                return GBMSGUI.ReturnCodes.Ret_BusyAcquiring;

            if (OnSession)
            {
                SetError(GetLocalizedString("Err_MSGUI_Session_Already_Open"), ErrorCodes.GBMSGUI_ERR_Session_Already_Open);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if ((AcquisitionMode != (int)AcquisitionModes.SingleAcquisition) &&
                (AcquisitionMode != (int)AcquisitionModes.MultipleAcquisition))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return ReturnCodes.Ret_Failure;
            }

            // if device is not defined, return error
            byte DeviceID;
            String SerialNum;
            ret = GBMSAPI_NET_DeviceSettingRoutines.GBMSAPI_NET_GetCurrentDevice(out DeviceID, out SerialNum);
            if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                SetError(GetMSAPIErrorMessage("GBMSAPI_NET_GetCurrentDevice", ret), ret);
                return ReturnCodes.Ret_Failure;
            }

            // check serial number to see if Full Enhanced SDK is supported
            String CharToCheck = SerialNum.Substring(2, 1);
            if ((CharToCheck.ToUpper() == "B") || (CharToCheck.ToUpper() == "L"))
            {
                SetError(GetLocalizedString("Msg_Full_Enhanced_SDK_Not_Supported"), ErrorCodes.GBMSGUI_ERR_Full_Enhanced_SDK_Not_Supported);
                return ReturnCodes.Ret_Failure;
            }

            // 1.9.15.0 - check GBFINIMG license
            NW_GBFINIMG_LicenseUSBScannerCheck GBFINIMGLicense;
            NW_GBFINIMG_ProcessImage.NW_GBFINIMG_GetLicenseInfo(out GBFINIMGLicense);
	        if (GBFINIMGLicense.Result != NW_GBFINIMG_UsbDriverErrorCodeValues.NoError)
	        {
		        if (GBFINIMGLicense.Result == NW_GBFINIMG_UsbDriverErrorCodeValues.UsbDeviceNotFound)
		        {
                    SetError(GetLocalizedString("Msg_Full_Enhanced_SDK_Not_Supported"), ErrorCodes.GBMSGUI_ERR_Full_Enhanced_SDK_Not_Supported);
                    return ReturnCodes.Ret_Failure;
		        }
		        if (GBFINIMGLicense.Result == NW_GBFINIMG_UsbDriverErrorCodeValues.UsbDriver)
		        {
		            String ErrMsg;
                    ErrMsg = GetLocalizedString("Err_LL_USB_Driver");
                    ErrMsg = ErrMsg + " (" + GBFINIMGLicense.UsbDriverErrorCodeDetail.ToString("X") + ")";
                    SetError(ErrMsg, ErrorCodes.GBMSGUI_ERR_Full_Enhanced_SDK_Not_Supported);	// Boh???
                    return ReturnCodes.Ret_Failure;
                }
	        }

            // get characteristics of current device
            uint Features;
            ret = GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetDeviceFeatures(out Features);
            if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                SetError(GetMSAPIErrorMessage("GBMSAPI_NET_GetAdditionalDeviceFeatures", ret), ret);
                return ReturnCodes.Ret_Failure;
            }

            if (CheckMask(Features, GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_1000DPI_RESOLUTION))
                Resolution1000Dpi = true;
            else
                Resolution1000Dpi = false;

            if (CheckMask(Features, GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_AUTO_CAPTURE_BLOCKING))
                BlockAutocaptureSupported = true;
            else
                BlockAutocaptureSupported = false;

            if (CheckMask(Features, GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_IMAGE_ROTATION))
                ImageRotationSupported = true;
            else
                ImageRotationSupported = false;

            // 1.9.12.0
            if (CheckMask(Features, GBMSAPI_NET_DeviceFeatures.GBMSAPI_NET_DF_DRY_SKIN_IMG_ENHANCE))
                DryFingerImageEnhancementSupported = true;
            else
                DryFingerImageEnhancementSupported = false;

            // get supported scan options
            ret = GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetSupportedScanOptions(out SupportedScanOptions);
            if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                SetError(GetMSAPIErrorMessage("GBMSAPI_NET_GetSupportedScanOptions", ret), ret);
                return ReturnCodes.Ret_Failure;
            }

            SessionOptions = Options;

            // create scanned objects list
            ScannedObjectsList = new List<ScannedObject>();
            // init scanned objects list
            InitScannedObjectsList();
            SetDefaultSegmentsConfiguration();

            // pass reference to current instance of GBMSGUI (this) to acquisition form
            frmAcquisition = new AcquisitionForm(this);

            if (AcquisitionMode == (int)AcquisitionModes.SingleAcquisition)
                frmAcquisition.SingleAcquisition = true;
            else
                frmAcquisition.SingleAcquisition = false;

            frmAcquisition.PersonID = PersonID;

            // window size and position
            if (CfgData.AcqWindowWidth != -1)
            {
                frmAcquisition.Left = CfgData.AcqWindowLeft;
                frmAcquisition.Top = CfgData.AcqWindowTop;
                frmAcquisition.Width = CfgData.AcqWindowWidth;
                frmAcquisition.Height = CfgData.AcqWindowHeight;
                if (CfgData.AcqWindowMaximized)
                    frmAcquisition.WindowState = FormWindowState.Maximized;
                else
                    frmAcquisition.WindowState = FormWindowState.Normal;
            }

            // init DSBeep (to avoid delay on first call)
            // pass the value to initialize COM for using DirectSound (if used)
            // call by default with 0x02 (COINIT_APARTMENTTHREADED) (objbase.h)
            // if application need it, change to 0x00 (COINIT_MULTITHREADED)
            DSInit(0x02, false);

            OnSession = true;
            return ReturnCodes.Ret_Success;
        }

        public int SetImageSize(uint ScannedObjectID, uint Width, uint Height)
        {
            bool Result = true;

            if (!OnSession)
            {
                SetError(GetLocalizedString("Err_MSGUI_Session_Not_Open"), ErrorCodes.GBMSGUI_ERR_Session_Not_Open);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if (!IsValidScannedObjectID(ScannedObjectID))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            // get max size for object type on current scanner
            uint MaxImageSizeX, MaxImageSizeY;
            bool FlatScanArea = true;
            if (GBMSGUI.IsRolled(ScannedObjectID))
                FlatScanArea = false;
            else if (GBMSGUI.IsFlatSingleFinger(ScannedObjectID)
                && CfgData.FlatFingerOnRollArea
                && GBMSGUI.CheckMask(SupportedScanOptions, GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA))
                FlatScanArea = false;

            int ret = GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetImageSize(
                GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScannedObjectID),
                FlatScanArea, false, out MaxImageSizeX, out MaxImageSizeY);
            if (ret != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                SetError(GetMSAPIErrorMessage("GBMSAPI_NET_GetImageSize", ret), ret);
                return ReturnCodes.Ret_Failure;
            }

            // min size
            uint MinImageSizeX, MinImageSizeY;
            if (Resolution1000Dpi)
                MinImageSizeX = MinImageSizeY = 400;
            else
                MinImageSizeX = MinImageSizeY = 200;
            if ((Width < MinImageSizeX) || (Height < MinImageSizeY))
            {
                // set anyway to min size, so error can be treated as a warning
                if (Width < MinImageSizeX)
                    Width = MinImageSizeX;
                if (Height < MinImageSizeY)
                    Height = MinImageSizeY;
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                Result = false;
                //return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            // max size
            if ((Width > MaxImageSizeX) || (Height > MaxImageSizeY))
            {
                // set anyway to max size, so error can be treated as a warning
                if (Width > MaxImageSizeX)
                    Width = MaxImageSizeX;
                if (Height > MaxImageSizeY)
                    Height = MaxImageSizeY;
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                Result = false;
                //return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            ScannedObject ScanObj = new ScannedObject(ScannedObjectID);
            ScanObj = ScannedObjectsList[ScannedObjectsList.IndexOf(ScanObj)];

            ScanObj.ImageSizeX = Width;
            ScanObj.ImageSizeY = Height;

            if (Result)
                return ReturnCodes.Ret_Success;
            else 
                return GBMSGUI.ReturnCodes.Ret_Failure;
        }

        public int SetContrastThresholds(uint ScannedObjectID, int Threshold1, int Threshold2)
        {
            if (!OnSession)
            {
                SetError(GetLocalizedString("Err_MSGUI_Session_Not_Open"), ErrorCodes.GBMSGUI_ERR_Session_Not_Open);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if (!IsValidScannedObjectID(ScannedObjectID))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if ((Threshold1 < 0) || (Threshold2 < 0) || (Threshold1 > 255) || (Threshold2 > 255)
                || (Threshold1 > Threshold2))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            ScannedObject ScanObj = new ScannedObject(ScannedObjectID);
            ScanObj = ScannedObjectsList[ScannedObjectsList.IndexOf(ScanObj)];

            ScanObj.ContrastThreshold1 = Threshold1;
            ScanObj.ContrastThreshold2 = Threshold2;

            return ReturnCodes.Ret_Success;
        }

        public int SetQualityThresholds(uint ScannedObjectID, int Threshold1, int Threshold2)
        {
            if (!OnSession)
            {
                SetError(GetLocalizedString("Err_MSGUI_Session_Not_Open"), ErrorCodes.GBMSGUI_ERR_Session_Not_Open);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if (!IsValidScannedObjectID(ScannedObjectID))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            int MaxThreshold;
            if (CfgData.QualityAlgorithm == GBMSGUI.QualityAlgorithms.NFIQAlgorithm)
                MaxThreshold = 5;
            else
                MaxThreshold = 100;

            // 0 is permitted; both 0 means no thresholds
            if ((Threshold1 < 0) || (Threshold1 > MaxThreshold) ||
                (Threshold2 < 0) || (Threshold2 > MaxThreshold) ||
                (Threshold1 > Threshold2))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            ScannedObject ScanObj = new ScannedObject(ScannedObjectID);
            ScanObj = ScannedObjectsList[ScannedObjectsList.IndexOf(ScanObj)];

            ScanObj.QualityThreshold1 = Threshold1;
            ScanObj.QualityThreshold2 = Threshold2;

            return ReturnCodes.Ret_Success;
        }

        public int SetSegmentImageSize(uint ScannedObjectID, int Index, uint Width, uint Height)
        {
            if (!OnSession)
            {
                SetError(GetLocalizedString("Err_MSGUI_Session_Not_Open"), ErrorCodes.GBMSGUI_ERR_Session_Not_Open);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if (!IsValidScannedObjectID(ScannedObjectID) && !GBMSGUI.IsSlap(ScannedObjectID))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if (Index < 0 || Index > 3)
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            int SegmentIndex;
            // in case of left slap, reverse index
            if (ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
                SegmentIndex = 3 - Index;
            else if (ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_LEFT)
                SegmentIndex = 1 - Index;
            else
                SegmentIndex = Index;

            bool Result = true;

            // min size
            uint MinImageSizeX, MinImageSizeY;
            if (Resolution1000Dpi)
                MinImageSizeX = MinImageSizeY = 400;
            else
                MinImageSizeX = MinImageSizeY = 200;
            if ((Width < MinImageSizeX) || (Height < MinImageSizeY))
            {
                // set anyway to min size, so error can be treated as a warning
                if (Width < MinImageSizeX)
                    Width = MinImageSizeX;
                if (Height < MinImageSizeY)
                    Height = MinImageSizeY;
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                Result = false;
                //return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            // check that size requested is not bigger than slap
            ScannedObject ScanObj = new ScannedObject(ScannedObjectID);
            ScanObj = ScannedObjectsList[ScannedObjectsList.IndexOf(ScanObj)];
            if ((Width > ScanObj.ImageSizeX) || (Height > ScanObj.ImageSizeY))
            {
                // set anyway to max size, so error can be treated as a warning
                if (Width > ScanObj.ImageSizeX)
                    Width = ScanObj.ImageSizeX;
                if (Height > ScanObj.ImageSizeY)
                    Height = ScanObj.ImageSizeY;
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                Result = false;
                //return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            SegmCfgData[SegmentIndex].ImageSizeX = (int)Width;
            SegmCfgData[SegmentIndex].ImageSizeY = (int)Height;

            if (Result)
                return ReturnCodes.Ret_Success;
            else
                return GBMSGUI.ReturnCodes.Ret_Failure;
        }

        public int SetSegmentQualityThresholds(uint ScannedObjectID, int Index, int Threshold1, int Threshold2)
        {
            if (!OnSession)
            {
                SetError(GetLocalizedString("Err_MSGUI_Session_Not_Open"), ErrorCodes.GBMSGUI_ERR_Session_Not_Open);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if (!IsValidScannedObjectID(ScannedObjectID) && !GBMSGUI.IsSlap(ScannedObjectID))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if (Index < 0 || Index > 3)
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            int SegmentIndex;
            // in case of left slap, reverse index
            if (ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
                SegmentIndex = 3 - Index;
            else if (ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_LEFT)
                SegmentIndex = 1 - Index;
            else
                SegmentIndex = Index;

            int MaxThreshold;
            if (CfgData.QualityAlgorithm == GBMSGUI.QualityAlgorithms.NFIQAlgorithm)
                MaxThreshold = 5;
            else
                MaxThreshold = 100;

            // 0 is permitted; both 0 means no thresholds
            if ((Threshold1 < 0) || (Threshold1 > MaxThreshold) ||
                (Threshold2 < 0) || (Threshold2 > MaxThreshold) ||
                (Threshold1 > Threshold2))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            SegmCfgData[SegmentIndex].QualityThreshold1 = Threshold1;
            SegmCfgData[SegmentIndex].QualityThreshold2 = Threshold2;

            return ReturnCodes.Ret_Success;
        }

        public int SetSegmentUnavailabilityReason(uint ScannedObjectID, int Index, int Reason)
        {
            if (!OnSession)
            {
                SetError(GetLocalizedString("Err_MSGUI_Session_Not_Open"), ErrorCodes.GBMSGUI_ERR_Session_Not_Open);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if (!IsValidScannedObjectID(ScannedObjectID) && !GBMSGUI.IsSlap(ScannedObjectID))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if (Index < 0 || Index > 3)
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            int SegmentIndex;
            // in case of left slap, reverse index
            if (ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT)
                SegmentIndex = 3 - Index;
            else if (ScannedObjectID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_LEFT)
                SegmentIndex = 1 - Index;
            else
                SegmentIndex = Index;

            if ((Reason != GBMSGUI.UnavailabilityReason.Amputated)
                && (Reason != GBMSGUI.UnavailabilityReason.Unprintable))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            SegmCfgData[SegmentIndex].UnavailabilityReason = Reason;

            return ReturnCodes.Ret_Success;
        }

        public int SetAfterStartCallback(AfterStartCallback Callback, IntPtr Param)
        {
            AfterStartCallbackRef = Callback;
            AfterStartCallbackParam = Param;

            return ReturnCodes.Ret_Success;
        }

        public int Acquire(uint ScannedObjectID, uint Options, Byte[] Image, out int ImageSizeX, out int ImageSizeY, out int Dpi)
        {
            ImageSizeX = -1;
            ImageSizeY = -1;

            if (Resolution1000Dpi)
                Dpi = 1000;
            else
                Dpi = 500;

            if (!OnSession)
            {
                SetError(GetLocalizedString("Err_MSGUI_Session_Not_Open"), ErrorCodes.GBMSGUI_ERR_Session_Not_Open);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if (OnAcquisition)
                return GBMSGUI.ReturnCodes.Ret_BusyAcquiring;

            AcquisitionOptions = Options;

            // can't perform sequence check if segmentation is not active
            if (CheckMask(SessionOptions, SessionOption.SequenceCheck) &&
                !CheckMask(AcquisitionOptions, AcquisitionOption.Segmentation))
            {
                SetError(GetLocalizedString("Err_MSGUI_Segmentation_Required"), ErrorCodes.GBMSGUI_ERR_Segmentation_Required);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if (!IsValidScannedObjectID(ScannedObjectID))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if (Image == null)
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            ScannedObject ScanObj = new ScannedObject(ScannedObjectID);
            // retrieve object in the list
            ScanObj = ScannedObjectsList[ScannedObjectsList.IndexOf(ScanObj)];

            OnAcquisition = true;
            frmAcquisition.ScanResult = -1;
            frmAcquisition.ScannedObject = ScanObj;

            // show dialog in non modal way
            frmAcquisition.Enabled = true;
            if (!frmAcquisition.Visible)
                frmAcquisition.Show();

            // call Acquire method of the form
            frmAcquisition.Acquire();

            // wait for result
            int SleepCounter = 0;
            while ((frmAcquisition.ScanResult == -1) || (frmAcquisition.Acquiring))
            {
                Application.DoEvents();
                // decrease the number of sleep calls...
                if (SleepCounter == 5)
                {
                    // do not let application use 100% of CPU
                    Thread.Sleep(1);
                    SleepCounter = 0;
                }
                SleepCounter++;
            }

            frmAcquisition.AcquireEnd();

            // after slap, clear segments settings
            if (IsSlap(ScannedObjectID))
                SetDefaultSegmentsConfiguration();

            if (frmAcquisition.SingleAcquisition)
                frmAcquisition.Hide();
            //else
            //    frmAcquisition.Enabled = false;

            OnAcquisition = false;

            if (frmAcquisition.ScanResult == GBMSGUI.ReturnCodes.Ret_Success)
            {
                ScanObj.Acquired = true;

                // copy image
                ImageSizeX = frmAcquisition.bmpImage.Width;
                ImageSizeY = frmAcquisition.bmpImage.Height;
                BitmapToRawImage(frmAcquisition.bmpImage, Image);
            }

            // free image
            if (frmAcquisition.bmpImage != null)
                frmAcquisition.bmpImage.Dispose();

            // force Grarbage collection
            GC.Collect();

            return frmAcquisition.ScanResult;
        }

        public int GetIAFISQuality(uint ScannedObjectID, int QualityAlgorithm, out int Quality)
        {
            Quality = -1;

            if (!OnSession)
            {
                SetError(GetLocalizedString("Err_MSGUI_Session_Not_Open"), ErrorCodes.GBMSGUI_ERR_Session_Not_Open);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if (!IsValidScannedObjectID(ScannedObjectID))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            ScannedObject ScanObj = new ScannedObject(ScannedObjectID);
            // retrieve object in the list
            ScanObj = ScannedObjectsList[ScannedObjectsList.IndexOf(ScanObj)];

            if (QualityAlgorithm == GBMSGUI.QualityAlgorithms.GBAlgorithm)
                Quality = ScanObj.GBQuality;
            else
                Quality = ScanObj.NFIQQuality;

            return ReturnCodes.Ret_Success;
        }

        public int GetUnavailabilityReason(uint ScannedObjectID, out int Reason)
        {
            Reason = -1;

            if (!OnSession)
            {
                SetError(GetLocalizedString("Err_MSGUI_Session_Not_Open"), ErrorCodes.GBMSGUI_ERR_Session_Not_Open);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if (!IsValidScannedObjectID(ScannedObjectID))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            ScannedObject ScanObj = new ScannedObject(ScannedObjectID);
            // retrieve object in the list
            ScanObj = ScannedObjectsList[ScannedObjectsList.IndexOf(ScanObj)];

            Reason = ScanObj.UnavailabilityReason;

            return ReturnCodes.Ret_Success;
        }

        public int GetSegmentationResult(int Index, int QualityAlgorithmID,
            out bool SegmentAvailable, out int Left, out int Top, out int Right, out int Bottom,
            out int UnavailabilityReason, out int FingerprintQuality,
            Byte[] ImagePtr, out int ImageSizeX, out int ImageSizeY, out int Dpi)
        {
            SegmentAvailable = false;
            Left = -1;
            Top = -1;
            Right = -1;
            Bottom = -1;
            FingerprintQuality = -1;
            UnavailabilityReason = -1;
            ImageSizeX = -1;
            ImageSizeY = -1;
            Dpi = -1;

            if (!OnSession)
            {
                SetError(GetLocalizedString("Err_MSGUI_Session_Not_Open"), ErrorCodes.GBMSGUI_ERR_Session_Not_Open);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if (Index < 0 || Index > 3)
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return GBMSGUI.ReturnCodes.Ret_Failure;
            }

            if ((QualityAlgorithmID != GBMSGUI.QualityAlgorithms.GBAlgorithm)
                && (QualityAlgorithmID != GBMSGUI.QualityAlgorithms.NFIQAlgorithm))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return ReturnCodes.Ret_Failure;
            }

            if (Resolution1000Dpi)
                Dpi = 1000;
            else
                Dpi = 500;

            return frmAcquisition.GetSegmentationResult(Index, QualityAlgorithmID,
                out SegmentAvailable, out Left, out Top, out Right, out Bottom,
                out UnavailabilityReason, out FingerprintQuality, 
                ImagePtr, out ImageSizeX, out ImageSizeY);
        }

        public int StopSession()
        {
            if (OnAcquisition)
                return GBMSGUI.ReturnCodes.Ret_BusyAcquiring;

            // save window size and position
            CfgData.AcqWindowLeft = frmAcquisition.Left;
            CfgData.AcqWindowTop = frmAcquisition.Top;
            CfgData.AcqWindowWidth = frmAcquisition.Width;
            CfgData.AcqWindowHeight = frmAcquisition.Height;
            if (frmAcquisition.WindowState == FormWindowState.Maximized)
                CfgData.AcqWindowMaximized = true;
            else
                CfgData.AcqWindowMaximized = false;

            frmAcquisition.ForceClose = true;
            frmAcquisition.Close();
            if (frmAcquisition.gcAcqFormHandle.IsAllocated)
                frmAcquisition.gcAcqFormHandle.Free();
            if (AcquisitionForm.gcAcqFormHandle_St.IsAllocated)
                AcquisitionForm.gcAcqFormHandle_St.Free();
            frmAcquisition.Dispose();

            ScannedObjectsList.Clear();
            SequenceCheckData.Clear();

            //if (LCDPresent)
            GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_VUI_LCD_SetLogoScreen();

            // force Grarbage collection
            GC.Collect();

            OnSession = false;
            return ReturnCodes.Ret_Success;
        }

        public String GetErrorMessage()
        {
            return ErrorMessage;
        }

        public int GetErrorCode()
        {
            return ErrorCode;
        }

        internal void SetError(String ErrMsg, int ErrCode)
        {
            ErrorMessage = ErrMsg;
            ErrorCode = ErrCode;
        }

        // set error string for GBMSAPI error code
        public String GetMSAPIErrorMessage(String FuncName, int ErrCode)
        {
            String ErrMsg;

            switch (ErrCode)
            {
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_USB_DRIVER:
                    ErrMsg = GetLocalizedString("Err_LL_USB_Driver");
                    uint USBErr;
                    GBMSAPI_NET_AuxiliaryRoutines.GBMSAPI_NET_GetUSBErrorCode(out USBErr);
                    ErrMsg = ErrMsg + " (" + USBErr.ToString("X") + ")";
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_DEVICE_NOT_FOUND:
                    ErrMsg = GetLocalizedString("Err_LL_Device_Not_Found");
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_USB_THREAD:
                    ErrMsg = GetLocalizedString("Err_LL_USB_Thread");
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_PARAMETER:
                    ErrMsg = GetLocalizedString("Err_LL_Parameter");
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_SCANNER_NOT_CONFIGURED:
                    ErrMsg = GetLocalizedString("Err_LL_Scanner_Not_Configured");
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_DEVICE_NOT_RESPONDING:
                    ErrMsg = GetLocalizedString("Err_LL_Device_Not_Responding");
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_SCANNER_COMMUNICATION:
                    ErrMsg = GetLocalizedString("Err_LL_Scanner_Communication");
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_UNAVAILABLE_OPTION:
                    ErrMsg = GetLocalizedString("Err_LL_Unavailable_Option");
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED:
                    ErrMsg = GetLocalizedString("Err_MSAPI_Specific_DLL_Not_Loaded");
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_METHOD_NOT_SUPPORTED:
                    ErrMsg = GetLocalizedString("Err_MSAPI_Method_Not_Supported");
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED:
                    ErrMsg = GetLocalizedString("Err_MSAPI_Object_Type_Not_Supported");
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED:
                    ErrMsg = GetLocalizedString("Err_MSAPI_Scan_Area_Not_Supported");
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_ACQUISITION_THREAD:
                    ErrMsg = GetLocalizedString("Err_MSAPI_Acquisition_Thread");
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
                    ErrMsg = GetLocalizedString("Err_MSAPI_Acquisition_Already_Started");
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_FEATURE_NOT_SUPPORTED:
                    ErrMsg = GetLocalizedString("Err_MSAPI_Feature_Not_Supported");
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_CURRENT_DEV_NOT_SET:
                    ErrMsg = GetLocalizedString("Err_MSAPI_Current_Device_Not_Set");
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_MEMORY_ALLOCATION:
                    ErrMsg = GetLocalizedString("Err_MSAPI_Memory_Allocation");
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_GENERIC:
                    ErrMsg = GetLocalizedString("Err_MSAPI_Generic");
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION:
                    ErrMsg = GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING;
                    break;
                case GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_INTERNAL:
                    ErrMsg = GetLocalizedString("Err_MSAPI_Internal");
                    break;
                default:
                    ErrMsg = GetLocalizedString("Err_Unknown") + " (" + ErrCode + ")";
                    break;
            }

            ErrMsg = ErrMsg + " (" + FuncName + ")";

            return ErrMsg;
        }

        internal String GetLocalizedString(String StringID)
        {
            if (CfgData.Culture != null)
                return LocRM.GetString(StringID, CfgData.Culture);
            else
                return LocRM.GetString(StringID);
        }

        // initializes internal list of all scannable objecs
        private void InitScannedObjectsList()
        {
            ScannedObject ScanObj;

            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_THUMB);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_INDEX);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_MIDDLE);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_RING);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_LITTLE);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_THUMB);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_INDEX);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_MIDDLE);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_RING);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_LITTLE);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_THUMB);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_INDEX);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_MIDDLE);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_RING);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_LITTLE);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_THUMB);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_INDEX);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_MIDDLE);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_RING);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_LITTLE);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_LOWER_HALF_PALM_LEFT);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_UPPER_HALF_PALM_LEFT);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_WRITER_PALM_LEFT);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_LOWER_HALF_PALM_RIGHT);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_UPPER_HALF_PALM_RIGHT);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_WRITER_PALM_RIGHT);
            ScannedObjectsList.Add(ScanObj);
            //V1.1 - DS40
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_LEFT);
            ScannedObjectsList.Add(ScanObj);
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_RIGHT);
            ScannedObjectsList.Add(ScanObj);
            //V1.2 - DS40
            ScanObj = new ScannedObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_INDEXES);
            ScannedObjectsList.Add(ScanObj);

            // adjust default image sizes if resolution is 1000 Dpi
            if (Resolution1000Dpi)
            {
                for (int i = 0; i < ScannedObjectsList.Count; i++)
                {
                    ScannedObjectsList[i].ImageSizeX *= 2;
                    ScannedObjectsList[i].ImageSizeY *= 2;
                }
            }
        }

        private void SetDefaultSegmentsConfiguration()
        {
            for (int i = 0; i < 4; i++)
            {
                SegmCfgData[i].ImageSizeX = 500;
                SegmCfgData[i].ImageSizeY = 500;
                if (Resolution1000Dpi)
                {
                    SegmCfgData[i].ImageSizeX *= 2;
                    SegmCfgData[i].ImageSizeY *= 2;
                }
                SegmCfgData[i].UnavailabilityReason = -1;
                SegmCfgData[i].QualityThreshold1 = 2;
                SegmCfgData[i].QualityThreshold2 = 3;
            }
        }

        // DEBUG
        public static void LogToFile(String Msg)
        {
            return;

            // Create a writer and open the file:
            StreamWriter log;

            if (!File.Exists("GBMSGUI.log"))
            {
                log = new StreamWriter("GBMSGUI.log");
            }
            else
            {
                log = File.AppendText("GBMSGUI.log");
            }

            // Write to the file:
            log.WriteLine(DateTime.Now + " - " + Msg);

            // Close the stream:
            log.Close();
        }

        // 1.9.12.0
        public int SelectFingerContactEvaluationMode(int Mode)
        {
            if ((Mode != FingerContactEvaluationMode.Contrast)
                && (Mode != FingerContactEvaluationMode.DryWetWarning))
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return ReturnCodes.Ret_Failure;
            }

            if ((Mode == FingerContactEvaluationMode.DryWetWarning) &&
                !DryFingerImageEnhancementSupported)
            {
                SetError(GetLocalizedString("Err_MSGUI_Invalid_Parameter"), ErrorCodes.GBMSGUI_ERR_Invalid_Parameter);
                return ReturnCodes.Ret_Failure;
            }

            // 1.9.14.0
            if (Mode == FingerContactEvaluationMode.Contrast)
            {
                // if Dry/wet warning is not shown, can't block autocapture...
                CfgData.BlockAutoCaptureMask &= ~GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_DRY_FINGER;
                CfgData.BlockAutoCaptureMask &= ~GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_WET_FINGER;
            }
            else
            {
                CfgData.BlockAutoCaptureMask |= GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_DRY_FINGER;
                CfgData.BlockAutoCaptureMask |= GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_WET_FINGER;
            }

            FingerContactEvalMode = Mode;
            return ReturnCodes.Ret_Success;
        }
    }
}
