#ifndef GBMSAPI_LIBRARY_H
#define GBMSAPI_LIBRARY_H

#ifdef GBMSAPI_EXPORTS
#define GBMSAPI_API __declspec(dllexport)
#else
#define GBMSAPI_API __declspec(dllimport)
#endif

#include "windows.h"
#include "GBMSAPI_Defines.h"

#ifdef __cplusplus
extern "C"
{
#endif

	/*******************************************************************************
	********************************************************************************
	******** GENERAL INTRODUCTION **************************************************
	********************************************************************************
	*******************************************************************************/
	//
	//	This library allows more than one Green Bit device to be attached to the USB
	//	bus, both of the same (if the device supports the 
	//	GBMSAPI_DF_MULTIPLE_USB_CONNECTIONS_ALLOWED feature - e.g. two DactyScan84) 
	//	and of different (e.g. one DactyScan84 and one Poliscan2) type, or a mixed
	//	combination of them (e.g. two DS84 and a PS2).
	//	In order to know what devices are attached to the USB port, the application
	//	can call the GBMSAPI_GetAttachedDeviceList(...) function.
	//	In order to switch from a device to another the application must use the
	//	GBMSAPI_SetCurrentDevice(...) function.
	//	When this library is loaded, a search is performed on the USB bus, and the
	//	first found device is set as the current device (see also
	//	GBMSAPI_SetCurrentDevice(...) function).
	//	See notes 1 and 2 for useful warnings.
	//
	//	NOTE1:
	//		Even if this library allows more than one device to be attached to the
	//		USB bus, only one of them will be used at once.
	//
	//	NOTE2:
	//		If any of the functions defined in this SDK returns
	//		GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED error code, this means that
	//		the specific device DLL is not present. In such a case it's recommended
	//		to reinstall the library, ensuring that the specific DLL has been copied
	//		either in the application or in the <System> PC directory.
	//
	////////////////////////////////////////////////////////////////////////////////


	/*******************************************************************************
	********************************************************************************
	*				SECTION 0													   *
	*																			   *
	*				DEVICE SETTINGS												   *
	********************************************************************************
	*******************************************************************************/
	// VER 2800
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_SetScannerLibrariesPath
	//
	// Description: 
	//	This function sets the path where the scanner libraries should be installed.
	//	When loading the scanner libraries, the GBMSAPI will follow this logic:
	//
	//	1) Searchs in the path set through this function
	//	2) If this function has never been called or the step (1) fails, searchs
	//		in the same path where the GBMSAPI is found
	//	3) If step 2 fails, libraries are searched following the OS logic (in the
	//		application path or in the system directories
	//
	//	This function should be called before the GBMSAPI_GetAttachedDeviceList
	//	and before GBMSAPI_SetCurrentDevice(...) if	user wants to install libraries
	//	in a specified path
	//
	// Parameters:
	//	<PathToBeSet>	(input):
	//		String containing the path to be set. It must be '\0' terminated and its
	//		last char must be '\\'
	//
	// Return:
	////////////////////////////////////////////////////////////////////////////////
	void GBMSAPI_API __stdcall GBMSAPI_SetScannerLibrariesPath(
		char *PathToBeSet
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetAttachedDeviceList
	//
	// Description: 
	//	This function returns the number and a list of the devices currently 
	//	attached to the USB bus.
	//
	// Parameters:
	//	<AttachedDeviceList>	(output):
	//		List of the devices attached to the USB bus. The number of its items is
	//		given in the <AttachedDeviceNumber> parameter.
	//		Valid only if GBMSAPI_ERROR_CODE_NO_ERROR is returned as error code and
	//		<AttachedDeviceNumber> is greater than 0.
	//
	//	<AttachedDeviceNumber>	(output):
	//		Number of devices attached to the USB port.
	//		Valid only if GBMSAPI_ERROR_CODE_NO_ERROR is returned as error code.
	//
	//	<USBErrorCode>	(output):
	//		In case GBMSAPI_ERROR_CODE_USB_DRIVER is returned as error, USB error code
	//		is registered here. <USBErrorCode> must	be interpreted in hexadecimal 
	//		representation. This error should be visualized and reported to the
	//		GreenBit technical support.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		<AttachedDeviceList> or <AttachedDeviceNumber> or <USBErrorCode> is NULL
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_USB_DRIVER
	//		Error in communication with USB driver. Don't call 
	//		GBMSAPI_GetUSBErrorCode(...) function, since USB error code is directly
	//		reported in <USBErrorCode> parameter.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetAttachedDeviceList(
		GBMSAPI_DeviceInfoStruct AttachedDeviceList[GBMSAPI_MAX_PLUGGED_DEVICE_NUM],
		int *AttachedDeviceNumber,
		unsigned long *USBErrorCode
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_SetCurrentDevice
	//
	// Description: 
	//	This function sets the device whose characteristics are passed as parameters
	//	as the current device to be used.
	//
	// Parameters:
	//	<DeviceID>	(input):
	//		ID of the device model: see DEVICE NAME section in GBMSAPI_Defines.h
	//
	//	<DeviceSerialNumber>	(input):
	//		Device Serial number: will be taken in consideration only if different
	//		from NULL and device supports GBMSAPI_DF_MULTIPLE_USB_CONNECTIONS_ALLOWED
	//		feature.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application.
	//
	//	- GBMSAPI_ERROR_CODE_MEMORY_ALLOCATION
	//		Cannot allocate memory for setting device
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		<DeviceID> not recognized
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_SetCurrentDevice(
		unsigned char DeviceID,
		char *DeviceSerialNumber
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetCurrentDevice
	//
	// Description: 
	//	This function returns the IDs for the current device being used.
	//
	// Parameters:
	//	<DeviceID>	(output):
	//		ID of the device model: will be taken in consideration only if different
	//		from NULL. See DEVICE NAME section in GBMSAPI_Defines.h
	//
	//	<DeviceSerialNumber>	(output):
	//		Device Serial number: will be taken in consideration only if different
	//		from NULL and it was set with a preceding call of 
	//		GBMSAPI_SetCurrentDevice(...). It must be allocated by the application
	//		and its length must be >= GBMSAPI_MAX_SN_LENGHT
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application.
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		<DeviceID> is NULL
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetCurrentDevice(
		unsigned char *DeviceID,
		char *DeviceSerialNumber
	);

	// VER2700

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetDeviceNameAndVersion
	//
	// Description: 
	//	This function returns in wide char text format firmware name and version.
	//
	// Parameters:
	//	<NameAndVersion>	(output):
	//		Device firmware name and version; it must be allocated from the
	//		application, and its size must be >= 
	//		GBMSAPI_MAX_CHARS_IN_DEVICE_NAME_AND_INFO (see GBMSAPI_Defines.h)
	//
	//	NOTE1:
	//		This function is supported only by some devices.
	//		Otherwise it returns GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1
	//		
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetDeviceNameAndVersion(
		wchar_t * NameAndVersion
	);

	/*******************************************************************************
	********************************************************************************
	*				SECTION 1													   *
	*																			   *
	*				DEVICE CHARACTERISTICS										   *
	********************************************************************************
	*******************************************************************************/

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetScannableTypes
	//
	// Description: 
	//	This function returns, in the <ScannableTypesMask> output parameter, a
	//	list of the types supported by the device, as they are described into
	//	the "SCANNABLE BIOMETRIC TYPES" section of the GBMSAPI_Defines.h file.
	//
	// Parameters:
	//	<ScannableTypesMask>	(output):
	//		List of the types that can be acquired by the device, in form of a bit
	//		mask.
	//		Valid only if GBMSAPI_ERROR_CODE_NO_ERROR is returned as error code
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		<ScannableTypesMask> is NULL
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application.
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetScannableTypes
		(
		unsigned int *ScannableTypesMask
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetTypeFromObject
	//
	// Description: 
	//	This function returns the object type corresponding to the object passed as
	//	a parameter, as they are described into the "SCANNABLE BIOMETRIC TYPES"
	//	section of the GBMSAPI_Defines.h file.
	//
	// Parameters:
	//	<ObjToScan>	(input):
	//		Must be one of the objects defined in the "SCANNABLE OBJECTS" section of
	//		the GBMSAPI_Defines.h file.
	//
	// Return:
	//	- different from GBMSAPI_SBT_NO_OBJECT_TYPE:
	//		success
	//
	//	- GBMSAPI_SBT_NO_OBJECT_TYPE:
	//		<ObjToScan> was not recognized
	////////////////////////////////////////////////////////////////////////////////
	unsigned int GBMSAPI_API __stdcall GBMSAPI_GetTypeFromObject(
		unsigned int ObjToScan
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetDeviceFeatures
	//
	// Description: 
	//	This function returns, in the <DeviceFeatures> output parameter, a
	//	list of the features supported by the device, as they are 
	//	described in the "DEVICE FEATURES" section of the GBMSAPI_Defines.h
	//	file.
	//
	// Parameters:
	//	<DeviceFeatures>	(output):
	//		List of the features supported by the device, in form of a 
	//		bit mask.
	//		Valid only if GBMSAPI_ERROR_CODE_NO_ERROR is returned as error code
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application.
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		<DeviceFeatures> is NULL
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetDeviceFeatures
		(
		unsigned int *DeviceFeatures
		);

	// VER 2.7.1.0
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_SetRollAreaStandard
	//
	// Description: 
	//	This function allows the user to set what is the standard the rolled image
	//	should be acquired with: IQS standard (1.6 x 1.6 inch) or GA chinese standard
	//	(1.28 x 1.28 inch).
	//	Call this function before GBMSAPI_StartAcquisition(...) in order to set the
	//	desired standard.
	//
	// NOTE:
	//	If requested standard by means of <RollAreaStandard> is not supported,
	//	GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED is returned and the function is
	//	not executed.
	//
	// Parameters:
	//	<RollAreaStandard>	(input):
	//		Can be unsigned GBMSAPI_DF_ROLL_AREA_IQS or GBMSAPI_DF_IQS_GA (see "DEVICE 
	//		FEATURES" section in GBMSAPI_Defines.h file)
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application.
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		<RollAreaStandard> is out of allowed range
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		Requested <RollAreaStandard> is not supported
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_SetRollAreaStandard
	(
		unsigned int RollAreaStandard
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetSupportedScanOptions
	//
	// Description: 
	//	This function returns, in the <SupportedScanOptions> output parameter, a
	//	list of the scan options supported by the device, as they are 
	//	described in the "ACQUISITION OPTIONS" section of the GBMSAPI_Defines.h
	//	file.
	//
	// Parameters:
	//	<SupportedScanOptions>	(output):
	//		List of the scan options supported by the device, in form of a 
	//		bit mask.
	//		Valid only if GBMSAPI_ERROR_CODE_NO_ERROR is returned as error code
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application.
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		<SupportedScanOptions> is NULL
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetSupportedScanOptions
	(
		unsigned int *SupportedScanOptions
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetOptionalExternalEquipment
	//
	// Description: 
	//	This function returns, in the <OptionalExternalEquipment> output parameter, a
	//	list of the external devices supported by the device, as they are 
	//	described in the "OPTIONAL EXTERNAL EQUIPMENT" section of the GBMSAPI_Defines.h
	//	file.
	//
	// Parameters:
	//	<OptionalExternalEquipment>	(output):
	//		List of the external devices supported by the device, in form of a 
	//		bit mask.
	//		Valid only if GBMSAPI_ERROR_CODE_NO_ERROR is returned as error code
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application.
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		<OptionalExternalEquipment> is NULL
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetOptionalExternalEquipment
		(
		unsigned int *OptionalExternalEquipment
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetAvailableImageInfo
	//
	// Description: 
	//	This function returns, in the <AvailableImageInfo> output parameter, a
	//	list of the information (see "AVAILABLE IMAGE INFO" section of the 
	//	GBMSAPI_Defines.h file) supported by the device for the <ObjectTypeToScan> 
	//	type of the object
	//	(see "SCANNABLE BIOMETRIC TYPES" section of the GBMSAPI_Defines.h file)
	//
	// Parameters:
	//	<ObjectTypeToScan>			(input):
	//		This type must be supported by the device (see 
	//		GBMSAPI_GetScannableTypes(...)), otherwise 
	//		GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED will be returned.
	//
	//	<AvailableImageInfo>	(output):
	//		List of the information for the <ObjectTypeToScan> type, in form of a 
	//		bit mask.
	//		Valid only if GBMSAPI_ERROR_CODE_NO_ERROR is returned as error code
	//
	//	
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application.
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		<AvailableImageInfo> is NULL
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED
	//		<ObjectTypeToScan> type is not supported by the device
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetAvailableImageInfo
		(
		unsigned int ObjectTypeToScan,
		unsigned int *AvailableImageInfo
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetImageSize
	//
	// Description: 
	//	Returns the size (both width and height) of the image returned by the 
	//	scanner during the acquisition process,	depending on the object to be 
	//	scanned, which area (see GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA option in GBMSAPI_Defines.h
	//	file) the object should be scanned on and the acquisition phase (see 
	//	ACQUISITION PROCESS DESCRIPTION section in GBMSAPI_Defines.h file) passed as 
	//	input parameters.
	//
	// Parameters:
	//	<ObjectTypeToScan>			(input):
	//		This type must be supported by the device (see 
	//		GBMSAPI_GetScannableTypes(...)), otherwise 
	//		GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED will be returned.
	//
	//	<FlatScanArea>			(input):
	//		Scanning area where the object has to be acquired.
	//		For more details about "flat" and "roll" area meaning, see the 
	//		GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA option in GBMSAPI_Defines.h file.
	//
	//		NOTE:
	//		Even if object type or device does not support this area, image size is 
	//		anyway provided in the <ImageSizeX> and <ImageSizeY> parameters, but
	//		a GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED error code is returned: treat it
	//		simply as a warning. 
	//		
	//		Values to pass:
	//
	//		- TRUE: Flat area
	//		- FALSE: Roll area
	//
	//	<Preview> (input):
	//		Acquisition phase: in order to save USB trafic and image processing,
	//		sometimes image size is different between PREVIEW and FULL-RESOLUTION 
	//		phase. For more details about acquisition phases see ACQUISITION PROCESS
	//		DESCRIPTION section in GBMSAPI_Defines.h file. 
	//		
	//		Values to pass:
	//
	//		- TRUE: PREVIEW phase
	//		- FALSE: FULL-RESOLUTION phase
	//
	//	<ImageSizeX> (output):
	//		valid only if GBMSAPI_ERROR_CODE_NO_ERROR or GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED 
	//		is returned. Image Width.
	//
	//	<ImageSizeY> (output):
	//		valid only if GBMSAPI_ERROR_CODE_NO_ERROR or GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED 
	//		is returned. Image Height.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		<ImageSizeX> or <ImageSizeY> is NULL
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED
	//		<ObjectTypeToScan> is not supported by the device
	//
	//	- GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED
	//		Scan area not supported for the <ObjectTypeToScan> type. 
	//		TREAT AS A WARNING!!!! 
	//
	//	- GBMSAPI_ERROR_CODE_GENERIC
	//		Something went wrong in size values returned from low level
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetImageSize
		(
		unsigned int	ObjectTypeToScan,
		BOOL			FlatScanArea,
		BOOL			Preview,
		unsigned int	*ImageSizeX,
		unsigned int	*ImageSizeY
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetMaxImageSize
	//
	// Description: 
	//	Returns the size (both width and height) of the biggest object capturable
	//	by the device. It can be useful to alloc memory before beginning 
	//	acquisition.
	//
	// Parameters:
	//	<ImageMaxSizeX> (output):
	//		valid only if GBMSAPI_ERROR_CODE_NO_ERROR is returned. Image Width.
	//
	//	<ImageMaxSizeY> (output):
	//		valid only if GBMSAPI_ERROR_CODE_NO_ERROR is returned. Image Height.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		<ImageMaxSizeX> or <ImageMaxSizeY> is NULL
	//
	//	- GBMSAPI_ERROR_CODE_GENERIC
	//		Something went wrong in size values returned from low level
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetMaxImageSize
		(
		unsigned int	*ImageMaxSizeX,
		unsigned int	*ImageMaxSizeY
		);

	// VER 2700
	// VERY IMPORTANT NOTE: USE THIS FUNCTION TO GET THE IMAGE SIZE FOR THE
	// CALIBRATION IMAGE
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetCalibrationImageSize
	//
	// Description: 
	//	Returns the size (both width and height) of the calibration image produced 
	//	by the scanner,	depending on the acquisition area (see 
	//	GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA option in GBMSAPI_Defines.h
	//	file) passed as input parameter.
	//	They can be used for memory allocation before using
	//	GBMSAPI_GetCalibrationImage() function.
	//
	// Parameters:
	//
	//	<FlatScanArea>			(input):
	//		Scanning area where the object has to be acquired.
	//		For more details about "flat" and "roll" area meaning, see the 
	//		GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA option in GBMSAPI_Defines.h file.
	//
	//		NOTE:
	//		Even if object type or device does not support this area, image size is 
	//		anyway provided in the <ImageSizeX> and <ImageSizeY> parameters, but
	//		a GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED error code is returned: treat it
	//		simply as a warning. 
	//		
	//		Values to pass:
	//
	//		- TRUE: Flat area
	//		- FALSE: Roll area
	//
	//	<ImageSizeX> (output):
	//		valid only if GBMSAPI_ERROR_CODE_NO_ERROR or GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED 
	//		is returned. Image Width.
	//
	//	<ImageSizeY> (output):
	//		valid only if GBMSAPI_ERROR_CODE_NO_ERROR or GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED 
	//		is returned. Image Height.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		<ImageSizeX> or <ImageSizeY> is NULL
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED
	//		<ObjectTypeToScan> is not supported by the device
	//
	//	- GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED
	//		Scan area not supported for the <ObjectTypeToScan> type. 
	//		TREAT AS A WARNING!!!! 
	//
	//	- GBMSAPI_ERROR_CODE_GENERIC
	//		Something went wrong in size values returned from low level
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetCalibrationImageSize
	(
		BOOL			FlatScanArea,
		unsigned int	*ImageSizeX,
		unsigned int	*ImageSizeY
	);

	/*******************************************************************************
	********************************************************************************
	*				SECTION 2													   *
	*																			   *
	*				SCAN SETTINGS												   *
	********************************************************************************
	*******************************************************************************/

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_SetSelectImageTimeout
	//
	// Description: 
	//	This function specifies the maximum time period for SELECT_IMAGE phase of 
	//	auto-capture procedure. 
	//	Call this function before GBMSAPI_StartAcquisition(). Enough to call once after 
	//	GBMSAPI_SetCurrentDevice(...) function.
	//	In applications where scanning time is not critical parameter, it could be
	//	recommended to set values about 3000-3500 ms. With higher value auto-capture
	//	more frequently achieves the stabilization condition, and, in other words,
	//	the best possible fingerprint quality.
	//
	//	NOTE1:
	//		This function applies only to those devices that provide auto-capture,
	//		such as DactyScan84. If called for other devices it returns
	//		GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED. It should be treated as a warning.
	//
	// Parameters:
	//	<SelectImageTimeout> (input):
	//		SELECT_IMAGE phase timeout in ms.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED:
	//		See NOTE1
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER
	//		<SelectImageTimeout> less than minimal value
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_SetSelectImageTimeout
		(
		unsigned int SelectImageTimeout
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_SetLowerHalfPalmCompletenessThreshold
	//
	// Description: 
	//	This function specifies pattern completeness threshold value for a 
	//	GBMSAPI_SBT_FLAT_LOWER_HALF_PALM object when acquisition in auto-capture mode
	//	(see AUTO-CAPTURE PROCESS section in GBMSAPI_Defines.h) is used.
	//	Auto-capture procedure doesn't stop while the specified threshold is not 
	//	achieved.
	//	Call this function before GBMSAPI_StartAcquisition() to adjust the threshold value. 
	//	Enough to call once after having called the GBMSAPI_SetCurrentDevice(...).
	//	Default value 90% is used if the function is not called.
	//	To obtain the pattern completeness value associated with the current image 
	//	see the GBMSAPI_GetLowerHalfPalmCompleteness() function.
	//
	//	NOTE1:
	//		This function is supported only if the device supports 
	//		GBMSAPI_SBT_FLAT_LOWER_HALF_PALM objects.
	//		Otherwise it returns GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED.
	//
	// Parameters:
	//	<CompletenessThreshold>	(input):
	//		availbale range 0-100%, in percents.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		<CompletenessThreshold> not in range
	//
	//	- GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED
	//		see NOTE1
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_SetLowerHalfPalmCompletenessThreshold
		(
		unsigned char CompletenessThreshold
		);

	// ver 2300
	// SetFrameRate and GBMSAPI_EnableManualLEDControl
	// VER 2700: CONSIDER THIS FUNCTION AS OBSOLETE: it is replaced by
	// GBMSAPI_SetFrameRateComplete(...)
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_SetFrameRate
	//
	// Description: 
	//	This function specifies the required nominal frame rate. It could be useful
	//	to adjust resource consumption in order to fit specific PC/USB/CPU 
	//	limitations of a given application.
	//	If the application does not have any limitations, maximum value is 
	//	recommended for the best performance of the auto-capture system.
	//
	//	NOTE1:
	//		This function is supported only by DactyScan 26 and DactyScan 84
	//		devices.
	//		Otherwise it returns GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED.
	//
	//	NOTE2:
	//		This function must be called outside of the acquisition.
	//		Otherwise it returns GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED.
	//
	//
	// Parameters:
	//	<FlatScanArea>			(input):
	//		Scanning area (see the GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA option in 
	//		GBMSAPI_Defines.h file).
	//
	//		NOTE:
	//		If device does not support this area, frame rate will not be set
	//		and a GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED error code is returned. 
	//		
	//		Values to pass:
	//
	//		- TRUE: Flat area
	//		- FALSE: Roll area
	//
	//	<FrameRate>	(input):
	//		Limits for this parameter are specified in the 
	//		"FRAME RATE RANGE FOR DEVICES" section of the GBMSAPI_Defines.h file.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		see NOTE2
	//
	//	- GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED
	//		Scan area not supported.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED
	//		see NOTE1
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER
	//		see <FrameRate> parameter
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_SetFrameRate(
		BOOL			FlatScanArea,
		double			FrameRate
	);

	// ver 2700: added GBMSAPI_SetFrameRateComplete
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_SetFrameRateComplete
	//
	// Description: 
	//	This function specifies the required nominal frame rate. It could be useful
	//	to adjust resource consumption in order to fit specific PC/USB/CPU 
	//	limitations of a given application.
	//	If the application does not have any limitations, maximum value is 
	//	recommended for the best performance of the auto-capture system.
	//
	//	NOTE1:
	//		This function is not supported by all Green Bit devices. In order to know
	//		whether this feature is supported, see the <DeviceFeatures> parameter of
	//		the GBMSAPI_GetDeviceFeatures(...) function and see whether
	//		GBMSAPI_DF_FRAME_RATE_SETTING bit is set.Otherwise it returns
	//		GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED.
	//
	//	NOTE2:
	//		This function must be called outside of the acquisition.
	//		Otherwise it returns GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED.
	//
	//
	// Parameters:
	//	<FrameRateOptions>			(input):
	//		Frame rate setting options (see the FRAME RATE OPTIONS paragraph in 
	//		GBMSAPI_Defines.h file).
	//
	//	<FrameRate>	(input):
	//		In order to get the range for this parameters, see the 
	//		GBMSAPI_GetFrameRateRange(...) function. If parameter value is outside
	//		range, GBMSAPI_ERROR_CODE_PARAMETER is returned.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		see NOTE2
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED
	//		see NOTE1
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER
	//		see <FrameRateOptions> parameter
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_SetFrameRateComplete(
		unsigned int	FrameRateOptions,
		double			FrameRate
	);

	// ver 2700: added GBMSAPI_GetFrameRateRange
	// VER 2800: CONSIDER THIS FUNCTION AS OBSOLETE: it is replaced by
	// GBMSAPI_GetFrameRateRange_Global(...)
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetFrameRateRange
	//
	// Description: 
	//	This function gets the range of the <FrameRateOptions> parameter of the
	//	"GBMSAPI_SetFrameRateComplete(...)" function. It returns the max, min and
	//	default value of the nominal frame rate.
	//
	//	NOTE1:
	//		This function is not supported by all Green Bit devices. In order to know
	//		whether this feature is supported, see the <DeviceFeatures> parameter of
	//		the GBMSAPI_GetDeviceFeatures(...) function and see whether
	//		GBMSAPI_DF_FRAME_RATE_SETTING bit is set.Otherwise it returns
	//		GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED.
	//
	//
	// Parameters:
	//	<FrameRateOptions>			(input):
	//		Frame rate setting options (see the FRAME RATE OPTIONS paragraph in 
	//		GBMSAPI_Defines.h file).
	//
	//	<MaxFrameRate>	(output):
	//		Max value of nominal frame rate.
	//
	//	<MinFrameRate>	(output):
	//		Min value of nominal frame rate.
	//
	//	<DefFrameRate>	(output):
	//		Default value of nominal frame rate.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED
	//		see NOTE1
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER
	//		see <FrameRate> parameter
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetFrameRateRange(
		unsigned int	FrameRateOptions,
		double			*MaxFrameRate,
		double			*MinFrameRate,
		double			*DefFrameRate
	);

	// VER 2800
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetFrameRateRange_Global
	//
	// Description: 
	//	This function is similar to the GBMSAPI_GetFrameRateRange(...) one, with the
	//	difference that it can be called even when the scanner specified in the 
	//	<DeviceID> parameter is not plugged.
	//
	//	NOTE1:
	//		This function is not supported by all Green Bit devices. In order to know
	//		whether this feature is supported, see the <DeviceFeatures> parameter of
	//		the GBMSAPI_GetDeviceFeatures(...) function and see whether
	//		GBMSAPI_DF_FRAME_RATE_SETTING bit is set.Otherwise it returns
	//		GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED.
	//
	//
	// Parameters:
	//	<DeviceID>			(input):
	//		Specifies the scanner type whose Frame rate range needs to be got: 
	//		see DEVICE NAME section in GBMSAPI_Defines.
	//
	//	<FrameRateOptions>			(input):
	//		Frame rate setting options (see the FRAME RATE OPTIONS paragraph in 
	//		GBMSAPI_Defines.h file).
	//
	//	<RollAreaStandard>	(input):
	//		Can be unsigned GBMSAPI_DF_ROLL_AREA_IQS or GBMSAPI_DF_IQS_GA (see "DEVICE 
	//		FEATURES" section in GBMSAPI_Defines.h file)
	//
	//	<MaxFrameRate>	(output):
	//		Max value of nominal frame rate.
	//
	//	<MinFrameRate>	(output):
	//		Min value of nominal frame rate.
	//
	//	<DefFrameRate>	(output):
	//		Default value of nominal frame rate.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED
	//		see NOTE1
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER
	//		see <FrameRate> parameter
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetFrameRateRange_Global(
		unsigned char	DeviceID,
		unsigned int	FrameRateOptions,
		unsigned int	RollAreaStandard,
		double			*MaxFrameRate,
		double			*MinFrameRate,
		double			*DefFrameRate
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_EnableManualLEDControl
	//
	// Description: 
	//	This function enables the application to manually control the graphical LEDs
	//	during acquisition for devices supporting them (at the present release, 
	//	DactyScan26i, DactyScan84 and DactyScan40).
	//	It is enough to call this function once after
	//	the library has been loaded. If called, the set value keeps on being valid
	//	until a new call to this function is made or the library is unloaded.
	//	After the library is loaded, the default value is FALSE (automatic LED 
	//	control during acquisition).
	//
	//	NOTE1:
	//		This function is supported only by DactyScan 26, DactyScan 84
	//		and DactyScan40 devices. Otherwise it has no effect.
	//
	//	NOTE2:
	//		This function must be called outside of the acquisition.
	//		Otherwise it returns GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED.
	//
	//
	// Parameters:
	//	<Enable>			(input):
	//		- TRUE: Enable manual control during acquisition
	//		- FALSE: Disable manual control during acquisition
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		see NOTE2
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_EnableManualLEDControl(
		BOOL			Enable
	);
	// end ver 2300

	// Ver 2500
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_SetImageRotation
	//
	// Description: 
	//	This function enables the application to set the rotation of the image
	//	coming from the scanner.
	//	Rotation is not performed if this function is not called.
	//	It is enough to call this function once after GBMSAPI_SetCurrentDevice().
	//	If called, the set value keeps on being valid
	//	until a new call to this function is made or the GBMSAPI_SetCurrentDevice()
	//	is called again.
	//
	//	NOTE1:
	//		To know whether this function is supported, check that 
	//		GBMSAPI_DF_IMAGE_ROTATION feature is supported. Otherwise the function
	//		returns GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED.
	//
	//	NOTE2:
	//		This function must be called outside of the acquisition.
	//		Otherwise it returns GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED.
	//
	//	Ver 2700 
	//	NOTE3:
	//		For MS500 device and its derivates (MC500 and MSC500) this function
	//		works for objects acquired on the FLAT area only, otherwise it
	//		has no effect, and the only	valid rotation values are 0 and 180 degrees
	//		(GBMSAPI_IMG_ROTATE_180), otherwise it returns GBMSAPI_ERROR_CODE_PARAMETER
	//		error.
	//	end Ver 2700
	//
	//
	// Parameters:
	//	<ImageRotation>			(input):
	//		see values in "IMAGE ROTATION" section of the GBMSAPI_Defines.h
	//		file
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		see NOTE1
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		see NOTE2
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		see NOTE3
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_SetImageRotation(
		int  ImageRotation
	);

	// VER 2.9.0.0
	////////////////////////////////////////////////////////////////////////////////
	// EnableDrySkinImgEnhance
	//
	// Description:
	//	This function enables/disables an image enhancement feature that helps to
	//	improve the details of ridges and valleys in case of dry/hard finger skin.
	//
	//	NOTE1:
	//		the dry-finger-enhancement feature is disabled by default.
	//
	//	When the feature is active, detected good-enough contact area with the
	//	fingerprint is denoted by darker (gray) background, while area where contact
	//	with the finger is not sufficiently good remains with white background.
	//	This different-tone background approach provides a visual feedback that 
	//	allows to improve dry-fingers acquisition applying more intensive pressing
	//	on the finger where is necessary in case of dry/hard skin.
	//
	//	NOTE2:
	//		the dry-finger-enhancement feature can be activated regradless of actual
	//		finger skin conditions (dry/normal/wet), so, it can be used as a 
	//		permanently-active feature for every finger/palm acquisition.
	//		Dark background in image will be cleaned by the successive call of the
	//		MC517_ImageFinalization() function to ensure usual white background for
	//		every acquired fingerprint/palmprint.
	//
	//	Enough to call this function once after loading DLL, but also the function 
	//	permits to enable/disable alternately this feature during an acquisition
	//	process.
	//
	//	Dry-finger-enhancement feature is also activable by MC517.ini file.
	//
	//	NOTE3:
	//		Before calling this function, check that GBMSAPI_DF_DRY_SKIN_IMG_ENHANCE
	//		feature is supported. Otherwise it returns
	//		GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED
	//
	//
	// Parameters:
	//	<Enable> (input):
	//		pass 1 (TRUE) to enable, 0 (FALSE) to disable
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		see NOTE1
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_EnableDrySkinImgEnhance(
		unsigned char Enable
	);

	/*******************************************************************************
	********************************************************************************
	*				SECTION 3													   *
	*																			   *
	*				SCAN FUNCTIONS												   *
	********************************************************************************
	*******************************************************************************/
	// Functions that have to be called in order to start and stop the acquisition
	// process are listed here. 

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_StartAcquisition
	//
	// Description: 
	//	This function makes the device ready for the acquisition and starts the
	//	acquisition thread.
	//
	// Parameters:
	//	<ObjectToScan>			(input):
	//		It should be choosen between the objects defined in the "SCANNABLE
	//		OBJECTS" section in the GBMSAPI_Defines.h file, otherwise 
	//		GBMSAPI_ERROR_CODE_PARAMETER will be returned.
	//		Its type must be supported by the device (see 
	//		GBMSAPI_GetScannableTypes(...)), otherwise 
	//		GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED will be returned.
	//
	//	<OptionMask> (input):
	//		Scanning option. For more detailed information see the ACQUISITION 
	//		OPTIONS section in the GBMSAPI_Defines.h file.
	//
	//	<AcquisitionEventsManagerCallbackPtr> (input):
	//		Pointer to the function that will be called within the acquisition
	//		thread. For more detailed information see the ACQUISITION EVENTS 
	//		MANAGER CALLBACK section in the GBMSAPI_Defines.h file.
	//		This parameter can be NULL.
	//
	//	<AcquisitionEventsManagerParameters> (input):
	//		Parameter that will be passed to the
	//		<AcquisitionEventsManagerCallbackPtr>(...) function. For more detailed
	//		information see the ACQUISITION EVENTS MANAGER CALLBACK section in the
	//		GBMSAPI_Defines.h file.
	//		This parameter can be NULL.
	//
	//	<DisplayOptionMask> (input):
	//		Display option. For more detailed information see the DISPLAY 
	//		OPTIONS section in the GBMSAPI_Defines.h file. If device does not support
	//		the GBMSAPI_OED_VUI_LCD feature this parameter is not considered.
	//
	//	<ContrastLimitToDisplayOnLCD> (input):
	//		In case when the developer wants to show a customized contrast
	//		value during acquisition, this parameter allows to set the contrast
	//		limit that has to be shown. This value will not be taken in
	//		care in the following cases:
	//
	//		-	The scanner does not support the GBMSAPI_OED_VUI_LCD feature
	//		-	The scanner does not support the GBMSAPI_AII_FINGERPRINT_CONTRAST
	//			feature
	//		-	The type of the <ObjectToScan> parameter is 
	//			GBMSAPI_SBT_ROLL_SINGLE_FINGER
	//
	//		Set to 0 in order to show no limit.
	//		VER 2.9.0.0
	//		If this parameter is set to 0, the scanners supporting the
	//		GBMSAPI_DF_DRY_SKIN_IMG_ENHANCE feature (see the "DEVICE FEATURES"
	//		section in the "GBMSAPI_Defines.h" file) will display a warning in case of
	//		dry or wet finger together with the corresponding diagnostic message
	//		instead of the contrast value.
	//
	//	<CompletenessLimitToDisplayOnLCD> (input):
	//		In case when the developer wants to show a customized palm completeness
	//		value during acquisition, this parameter allows to set the contrast
	//		limit that has to be shown. This value will not be taken in
	//		care in the following cases:
	//
	//		-	The scanner does not support the GBMSAPI_OED_VUI_LCD feature
	//		-	The scanner does not support the 
	//			GBMSAPI_AII_LOWER_HALF_PALM_COMPLETENESS feature
	//		-	The type of the <ObjectToScan> parameter is different from
	//			GBMSAPI_SBT_FLAT_LOWER_HALF_PALM
	//
	//		Set to 0 in order to show no limit
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER
	//		<ObjectToScan> is not one of the objects defined in the "SCANNABLE
	//		OBJECTS" section.
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED
	//		<ObjectToScan> type is not supported by the device.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED
	//		The acquisition thread is already running	
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_THREAD
	//		Error in starting acquisition thread
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_StartAcquisition
		(
		unsigned int ObjectToScan,
		unsigned int OptionMask,
		GBMSAPI_AcquisitionEventsManagerCallback AcquisitionEventsManagerCallbackPtr,
		void *AcquisitionEventsManagerParameters,
		unsigned int DisplayOptionMask,
		unsigned char ContrastLimitToDisplayOnLCD,
		unsigned char CompletenessLimitToDisplayOnLCD
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_StopAcquisition
	//
	// Description: 
	//	This fnction stops the current acquisition process (illuminator is switched
	//	off). 
	//
	// Parameters:
	//	NONE.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_StopAcquisition(void);
	
	//	VER2500: added manual roll preview stop for DS84 device
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_ROLL_StopPreview
	//
	// Description: 
	//	This function stops the preview phase for rolled obiects acquisition.
	//
	// NOTE1:
	//	At moment this function is supported by the DS84 device only. Don't call
	//	for other devices, otherwise GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED
	//	will be returned
	//
	// NOTE2:
	//	Call this function only when the acquisition is in preview phase,
	//	otherwise GBMSAPI_ERROR_CODE_ACQUISITION_THREAD will be returned
	//
	// Parameters:
	//	NONE.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED:
	//		See NOTE1.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_THREAD:
	//		See NOTE2.
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED:
	//		This function works on rolled objects only.
	////////////////////////////////////////////////////////////////////////////////
	int __stdcall GBMSAPI_ROLL_StopPreview(void);

	/*******************************************************************************
	********************************************************************************
	*				SECTION 4													   *
	*																			   *
	*				AFTER START FUNCTIONS										   *
	********************************************************************************
	*******************************************************************************/
	// Functions that have to be called inside the 
	// <AcquisitionEventsManagerCallbackPtr> (see
	// GBMSAPI_StartAcquisition(...)) in order to manage the GBMSAPI_AE_SCANNER_STARTED
	// event are listed here.

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_SetClippingRegionSize
	//
	// Description: 
	//	Function activates auto-clipping option and specifies clipping region
	//	size. There is no clipping by default.
	//	Auto-clipping option allows generating image of a size less than original
	//	size taken from the scanner. 
	//	Clipping region is positioned automatically following the actual position of
	//	the object being scanned within the original image.
	//	Call this function inside <AcquisitionEventsManagerCallbackPtr> when the
	//	GBMSAPI_AE_SCANNER_STARTED has occurred.
	//	To proper functioning it is required that <ObjectTypeToScan> parameter of
	//	GBMSAPI_StartAcquisition(...) really corresponds to the currently scanned object.
	//	See also GBMSAPI_GetClippingRegionPosition(...) function description below.
	//
	//	NOTE1:
	//		for GBMSAPI_SBT_PHOTO objects auto-clipping is not supported.
	//
	//	NOTE2: 
	//		Since image size can be different in PREVIEW and in IQS phase, it's
	//		agreed that <ClippingRegionWidth> and <ClippingRegionHeight> are
	//		related to the size of the full resolution image (FULL-RESOLUTION). 
	//		In order to	get FULL-RESOLUTION image size see GBMSAPI_GetImageSize.
	//
	// Parameters:
	//	<ClippingRegionWidth>	(input):
	//		specifies the width of clipping region (X-size).
	//
	//	<ClippingRegionHeight>	(input):
	//		specifies the height of clipping region (Y-size).
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED
	//		<ObjectTypeToScan> parameter of GBMSAPI_StartAcquisition(...) function was
	//		GBMSAPI_SBT_PHOTO (see NOTE1)
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file. 
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_SetClippingRegionSize
		(
		unsigned int	ClippingRegionWidth,
		unsigned int	ClippingRegionHeight
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_SetDiagnosticSensitivity
	//
	// Description: 
	//	Function is used to tune generation of all diagnostic messages for the 
	//	given scan area except those of roll diagnostic (to tune 
	//	GBMSAPI_DIAGNOSTIC_MSG_VSROLL_... messages see 
	//	GBMSAPI_ROLL_SetDiagnosticSensitivity(...) function).
	//  Function gives the possibility to change default internal 
	//	threshold/parameter values that are used in diagnostic procedures to 
	//	generate diagnostic messages. If default diagnostic behavoiur does not
	//	satisfy specific application requirements, it is possible to tune these
	//	values. In such a case the application developer should contact GREEN BIT
	//	technical support in order to get the required values of <ParameterType>
	//	and <*ParameterValuePtr> specifiyng his concrete requirements.
	//	It has effect only for the current acquisition.
	//	Call this function inside <AcquisitionEventsManagerCallbackPtr> when the
	//	GBMSAPI_AE_SCANNER_STARTED has occurred.
	//
	//	NOTE1:
	//		This function is not supported by all of the GreenBit devices, therefore
	//		check carefully device documentation (in particular the specific SDK
	//		header file). If called for device that do not support tuning, it
	//		returns GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED, but it should be treated as a
	//		warning.
	//
	// Parameters:
	//	<FlatScanArea>	(input):
	//		Scanning window area.
	//		For more details about "flat" and "roll" area meaning, see the 
	//		GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA option in GBMSAPI_Defines.h file.
	//
	//		NOTE2:
	//		Even if device does not support this area, diagnostic sensitivity 
	//		setting is anyway provided, but	a GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED
	//		error code is returned: treat it
	//		simply as a warning. 
	//		
	//		Values to pass:
	//
	//		- TRUE: Flat area
	//		- FALSE: Roll area
	//
	//	<ParameterType> (input): 
	//		selects parameter to be modified (ask GREEN BIT	technical support)
	//
	//	<ParameterValuePtr> (input):
	//		pointer to location with new value specified (ask GREEN BIT
	//		technical support)
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED
	//		see NOTE1
	//
	//	- GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED
	//		see NOTE2
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file. 
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_SetDiagnosticSensitivity(
		BOOL	FlatScanArea,
		int		ParameterType,
		void	*ParameterValuePtr
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_ROLL_SetDiagnosticSensitivity
	//
	// Description: 
	//	Function is used to tune generation of all roll diagnostic messages 
	//	(GBMSAPI_DIAGNOSTIC_MSG_VSROLL_...).
	//  Function gives the possibility to change default internal threshold/parameter
	//	values that are used in diagnostic procedures to generate diagnostic 
	//	messages. If default diagnostic behavoiur does not satisfy specific 
	//	application requirements, it is possible to tune these values.  In this case
	//	the application developer should contact GREEN BIT technical support to get
	//	required values of <ParameterType> and <*ParameterValuePtr> specifiyng his
	//	concrete requirements.
	//	It has effect only for the current acquisition.
	//	Call this function inside <AcquisitionEventsManagerCallbackPtr> when the
	//	GBMSAPI_AE_SCANNER_STARTED has occurred.
	//
	//	NOTE1:
	//		This function is supported only if the device supports ROLLED objects.
	//		Otherwise it returns GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED
	//
	// Parameters:
	//	<ParameterType> (input):
	//		selects parameter to be modified (ask GREEN BIT	technical support)
	//
	//	<ParameterValuePtr> (input):
	//		pointer to location with new value specified (ask GREEN BIT technical
	//		support)
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED
	//		see NOTE1
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file. 
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_ROLL_SetDiagnosticSensitivity(
		int  ParameterType,
		void  *ParameterValuePtr
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_ROLL_SetPreviewTimeout
	//
	// Description: 
	//	This function sets the time duration for the finger to stay still in the
	//	PREVIEW phase of a ROLLED object acquisition in case automatic stop preview
	//	hase been choosen: see ROLLED FINGERPRINT 
	//	ACQUISITION section in GBMSAPI_Defines.h file for more detailed information.
	//	Once this function has been called, the setting will be kept for the 
	//	following acquisitions.
	//	If not called, default time duration is 1500 ms.
	//	Call this function inside <AcquisitionEventsManagerCallbackPtr> when the
	//	GBMSAPI_AE_SCANNER_STARTED has occurred.
	//
	//	NOTE1:
	//		This function is supported only if the device supports ROLLED objects.
	//		Otherwise it returns GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED
	//	
	//	NOTE2:
	//		Call this function only if the <ObjectTypeToScan> parameter of the
	//		GBMSAPI_StartAcquisition(...) function was GBMSAPI_SBT_ROLL_SINGLE_FINGER, 
	//		otherwise GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED will be returned.
	//	
	// Parameters:
	//	<Timeout> (input):
	//		timeout in milliseconds (min 500; max 3000; default 1500)
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED
	//		see NOTE1
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED:
	//		See NOTE2
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file. 
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_ROLL_SetPreviewTimeout(
		unsigned int Timeout
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_ROLL_SetArtefactCleaningDepth
	//
	// Description: 
	//	This function specifies the depth of artefact regions cleaning at the edge of
	//	fingerprint.
	//	Due to the fact that the most frequently rolling artefact regions appear on
	//	fingerprint edge, it is sensible to wipe erroneous ridges from these regions
	//	extending light outer space inside artefact region for programmable depth. 
	//	Default value is 1 - cleaning is performed for max. 0.8 mm - if this 
	//	function is not called. 
	//	Call this function inside <AcquisitionEventsManagerCallbackPtr> when the
	//	GBMSAPI_AE_SCANNER_STARTED has occurred.
	//
	//	NOTE1:
	//		This function is supported only if the device supports ROLLED objects.
	//		Otherwise it returns GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED
	//	
	//	NOTE2:
	//		Call this function only if the <ObjectTypeToScan> parameter of the
	//		GBMSAPI_StartAcquisition(...) function was GBMSAPI_SBT_ROLL_SINGLE_FINGER, 
	//		otherwise GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED will be returned.
	//	
	// Parameters:
	//	<ArtefactCleaningDepth> (input): 
	//		see ROLL FUNCTIONALITY DEFINES section in GBMSAPI_Defines.h:
	//
	//		- GBMSAPI_ARTEFACT_CLEANING_DEPTH_ZERO:
	//			cleaning disable 
	//
	//		- [1;GBMSAPI_ARTEFACT_CLEANING_DEPTH_MAX]:
	//			edge cleaning enable for the specified depth (in 0.8 mm units)
	//
	//		- GBMSAPI_ARTEFACT_CLEANING_DEPTH_UNLIMITED:
	//			edge cleaning enable without any depth limitations, also internal
	//			artefact regions are cleared
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED
	//		see NOTE1
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED:
	//		See NOTE2
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_ROLL_SetArtefactCleaningDepth(
		unsigned char ArtefactCleaningDepth
		);

	/*******************************************************************************
	********************************************************************************
	*				SECTION 5													   *
	*																			   *
	*				ACQUISITION CALLBACK FUNCTIONS FOR ACQUIRED FRAME			   *
	********************************************************************************
	*******************************************************************************/
	// Functions that should be called inside the 
	// GBMSAPI_AcquisitionEventsManagerCallback(...) (see ACQUISITION EVENTS MANAGER CALLBACK section of 
	// the GBMSAPI_Defines.h file) when the GBMSAPI_AE_VALID_FRAME_ACQUIRED event has
	// occurred can be divided in three groups:
	//
	// 1)	FLAT objects AUTO-CAPTURE utilities:
	//		GBMSAPI_GetAutoCapturePhase(...) and GBMSAPI_SetAutoCaptureBlocking(...)
	//
	// 2)	Image info utilities:
	//		GBMSAPI_GetFingerprintContrast(...), GBMSAPI_GetFingerprintSize(...),
	//		GBMSAPI_GetLowerHalfPalmCompleteness(...),
	//		GBMSAPI_GetClippingRegionPosition(...).
	//
	// 3)	General utilities:
	//		GBMSAPI_VUI_LED_BlinkDuringAcquisition(...)
	//
	// NOTE1:
	//	The functions belonging to group (2) - Image info uilities - can be called
	//	also when acquisition has been stopped (that is, when the 
	//	GBMSAPI_AE_ACQUISITION_END event has occurred).

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LED_BlinkDuringAcquisition
	//
	// Description: 
	//	This function makes the Finger indicator LED(s) be switched on during the 
	//	acquisition process blink. Call inside the 
	//	AcquisitionEventsManagerCallbackPtr(...) function, if <OccurredEventCode> is
	//	different from GBMSAPI_AE_ACQUISITION_ERROR or GBMSAPI_AE_ACQUISITION_END,
	//	otherwise it has no effect.
	//
	//	NOTE1:
	//		Call this function only if device supports the GBMSAPI_OED_VUI_LED
	//		equipment, otherwise GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED will be
	//		returned. This error can be considered as a warning
	//
	// Parameters:
	//	<Blink>	(output):
	//		-	TRUE:
	//			blink at fixed frequency (2 times per second)
	//		-	FALSE:
	//			no blink
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED:
	//		Device does not support GBMSAPI_OED_VUI_LED equipment.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LED_BlinkDuringAcquisition(
		BOOL Blink
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetAutoCapturePhase
	//
	// Description: 
	//	This function returns the current phase of the internal auto-capture process 
	//	in case when auto-capture mode for FLAT objects has been selected - by 
	//	setting the GBMSAPI_AO_AUTOCAPTURE bit of the <OptionMask> parameter of the
	//	GBMSAPI_StartAcquisition(...) function.
	//	To implement acquisition process correctly, the application should be 
	//	informed about the current phase of the auto-capture process - in particular,
	//	to implement WAIT_FINGER timeout (maximum period of time that could be spent
	//	for finger awaiting). 
	//	Before breaking acquisition process by timeout, the application should check
	//	if auto-capture process is still at WAIT_FINGER phase. It is recommended do
	//	not break process at SELECT_IMAGE phase because, in fact, finger is already
	//	appeared on the scanner surface and selected fingerprint image will be
	//	available soon. 
	//	Call this function inside <AcquisitionEventsManagerCallbackPtr>(...) when
	//	the GBMSAPI_AE_VALID_FRAME_ACQUIRED event has occurred.
	//
	//	NOTE1:
	//		This function should not be used for ROLLED objects acquisition, 
	//		otherwise GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED will be returned.
	//
	//	NOTE2:
	//		Before calling this function, check that AUTO-CAPTURE mode is supported
	//		by the device, by calling the GBMSAPI_GetScanOptions() function:
	//		if current device does not support this option, 
	//		GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED will be returned.
	//
	// Parameters:
	//	<AcquisitionPhase>	(output):
	//		see values in the "ACQUISITION PHASE" section of the GBMSAPI_Defines.h
	//		file. Valid only if GBMSAPI_ERROR_CODE_NO_ERROR is returned.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED
	//		see NOTE1
	//
	//	- GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED
	//		see NOTE2
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetAutoCapturePhase(
		int *AcquisitionPhase
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_SetAutoCaptureBlocking
	//
	// Description: 
	//	This function allows to block temporarily auto-capture system to prevent 
	//	automatic acquisition stop in cases when application itself decides to 
	//	prohibit current image selection analyzing incoming fingerprint images or 
	//	the corresponding diagnostic messages. 
	//	This function could be applied in cases when a problem appears during
	//	acqusition process due to improper user behaviour (and is easy-evitable) 
	//	- for example, when <GBMSAPI_Diagnostic> parameter of 
	//	<AcquisitionEventsManagerCallbackPtr> returns diagnostic messages: 
	//
	//		-	GBMSAPI_DIAGNOSTIC_MSG_SCANNER_SURFACE_NOT_NORMA (the finger has been 
	//			placed too much early)
	//		-	GBMSAPI_DIAGNOSTIC_MSG_FLAT_FINGER_SLIDING (the finger is sliding on
	//			the scanning surface)
	//		-	GBMSAPI_DIAGNOSTIC_MSG_FLAT_FINGER_OUT_OF_REGION_... (the finger is
	//			positioned improperly)
	//		-	GBMSAPI_DIAGNOSTIC_MSG_FLAT_FINGER_DISPLACED_DOWN (the finger is
	//			positioned improperly)
	//
	//	The operator should be warned that auto-capture is blocked due to improper
	//	user behaviour. For example, in case when the DactyScan84 device is being
	//	used, the blinking yellow finger indicator can be used together with
	//	GBMSAPI_SetAutoCaptureBlocking(...) function.
	//	Blocking has no effect for the current acquisition if this function is not
	//	called. 
	//	It is responsibility of the application to turn blocking off if it was turned
	//	on before in order to enable auto-capture again. 
	//	Call this function inside <AcquisitionEventsManagerCallbackPtr>(...) when
	//	the GBMSAPI_AE_VALID_FRAME_ACQUIRED event has occurred.
	//
	//	NOTE1:
	//		This function should not be used for ROLLED objects acquisition, 
	//		otherwise GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED will be returned.
	//
	//	NOTE2:
	//		Before calling this function, check that AUTO-CAPTURE mode is supported
	//		by the device, by calling the GBMSAPI_GetScanOptions() function:
	//		if current device does not support this option, 
	//		GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED will be returned.
	//
	//	NOTE3:
	//		Call this function only if GBMSAPI_DF_AUTO_CAPTURE_BLOCKING feature is
	//		supported (see DEVICE FEATURES section of the GBMSAPI_Defines.h
	//		file. Otherwise GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED will be returned.
	//
	// Parameters:
	//	<Blocking>	(input): 
	//		TRUE:	turn blocking on
	//		FALSE:	turn blocking off
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED
	//		see NOTE1
	//
	//	- GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED
	//		see NOTE2
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED
	//		see NOTE3
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_SetAutoCaptureBlocking(
		BOOL Blocking
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetFingerprintContrast
	//
	// Description: 
	//	This function returns contrast value of the current/best flat fingerprint 
	//	image - single or slap. 
	//	This value could be useful for some applications, where it is important to 
	//	perform rough estimation of fingerprint image immediately during acquisition.
	//	Fingerprint contrast is determined as the average difference between dark 
	//	(ridges) and light (valleys) pixels within fingerprint region in image. 
	//	Call this function inside <AcquisitionEventsManagerCallbackPtr>(...) when
	//	the GBMSAPI_AE_VALID_FRAME_ACQUIRED (or eventually GBMSAPI_AE_ACQUISITION_END)
	//	event has occurred.
	//
	//	NOTE1:
	//		This function should not be used for ROLLED objects acquisition, 
	//		otherwise GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED will be returned.
	//
	//	NOTE2:
	//		Contrast value could be inappropriate, if diagnostic message 
	//		GBMSAPI_DIAGNOSTIC_MSG_SCANNER_SURFACE_NOT_NORMA or 
	//		GBMSAPI_DIAGNOSTIC_MSG_SCANNER_FAILURE is present for the current frame.
	//
	//	NOTE3:
	//		If clipping has been activated, contrast value is calculated only 
	//		inside the current clipping region.
	//
	//	NOTE4:
	//		Call this function only if GBMSAPI_AII_FINGERPRINT_CONTRAST feature is
	//		supported (see AVAILABLE IMAGE INFO section of the GBMSAPI_Defines.h
	//		file). Otherwise GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED will be returned.
	//	
	// Parameters:
	//	<Contrast>	(output):
	//		contrast value in range 0-255. Valid only if GBMSAPI_ERROR_CODE_NO_ERROR is
	//		returned. 
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		<Contrast> is NULL.
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED
	//		see NOTE1
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED
	//		see NOTE4
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetFingerprintContrast(
		unsigned char *Contrast
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetFingerprintSize
	//
	// Description: 
	//	This function returns size of the current/best flat fingerprint image -
	//	single or slap. 
	//	This value could be useful for some applications, where it is important to 
	//	perform rough estimation of fingerprint image immediately during acquisition.
	//	Fingerprint size is determined as the total ridges/valleys square inside
	//	image.  
	//	Call this function inside <AcquisitionEventsManagerCallbackPtr>(...) when
	//	the GBMSAPI_AE_VALID_FRAME_ACQUIRED (or eventually GBMSAPI_AE_ACQUISITION_END)
	//	event has occurred.
	//
	//	NOTE1:
	//		This function should not be used for ROLLED objects acquisition, 
	//		otherwise GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED will be returned.
	//
	//	NOTE2:
	//		Fingerprint size value could be inappropriate, if diagnostic message 
	//		GBMSAPI_DIAGNOSTIC_MSG_SCANNER_SURFACE_NOT_NORMA or 
	//		GBMSAPI_DIAGNOSTIC_MSG_SCANNER_FAILURE is present for the current frame.
	//
	//	NOTE3:
	//		If clipping has been activated, fingerprint size value is calculated 
	//		only inside the current clipping region.
	//
	//	NOTE4:
	//		Call this function only if GBMSAPI_AII_FINGERPRINT_SIZE feature is
	//		supported (see AVAILABLE IMAGE INFO section of the GBMSAPI_Defines.h
	//		file). Otherwise GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED will be returned.
	//
	// Parameters:
	//	<FingerprintSize>	(output):
	//		fingerprint size in mm^2. Valid only if GBMSAPI_ERROR_CODE_NO_ERROR is
	//		returned. 
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		<FingerprintSize> is NULL.
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED
	//		see NOTE1
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED
	//		see NOTE4
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetFingerprintSize(
		unsigned int *FingerprintSize
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetLowerHalfPalmCompleteness
	//
	// Description: 
	//	This function returns pattern completeness value for a lower half-palm image
	//	(GBMSAPI_SBT_FLAT_LOWER_HALF_PALM objects).
	//	This value is useful for applications where it is important to perform 
	//	real-time estimation of half-palm pattern completess during acquisition and,
	//	thus, to prevent generation of incomplete lower half-palm scans.
	//	This value specifically refers to central or border "empty" zones that 
	//	appear due to the fact that concave palm surface is scanned on flat scanner
	//	surface. A certain enforcement should be applied while pressing palm down to 
	//	the scanner surface in order to eliminate or significantly diminish these
	//	"empty" zones.
	//	In case of auto-capture acqusition, 
	//	GBMSAPI_SetLowerHalfPalmCompletenessThreshold(...) function could be used to
	//	adjust the threshold value. 
	//	Call this function inside <AcquisitionEventsManagerCallbackPtr>(...) when
	//	the GBMSAPI_AE_VALID_FRAME_ACQUIRED (or eventually GBMSAPI_AE_ACQUISITION_END)
	//	event has occurred.
	//
	//	NOTE1:
	//		This function should be used only for GBMSAPI_SBT_FLAT_LOWER_HALF_PALM
	//		objects acquisition, otherwise GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED will be 
	//		returned.
	//
	//	NOTE2:
	//		Pattern completeness value could be inappropriate, if diagnostic message 
	//		GBMSAPI_DIAGNOSTIC_MSG_SCANNER_SURFACE_NOT_NORMA or 
	//		GBMSAPI_DIAGNOSTIC_MSG_SCANNER_FAILURE is present for the current frame.
	//
	//	NOTE3:
	//		Pattern completeness value is calculated independently on the current 
	//		clipping region if any.
	//
	//	NOTE4:
	//		Call this function only if GBMSAPI_AII_LOWER_HALF_PALM_COMPLETENESS feature is
	//		supported (see AVAILABLE IMAGE INFO section of the GBMSAPI_Defines.h
	//		file). Otherwise GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED will be returned.
	//
	// Parameters:
	//	<LowerHalfPalmCompleteness>	(output):
	//		lower half-palm pattern completeness value in percents 0-100%. 
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		<LowerHalfPalmCompleteness> is NULL.
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED
	//		see NOTE1
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED
	//		see NOTE4
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetLowerHalfPalmCompleteness(
		unsigned char *LowerHalfPalmCompleteness
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetClippingRegionPosition
	//
	// Description: 
	//	This function obtains automatically detected clipping region position within
	//	the current/best image frame. 
	//	Call this function inside <AcquisitionEventsManagerCallbackPtr>(...) when
	//	the GBMSAPI_AE_VALID_FRAME_ACQUIRED (or eventually GBMSAPI_AE_ACQUISITION_END)
	//	event has occurred.
	//
	//	NOTE1:
	//		<FramePtr> parameter of <AcquisitionEventsManagerCallbackPtr>(...) is
	//		always an image of original size, and it is responsibility of the 
	//		application to cut the clipping region using its known size and position.
	//
	//	NOTE2:
	//		For FLAT objects acquisition this function can be called for each
	//		acquired frame. In case of ROLLED objects, it's recommended this
	//		function should be called only after the acquisition has been stopped 
	//		(that is: when the GBMSAPI_AE_ACQUISITION_END event has occurred).
	//
	//	NOTE3:
	//		The coordinates and size of the clipping region are referred to the last
	//		acquired frame size (see <FrameSizeX> and <FrameSizeY> parameters of
	//		the GBMSAPI_AcquisitionEventsManagerCallback(...) callback
	//
	// Parameters:
	//	<ClippingRegionPosX>	(output):
	//		X-coordinate of the left-top corner of clipping region inside image. If
	//		value is -1, clipping has not been performed.
	//
	//	<ClippingRegionPosY>	(output):
	//		Y-coordinate of the left-top corner of clipping region inside image. If
	//		value is -1, clipping has not been performed.
	//
	//	<ClippingRegionSizeX>	(output):
	//		X-size of clipping region inside image.
	//
	//	<ClippingRegionSizeY>	(output):
	//		Y-size of clipping region inside image.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		<ClippingRegionPosX> or <ClippingRegionPosY> is NULL.
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetClippingRegionPosition(
		int *ClippingRegionPosX,
		int *ClippingRegionPosYY,
		unsigned int *ClippingRegionSizeX,
		unsigned int *ClippingRegionSizeY
		);

	/*******************************************************************************
	********************************************************************************
	*				SECTION 6													   *
	*																			   *
	*				FINAL ACQUIRED IMAGE FUNCTIONS								   *
	********************************************************************************
	*******************************************************************************/
	// Functions that should be called inside the 
	// GBMSAPI_AcquisitionEventsManagerCallback(...) (see ACQUISITION EVENTS MANAGER CALLBACK section of 
	// the GBMSAPI_Defines.h file) when the GBMSAPI_AE_ACQUISITION_END event has
	// occurred are listed here.  	

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_ImageFinalization
	//
	// Description: 
	//	This function finalizes the fingerprint image previously returned in the
	//	<FramePtr> parameter of the <AcquisitionEventsManagerCallbackPtr>(...) 
	//	function (see GBMSAPI_StartAcquisition() function).
	//	In particular, this function performs elimination of latent prints 
	//	originated by recent scans around currently scanned fingerprint region. 
	//	This ensures that there are no "foreign" ridge/valley regions attached to
	//	the border of the scanned fingerprint. 
	//	This processing is based on comparing of local contrast values of initial
	//	frame (without fingerprint image inside) to local contrast values of 
	//	acquired frame containing fingerprint image inside. All local regions whose
	//	contrast value has not been increased are cleared to be white. 
	//	This processing strongly requires that the final fingerprint image returned 
	//	in the <FramePtr> parameter of the <AcquisitionEventsManagerCallbackPtr>(...)
	//	function has not been accompanied by diagnostic messages 
	//	GBMSAPI_DIAGNOSTIC_MSG_SCANNER_SURFACE_NOT_NORMA or 
	//	GBMSAPI_DIAGNOSTIC_MSG_SCANNER_FAILURE.  
	//	Call this function inside <AcquisitionEventsManagerCallbackPtr>(...) when
	//	the GBMSAPI_AE_ACQUISITION_END event has occurred.
	//
	//	NOTE1:
	//		This function can not be used to process fingerprint images generated 
	//		outside.
	//
	//	NOTE2:
	//		Don't apply this function to ROLLED composite fingerprint image when
	//		when acquisition is stopped because this image has been already 
	//		internally finalized.
	//
	//	NOTE3:
	//		Don't apply this function GBMSAPI_SBT_PHOTO objects.
	//
	// Parameters:
	//	<FrameToFinalize>	(input/output):
	//		pointer to the last frame returned in the <FramePtr> parameter of the 
	//		<AcquisitionEventsManagerCallbackPtr>(...) function.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_ImageFinalization(
		unsigned char *FrameToFinalize
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_ColorReconstruction
	//
	// Description: 
	//	This function produces a true RGB-format output image from input raw image
	//	acquired for GBMSAPI_SBT_PHOTO objects.  
	//	Call this function inside <AcquisitionEventsManagerCallbackPtr>(...) when
	//	the GBMSAPI_AE_ACQUISITION_END event has occurred.
	//
	// Parameters:
	//	<MonochromeImage>	(input):
	//		pointer to the last frame returned in the <FramePtr> parameter of the 
	//		<AcquisitionEventsManagerCallbackPtr>(...) function.
	//
	//	<SizeX> 	(input):
	//		specifies X-size of the image. It should be	obtained as the <ImageSizeX>
	//		output parameter of the GBMSAPI_GetImageSize(...) function.
	//
	//	<SizeY> 	(input):
	//		specifies Y-size of the image. It should be	obtained as the <ImageSizeY>
	//		output parameter of the GBMSAPI_GetImageSize(...) function.
	//
	//	<ColorImage>	(input/output):
	//		pointer to a buffer where the output color image array in 3-byte RGB 
	//		format (without header) will be stored.	Must be allocated by the
	//		application, and its size, in bytes, must be >= 3 * <SizeX> * <SizeY>.
	//
	//	<RedKoef, GreenKoef, BlueKoef>	(input):
	//		"Bayer to RGB" color transformation	coefficients, standard value 1.0,
	//		can be used to adjust colors 
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED:
	//		The GBMSAPI_StartAcquisition() function was called with an object type
	//		different from GBMSAPI_SBT_PHOTO.
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_ColorReconstruction(
		unsigned char *MonochromeImage, 
		long  SizeX,long  SizeY,
		RGBTRIPLE *ColorImage, 
		double RedKoef,double GreenKoef,double BlueKoef
		);

	// VER 2600
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetFrameStatistic
	//
	// Description: 
	//	This function returns the number of acquired frame and lost frame, 
	//	calculated during the last acquisition.
	//
	// Parameters:
	//	<AcquiredFrames>	(output):
	//		number of acquired frames during last acquisition.
	//
	//	<LostFrames>	(output):
	//		number of lost frames during last acquisition. 
	//
	//	NOTE: Call this function after acquisition end
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetFrameStatistic(
		unsigned long *AcquiredFrames,
		unsigned long *LostFrames
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_ROLL_GetCompositeImageInfo
	//
	// Description: 
	//	This function returns useful information collected during composition 
	//	process about the final composite fingerprint image.  
	//	Call this function inside <AcquisitionEventsManagerCallbackPtr>(...) when
	//	the GBMSAPI_AE_ACQUISITION_END event has occurred.
	//	
	//	NOTE1:
	//		Call this function only if the <ObjectTypeToScan> parameter of the
	//		GBMSAPI_StartAcquisition(...) function was GBMSAPI_SBT_ROLL_SINGLE_FINGER,
	//		otherwise GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED will be returned.
	//
	// Parameters:
	//	<RolledArtefactSize> (output):
	//		if equal to NULL, value is not returned; otherwise returns the overall
	//		square of rolling arftefact regions within the final composite 
	//		fingerprint image expressed in mm^2.
	//		The lower this number the better fingerprint quality.
	//		Recommended threshold values to estimate present artefacts are: 
	//
	//		good			<= 7 mm2	(could be displayed with green indicator
	//									if the LED indicator is provided) 
	//		medium			<= 14 mm2	(could be displayed with yellow indicator
	//									if the LED indicator is provided) 
	//		unacceptable	> 14 mm2	(could be displayed with red indicator
	//									if the LED indicator is provided)
	//
	//	<MarkerFrame> (output):
	//		if equal to NULL, value is not returned; otherwise returns pointer to
	//		marker frame buffer whose size can be obtained by calling the 
	//		GBMSAPI_GetImageSize(...) function. Marker frame
	//		contains pixel markers (bit masks) corresponding to pixels of the final
	//		composite image. These markers convey specific information about the
	//		corresponding pixel. The markers are defined into the "ROLL 
	//		FUNCTIONALITY DEFINES" section of the GBMSAPI_Defines.h file.
	//
	//	<NotWipedArtefactFrame> (output):
	//		if equal to NULL, value is not returned; otherwise returns the pointer
	//		to the frame buffer whose size can be obtained by calling the 
	//		GBMSAPI_GetImageSize(...) function. This frame
	//		contains a copy of the final composite fingerprint image that completely
	//		corresponds to the composite image returned in the <FramePtr> parameter
	//		of the <AcquisitionEventsManagerCallbackPtr>(...) function excluding
	//		artefact region	wiping (if enabled). In this copy all artefact regions
	//		are not wiped. This copy could be used to tune the value of
	//		<ArtefactCleaningDepth>	parameter comparing wiped and not-wiped images.
	//
	//  <CompositeFingerprintSize> (output):
	//		if equal to NULL, value is not returned; otherwise returns the size of
	//		the final composite fingerprint expressed in mm^2. Fingerprint size is
	//		determined as the total ridges/valleys square inside image.
	//
	//  <CompositeFingerprintContrast> (output):
	//		if equal to NULL, value is not returned; otherwise returns the contrast
	//		of the final composite fingerprint expressed in grayscales. Fingerprint
	//		contrast is determined as the average difference between dark (ridges)
	//		and light (valleys) pixels within fingerprint region in image.
	//
	//  <FlatFingerprintSize> (output):
	//		if equal to NULL, value is not returned; otherwise returns the size of 
	//		the biggest input flat fingerprint image expressed in mm^2. Fingerprint
	//		size is determined as the total ridges/valleys square inside image.
	//
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED:
	//		The GBMSAPI_StartAcquisition() function was called with an object type
	//		different from GBMSAPI_SBT_ROLL_SINGLE_FINGER.
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_ROLL_GetCompositeImageInfo( 
		unsigned int	*RolledArtefactSize,
		unsigned char   **MarkerFrame,	   
		unsigned char   **NotWipedArtefactFrame,		
		unsigned int	*CompositeFingerprintSize,
		unsigned char   *CompositeFingerprintContrast,	
		unsigned int	*FlatFingerprintSize
		);

	/*******************************************************************************
	********************************************************************************
	*				SECTION 7													   *
	*																			   *
	*				AUXILIARY FUNCTIONS											   *
	********************************************************************************
	*******************************************************************************/
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_SetCalibrationImage
	//
	// Description: 
	//	Function sets refreshed calibration image for the selected scanning area to 
	//	be used instead of default calibration image in calibration process.
	//	Calibration process is applied to every image frame produced by scanner.
	//	Call this function before GBMSAPI_StartAcquisition(...) function. It's enough to
	//	call once after device initialization (i.e. after GBMSAPI_SetCurrentDevice()
	//	function has been called).
	//	If this function is not called, default (factory-preprogrammed) calibration
	//	image will be used in calibration process.
	//
	//	NOTE: 
	//		refreshed calibration image should be obtained before by using the
	//		GBMSAPI_GetCalibrationImage(...) function.
	//
	//	NOTE2: 
	//		Before calling this function, make sure that an acquisition is not in
	//		still in progress, otherwise GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED
	//		will be returned.
	//
	//
	// Parameters:
	//	<FlatScanArea>			(input):
	//		Scanning area (see the GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA option in 
	//		GBMSAPI_Defines.h file).
	//
	//		NOTE1:
	//		If device does not support this area, calibration image will not
	//		be set and a GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED error code is returned. 
	//		
	//		Values to pass:
	//
	//		- TRUE: Flat area
	//		- FALSE: Roll area
	//
	//	<CalibrationImage>	(input):
	//		pointer to a buffer allocated by the application that contains refreshed
	//		calibration image. If <CalibrationImage> is NULL then default
	//		(factory-preprogrammed) calibration image will be used.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		see NOTE2
	//
	//	- GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED
	//		Scan area not supported.
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_SetCalibrationImage(
		BOOL			FlatScanArea,
		unsigned char	*CalibrationImage
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetCalibrationImage
	//
	// Description: 
	//	Function generates for the given scan area a refreshed calibrarion image 
	//	used for calibration process.
	//  GreenBit scanners keep on-board their default calibrarion image programmed 
	//	at production stage. This calibration image is used always by default in
	//	calibration process. Refreshed calibrarion image can be used instead of
	//	default calibration image if it is explicitly requested (see
	//	GBMSAPI_SetCalibrationImage(...) function). 
	//	Scanner calibration process is used to compensate possible variation of 
	//	scanner characteristics, that are of two types:
	//
	//	- permanent: illuminator non-uniformity specific for each individual scanner
	//
	//	- temporary: due to use (little dust particles or insignificant 
	//		imperfections) and due to environmental changes (humidity, temperature).
	//
	//	Calibration process is applied to every image frame produced by scanner.
	//	Therefore, it is recommended to refresh calibration image periodically for 
	//	every scanner. The application is responsible to save refreshed calibration
	//	image, for example, to a file. It is recommended to use scanner serial 
	//	number (see GBMSAPI_GetScannerStatistics) and scanner type to constitute image
	//	file name (ex.: "D8R00000001_PARTFRAME.RAW"); in this way the calibration 
	//	image file is explicitly linked to its own scanner.
	//  It is prohibited to call this function inside another scanning process 
	//	(after GBMSAPI_StartAcquisition(...) and until GBMSAPI_StopAcquisition(...)), because
	//	the function implements internal scanning process to generate the calibration
	//	image.
	//
	//	Automatic diagnostic during refreshed calibrarion image generation:
	//		During refreshed calibration image generation procedure an automatic 
	//		diagnostic check is executed. If refreshed calibration image is not 
	//		suitable, the corresponding diagnostic messages are present in parameter
	//		<CalibrationDiagnostic>. If at least one diagnostic message present, it 
	//		is strongly recommended to repeat calibration image generation performing 
	//		before all necessary actions to eliminate diagnostic messages. 
	//		Calibration diagnostic sensitivity is tuned to prohibit grave and permit
	//		natural level of dirtiness/defects.
	//		Before calibrarion image generation procedure the scanning surface should
	//		be cleaned carefully and no object (finger, etc.) should be placed on it.
	//		There should not be any scratches and other visible defects on the 
	//		scanning surface. 
	//		At the moment of calibration image generation it is recommemned to cover
	//		scanning surface with a dark shield to ensure that there is no strong 
	//		external light source influence (ex.: strong direct sun rays).
	//
	//	NOTE1: 
	//		Before calling this function, make sure that an acquisition is not in
	//		still in progress, otherwise GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED
	//		will be returned.
	//
	//
	// Parameters:
	//	<FlatScanArea>			(input):
	//		Scanning area (see the GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA option in 
	//		GBMSAPI_Defines.h file).
	//
	//		NOTE:
	//		If device does not support this area, calibration image will not
	//		be got and a GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED error code is returned. 
	//		
	//		Values to pass:
	//
	//		- TRUE: Flat area
	//		- FALSE: Roll area
	//
	//	<CalibrationImage>	(input/output):
	//		pointer to a buffer allocated by the application that contains refreshed
	//		calibration image whose size can be got by calling the 
	//		GBMSAPI_GetCalibrationImage(...) function with <FlatScanArea> as <FlatScanArea>
	//		parameter.
	//
	//	<CalibrationDiagnostic>	(output) 32-bit diagnostic word:
	//
	//		bit value '1' - diagnostic message is present, 
	//		bit value '0' - diagnostic message is absent
	//		
	//		Possible diagnostic messages are:
	//
	//		- GBMSAPI_DIAGNOSTIC_MSG_SCANNER_SURFACE_NOT_NORMA 
	//		- GBMSAPI_DIAGNOSTIC_MSG_SCANNER_FAILURE 
	//		- GBMSAPI_DIAGNOSTIC_MSG_EXT_LIGHT_TOO_STRONG
	//
	//		All these diagnostic messages have the same code (bit position) as
	//		described in DIAGNOSTIC MESSAGES section of the GBMSAPI_Defines.h file.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		see NOTE1
	//
	//	- GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED
	//		Scan area not supported.
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetCalibrationImage(
		BOOL			FlatScanArea,
		unsigned char	*CalibrationImage,
		unsigned int	*CalibrationDiagnostics
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetMultiScanAPIVersion
	//
	// Description: 
	//	This function obtains library current version. 
	//	These fields are changed upon any modification of the dll module.
	//
	// Parameters:
	//	<VersionField1>	(output):
	//		corresponds to the first field of module version.
	//
	//	<VersionField2>	(output): 
	//		corresponds to the second field of module version.
	//
	//	<VersionField3>	(output):
	//		corresponds to the third field of module version.
	//
	//	<VersionField4> (output):
	//		corresponds to the fourth field of module version.
	//
	//	Example: if module version is <1.2.3.4>, then VersionField1=1, 
	//		VersionField2=2, etc.
	//
	// Return:
	//	NONE.
	////////////////////////////////////////////////////////////////////////////////
	void GBMSAPI_API __stdcall GBMSAPI_GetMultiScanAPIVersion
	(
		unsigned char *VersionField1,
		unsigned char *VersionField2,
		unsigned char *VersionField3,
		unsigned char *VersionField4
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetUnderlyingLibraryDllVersion
	//
	// Description: 
	//	This function obtains the current version and the file name of the low level
	//	library for the	current device (i.e. the device set by calling the
	//	GBMSAPI_SetCurrentDevice() function). For example, for DactyScan84 device,
	//	library name will be "DS84u.dll"
	//	These fields are changed upon any modification of the dll module.
	//
	// Parameters:
	//	<VersionField1>	(output):
	//		corresponds to the first field of module version.
	//
	//	<VersionField2>	(output): 
	//		corresponds to the second field of module version.
	//
	//	<VersionField3>	(output):
	//		corresponds to the third field of module version.
	//
	//	<VersionField4> (output):
	//		corresponds to the fourth field of module version.
	//
	//	<LowLevelDllName>	(output):
	//		string containing the dll name. Must be allocated by the application and
	//		its size (in characters) should be MAX_PATH (see windef.h standard
	//		library).
	//
	//	Example: if module version is <1.2.3.4>, then VersionField1=1, 
	//		VersionField2=2, etc.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetUnderlyingLibraryDllVersion(
		unsigned char *VersionField1,
		unsigned char *VersionField2,
		unsigned char *VersionField3,
		unsigned char *VersionField4,
		char *LowLevelDllName
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetUSBErrorCode
	//
	// Description: 
	//	If the <USBErrorCode> parameter is not NULL, this function reports error code
	//	of USB device driver. Call this function to get detailed USB error code only
	//	if one of SDK functions has returned GBMSAPI_ERROR_CODE_USB_DRIVER.
	// 
	//	Important to know: USB driver error code E0001005h means that another device
	//	currently uses USB device driver (driver is busy). 
	//  For example, if there are two DS84 devices on USB bus and one of them is in
	//	use, an attempt to communicate with the second one will generate this error 
	//	code. The application is responsible to resolve the USB device driver 
	//	sharing conflict.
	//	
	// Parameters:
	//	<USBErrorCode> (output):
	//		valid only if GBMSAPI_ERROR_CODE_NO_ERROR is returned. <USBErrorCode> must
	//		be interpreted in hexadecimal representation. This error should be 
	//		visualized and reported	to the GreenBit technical support.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetUSBErrorCode(
		unsigned long *USBErrorCode
		);

	// VER2700
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetScannerStatistics
	//
	// Description: 
	//	This function eturns statistical data automatically collected by the 
	//	scanner.
	//
	// Parameters:
	//	<Counter> (output):
	//		valid only if GBMSAPI_ERROR_CODE_NO_ERROR is returned. Number of times
	//		the scanner was used. Not resettable.
	//
	//	<ProductionDate> (output):
	//		valid only if GBMSAPI_ERROR_CODE_NO_ERROR is returned. Date which
	//		statistics collection starts from, in seconds from Jan 1th, 1970.
	//		Not resettable.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		This feature is not supported by the scanner.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetScannerStatistics(
		unsigned int *Counter,
		unsigned int *ProductionDate
	);

	/*******************************************************************************
	********************************************************************************
	*				SECTION 8													   *
	*																			   *
	*				EXTERNAL DEVICES CONTROL									   *
	********************************************************************************
	*******************************************************************************/
	// Please refer to the OPTIONAL EXTERNAL EQUIPMENT section in the
	// GBMSAPI_Defines.h file for more details about usage of the external
	// devices.

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_Sound
	//
	// Description: 
	//	This function forces the device to produces an audible signal using on-board
	//	buzzer.
	//
	//	NOTE1: 
	//		It is recommended to call this function outside acquisition process -
	//		before GBMSAPI_StartAcquisition(...) or after 
	//		GBMSAPI_StopAcquisition(...).
	//
	//	NOTE2: 
	//		Call this function only if GBMSAPI_OED_SOUND feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	// Parameters:
	//	<Tone> (input): 
	//		specifies the tone of signal (frequency) and is in range 1-63.
	//		Signal frequency is function of  <Tone> and is calculated by using the
	//		following formula:
	//
	//			Frequency(KHz)=1000/(Tone*20).
	//
	//		NOTE:
	//			It is recommended to use value 12 (4 KHz) because it
	//			produces the most aloud sound.
	//
	//	<Duration> (input):
	//		specifies the duration of signal in steps of 200ms. Available range is
	//		1-15. (ex.: value 2 produces signal with duration 400 ms).
	//
	//	<Iterations> (input):
	//		specifies the number of iterations.	Available values are 0 (used to 
	//		break the previously programmed signal) and 1-15 (number of repetitive
	//		iterations).  
	//
	//		NOTE:
	//			Pause duration between two successive signals is equal to the 
	//			duration of the signal itself.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE3.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_Sound(
		unsigned char Tone,
		unsigned char Duration, 
		unsigned char Iterations
		);

	// VER2700
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_GetUserDataSize
	//
	// Description: 
	//	This function returns the size, in bytes, of the user available data 
	//	storage area.
	//
	//	NOTE: 
	//		Call this function only if GBMSAPI_DF_PERMANENT_USER_DATA_STORAGE 
	//		feature is supported. 
	//		See DEVICE FEATURES section in GBMSAPI_Defines.h file.
	//
	// Parameters:
	//	<UserDataSize> (output):
	//		valid only if GBMSAPI_ERROR_CODE_NO_ERROR is returned. Size in byte.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_GetUserDataSize(int *UserDataSize);

	// VER2700
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_ReadUserData
	//
	// Description: 
	//	This function reads a specified size of the user available data storage 
	//	area starting from specified address.
	//	Data integrity is not guarantee so it's user responsibility to implements a
	//	proper check (CRC).
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_DF_PERMANENT_USER_DATA_STORAGE 
	//		feature is supported. 
	//		See DEVICE FEATURES section in GBMSAPI_Defines.h file.
	//
	//	NOTE2: 
	//		<Address> + <Size> should not be greater than the value returned by 
	//		GBMSAPI_GetUserDataSize() function, otherwise 
	//		GBMSAPI_ERROR_CODE_PARAMETER will be returned
	//
	// Parameters:
	//	<Address> (input):
	//		Address from which the data will be read. Range from 0 to 
	//		(GBMSAPI_GetUserDataSize() - 1)
	//
	//	<Size> (input):
	//		size, in bytes, of the required data.
	//
	//	<Data> (output):
	//		valid only if GBMSAPI_ERROR_CODE_NO_ERROR is returned. Pointer to a
	//		buffer in which the data, read from the scanner, will be available.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER
	//		See NOTE2.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_ReadUserData(
		unsigned int Address, 
		unsigned int Size,
		void* Data
	);

	// VER2700
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_WriteUserData
	//
	// Description: 
	//	This function writes a specified size of the user available data storage 
	//	area starting from specified address.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_DF_PERMANENT_USER_DATA_STORAGE 
	//		feature is supported. 
	//		See DEVICE FEATURES section in GBMSAPI_Defines.h file.
	//
	//	NOTE2: 
	//		<Address> + <Size> should not be greater than the value returned by 
	//		GBMSAPI_GetUserDataSize() function, otherwise 
	//		GBMSAPI_ERROR_CODE_PARAMETER will be returned
	//
	//	NOTE3: 
	//		Data are permanently saved in a EEPROM (or in a Flash) memory and 
	//		can be rewritten many time; however this storage area cannot be used 
	//		for repetitive R/W operations because the mean life of EEPROM memories
	//		is a million write operations.
	//
	// Parameters:
	//	<Address> (input):
	//		Address at which the data will be written. Range from 0 to 
	//		(GBMSAPI_GetUserDataSize() - 1)
	//
	//	<Size> (input):
	//		Size, in bytes, of the data to be written.
	//
	//	<Data> (input):
	//		Pointer to a buffer containing the data to be written.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER
	//		See NOTE2.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_WriteUserData(
		unsigned int Address, 
		unsigned int Size,
		void* Data
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_SetFingerIndicatorVUIState
	//
	// Description: 
	//	This function puts the specified indicator(s) to the desired state.
	//	Function affects the state of all indicators marked in parameter 
	//	<IndicatorMask>. 
	//	The state of non-marked indicators is not changed.
	//
	//	NOTE1:
	//		During acquisition, if this function is called, it has no effect,
	//		since LED control is automatically performed. The only option left
	//		to developer is to make the LEDs blink (see the
	//		GBMSAPI_VUI_LED_BlinkDuringAcquisition(...) function).
	//
	//	NOTE2: 
	//		It is recommended to use the following color scheme for finger 
	//		indicators:
	//
	//		1.	continuous yellow to indicate acquisition in progress (normal
	//			conditions) - this is automatically performed during acquisition.
	//
	//		2.	blinking yellow to indicate acquisition in progress when 
	//			<GB_Diagnostic> parameter of the 
	//			GBMSAPI_AcquisitionEventsManagerCallback(...) assumes the following
	//			values caused by possible improper user behaviour:
	//
	//			-	GBMSAPI_DM_SCANNER_SURFACE_NOT_NORMA (finger has been
	//				placed too early)
	//			-	GBMSAPI_DM_FLAT_FINGER_SLIDING (finger is sliding on
	//				scanning surface)
	//			-	GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_... (finger has 
	//				been positioned improperly)
	//			-	GBMSAPI_DM_FLAT_FINGER_DISPLACED_DOWN (finger has been
	//				positioned improperly)
	//
	//			Blinking yellow can be used together with the
	//			GBMSAPI_SetAutoCaptureBlocking(...) function in order to indicate that
	//			auto-capture is prohibited due to improper user behaviour.
	//			During acquisition blinking can be set by means of the
	//			GBMSAPI_VUI_LED_BlinkDuringAcquisition(...) function.
	//
	//		3.	short-period green/red to indicate end of acquisition process and
	//			the correspoding acquisition result (positive/negative
	//			correspondingly).
	//
	//	NOTE3: 
	//		Call this function only if GBMSAPI_OED_VUI_LED feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	// Parameters:
	//	<IndicatorMask> (input): 
	//		Can be choosen between the values defined in the LED MASK chapter of the
	//		VISUAL INTERFACE LEDS section in GBMSAPI_Defines.h file.
	//
	//	<Color> (input): 
	//		Can be choosen between the values defined in the LED COLORS chapter of 
	//		the VISUAL INTERFACE LEDS section in GBMSAPI_Defines.h file.
	//
	//	<Blink> (input):
	//		- TRUE:		blink
	//		- FALSE:	no blink
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE3. Consider as a warning
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_SetFingerIndicatorVUIState(
		unsigned int IndicatorMask,
		int Color,
		BOOL Blink
		);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_CheckPedalState
	//
	// Description: 
	//	This function returns the current state of externally-connected pedal. 
	//	Function can be used only if acquisition process is not in progress.
	//	With this function an application can use pedal as an additional control 
	//	tool between acqusitions (basic pedal usage - operator clicks pedal to stop
	//	the acquisition process to select the current image)
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LED feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	// Parameters:
	//	<PedalState> (output):
	//		- TRUE:		pedal is pressed
	//		- FALSE:	pedal is not pressed
	//
	//		NOTE:
	//			If a pedal has been pressed and released between calls of this
	//			function, <PedalState> will be assigned TRUE once to inform about
	//			the lost pressing.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND
	//		No devices were found.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_CheckPedalState(
		unsigned char *PedalState
		);	

	// VER 2.8.0.0
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_GetLcdFeatures
	//
	// Description: 
	//	This function returns, in the <LcdFeatures> output parameter, a
	//	list of the types supported by the device, as they are described into
	//	the "SCANNABLE BIOMETRIC TYPES" section of the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	// Parameters:
	//	<LcdFeatures> (output):
	//		List of the features supported by the lcd display, in form of a bit
	//		mask.
	//		Valid only if GBMSAPI_ERROR_CODE_NO_ERROR is returned as error code
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_GetLcdFeatures(
		unsigned int *LcdFeatures
	);

	// VER 2.8.0.0
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetLanguage
	//
	// Description: 
	//	This function allows the developer to set the VUI LCD interface language
	//	in idioms other than English.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2: 
	//		Call this function only if GBMSAPI_VILCD_LF_LANGUAGE_SETTING feature is
	//		supported. 
	//		See DISPLAY FEATURES section in GBMSAPI_Defines.h file.
	//
	//	NOTE3:
	//		Call this function only if no acquisition is in progress, since during
	//		acquisition LCD is automatically controlled. It can be called:
	//		
	//		1)	before calling GBMSAPI_StartAcquisition(...) function
	//
	//		2)	when GBMSAPI_AcquisitionEventsManagerCallback(...) is called with
	//			GBMSAPI_AE_ACQUISITION_END or GBMSAPI_AE_ACQUISITION_ERROR as
	//			<OccurredEventCode> parameter		
	//
	//		See also GBMSAPI_StartAcquisition(...) function. Otherwise 
	//		GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED will be returned.
	//
	// Parameters:
	//	<ISO639Lang> (input):
	//		ISO639Lang: ISO 639-1 language code
	//						"en" for english                
	//						"zh" for chinese
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1 and NOTE2.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		See NOTE3.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetLanguage(
		char * ISO639Lang
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetLogoScreen
	//
	// Description: 
	//	This function allows to set LOGO screen on LCD. Use this function if, after
	//	an acquisition where you had choosen to set STOP-SCREEN as the final screen
	//	(see <DisplayOptionMask> parameter of GBMSAPI_StartAcquisition(...) function),
	//	you want to return to LOGO SCREEN (for example after a button pressure)
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2:
	//		Call this function only if no acquisition is in progress, since during
	//		acquisition LCD is automatically controlled. It can be called:
	//		
	//		1)	before calling GBMSAPI_StartAcquisition(...) function
	//
	//		2)	when GBMSAPI_AcquisitionEventsManagerCallback(...) is called with
	//			GBMSAPI_AE_ACQUISITION_END or GBMSAPI_AE_ACQUISITION_ERROR as
	//			<OccurredEventCode> parameter		
	//
	//		See also GBMSAPI_StartAcquisition(...) function. Otherwise 
	//		GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED will be returned.
	//
	// Parameters:
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		See NOTE2.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetLogoScreen(void);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen
	//
	// Description: 
	//	This function allows to choose an image to visualize on LCD.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2:
	//		Call this function only if no acquisition is in progress, since during
	//		acquisition LCD is automatically controlled. It can be called:
	//		
	//		1)	before calling GBMSAPI_StartAcquisition(...) function
	//
	//		2)	when GBMSAPI_AcquisitionEventsManagerCallback(...) is called with
	//			GBMSAPI_AE_ACQUISITION_END or GBMSAPI_AE_ACQUISITION_ERROR as
	//			<OccurredEventCode> parameter		
	//
	//		See also GBMSAPI_StartAcquisition(...) function. Otherwise 
	//		GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED will be returned.
	//
	//	NOTE3:
	//		Call this function only if currently set screen is STOP-SCREEN,
	//		otherwise function has no effect.
	//
	// Parameters:
	//	<MessageToSet> (input):
	//		Must be one of the values defined in the PRELOADED DIAGNOSTIC MESSAGES
	//		or PRELOADED GENERAL MESSAGES paragraph of the VISUAL INTERFACE LCD 
	//		section in GBMSAPI_Defines.h file. Otherwise function has no effect.
	//
	//	<MessageArea> (input):
	//		Must be one of the values defined in the PRELOADED MESSAGES AREA 
	//		paragraph of the VISUAL INTERFACE LCD section in GBMSAPI_Defines.h file.
	//		Otherwise function has no effect.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		See NOTE2.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(
		BYTE MessageToSet,
		BYTE MessageArea
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetCustomerMessageOnStopScreen
	//
	// Description: 
	//	This function is used to write customer message on LCD. Use only if
	//	necessary, normally is better to use the 
	//	GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(...).
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2:
	//		Call this function only if no acquisition is in progress, since during
	//		acquisition LCD is automatically controlled. It can be called:
	//		
	//		1)	before calling GBMSAPI_StartAcquisition(...) function
	//
	//		2)	when GBMSAPI_AcquisitionEventsManagerCallback(...) is called with
	//			GBMSAPI_AE_ACQUISITION_END or GBMSAPI_AE_ACQUISITION_ERROR as
	//			<OccurredEventCode> parameter		
	//
	//		See also GBMSAPI_StartAcquisition(...) function. Otherwise 
	//		GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED will be returned.
	//
	//	NOTE3:
	//		Call this function only if currently set screen is STOP-SCREEN,
	//		otherwise function has no effect.
	//
	// Parameters:
	//	<MessageToSet> (input):
	//		Must be an ANSI string '\0' terminated. Its length cannot be more than
	//		GBMSAPI_VILCD_MAX_CUSTOM_MSG_LEN, otherwise message will be
	//		truncated. If NULL function will not be executed.
	//
	//	<MessageToSetType> (input):
	//		Must be one among GBMSAPI_VILCD_CUSTOM_MSG_TYPE_INFO_MSG and
	//		GBMSAPI_VILCD_CUSTOM_MSG_TYPE_ERROR_MSG. Otherwise 
	//		GBMSAPI_VILCD_CUSTOM_MSG_TYPE_INFO_MSG option will be used. In the
	//		first case (GBMSAPI_VILCD_CUSTOM_MSG_TYPE_INFO_MSG) the message appears
	//		on LCD with BLACK character on WHITE background, in the second one
	//		(GBMSAPI_VILCD_CUSTOM_MSG_TYPE_ERROR_MSG) the message appears on LCD with
	//		WHITE character on BLACK background
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		See NOTE2.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetCustomerMessageOnStopScreen(
		char * MessageToSet,
		BYTE MessageToSetType
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_EnableOKButtonOnStopScreen
	//
	// Description: 
	//	This function allows to enable OK button on STOP-SCREEN. If it
	//	is not called, the OK button on the STOP SCREEN will not be enabled and its
	//	pressure cannot be caught by the application.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2:
	//		Call this function only if no acquisition is in progress, since during
	//		acquisition LCD is automatically controlled. It can be called:
	//		
	//		1)	before calling GBMSAPI_StartAcquisition(...) function
	//
	//		2)	when GBMSAPI_AcquisitionEventsManagerCallback(...) is called with
	//			GBMSAPI_AE_ACQUISITION_END or GBMSAPI_AE_ACQUISITION_ERROR as
	//			<OccurredEventCode> parameter		
	//
	//		See also GBMSAPI_StartAcquisition(...) function. Otherwise 
	//		GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED will be returned.
	//
	//	NOTE3:
	//		Call this function only if currently set screen is STOP-SCREEN,
	//		otherwise function has no effect.
	//
	// Parameters:
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		See NOTE2.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_EnableOKButtonOnStopScreen(void);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetQualityLimitOnStopScreen
	//
	// Description: 
	//	This function writes on STOP ACQUISITION SCREEN of the LCD the limit value
	//	of quality.
	//	Call this function before calling 
	//	GBMSAPI_VUI_LCD_SetQualityOnStopScreen(...), otherwise display
	//	behaviour could be unpredictable.
	//  For details about display working, see the VISUAL INTERFACE LCD section in
	//	the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2:
	//		Call this function only if no acquisition is in progress, since during
	//		acquisition LCD is automatically controlled. It can be called:
	//		
	//		1)	before calling GBMSAPI_StartAcquisition(...) function
	//
	//		2)	when GBMSAPI_AcquisitionEventsManagerCallback(...) is called with
	//			GBMSAPI_AE_ACQUISITION_END or GBMSAPI_AE_ACQUISITION_ERROR as
	//			<OccurredEventCode> parameter		
	//
	//		See also GBMSAPI_StartAcquisition(...) function. Otherwise 
	//		GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED will be returned.
	//
	//	NOTE3:
	//		Call this function only if currently set screen is STOP-SCREEN,
	//		otherwise function has no effect.
	//
	//	NOTE4:
	//		For colour display only, call this function before calling
	//		GBMSAPI_VUI_LCD_SetQualityIntermediateLimitOnStopScreen(...),
	//		otherwise this latter will have no effect.
	//
	// Parameters:
	//	<QualityLimit> (input):
	//		quality limit value to be written.
	//
	//	<Direction> (input):
	//		Tells if the quality value is valid when it is smaller or greater than
	//		<QualityLimit>. The values it can assume are defined in the LIMITS
	//		DIRECTION paragraph of the VISUAL INTERFACE LCD section in the
	//		GBMSAPI_Defines.h file
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		See NOTE2.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetQualityLimitOnStopScreen(
		BYTE QualityLimit,
		BYTE Direction
	);

	// VER 2.8.0.0
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetQualityIntermediateLimitOnStopScreen
	//
	// Description: 
	//	This function sets the intermediate limit for Quality on 
	//	STOP-SCREEN.
	//  For details about display working, see the VISUAL INTERFACE LCD section in
	//	the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		Call this function after the 
	//		GBMSAPI_VUI_LCD_SetQualityLimitOnStopScreen(...) function and
	//		before the GBMSAPI_VUI_LCD_SetQualityOnStopScreen(...) one.
	//		If this function is not called, only RED and GREEN values will be shown.
	//		If the function calling order indicated above is not respected, the
	//		display could have an unpredictable behaviour.
	//
	//	NOTE2:
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported 
	//		(see OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file)
	//		and if GBMSAPI_VILCD_LF_INTERMEDIATE_LIMIT feature see 
	//		DISPLAY FEATURES paragraph in GBMSAPI_Defines.h)as well, otherwise 
	//		GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED will be returned
	//	
	//
	// Parameters:
	//	<QualityIntermediateLimit>			(input):
	//		The value of intermediate limit that should be shown.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED
	//		<ObjectToScan> type is not supported by the device.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED
	//		The acquisition thread is already running
	//
	//	- GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED:
	//		See NOTE2
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetQualityIntermediateLimitOnStopScreen(
		BYTE QualityIntermediateLimit
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetQualityOnStopScreen
	//
	// Description: 
	//	This function allows to set quality value on the STOP-SCREEN.
	//	Call this function after GBMSAPI_VUI_LCD_SetQualityLimitOnStopScreen(...)
	//	and, for colour display only, after
	//	GBMSAPI_VUI_LCD_SetQualityIntermediateLimitOnStopScreen(...) (if called,
	//	see comments for the function).
	//  For details about display working, see the VISUAL INTERFACE LCD section in
	//	the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2:
	//		Call this function only if no acquisition is in progress, since during
	//		acquisition LCD is automatically controlled. It can be called:
	//		
	//		1)	before calling GBMSAPI_StartAcquisition(...) function
	//
	//		2)	when GBMSAPI_AcquisitionEventsManagerCallback(...) is called with
	//			GBMSAPI_AE_ACQUISITION_END or GBMSAPI_AE_ACQUISITION_ERROR as
	//			<OccurredEventCode> parameter		
	//
	//		See also GBMSAPI_StartAcquisition(...) function. Otherwise 
	//		GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED will be returned.
	//
	//	NOTE3:
	//		Call this function only if currently set screen is STOP-SCREEN,
	//		otherwise function has no effect.
	//
	// Parameters:
	//	<Quality> (input):
	//		quality value to be written.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		See NOTE2.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetQualityOnStopScreen(
		BYTE Quality
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetArtefactLimitOnStopScreen
	//
	// Description: 
	//	This function writes on STOP SCREEN of the LCD the limit value
	//	of rolled artefacts.
	//	Call this function before calling 
	//	GBMSAPI_VUI_LCD_SetArtefactOnStopScreen(...), otherwise display
	//	behaviour could be unpredictable.
	//  For details about display working, see the VISUAL INTERFACE LCD section in
	//	the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2:
	//		Call this function only if no acquisition is in progress, since during
	//		acquisition LCD is automatically controlled. It can be called:
	//		
	//		1)	before calling GBMSAPI_StartAcquisition(...) function
	//
	//		2)	when GBMSAPI_AcquisitionEventsManagerCallback(...) is called with
	//			GBMSAPI_AE_ACQUISITION_END or GBMSAPI_AE_ACQUISITION_ERROR as
	//			<OccurredEventCode> parameter		
	//
	//		See also GBMSAPI_StartAcquisition(...) function. Otherwise 
	//		GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED will be returned.
	//
	//	NOTE3:
	//		Call this function only if currently set screen is STOP-SCREEN,
	//		otherwise function has no effect.
	//
	//	NOTE4:
	//		For colour display only, call this function before calling
	//		GBMSAPI_VUI_LCD_SetArtefactIntermediateLimitOnStopScreen(...),
	//		otherwise this latter will have no effect.
	//
	// Parameters:
	//	<ArtefactLimit> (input):
	//		artefacts limit value to be written.
	//
	//	<Direction> (input):
	//		Tells if the artefacts value is valid when it is smaller or greater than
	//		<ArtefactLimit>. The values it can assume are defined in the LIMITS
	//		DIRECTION paragraph of the VISUAL INTERFACE LCD section in the
	//		GBMSAPI_Defines.h file
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		See NOTE2.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetArtefactLimitOnStopScreen(
		int ArtefactLimit,
		BYTE Direction
	);

	// VER 2.8.0.0
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetArtefactIntermediateLimitOnStopScreen
	//
	// Description: 
	//	This function sets the intermediate limit for Artefacts on 
	//	STOP-SCREEN.
	//  For details about display working, see the VISUAL INTERFACE LCD section in
	//	the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		Call this function after the 
	//		GBMSAPI_VUI_LCD_SetArtefactLimitOnStopScreen(...) function and
	//		before the GBMSAPI_VUI_LCD_SetArtefactOnStopScreen(...) one.
	//		If this function is not called, only RED and GREEN values will be shown.
	//		If the function calling order indicated above is not respected, the
	//		display could have an unpredictable behaviour.
	//
	//	NOTE2:
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported 
	//		(see OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file)
	//		and if GBMSAPI_VILCD_LF_INTERMEDIATE_LIMIT feature see 
	//		DISPLAY FEATURES paragraph in GBMSAPI_Defines.h)as well, otherwise 
	//		GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED will be returned
	//	
	//
	// Parameters:
	//	<ArtefactIntermediateLimit>			(input):
	//		The value of intermediate limit that should be shown.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED
	//		<ObjectToScan> type is not supported by the device.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED
	//		The acquisition thread is already running
	//
	//	- GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED:
	//		See NOTE2
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetArtefactIntermediateLimitOnStopScreen(
		int ArtefactIntermediateLimit
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetArtefactOnStopScreen
	//
	// Description: 
	//	This function writes on the LCD the rolled artefact amount of the image.
	//	Call this function after GBMSAPI_VUI_LCD_SetArtefactLimitOnStopScreen(...)
	//	and, for colour display only, after
	//	GBMSAPI_VUI_LCD_SetArtefactIntermediateLimitOnStopScreen(...) (if called,
	//	see comments for the function).
	//  For details about display working, see the VISUAL INTERFACE LCD section in
	//	the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2:
	//		Call this function only if no acquisition is in progress, since during
	//		acquisition LCD is automatically controlled. It can be called:
	//		
	//		1)	before calling GBMSAPI_StartAcquisition(...) function
	//
	//		2)	when GBMSAPI_AcquisitionEventsManagerCallback(...) is called with
	//			GBMSAPI_AE_ACQUISITION_END or GBMSAPI_AE_ACQUISITION_ERROR as
	//			<OccurredEventCode> parameter		
	//
	//		See also GBMSAPI_StartAcquisition(...) function. Otherwise 
	//		GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED will be returned.
	//
	//	NOTE3:
	//		Call this function only if currently set screen is STOP-SCREEN,
	//		otherwise function has no effect.
	//
	// Parameters:
	//	<Artefact> (input):
	//		Artefact amount to be written.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		See NOTE2.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetArtefactOnStopScreen(
		int Artefact
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetSegmentationOnStopScreen
	//
	// Description: 
	//	This function writes on the LCD the result of the fingers segmentation when
	//	2 thumbs or 4 fingers have been acquired.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2:
	//		Call this function only if no acquisition is in progress, since during
	//		acquisition LCD is automatically controlled. It can be called:
	//		
	//		1)	before calling GBMSAPI_StartAcquisition(...) function
	//
	//		2)	when GBMSAPI_AcquisitionEventsManagerCallback(...) is called with
	//			GBMSAPI_AE_ACQUISITION_END or GBMSAPI_AE_ACQUISITION_ERROR as
	//			<OccurredEventCode> parameter		
	//
	//		See also GBMSAPI_StartAcquisition(...) function. Otherwise 
	//		GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED will be returned.
	//
	//	NOTE3:
	//		Call this function only if currently set screen is STOP-SCREEN,
	//		otherwise function has no effect.
	//
	// Parameters:
	//	<Seg1> (input):
	//		result on position 1 (from left to right) of segmentation. It can
	//		assume values defined in the SEGMENTATION RESULTS VALUE paragraph
	//		of the VISUAL INTERFACE LCD in GBMSAPI_Defines.h file.
	//
	//	<Seg2> (input):
	//		result on position 2 (from left to right) of segmentation. It can
	//		assume values defined in the SEGMENTATION RESULTS VALUE paragraph
	//		of the VISUAL INTERFACE LCD in GBMSAPI_Defines.h file.
	//
	//	<Seg3> (input):
	//		result on position 3 (from left to right) of segmentation. It can
	//		assume values defined in the SEGMENTATION RESULTS VALUE paragraph
	//		of the VISUAL INTERFACE LCD in GBMSAPI_Defines.h file.
	//
	//	<Seg4> (input):
	//		result on position 4 (from left to right) of segmentation. It can
	//		assume values defined in the SEGMENTATION RESULTS VALUE paragraph
	//		of the VISUAL INTERFACE LCD in GBMSAPI_Defines.h file.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		See NOTE2.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetSegmentationOnStopScreen(
		BYTE Seg1,
		BYTE Seg2,
		BYTE Seg3,
		BYTE Seg4
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_EnableStartButtonOnLogoScreen
	//
	// Description: 
	//	This function allows to enable START button on LOGO SCREEN. If it
	//	is not called, the START button on the LOGO SCREEN will not be enabled and its
	//	pressure cannot be caught by the application.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2:
	//		Call this function only if no acquisition is in progress, since during
	//		acquisition LCD is automatically controlled. It can be called:
	//		
	//		1)	before calling GBMSAPI_StartAcquisition(...) function
	//
	//		2)	when GBMSAPI_AcquisitionEventsManagerCallback(...) is called with
	//			GBMSAPI_AE_ACQUISITION_END or GBMSAPI_AE_ACQUISITION_ERROR as
	//			<OccurredEventCode> parameter		
	//
	//		See also GBMSAPI_StartAcquisition(...) function. Otherwise 
	//		GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED will be returned.
	//
	//	NOTE3:
	//		Call this function only if currently set screen is LOGO-SCREEN,
	//		otherwise function has no effect.
	//
	// Parameters:
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		See NOTE2.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_EnableStartButtonOnLogoScreen(void);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_GetPressedButton
	//
	// Description: 
	//	This function reads buttons pressed on the touchscreen.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2:
	//		Call this function only if no acquisition is in progress, since during
	//		acquisition LCD is automatically controlled. It can be called:
	//		
	//		1)	before calling GBMSAPI_StartAcquisition(...) function
	//
	//		2)	when GBMSAPI_AcquisitionEventsManagerCallback(...) is called with
	//			GBMSAPI_AE_ACQUISITION_END or GBMSAPI_AE_ACQUISITION_ERROR as
	//			<OccurredEventCode> parameter		
	//
	//		See also GBMSAPI_StartAcquisition(...) function. Otherwise 
	//		GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED will be returned.
	//
	// Parameters:
	//	<PressedButton> (output):
	//		ID of the pressed button on the touchscreen. It can
	//		assume values defined in the PRESSED BUTTON IDs paragraph
	//		of the VISUAL INTERFACE LCD in GBMSAPI_Defines.h file.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		See NOTE2.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_GetPressedButton(BYTE *PressedButton);

	// VER 2.8.0.0
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimit
	//
	// Description: 
	//	This function sets the intermediate limit for lower half palm completeness on 
	//	ACQUISITION-SCREEN.
	//  For details about display working, see the VISUAL INTERFACE LCD section in
	//	the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		Call this function before the GBMSAPI_StartAcquisition(...) function.
	//		If this function is not called, only RED and GREEN values will be shown.
	//
	//	NOTE2:
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported 
	//		(see OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file)
	//		and if GBMSAPI_VILCD_LF_INTERMEDIATE_LIMIT feature see 
	//		DISPLAY FEATURES paragraph in GBMSAPI_Defines.h)as well, otherwise 
	//		GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED will be returned
	//	
	//
	// Parameters:
	//	<CompletenessIntermediateLimit>			(input):
	//		The value of intermediate limit that should be shown.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED
	//		<ObjectToScan> type is not supported by the device.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED
	//		The acquisition thread is already running
	//
	//	- GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED:
	//		See NOTE2
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimit(
		BYTE CompletenessIntermediateLimit
	);

	// VER 2.8.0.0
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetContrastIntermediateLimit
	//
	// Description: 
	//	This function sets the intermediate limit for contrast on 
	//	ACQUISITION-SCREEN.
	//  For details about display working, see the VISUAL INTERFACE LCD section in
	//	the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		Call this function before the GBMSAPI_StartAcquisition(...) function.
	//		If this function is not called, only RED and GREEN values will be shown.
	//
	//	NOTE2:
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported 
	//		(see OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file)
	//		and if GBMSAPI_VILCD_LF_INTERMEDIATE_LIMIT feature see 
	//		DISPLAY FEATURES paragraph in GBMSAPI_Defines.h)as well, otherwise 
	//		GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED will be returned
	//	
	//
	// Parameters:
	//	<ContrastIntermediateLimit>			(input):
	//		The value of intermediate limit that should be shown.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED
	//		<ObjectToScan> type is not supported by the device.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED
	//		The acquisition thread is already running
	//
	//	- GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED:
	//		See NOTE2
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetContrastIntermediateLimit(
		BYTE ContrastIntermediateLimit
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetContrastValueOnAcquisitionScreen
	//
	// Description: 
	//	This function allows to set contrast value on the ACQUISITION-SCREEN.
	//  For details about display working, see the VISUAL INTERFACE LCD section in
	//	the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2:
	//		Call this function only if acquisition is in progress, in function
	//		definition of GBMSAPI_AcquisitionEventsManagerCallback(...), and only if 
	//		GBMSAPI_DO_CUSTOMIZED_CONTRAST option has been set and currently
	//		scanned object type is not GBMSAPI_SBT_ROLL_SINGLE_FINGER. Otherwise it 
	//		has no effect.
	//
	// Parameters:
	//	<ContrastValue> (input):
	//		Contrast value to be set
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetContrastValueOnAcquisitionScreen(
		BYTE ContrastValue
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetCompletenessValueOnAcquisitionScreen
	//
	// Description: 
	//	This function allows to set LOWER HALF-PALM completeness value on the
	//	ACQUISITION-SCREEN.
	//  For details about display working, see the VISUAL INTERFACE LCD section in
	//	the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2:
	//		Call this function only if acquisition is in progress, in function
	//		definition of GBMSAPI_AcquisitionEventsManagerCallback(...), and only if 
	//		GBMSAPI_DO_CUSTOMIZED_CONTRAST option has been set and currently
	//		scanned object type is GBMSAPI_SBT_FLAT_LOWER_HALF_PALM. Otherwise it has
	//		no effect.
	//
	// Parameters:
	//	<CompletenessValue> (input):
	//		Completeness value to be set
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetCompletenessValueOnAcquisitionScreen(
		BYTE CompletenessValue
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetContrastValueOnStopScreen
	//
	// Description: 
	//	This function allows to set contrast value on the STOP-SCREEN.
	//	Call this function after GBMSAPI_VUI_LCD_SetContrastLimitOnStopScreen(...)
	//	and, for colour display only, after
	//	GBMSAPI_VUI_LCD_SetContrastIntermediateLimitOnStopScreen(...) (if called,
	//	see comments for the function).
	//  For details about display working, see the VISUAL INTERFACE LCD section in
	//	the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2:
	//		Call this function only if no acquisition is in progress, since during
	//		acquisition LCD is automatically controlled. It can be called:
	//		
	//		1)	before calling GBMSAPI_StartAcquisition(...) function
	//
	//		2)	when GBMSAPI_AcquisitionEventsManagerCallback(...) is called with
	//			GBMSAPI_AE_ACQUISITION_END or GBMSAPI_AE_ACQUISITION_ERROR as
	//			<OccurredEventCode> parameter		
	//
	//		See also GBMSAPI_StartAcquisition(...) function. Otherwise 
	//		GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED will be returned.
	//
	//	NOTE3:
	//		Call this function only if currently set screen is STOP-SCREEN,
	//		otherwise function has no effect.
	//
	// Parameters:
	//	<ContrastValue> (input):
	//		Contrast value to be set
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		See NOTE2.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetContrastValueOnStopScreen(
		BYTE ContrastValue
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetCompletenessValueOnStopScreen
	//
	// Description: 
	//	This function allows to set lower half palm completeness value on the 
	//	STOP-SCREEN.
	//	Call this function after 
	//	GBMSAPI_VUI_LCD_SetCompletenessLimitOnStopScreen(...)
	//	and, for colour display only, after
	//	GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen(...) (if
	//	called, see comments for the function).
	//  For details about display working, see the VISUAL INTERFACE LCD section in
	//	the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2:
	//		Call this function only if no acquisition is in progress, since during
	//		acquisition LCD is automatically controlled. It can be called:
	//		
	//		1)	before calling GBMSAPI_StartAcquisition(...) function
	//
	//		2)	when GBMSAPI_AcquisitionEventsManagerCallback(...) is called with
	//			GBMSAPI_AE_ACQUISITION_END or GBMSAPI_AE_ACQUISITION_ERROR as
	//			<OccurredEventCode> parameter		
	//
	//		See also GBMSAPI_StartAcquisition(...) function. Otherwise 
	//		GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED will be returned.
	//
	//	NOTE3:
	//		Call this function only if currently set screen is STOP-SCREEN,
	//		otherwise function has no effect.
	//
	// Parameters:
	//	<CompletenessValue> (input):
	//		Completeness value to be set
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
	//		See NOTE2.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetCompletenessValueOnStopScreen(
		BYTE CompletenessValue
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetContrastLimitOnStopScreen
	//
	// Description: 
	//	This function allows to set contrast value on the STOP-SCREEN.
	//	Call this function before calling 
	//	GBMSAPI_VUI_LCD_SetContrastValueOnStopScreen(...), otherwise display
	//	behaviour could be unpredictable.
	//  For details about display working, see the VISUAL INTERFACE LCD section in
	//	the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2:
	//		Call this function only if no acquisition is in progress, since during
	//		acquisition LCD is automatically controlled. It can be called:
	//		
	//		1)	before calling GBMSAPI_StartAcquisition(...) function
	//
	//		2)	when GBMSAPI_AcquisitionEventsManagerCallback(...) is called with
	//			GBMSAPI_AE_ACQUISITION_END or GBMSAPI_AE_ACQUISITION_ERROR as
	//			<OccurredEventCode> parameter		
	//
	//		See also GBMSAPI_StartAcquisition(...) function. Otherwise 
	//		GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED will be returned.
	//
	//	NOTE3:
	//		Call this function only if currently set screen is STOP-SCREEN,
	//		otherwise function has no effect.
	//
	//	NOTE4:
	//		For colour display only, call this function before calling
	//		GBMSAPI_VUI_LCD_SetContrastIntermediateLimitOnStopScreen(...),
	//		otherwise this latter will have no effect.
	//
	// Parameters:
	//	<ContrastLimit> (input):
	//		Contrast value to be set
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetContrastLimitOnStopScreen(BYTE ContrastLimit);

	// VER 2.8.0.0
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetContrastIntermediateLimitOnStopScreen
	//
	// Description: 
	//	This function sets the intermediate limit for contrast on 
	//	STOP-SCREEN.
	//  For details about display working, see the VISUAL INTERFACE LCD section in
	//	the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		Call this function after the 
	//		GBMSAPI_VUI_LCD_SetContrastLimitOnStopScreen(...) function and
	//		before the GBMSAPI_VUI_LCD_SetContrastValueOnStopScreen(...) one.
	//		If this function is not called, only RED and GREEN values will be shown.
	//		If the function calling order indicated above is not respected, the
	//		display could have an unpredictable behaviour.
	//
	//	NOTE2:
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported 
	//		(see OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file)
	//		and if GBMSAPI_VILCD_LF_INTERMEDIATE_LIMIT feature see 
	//		DISPLAY FEATURES paragraph in GBMSAPI_Defines.h)as well, otherwise 
	//		GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED will be returned
	//	
	//
	// Parameters:
	//	<ContrastIntermediateLimit>			(input):
	//		The value of intermediate limit that should be shown.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED
	//		<ObjectToScan> type is not supported by the device.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED
	//		The acquisition thread is already running
	//
	//	- GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED:
	//		See NOTE2
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetContrastIntermediateLimitOnStopScreen(
		BYTE ContrastIntermediateLimit
	);

	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetCompletenessLimitOnStopScreen
	//
	// Description: 
	//	This function allows to set completeness limit on the STOP-SCREEN.
	//	Call this function before calling 
	//	GBMSAPI_VUI_LCD_SetCompletenessValueOnStopScreen(...), otherwise display
	//	behaviour could be unpredictable.
	//  For details about display working, see the VISUAL INTERFACE LCD section in
	//	the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported. 
	//		See OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file.
	//
	//	NOTE2:
	//		Call this function only if no acquisition is in progress, since during
	//		acquisition LCD is automatically controlled. It can be called:
	//		
	//		1)	before calling GBMSAPI_StartAcquisition(...) function
	//
	//		2)	when GBMSAPI_AcquisitionEventsManagerCallback(...) is called with
	//			GBMSAPI_AE_ACQUISITION_END or GBMSAPI_AE_ACQUISITION_ERROR as
	//			<OccurredEventCode> parameter		
	//
	//		See also GBMSAPI_StartAcquisition(...) function. Otherwise 
	//		GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED will be returned.
	//
	//	NOTE3:
	//		Call this function only if currently set screen is STOP-SCREEN,
	//		otherwise function has no effect.
	//
	//	NOTE4:
	//		For colour display only, call this function before calling
	//		GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen(...),
	//		otherwise this latter will have no effect.
	//
	// Parameters:
	//	<CompletenessLimit> (input):
	//		Completeness value to be set
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
	//		See NOTE1.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetCompletenessLimitOnStopScreen(
		BYTE CompletenessLimit
	);

	// VER 2.8.0.0
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen
	//
	// Description: 
	//	This function sets the intermediate limit for lower half palm completeness on 
	//	STOP-SCREEN.
	//  For details about display working, see the VISUAL INTERFACE LCD section in
	//	the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		Call this function after the 
	//		GBMSAPI_VUI_LCD_SetCompletenessLimitOnStopScreen(...) function and
	//		before the GBMSAPI_VUI_LCD_SetCompletenessValueOnStopScreen(...) one.
	//		If this function is not called, only RED and GREEN values will be shown.
	//		If the function calling order indicated above is not respected, the
	//		display could have an unpredictable behaviour.
	//
	//	NOTE2:
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported 
	//		(see OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file)
	//		and if GBMSAPI_VILCD_LF_INTERMEDIATE_LIMIT feature see 
	//		DISPLAY FEATURES paragraph in GBMSAPI_Defines.h)as well, otherwise 
	//		GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED will be returned
	//	
	//
	// Parameters:
	//	<CompletenessIntermediateLimit>			(input):
	//		The value of intermediate limit that should be shown.
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED
	//		<ObjectToScan> type is not supported by the device.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED
	//		The acquisition thread is already running
	//
	//	- GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED:
	//		See NOTE2
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen(
		BYTE CompletenessIntermediateLimit
	);

	// VER 2.9.0.1
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_VUI_LCD_SetWetDryFingerDiagnosticIconOnStopScreen
	//
	// Description: 
	//	This function sets the icon that warns about wer or dry finger diagnostic on 
	//	STOP-SCREEN.
	//  For details about display working, see the VISUAL INTERFACE LCD section in
	//	the GBMSAPI_Defines.h file.
	//
	//	NOTE1: 
	//		When using this function, contrast and other quality parameters are not
	//		shown anymore on the LCD display
	//
	//	NOTE2:
	//		Call this function only if GBMSAPI_OED_VUI_LCD feature is supported 
	//		(see OPTIONAL EXTERNAL EQUIPMENT section in GBMSAPI_Defines.h file)
	//		and if GBMSAPI_VILCD_LF_INTERMEDIATE_LIMIT feature see 
	//		DISPLAY FEATURES paragraph in GBMSAPI_Defines.h)as well, otherwise 
	//		GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED will be returned
	//	
	//
	// Parameters:
	//	<DiagMessageToVisualize>			(input):
	//		The diagnostic message that has to be shown. Can be GBMSAPI_VILCD_MSG_DRY_FINGER
	//		or GBMSAPI_VILCD_MSG_WET_FINGER only, otherwise GBMSAPI_ERROR_CODE_PARAMETER
	//		will be returned
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED
	//		Specific device DLL not present in application or in <System> directory.
	//		In case this error is returned try to reinstall the application
	//
	//	- GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
	//		Call GBMSAPI_SetCurrentDevice(...) before calling this function.
	//
	//	- GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED
	//		<ObjectToScan> type is not supported by the device.
	//
	//	- GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED
	//		The acquisition thread is already running
	//
	//	- GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED:
	//		See NOTE2
	//
	//	- Other error codes:
	//		Failure in calling low level functions: see ERROR CODES section in the 
	//		GBMSAPI_Defines.h file.
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_VUI_LCD_SetWetDryFingerDiagnosticIconOnStopScreen(
		BYTE DiagMessageToVisualize
	);

	/*******************************************************************************
	********************************************************************************
	*				SECTION 9													   *
	*																			   *
	*				IMAGE PROCESSING UTILITIES									   *
	********************************************************************************
	*******************************************************************************/
	////////////////////////////////////////////////////////////////////////////////
	// GBMSAPI_ConvertImageFrom1000to500DPI
	//
	// Description: 
	//	This function converts image resolution from 1000 to 500 DPI.
	//	500 DPI image size will be <InputSizeX/2> and <InputSizeY/2>
	//
	//	NOTE1: 
	//		It is recommended to call this function only when acquisition process
	//		is finished.
	//
	//	NOTE2:
	//		Call this function only on 1000 DPI images (when GBMSAPI_EI_DPI_RES
	//		bit of the <EventInfo> parameter of the acquisition callback is set)
	//
	// Parameters:
	//	<Input1000DPIFrame> (input): 
	//		1000 DPI Frame
	//
	//	<Output500DPIFrame> (input/output):
	//		500 DPI frame to be returned. It must be already allocated and its
	//		size must be at less (InputSizeX * InputSizeY)/4
	//
	//	<InputSizeX> (input):
	//		X size of the input image
	//
	//	<InputSizeY> (input):
	//		Y size of the input image
	//
	// Return:
	//	- GBMSAPI_ERROR_CODE_NO_ERROR:
	//		success
	//
	//	- GBMSAPI_ERROR_CODE_PARAMETER:
	//		Input1000DPIFrame and/or Input500DPIFrame are NULL
	////////////////////////////////////////////////////////////////////////////////
	int GBMSAPI_API __stdcall GBMSAPI_ConvertImageFrom1000to500DPI(
		unsigned char *Input1000DPIFrame,
		unsigned char *Output500DPIFrame,
		unsigned int  InputSizeX,
		unsigned int  InputSizeY
	);


#ifdef __cplusplus
}
#endif

#endif GBMSAPI_LIBRARY_H