#include "acquisitionform.h"
#include "ui_acquisitionform.h"
#include "gbnfiq.h"
#include "ViewSegmForm.h"
#include "lfs_dll.h"
#include "Bozorth_dll.h"
#include "UnavailabilityReasonForm.h"

#define THRESHOLD_SMALLER_THAN  0
#define THRESHOLD_GREATHER_THAN 1

// Post process diagnostic
#define POST_PROC_DIAG_INCLINATION                      0x00000001
#define POST_PROC_DIAG_INCOMPLETE_PATTERN               0x00000002
#define POST_PROC_DIAG_INVALID_PATTERN                  0x00000004
#define POST_PROC_DIAG_SEGM_AMBIGUOUS_SLAP              0x00000008
#define POST_PROC_DIAG_SEGM_INCOMPLETE_SLAP             0x00000010
#define POST_PROC_DIAG_SEGM_POSSIBLE_INCOMPLTETE_SLAP   0x00000020
#define POST_PROC_DIAG_LEFT_RIGHT_MIXUP                 0x00000040
#define POST_PROC_DIAG_POSSIBLE_LEFT_RIGHT_MIXUP        0x00000080

// defines for Lfs get_minutiae
#define IMAGE_BIT_DEPTH         8
#define IMAGE_PIXEL_PER_MM      19.68   // 500 dpi

AcquisitionForm::AcquisitionForm(GBMSGUI *Ref, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::AcquisitionForm)
{
    MSGUIRef = Ref;

	setWindowFlags( windowFlags() | Qt::WindowMaximizeButtonHint);

    ui->setupUi(this);

	// DEBUG
	ReadIni();

    // connect OnAcquisitionEvent signal (emitted by the callback passed to GBMSAPI) to AcquisitionEventsManager slot;
    // this trick is necessary, because GBMSAPI calls the callback in a different thread, and from this
    // it is not possible to updtate the user interface
	// with Qt::BlockingQueuedConnection emit does not return until the slot finished
    QObject::connect(this, SIGNAL(OnAcquisitionEvent(unsigned int,int,unsigned int,unsigned char *,int,int,double,double,unsigned int)),
                     this, SLOT(AcquisitionEventsManager(unsigned int,int,unsigned int,unsigned char *,int,int,double,double,unsigned int)),
					 Qt::BlockingQueuedConnection);

    AcquisitionEventsManagerRet = -1;
    ScanResult = -1;
    mScannedObject = NULL;
    Acquiring = false;
    FramePtr = NULL;
    MarkerFrame = NULL;
    SegmTempBuff = NULL;
    NumSegments = 0;
	RotationBuffer1000DPI = NULL;

    memset(SegmImgDescr, 0, sizeof(GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR)*SEGM_IMG_MAX_NUM);

    // on Windows, OpeGL slow down performance!
#ifndef WIN32
    // enable OpenGL
    ui->graphicsView->setViewport(new QGLWidget());
    //ui->graphicsView->setViewport(new QGLWidget(this));
#endif
    ui->graphicsView->setViewportUpdateMode(QGraphicsView::FullViewportUpdate);

    // associate the scene with the graphics view
    ui->graphicsView->setScene(&scene);
    // add the frame to the scene
    scene.addItem(&FrameItem);

    // add clipping rectangle to the scene (hidden)
    ClipRectItem = new QClipRectItem(Qt::blue, 2);
    ClipRectItem->SetSizeOption(AllowMove);
    scene.addItem(ClipRectItem);
    ClipRectItem->setVisible(false);
    connect(ClipRectItem, SIGNAL(Changed()), this, SLOT(OnClipRectChanged()));

    // create gray palette
    GrayTable.resize(256);
    for( int i = 0; i < 256; i++ )
    {
       GrayTable[i] = qRgb( i, i, i );
    }

    ui->btnStop->setEnabled(false);
    ui->btnAccept->setEnabled(false);
    ui->btnRepeat->setEnabled(false);
    ui->btnBack->setEnabled(false);
    ui->btnSkip->setEnabled(false);
    ui->btnBreak->setEnabled(false);

    if (CheckMask(MSGUIRef->GetSessionOptions(), GBMSGUI_SESSION_OPTION_NO_ADVANCED_VIEW))
        ui->btnAdvancedView->setVisible(false);
    else
        ui->btnAdvancedView->setChecked(false);

	// 1.5.4.0
    if (MSGUIRef->IsDryFingerImageEnhancementSupported())
    {
		ui->chkDryFingerImageEnhancement->setChecked(CheckMask(MSGUIRef->GetSessionOptions(), GBMSGUI_SESSION_OPTION_DRY_FINGER_IMAGE_ENHANCEMENT));
		ui->chkDryFingerImageEnhancement->setEnabled(true);
    }
    else
    {
        ui->chkDryFingerImageEnhancement->setChecked(false);
        ui->chkDryFingerImageEnhancement->setEnabled(false);
    }

    ui->grpScanningInfo->setVisible(false);

    // context menu for segments
    ui->graphicsView->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(ui->graphicsView, SIGNAL(customContextMenuRequested(const QPoint&)),
        this, SLOT(PopUpSegmMenu(const QPoint&)));

    // signal mapper for segment menu items
    FindSegmentSignalMapper = new QSignalMapper(this);
    connect(FindSegmentSignalMapper, SIGNAL(mapped(int)), this, SLOT(OnSegmentSelected(int)));

    ViewSegmentSignalMapper = new QSignalMapper(this);
    connect(ViewSegmentSignalMapper, SIGNAL(mapped(int)), this, SLOT(OnViewSegment(int)));

    RemoveSegmentSignalMapper = new QSignalMapper(this);
    connect(RemoveSegmentSignalMapper, SIGNAL(mapped(int)), this, SLOT(OnRemoveSegment(int)));

    SegmBoxChangeSignalMapper = new QSignalMapper(this);
    connect(SegmBoxChangeSignalMapper, SIGNAL(mapped(int)), this, SLOT(OnSegmBoxChanged(int)));

	ListenerTimer = new QTimer();
	ListenerTimer->setInterval(50);
	connect(ListenerTimer, SIGNAL(timeout()), this, SLOT(OnListenerTimer()));

    // allocate memory for GBFINIMG
    SegmTempBuff = (unsigned char *)calloc(GBFINIMG_MEMORY_BUFFER_SIZE, sizeof(unsigned char));
    if (!SegmTempBuff)
    {
        // temporanea...
        QMessageBox::critical(this, tr("Error"), tr("Memory allocation error"), QMessageBox::Ok);
        return;
    }

    if (MSGUIRef->IsResolution1000Dpi())
    {
        // allocate buffer for rotation
        // get max size for object type on current scanner
		unsigned int MaxImageSizeX, MaxImageSizeY;
		GBMSAPI_GetMaxImageSize(&MaxImageSizeX, &MaxImageSizeY);
        unsigned int Size = MaxImageSizeX * MaxImageSizeY * GBFINIMG_ROTATION_BUFFER_FACTOR;
		RotationBuffer1000DPI = (unsigned char *)malloc(Size);
		if (RotationBuffer1000DPI == NULL)
		{
	        // temporanea...
		    QMessageBox::critical(this, tr("Error"), tr("Memory allocation error"), QMessageBox::Ok);
			return;
		}
        GBFINIMG_SetRotationBuffer1000DPI(RotationBuffer1000DPI, Size);
    }

}

AcquisitionForm::~AcquisitionForm()
{
    FreeSegmImgDescr();

    if (SegmTempBuff)
    {
        free (SegmTempBuff);
        SegmTempBuff = NULL;
    }

	if (RotationBuffer1000DPI)
	{
		free (RotationBuffer1000DPI);
		RotationBuffer1000DPI = NULL;
	}

    delete FindSegmentSignalMapper;
    delete ViewSegmentSignalMapper;
    delete RemoveSegmentSignalMapper;
    delete SegmBoxChangeSignalMapper;

    GrayTable.clear();
    delete ClipRectItem;
	delete ListenerTimer;

    delete ui;
}

// this function is STATIC
BOOL __stdcall AcquisitionForm::AcquisitionEventsManagerCallback(
        unsigned int OccurredEventCode,
        int GetFrameErrorCode,
        unsigned int EventInfo,
        unsigned char *FramePtr,
        int FrameSizeX,
        int FrameSizeY,
        double CurrentFrameRate,
        double NominalFrameRate,
        unsigned int GB_Diagnostic,
        void *UserDefinedParameters)
{
    AcquisitionForm *Form = (AcquisitionForm *)UserDefinedParameters;

	if (QThread::currentThread() == Form->thread())
	{
		// if the callback is in the same thread, call directly
		return Form->AcquisitionEventsManager(
			OccurredEventCode,
			GetFrameErrorCode,
			EventInfo,
			FramePtr,
			FrameSizeX,
			FrameSizeY,
			CurrentFrameRate,
			NominalFrameRate,
			GB_Diagnostic);
	}
	else
	{
	    // trick for getting the return value...
	    Form->SetAcquisitionEventsManagerRet(-1);

		// if the callback is called by a thread, use the emit
		// but the emit does not allow a return value...
		emit Form->OnAcquisitionEvent(
			OccurredEventCode,
			GetFrameErrorCode,
			EventInfo,
			FramePtr,
			FrameSizeX,
			FrameSizeY,
			CurrentFrameRate,
			NominalFrameRate,
			GB_Diagnostic);

		// get the return value
		return Form->GetAcquisitionEventsManagerRet();
	}

/*
    // trick for getting the return value...
    int ret;
    while ((ret = Form->GetAcquisitionEventsManagerRet()) == -1)
    {
        QApplication::processEvents();
#ifdef WIN32
        Sleep(1);
#else
        usleep(1000);
#endif
    }

    return ret;
*/
}

BOOL AcquisitionForm::AcquisitionEventsManager(
        unsigned int OccurredEventCode,
        int GetFrameErrorCode,
        unsigned int EventInfo,
        unsigned char *FramePtr,
        int FrameSizeX,
        int FrameSizeY,
        double CurrentFrameRate,
        double NominalFrameRate,
        unsigned int GB_Diagnostic)
{
    BOOL ret;

    switch (OccurredEventCode)
    {
        case GBMSAPI_AE_ACQUISITION_ERROR:
            ret = OnAcquisitionError(GetFrameErrorCode);
            break;

        case GBMSAPI_AE_SCANNER_STARTED:
            ret = OnScannerStarted();
            break;

        case GBMSAPI_AE_PREVIEW_PHASE_END:
            ret = OnPreviewPhaseEnd();
            break;

        case GBMSAPI_AE_VALID_FRAME_ACQUIRED:
			ret = OnValidFrame(EventInfo, FramePtr, FrameSizeX, FrameSizeY,
				CurrentFrameRate, NominalFrameRate, GB_Diagnostic, false);
            break;

        case GBMSAPI_AE_ACQUISITION_END:
            {
                bool AutomaticStop = false;
                bool Timeout = false;

                // check stop type in EventInfo
                if (CheckMask(EventInfo, GBMSAPI_EI_STOP_TYPE))
                    AutomaticStop = true;
                if (CheckMask(EventInfo, GBMSAPI_EI_ACQUISITION_TIMEOUT))
                    Timeout = true;

                if (!Timeout)
                {
                    // contains also a valid frame
                    if (FramePtr)
                        OnValidFrame(EventInfo, FramePtr, FrameSizeX, FrameSizeY,
                            CurrentFrameRate, NominalFrameRate, GB_Diagnostic, true);
                }

                ret = OnAcquisitionEnd(AutomaticStop, Timeout);
            }
            break;

        default:
            ret = 0;
            break;
    }

    AcquisitionEventsManagerRet = ret;
    return ret;
}

int AcquisitionForm::OnValidFrame(unsigned int EventInfo,
        unsigned char *FramePtr,
        int FrameSizeX,
        int FrameSizeY,
        double CurrentFrameRate,
        double NominalFrameRate,
        unsigned int GB_Diagnostic,
        bool FinalFrame)
{
    int ret = 0;
    int ClippingRegionPosX;
    int ClippingRegionPosY;
    unsigned int ClippingRegionSizeX;
    unsigned int ClippingRegionSizeY;
	bool PreviewFrame;

	// DEBUG
	//MSGUIRef->Beep(500, 10);

    // stop acquisition if form is closing for other reasons (skip, break, etc)
    if (ScanResult != -1)
        return 0;

	if (ui->lblScannedObject->text().isEmpty())
	{
		// show object name and picture
		QString ObjectName = GetScannedObjectName();
		ui->lblScannedObject->setText(ObjectName);
		ShowScannedObjectPictogram();
	}

    // save frame pointer and size
    this->FramePtr = FramePtr;
    this->FrameSizeX = FrameSizeX;
    this->FrameSizeY = FrameSizeY;

    // check if preview or acquisition phase
    if (CheckMask(EventInfo, GBMSAPI_EI_ACQUISITION_PHASE))
        PreviewFrame = false;
	else
		PreviewFrame = true;

    if (PreviewFrame)
    {
        // check if is full resolution image
        if (CheckMask(EventInfo, GBMSAPI_EI_PREVIEW_RES))
            FullResImage = true;
    }
    else
    {
        // final image is always full resolution
        FullResImage = true;
    }

    // show frame rate (if present)
    if (NominalFrameRate != 0)
        ShowFrameRate(CurrentFrameRate, NominalFrameRate);

    // save diagnostic for checking on stop
    LastDiagnostic = GB_Diagnostic;

    // clear masked diagnostic
    LastDiagnostic &= ~MSGUIRef->GetCfgData()->IgnoreDiagnosticMask;

	// first, check roll direction indication inside diagnostic mask
	if (CheckMask(LastDiagnostic, GBMSAPI_DM_VSROLL_ROLL_DIRECTION_LEFT))
		DisplayMessage(tr("Roll finger to the left"), false);
	else if (CheckMask(LastDiagnostic, GBMSAPI_DM_VSROLL_ROLL_DIRECTION_RIGHT))
		DisplayMessage(tr("Roll finger to the right"), false);
	else if (PreviewPhase)
		DisplayMessage(tr("Preview..."), false);

    // if BlockAutocapture option
    if (!GBMSGUI::IsRolled(mScannedObject->ScannedObjectID))
    {
        if (CheckMask(ScanOptions, GBMSAPI_AO_AUTOCAPTURE)
            && CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_BLOCK_AUTOCAPTURE)
            && MSGUIRef->IsBlockAutocaptureSupported()
            && !FinalFrame)
        {
            if (CheckMask(LastDiagnostic, MSGUIRef->GetCfgData()->BlockAutoCaptureMask) ||
                (MSGUIRef->GetCfgData()->BlockAutoCaptureContrast && (Contrast < mScannedObject->ContrastThreshold2)))
				BlockAutoCapture();
            else
				UnblockAutoCapture();
        }
    }

/*  moved in OnAcquisitionEnd
	if (FinalFrame && !GBMSGUI::IsRolled(mScannedObject->ScannedObjectID))
	{
        // 1.5.4.0 - restored check
        if (!CheckMask(LastDiagnostic, GBMSAPI_DM_SCANNER_SURFACE_NOT_NORMA)
            && !CheckMask(LastDiagnostic, GBMSAPI_DM_SCANNER_FAILURE))
        {
	        // finalize image
		    GBMSAPI_ImageFinalization(FramePtr);
		}
	}
*/

	if (!DontDrawFrame)	// DEBUG
	{
		// display image
		DisplayImage();
	}

    if (!GBMSGUI::IsRolled(mScannedObject->ScannedObjectID))
    {
        // show contrast
        GBMSAPI_GetFingerprintContrast(&Contrast);
        ui->progressContrast->SetValue(Contrast);
    }

    if (GBMSGUI::IsLowerPalm(mScannedObject->ScannedObjectID) && MSGUIRef->IsPalmCompletenessSupported())
    {
        // show palm completeness
        GBMSAPI_GetLowerHalfPalmCompleteness(&PalmCompleteness);
        ui->progressCompleteness->SetValue((int)PalmCompleteness);
    }

	// show clipping (for rolled, only at the end)
	if (!GBMSGUI::IsRolled(mScannedObject->ScannedObjectID) || FinalFrame)
	{
		ret = GBMSAPI_GetClippingRegionPosition(
					&ClippingRegionPosX,
					&ClippingRegionPosY,
					&ClippingRegionSizeX,
					&ClippingRegionSizeY);

		// in flat preview, clipping coordinates are relative to small image
		if (PreviewFrame)
		{
			ClippingRegionPosX *= PreviewScale;
			ClippingRegionPosY *= PreviewScale;
		}

		if ((ClippingRegionPosX >= 0) && (ClippingRegionPosY >=0)
			&& ((RealClipRect.width() != ImageSizeX) || (RealClipRect.height() != ImageSizeY)))
		{
			// do not touch RealClipRect size...
			RealClipRect.moveTopLeft(QPoint(ClippingRegionPosX, ClippingRegionPosY));
			ShowClipRectangle();
		}
		else
			ClipRectItem->setVisible(false);
	}

	// live segmentation
	if (GBMSGUI::IsSlap(mScannedObject->ScannedObjectID) &&
		!FinalFrame &&
		CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_SEGMENTATION) &&
		CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_SEGM_QUALITY_EVAL) &&
		CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_FULL_RES_PREVIEW)
		)
	{
		if (Contrast == 0)
		{
			ResetSegments();
			// clear diagnostic
			PrevPostProcessDiag = PostProcessDiag;
			PostProcessDiag = 0;
		}
		else
		{
			bool res = LiveSegmentation();
			if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_BLOCK_AUTOCAPTURE)
				&& MSGUIRef->IsBlockAutocaptureSupported())
			{
				if (!res)
					BlockAutoCapture();
				else
					UnblockAutoCapture();
			}
		}
	}

	// moved here because now displays both acquisition diag and post-processing diag (live segm)
    DisplayDiagnostic();

    return 1;
}

void AcquisitionForm::BlockAutoCapture()
{
    GBMSAPI_SetAutoCaptureBlocking(true);

    if (MSGUIRef->IsLedPresent())
        // notify also blinking led(s)
        GBMSAPI_VUI_LED_BlinkDuringAcquisition(1);

    // show message in list
    AddMessageToList(tr("Autocapture blocked"), QIcon(":/Icons/Warning.png"));
}

void AcquisitionForm::UnblockAutoCapture()
{
    GBMSAPI_SetAutoCaptureBlocking(false);
    if (MSGUIRef->IsLedPresent())
        // stop blinking led(s)
        GBMSAPI_VUI_LED_BlinkDuringAcquisition(0);

    // remove message
    RemoveMessageFromList(tr("Autocapture blocked"));
}

void AcquisitionForm::DisplayImage()
{
    // create image from FramePtr
    QImage ImgFrame(FrameSizeX, FrameSizeY, QImage::Format_Indexed8);
	unsigned char *pImage = FramePtr;
	for (int y=0; y<FrameSizeY; y++, pImage += FrameSizeX)
		memcpy(ImgFrame.scanLine(y), pImage, FrameSizeX);
    ImgFrame.setColorTable(GrayTable);
	QSize DrawSize(ImgFrame.width(), ImgFrame.height());

	if (DrawScaled)
	{
		DrawSize.scale(ui->graphicsView->width(), ui->graphicsView->height(), Qt::KeepAspectRatio);
		FrameItem.SetSize(DrawSize);
	}
	else
	{
		// scale image for view
		ImgFrame = ImgFrame.scaled(ui->graphicsView->width(), ui->graphicsView->height(), Qt::KeepAspectRatio);
	}

    FrameItem.SetImage(ImgFrame);

    /*
    QPixmap ScaledPixmap = QPixmap::fromImage(ImgScaled);

    if (MarkerFrame)
        DrawMarkerFrame(&ScaledPixmap);

    FrameItem.setPixmap(ScaledPixmap);  // setPixmap is toooooo slow!
    */

    // resize the scene, if necessary
	if (DrawScaled)
	{
		if ((scene.width() != DrawSize.width()) || (scene.height() != DrawSize.height()))
		{
			scene.setSceneRect(0, 0, DrawSize.width(), DrawSize.height());
			ui->graphicsView->fitInView(&FrameItem, Qt::KeepAspectRatio);
		}
	}
	else
	{
		if ((scene.width() != ImgFrame.width()) || (scene.height() != ImgFrame.height()))
		{
			scene.setSceneRect(0, 0, ImgFrame.width(), ImgFrame.height());
			ui->graphicsView->fitInView(&FrameItem, Qt::KeepAspectRatio);
		}
	}

    if (MarkerFrame && !CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_NO_ARTEFACTS_DISPLAY))
        DrawMarkerFrame();
}

void AcquisitionForm::DrawMarkerFrame()
{
    int i, x, y, xScaled, yScaled;
    QPainterPath path;

    double ScaleFactor = (double)scene.width()/(double)ImageSizeX;

    for (i = 0; i < (FrameSizeX * FrameSizeY); i++)
    {
        if (MarkerFrame[i] == 0)
            continue;

        // draw only pixels marked as artefact border
        if (CheckMask(MarkerFrame[i], GBMSAPI_ARTEFACT_REGION_MARKED_BORDER))
        {
            // get real coordinates
            y = i / (int)FrameSizeX;
            x = i % (int)FrameSizeX;

            xScaled = floor(x*ScaleFactor + 0.5);
            yScaled = floor(y*ScaleFactor + 0.5);

            // draw point
            path.addRect(xScaled, yScaled, 1, 1);
        }
    }

    FrameItem.SetMarkerPath(path);
}

int AcquisitionForm::OnScannerStarted()
{
	// turn on acquisition flag
    Acquiring = true;
    PreviewPhase = true;

	// 1.5.2.0 - moved in StartScanning
    //ui->btnAccept->setEnabled(false);
    //ui->btnRepeat->setEnabled(false);
	// 1.5.2.0 - re-enable buttons
	ui->btnSkip->setEnabled(true);
    if (MSGUIRef->GetAcquisitionMode() == GBMSGUI_AM_MULTIPLE_ACQUISITION)
		ui->btnBack->setEnabled(true);
	ui->btnBreak->setEnabled(true);

    ui->btnStop->setEnabled(true);
    ui->btnStop->setFocus();

    RealClipRect.setWidth(mScannedObject->ImageSizeX);
    RealClipRect.setHeight(mScannedObject->ImageSizeY);

    int ret;
    ret = GBMSAPI_SetClippingRegionSize(RealClipRect.width(), RealClipRect.height());
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        MSGUIRef->SetError(MSGUIRef->GetMSAPIErrorMessage("GBMSAPI_SetClippingRegionSize", ret), ret);
        ScanResult = GBMSGUI_RET_FAILURE;
        Acquiring = false;
        return 0;
    }

    return 1;
}

int AcquisitionForm::OnPreviewPhaseEnd()
{
	PreviewPhase = false;

    // for rolled, roll direction indication is shown
    if (!GBMSGUI::IsRolled(mScannedObject->ScannedObjectID))
    {
        DisplayMessage(tr("Acquiring and transferring image..."), false);
        QApplication::processEvents();
    }

    if (GBMSGUI::IsRolled(mScannedObject->ScannedObjectID) &&
        !CheckMask(ScanOptions, GBMSAPI_AO_NO_ROLL_PREVIEW))
    {
        // signal to start rolling
        if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_SOUND_ON_ROLL_PREVIEW_END))
        {
            if (MSGUIRef->IsSoundPresent())
                GBMSAPI_Sound(12, 1, 1);
            else
                MSGUIRef->Beep(500, 200);
        }
    }

    return 1;
}

int AcquisitionForm::OnAcquisitionEnd(bool AutomaticStop, bool Timeout)
{
    // turn off acquisition flag
    Acquiring = false;

	ToDoFinalization = false;	// 1.5.6.0

    // if skip or break pressed, return immediately
    if (ScanResult != -1)
        return 0;

    DisplayMessage("", false);
    ui->btnStop->setEnabled(false);

    if (MSGUIRef->IsLCDPresent())
        // clear message on LCD
        GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_AND_DIAG_EMPTY, GBMSAPI_VILCD_MSG_AREA);

    // check timeout
    if (Timeout)
    {
        DisplayMessage(tr("Acquisition timeout"), true);
        if (MSGUIRef->IsLCDPresent())
            GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_ACQUISITION_TIME_OUT, GBMSAPI_VILCD_MSG_AREA);
        ui->btnRepeat->setEnabled(true);
        ui->btnRepeat->setFocus();

        // start listen pedal and LCD for Accept or Repeat actions
        if (MSGUIRef->IsPedalPresent())
            StartListenPedal(PedalPressing);

        if (MSGUIRef->IsLCDPresent())
            ListenerTimer->start();

        return 1;
    }

    // allow to move clipping rectangle
    if (ClipRectItem->isVisible())
        ClipRectItem->SetEnabled(true);

    // check stop type
    if (CheckMask(ScanOptions, GBMSAPI_AO_AUTOCAPTURE) && !AutomaticStop)
    {
        // stopped manually before autocapture
        AddMessageToList(tr("Acquisition stopped before autocapture"), QIcon(":/Icons/Warning.png"));
    }

    // if roll is stopped manually, error
    if (GBMSGUI::IsRolled(mScannedObject->ScannedObjectID) && !AutomaticStop)
    {
        DisplayMessage(tr("Rotation not completed"), true);
        if (MSGUIRef->IsLCDPresent())
            GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_ROTATION_NO_COMPLETED, GBMSAPI_VILCD_MSG_AREA);
        ui->btnRepeat->setEnabled(true);
        ui->btnRepeat->setFocus();

        // start listen pedal and LCD for Accept or Repeat actions
        if (MSGUIRef->IsPedalPresent())
            StartListenPedal(PedalPressing);

        if (MSGUIRef->IsLCDPresent())
            ListenerTimer->start();

        return 1;
    }

    // Contrast = 0 means no object detected
    if ((Contrast == 0) && !GBMSGUI::IsRolled(mScannedObject->ScannedObjectID))
    {
        ResetSegments();

        DisplayMessage(tr("No object detected"), true);
        if (MSGUIRef->IsLCDPresent())
            GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_NO_FINGERPRINT_DETECTED, GBMSAPI_VILCD_MSG_AREA);
        ui->btnRepeat->setEnabled(true);
        ui->btnRepeat->setFocus();

        // start listen pedal and LCD for Accept or Repeat actions
        if (MSGUIRef->IsPedalPresent())
            StartListenPedal(PedalPressing);

        if (MSGUIRef->IsLCDPresent())
            ListenerTimer->start();

        return 1;
    }

    if (!GBMSGUI::IsRolled(mScannedObject->ScannedObjectID))
    {
        // notify acquisition process end (not for rolled, too many sounds...)
        if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_SOUND))
        {
            if (MSGUIRef->IsSoundPresent())
                GBMSAPI_Sound(50, 2, 1);
            else
                MSGUIRef->Beep(500, 200);
        }
    }

    // not yet supported in linux
#ifdef WIN32
    // show contrast on LCD only for objects without any other information
    // if Quality option is active, quality is shown instead of contrast
    if (GBMSGUI::IsPalm(mScannedObject->ScannedObjectID) && MSGUIRef->IsLCDPresent()
        && !CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_PALM_PRINT_QUALITY_CALCULATION))
    {
        // 1.5.5.0 - except if dry/wet warning is selected
		if (!(MSGUIRef->IsDryFingerImageEnhancementSupported() &&
			(MSGUIRef->GetFingerContactEvalMode() == GBMSGUI_FINGER_CONTACT_EVAL_MODE_DRY_WET_WARNING)))
		{
			GBMSAPI_VUI_LCD_SetContrastLimitOnStopScreen(mScannedObject->ContrastThreshold2);
			if (MSGUIRef->IsLCDMidLimitSupported())
				GBMSAPI_VUI_LCD_SetContrastIntermediateLimitOnStopScreen(mScannedObject->ContrastThreshold1);
			GBMSAPI_VUI_LCD_SetContrastValueOnStopScreen(Contrast);
		}
    }
#endif

    if (GBMSGUI::IsRolled(mScannedObject->ScannedObjectID))
        GetRolledInfo();

    if (!GBMSGUI::IsRolled(mScannedObject->ScannedObjectID))
	{
        // 1.5.4.0 - restored check
        if (!CheckMask(LastDiagnostic, GBMSAPI_DM_SCANNER_SURFACE_NOT_NORMA)
            && !CheckMask(LastDiagnostic, GBMSAPI_DM_SCANNER_FAILURE))
        {
			// 1.5.6.0 - set flag, image will be finalized after ImageProcessing
			ToDoFinalization = true;
	        // finalize image
		    //GBMSAPI_ImageFinalization(FramePtr);
		}
	}

    // call segmentation for slaps, ProcessImage for others (except Upper palm)
    if (GBMSGUI::IsSlap(mScannedObject->ScannedObjectID))
    {
        if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_SEGMENTATION))
        {
			// Slap4 now has automatic mode (GBFINIMG 3100)
            if (CheckUnavailableFingers())
			{
				// if some fingers are marked unavailable, switch to manual mode
                AddMessageToList(tr("Some fingers are marked as unavailable"), QIcon(":/Icons/Warning.png"));
			}

            // perform segmentation
			ResetSegments();    // NEEDED when LiveSegmentation is active... to clear old data
            SegmentationResult = DoSegmentation(false, false);

            // center clipping rectangle with segment bounding boxes
            if (ClipRectItem->isVisible() && SegmentationResult)
                CenterClipWithBoxes();
        }
    }
    else if (!GBMSGUI::IsUpperPalm(mScannedObject->ScannedObjectID))
    {
        // call GBFINIMG only if needed
        if ((MSGUIRef->GetCfgData()->QualityAlgorithm == GBMSGUI_GB_ALGORITHM) ||
            CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_DETECT_INCLINATION) ||
            CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_DETECT_INCOMPLETE_PATTERN) ||
            CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_DETECT_INVALID_PATTERN) ||
            CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_REMOVE_HALO_LATENT) ||
            CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_ROTATE_FINGER))
        {
            // call ProcessImage
            ProcessImageResult = ProcessImage();
        }
        else
            // set to true for CheckResult
            ProcessImageResult = true;
    }

	// 1.5.6.0 - call finalization after ProcessImage
	if (ToDoFinalization)
	{
        // finalize image
	    GBMSAPI_ImageFinalization(FramePtr);
		DisplayImage();
	}

    // clear message
    DisplayMessage("", false);
    if (MSGUIRef->IsLCDPresent())
        GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_AND_DIAG_EMPTY, GBMSAPI_VILCD_MSG_AREA);

    ui->btnRepeat->setEnabled(true);

    if (PostProcessDiag != 0)
        DisplayDiagnostic();

    // determine the result
    bool Result = CheckResult();

    // check Autoaccept
    if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_AUTOACCEPT))
    {
        if (Result == true)
        {
            on_btnAccept_clicked();
            return 1;
        }
        else
        {
            // show message
            DisplayMessage(tr("Autoaccept conditions not satisfied"), true);
            if (MSGUIRef->IsLCDPresent())
                GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_AUTOACEPT_CONDITION_NO_SATISFIED,
                    GBMSAPI_VILCD_MSG_AREA);
        }
    }

    // start listen pedal and LCD for Accept or Repeat actions
    if (MSGUIRef->IsPedalPresent())
        StartListenPedal(PedalPressing);

    if (MSGUIRef->IsLCDPresent())
        ListenerTimer->start();

    return 1;
}

bool AcquisitionForm::CheckResult()
{
    bool Result = true;

    // clear message
    DisplayMessage("", false);

    // diagnostic
    if (LastDiagnostic != 0)
        Result = false;

    if (PostProcessDiag != 0)
        Result = false;

    // for single finger (flat or rolled), get quality
    if (GBMSGUI::IsSingleFinger(mScannedObject->ScannedObjectID))
    {
        mScannedObject->NFIQQuality = GetNFIQQuality(FramePtr, FrameSizeX, FrameSizeY);
    }

    if (GBMSGUI::IsSlap(mScannedObject->ScannedObjectID))
    {
        // for slaps, if segmentation option, check segments
        if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_SEGMENTATION))
        {
            if (SegmentationResult)
            {
                // for LCD
                unsigned char SegmResults[4] =
                {
                    GBMSAPI_VILCD_SEG_NOT_OK,
                    GBMSAPI_VILCD_SEG_NOT_OK,
                    GBMSAPI_VILCD_SEG_NOT_OK,
                    GBMSAPI_VILCD_SEG_NOT_OK
                };
                // for Leds
                unsigned int IndicatorsRed = 0;
                unsigned int IndicatorsGreen = 0;
                bool SegmOk;

                // check values on segments
                for (int i = 0; i < SegmData.count(); i++)
                {
                    if (SegmData[i].Found)
                    {
                        SegmOk = true;

                        SegmResults[i] = GBMSAPI_VILCD_SEG_OK;

                        // quality
                        if (!EvalSegmentIAFISQuality(i))
                        {
                            SegmOk = false;
                            SegmResults[i] = GBMSAPI_VILCD_SEG_NOT_OK;
                        }

                        // pattern validity
                        if (SegmData[i].InvalidPatternDetected)
                        {
                            SegmOk = false;
                            SegmResults[i] = GBMSAPI_VILCD_SEG_NOT_OK;
                        }

                        // pattern completeness
                        if (SegmData[i].IncompletePatternDetected)
                        {
                            SegmOk = false;
                            SegmResults[i] = GBMSAPI_VILCD_SEG_NOT_OK;
                        }

                        // inclination
                        if (SegmData[i].InclinationDetected)
                        {
                            SegmOk = false;
                            SegmResults[i] = GBMSAPI_VILCD_SEG_NOT_OK;
                        }

                        // set green and red masks for led indicators
                        if (SegmOk)
                            IndicatorsGreen |= SetIndicatorMask(GBSegmId2ObjectID(SegmData[i].Finger, SegmData[i].Hand));
                        else
                            IndicatorsRed |= SetIndicatorMask(GBSegmId2ObjectID(SegmData[i].Finger, SegmData[i].Hand));

                        if (!SegmOk)
                            Result = false;
                    }
                    else if (SegmData[i].UnavailabilityReason != -1)
                        // show as unavailable
                        SegmResults[i] = GBMSAPI_VILCD_SEG_UNAVAILABLE;
                }

                if (MSGUIRef->IsLedPresent())
                {
                    // turn on leds
                    GBMSAPI_SetFingerIndicatorVUIState(IndicatorsGreen, GBMSAPI_VIL_COLOR_GREEN, false);
                    GBMSAPI_SetFingerIndicatorVUIState(IndicatorsRed, GBMSAPI_VIL_COLOR_RED, false);
                }

                if (MSGUIRef->IsLCDPresent())
                {
                    // show segmentation result on LCD
                    if (GBMSAPI_GetTypeFromObject(mScannedObject->ScannedObjectID) == GBMSAPI_SBT_FLAT_SLAP_4)
                        GBMSAPI_VUI_LCD_SetSegmentationOnStopScreen(
                            SegmResults[0], SegmResults[1], SegmResults[2], SegmResults[3]);
                    else
                        GBMSAPI_VUI_LCD_SetSegmentationOnStopScreen(GBMSAPI_VILCD_SEG_EMPTY,
                            SegmResults[0], SegmResults[1],
                            GBMSAPI_VILCD_SEG_EMPTY);
                }

                // verify position of segments bounding boxes
                if (!CheckSegmentBoundingBoxPositions())
                    Result = false;

                // verify overlap of segments bounding boxes
                if (!CheckSegmentBoundingBoxOverlap())
                    Result = false;
            }
            else // if (SegmentationResult)
            {
                DisplayMessage(tr("Segmentation failed"), true);
                // inform about manual mode
                AddMessageToList(tr("Right-click on image for manually locating segments"), QIcon(":/Icons/Info.png"));

                if (MSGUIRef->IsLCDPresent())
                {
                    GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_SEGMENTATION_FAILED,
                        GBMSAPI_VILCD_MSG_AREA);
                    GBMSAPI_VUI_LCD_SetSegmentationOnStopScreen(
                        GBMSAPI_VILCD_SEG_EMPTY,
                        GBMSAPI_VILCD_SEG_EMPTY,
                        GBMSAPI_VILCD_SEG_EMPTY,
                        GBMSAPI_VILCD_SEG_EMPTY);
                }

                ui->btnRepeat->setFocus();
                return false;
            }
        }
    }
    else    // not Slap
    {
        // for single finger (flat or rolled), get quality and minutiae
        if (GBMSGUI::IsSingleFinger(mScannedObject->ScannedObjectID) ||
            (GBMSGUI::IsPalm(mScannedObject->ScannedObjectID) && CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_PALM_PRINT_QUALITY_CALCULATION)))
        {
            if (!EvalIAIFSQuality())
                Result = false;
        }

        // show artifacts for rolled
        if (GBMSGUI::IsRolled(mScannedObject->ScannedObjectID))
            if (!EvalRolledArtefactsSize())
                Result = false;

        // sequence check for rolled
        if (GBMSGUI::IsRolled(mScannedObject->ScannedObjectID) &&
            CheckMask(MSGUIRef->GetSessionOptions(), GBMSGUI_SESSION_OPTION_SEQUENCE_CHECK))
        {
            if (!CheckSequence())
                Result = false;
        }

        if (!GBMSGUI::IsUpperPalm(mScannedObject->ScannedObjectID))
        {
            if (ProcessImageResult)
            {
                // Completeness
                if (GBMSGUI::IsLowerPalm(mScannedObject->ScannedObjectID))
                {
                    if (MSGUIRef->IsPalmCompletenessSupported())
                    {
                        // not yet supported in linux
#ifdef WIN32
                        if (MSGUIRef->IsLCDPresent())
                        {
                            GBMSAPI_VUI_LCD_SetCompletenessLimitOnStopScreen(MSGUIRef->GetCfgData()->LowerPalmCompletenessThreshold2);
                            if (MSGUIRef->IsLCDMidLimitSupported())
                                GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen(MSGUIRef->GetCfgData()->LowerPalmCompletenessThreshold1);
                            GBMSAPI_VUI_LCD_SetCompletenessValueOnStopScreen(PalmCompleteness);
                        }
#endif
                    }
                }
                else
                {
                    if (IncompletePatternDetected)
                        Result = false;
                }

                // Validity
                if (InvalidPatternDetected)
                    Result = false;

                // Inclination
                if (InclinationDetected)
                    Result = false;
            }
            else
            {
                Result = false;
            }
        }

        if (MSGUIRef->IsLedPresent())
        {
            // use led to show result
            int Color;
            if (Result)
                Color = GBMSAPI_VIL_COLOR_GREEN;
            else
                Color = GBMSAPI_VIL_COLOR_RED;
            GBMSAPI_SetFingerIndicatorVUIState(SetIndicatorMask(mScannedObject->ScannedObjectID), Color, false);
        }
    }

    // update user interface according to result
    if (Result)
    {
        // if all Ok, set focus on accept
        ui->btnAccept->setEnabled(true);
        ui->btnAccept->setFocus();

        DisplayMessage(tr("Acquisition successful"), false);
        if (MSGUIRef->IsLCDPresent())
            GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_ACQUISITION_SUCCESSFUL,
                GBMSAPI_VILCD_MSG_AREA);

        if (MSGUIRef->IsLCDPresent())
            // enable OK button on LCD
            GBMSAPI_VUI_LCD_EnableOKButtonOnStopScreen();
    }
    else
    {
        // enable accept anyway, but set focus on repeat
        ui->btnAccept->setEnabled(true);
        ui->btnRepeat->setFocus();

        // if not yet a message present
        if (ui->lblMessage->text() == "")
        {
            // tell the operator he should evaluate the result
            DisplayMessage(tr("Please, evaluate the result"), false);
            if (MSGUIRef->IsLCDPresent())
                GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_EVALUATE_THE_RESULT,
                    GBMSAPI_VILCD_MSG_AREA);
        }

        if (MSGUIRef->IsLCDPresent())
            // enable OK button on LCD
            GBMSAPI_VUI_LCD_EnableOKButtonOnStopScreen();
    }

    return Result;
}

int AcquisitionForm::OnAcquisitionError(int GetFrameErrorCode)
{
    Acquiring = false;

    MSGUIRef->SetError(MSGUIRef->GetMSAPIErrorMessage("AcquisitionEventsManager", GetFrameErrorCode), GetFrameErrorCode);
    ScanResult = GBMSGUI_RET_FAILURE;
    return 0;
}

int AcquisitionForm::Acquire()
{
    ScanResult = -1;
    mForceClose = false;

	// hide all
	ui->panDryWet->setVisible(false);
	ui->panScanningCompleteness->setVisible(false);
	ui->panArtefacts->setVisible(false);
	ui->panIAFISQuality->setVisible(false);

    // show object name and picture - moved when first frame arrives
    //QString ObjectName = GetScannedObjectName();
    //ui->lblScannedObject->setText(ObjectName);
    //ShowScannedObjectPictogram();

    // contrast progressbar
    ui->progressContrast->SetMaxValue(255);
    ui->progressContrast->SetThresholds(mScannedObject->ContrastThreshold1, mScannedObject->ContrastThreshold2);

    // show/hide specific UI parts according to object type
    if (GBMSGUI::IsSingleFinger(mScannedObject->ScannedObjectID) ||
        (GBMSGUI::IsPalm(mScannedObject->ScannedObjectID) &&
        CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_PALM_PRINT_QUALITY_CALCULATION)))
    {
        // IAFIS quality
        ui->panIAFISQuality->setVisible(true);
        // direction depends on algorythm
        if (MSGUIRef->GetCfgData()->QualityAlgorithm == GBMSGUI_NFIQ_ALGORITHM)
            SetQualityThresholds(mScannedObject->QualityThreshold1,
                mScannedObject->QualityThreshold2, THRESHOLD_SMALLER_THAN);
        else
            SetQualityThresholds(mScannedObject->QualityThreshold1,
                mScannedObject->QualityThreshold2, THRESHOLD_GREATHER_THAN);
    }
    else
        ui->panIAFISQuality->setVisible(false);

    if (GBMSGUI::IsRolled(mScannedObject->ScannedObjectID))
    {
        ui->panArtefacts->setVisible(true);
        SetArtefactsThresholds(MSGUIRef->GetCfgData()->ArtefactsThreshold1,
                               MSGUIRef->GetCfgData()->ArtefactsThreshold2,
                               THRESHOLD_SMALLER_THAN);
    }
    else
        ui->panArtefacts->setVisible(false);

    if (GBMSGUI::IsLowerPalm(mScannedObject->ScannedObjectID) && MSGUIRef->IsPalmCompletenessSupported())
    {
        // palm completeness
        ui->panScanningCompleteness->setVisible(true);
        ui->progressCompleteness->SetMaxValue(100);
        ui->progressCompleteness->SetThresholds(MSGUIRef->GetCfgData()->LowerPalmCompletenessThreshold1,
                                                MSGUIRef->GetCfgData()->LowerPalmCompletenessThreshold2);
        ui->panIAFISQuality->move(ui->panIAFISQuality->geometry().x(), ui->panScanningCompleteness->geometry().bottom());

        // set completeness threshold for low level
        GBMSAPI_SetLowerHalfPalmCompletenessThreshold(MSGUIRef->GetCfgData()->LowerPalmCompletenessThreshold2);

        // not yet supported in linux
#ifdef WIN32
        if (MSGUIRef->IsLCDPresent() && MSGUIRef->IsLCDMidLimitSupported())
            GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimit(MSGUIRef->GetCfgData()->LowerPalmCompletenessThreshold1);
#endif
    }
    else
    {
        ui->panScanningCompleteness->setVisible(false);
    }

	// 1.5.2.0 - moved in OnScannerStarted
    //ui->btnSkip->setEnabled(true);
    //ui->btnBreak->setEnabled(true);
    //if (MSGUIRef->GetAcquisitionMode() == GBMSGUI_AM_MULTIPLE_ACQUISITION)
    //    ui->btnBack->setEnabled(true);

    // if slap, set segments
    if (GBMSGUI::IsSlap(mScannedObject->ScannedObjectID) &&
            CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_SEGMENTATION))
    {
        InitSegmImgDescr();
        SetSegments();
    }

    SetPreviewScale();

    // enable rotation option oly for lower palm
	if (GBMSGUI::IsLowerPalm(mScannedObject->ScannedObjectID) && MSGUIRef->IsImageRotationSupported())
    {
        ui->chkRotate180->setEnabled(true);
    }
    else
    {
        ui->chkRotate180->setChecked(false);
        ui->chkRotate180->setEnabled(false);
    }

    // 1.5.4.0 - show Dry/wet diagnostic instead of contrast, if supported
    // and not setting changed
    if (MSGUIRef->IsDryFingerImageEnhancementSupported() && 
        (MSGUIRef->GetFingerContactEvalMode() == GBMSGUI_FINGER_CONTACT_EVAL_MODE_DRY_WET_WARNING))
    {
		ui->panDryWet->setVisible(true);
        ui->panContrast->setVisible(false);
    }
    else
    {
		ui->panDryWet->setVisible(false);
        ui->panContrast->setVisible(true);
    }

    StartScanning();

    return 0;
}

void AcquisitionForm::StartScanning()
{
    int ret;
    unsigned int DisplayOptions = 0;

    // reset scanned object
    mScannedObject->Acquired = false;
    mScannedObject->Completeness = 0;
    mScannedObject->GBQuality = 0;
    mScannedObject->NFIQQuality = 0;
    mScannedObject->RolledArtefactSize = 0;

    FramePtr = NULL;
    FrameSizeX = 0;
    FrameSizeY = 0;
    MarkerFrame = NULL;
    FullResImage = false;
    Contrast = 0;
    LastDiagnostic = 0;
	PrevDiagnostic = 0;
    PostProcessDiag = 0;
	PrevPostProcessDiag = 0;
    SegmentationResult = false;
    ProcessImageResult = false;
    IncompletePatternDetected = false;
    InvalidPatternDetected = false;
    InclinationDetected = false;

    // clear image
    //FrameItem.setPixmap(NULL);
    FrameItem.ClearImage();

	// 1.5.2.0 - disable all buttons
    ui->btnAccept->setEnabled(false);
    ui->btnRepeat->setEnabled(false);
	ui->btnSkip->setEnabled(false);
	ui->btnBack->setEnabled(false);
	ui->btnBreak->setEnabled(false);

    // 1.5.2.0 - hide object name and picture (will be shown when first frame arrives)
    ui->lblScannedObject->setText("");
    ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Hands_empty.bmp"));

    ResetSegments();
    ui->progressContrast->SetValue(0);
    CleanDiagnostic();
    ShowFrameRate(0, 0);
    ResetIAFISQuality();
    ResetArtefactsSize();
    ui->progressCompleteness->SetValue(0);

    ClipRectItem->SetEnabled(false);
    ClipRectItem->setVisible(false);
    ui->btnHelpDiag->setEnabled(false);
    ui->frmDryWet->setVisible(false);

    if (MSGUIRef->IsPedalPresent())
    {
        // check that pedal is not pressed before starting acquisition
        StartListenPedal(ReleasePedal);
        while (PedalInfo.PedalPressed)
        {
	        QApplication::processEvents();
#ifdef WIN32
			Sleep(1);
#else
			usleep(1000);
#endif
        }
        // disable timer before starting acquisition
        ListenerTimer->stop();
    }

    // if option active, rotate 180 degrees
	if (ui->chkRotate180->isEnabled())	// 1.5.2.0
	{
		if (ui->chkRotate180->isChecked())
			GBMSAPI_SetImageRotation(GBMSAPI_IMG_ROTATE_180);
		else
			GBMSAPI_SetImageRotation(GBMSAPI_IMG_ROTATE_NO);
	}


    // show start message
    DisplayMessage(tr("Diagnostic in progress, don't put finger..."), false);
    QApplication::processEvents();

    // set current options
    SetScanningOptions();

    // set to go to Stop screen after acquisition
    DisplayOptions |= GBMSAPI_DO_FINAL_SCREEN;

    // not yet supported in linux
#ifdef WIN32
    if (MSGUIRef->IsLCDPresent() && MSGUIRef->IsLCDMidLimitSupported())
        GBMSAPI_VUI_LCD_SetContrastIntermediateLimit(mScannedObject->ContrastThreshold1);
#endif

    ret = GBMSAPI_StartAcquisition(
        mScannedObject->ScannedObjectID,
        ScanOptions,
        AcquisitionEventsManagerCallback,
        (void *)this,
        DisplayOptions,
        mScannedObject->ContrastThreshold2,
        MSGUIRef->GetCfgData()->LowerPalmCompletenessThreshold2
        );

    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        MSGUIRef->SetError(MSGUIRef->GetMSAPIErrorMessage("GBMSAPI_StartAcquisition", ret), ret);
        ScanResult = GBMSGUI_RET_FAILURE;
        return;
    }

    // call afterstart callback, if defined
    GBMSGUI_AfterStartFuncPtr AfterStartCallback = MSGUIRef->GetAfterStartCallback();
    if (AfterStartCallback != NULL)
    {
        if (!AfterStartCallback(MSGUIRef->GetAfterStartCallbackParam()))
        {
            MSGUIRef->SetError(tr("AfterStartCallback Failed"), GBMSGUI_ERR_AFTERSTARTCALLBACK_FAILED);
            ScanResult = GBMSGUI_RET_FAILURE;
            Acquiring = false;
        }
    }

}

void AcquisitionForm::StopScanning()
{
    int ret;
    ret = GBMSAPI_StopAcquisition();
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        MSGUIRef->SetError(MSGUIRef->GetMSAPIErrorMessage("GBMSAPI_StopAcquisition", ret), ret);
        ScanResult = GBMSGUI_RET_FAILURE;
    }
}


void AcquisitionForm::on_btnAccept_clicked()
{
    // for slap, check segments presence
    if (GBMSGUI::IsSlap(mScannedObject->ScannedObjectID) &&
        CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_SEGMENTATION))
    {
        if (CheckMask(MSGUIRef->GetSessionOptions(), GBMSGUI_SESSION_OPTION_ASK_UNAVAILABILITY_REASON))
            if (!AreSegmentsAllFound())
                if (!AskUnavailabilityReason())
                    return;

        // ensure bounding boxes coordinates are inside the image
        CutBoundingBoxPositions();
    }

    // notify accept with sound
    if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_SOUND))
    {
        MSGUIRef->Beep(500, 200);
    }

    // show a message, while application saves image (can take long time for big images)
    DisplayMessage(tr("Saving image..."), false);
    QApplication::processEvents();
    if (MSGUIRef->IsLCDPresent())
        GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_SAVING_IMAGE,
            GBMSAPI_VILCD_MSG_AREA);

    ScanResult = GBMSGUI_RET_SUCCESS;
}

bool AcquisitionForm::Finished()
{
    if (ScanResult != -1)
        return true;
    else
        return false;
}

void AcquisitionForm::closeEvent(QCloseEvent *event)
{
    if (mForceClose)
        event->accept();
    else
    {
        event->ignore();
        // translate closing by "X" into Break
        on_btnBreak_clicked();
    }
}

void AcquisitionForm::ShowClipRectangle()
{
    QRect ClipRect;

    // calculate scale factor
    double ScaleFactor = (double)scene.width()/(double)ImageSizeX;

    ClipRect.setLeft(floor((double)RealClipRect.left() *ScaleFactor + 0.5));
    ClipRect.setTop(floor((double)RealClipRect.top()*ScaleFactor + 0.5));
    ClipRect.setWidth(floor((double)RealClipRect.width()*ScaleFactor + 0.5));
    ClipRect.setHeight(floor((double)RealClipRect.height()*ScaleFactor + 0.5));

    ClipRectItem->setRect(0, 0, ClipRect.width(), ClipRect.height());
    ClipRectItem->setPos(ClipRect.x(), ClipRect.y());
    ClipRectItem->setVisible(true);
}

void AcquisitionForm::resizeEvent(QResizeEvent *event)
{
    if (FramePtr)
    {
        DisplayImage();

        if (ClipRectItem->isVisible())
            // update posiition
            ShowClipRectangle();

        //if (GBMSGUI::IsSlap(mScannedObject->ScannedObjectID) &&
        //        CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_SEGMENTATION))
            ShowBoundingBoxes();
    }

    QWidget::resizeEvent(event);
}

void AcquisitionForm::showEvent(QShowEvent * event)
{
    QWidget::showEvent(event);
}


void AcquisitionForm::on_btnStop_clicked()
{
    int ret;

    if (GBMSGUI::IsRolled(mScannedObject->ScannedObjectID) &&
        PreviewPhase &&
        CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_ROLL_PREVIEW_MANUAL_STOP))
    {
        ret = GBMSAPI_ROLL_StopPreview();
        if (ret == GBMSAPI_ERROR_CODE_NO_ERROR)
        {
            return;
        }
        // ignore GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED error
        if (ret != GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED)
        {
            MSGUIRef->SetError(MSGUIRef->GetMSAPIErrorMessage("GBMSAPI_ROLL_StopPreview", ret), ret);
            ScanResult = GBMSGUI_RET_FAILURE;
            return;
        }
    }

    StopScanning();
}

void AcquisitionForm::on_btnRepeat_clicked()
{
    // inform about action
    DisplayMessage(tr("Repeating..."), false);
    QApplication::processEvents();

    if (MSGUIRef->IsLCDPresent())
        GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_REPEATING,
            GBMSAPI_VILCD_MSG_AREA);

    if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_SOUND))
    {
        MSGUIRef->Beep(650, 200);
    }

    // restart acquisition
    StartScanning();
}

void AcquisitionForm::GetAcquiredImageBuffer(unsigned char *ImageBuff, int *ImgWidth, int *ImgHeight)
{
    if (ClipRectItem->isVisible())
    {
        *ImgWidth = RealClipRect.width();
        *ImgHeight = RealClipRect.height();
        CutImage(FramePtr, FrameSizeX, FrameSizeY,
            RealClipRect.left(), RealClipRect.top(),
            ImageBuff, RealClipRect.width(), RealClipRect.height());
    }
    else
    {
        *ImgWidth = FrameSizeX;
        *ImgHeight = FrameSizeY;
        memcpy(ImageBuff, FramePtr, FrameSizeX*FrameSizeY);
    }
}

// returns name of the currently scanned object
QString AcquisitionForm::GetScannedObjectName()
{
    QString ObjectName;

    switch (mScannedObject->ScannedObjectID)
    {
        case GBMSAPI_SO_FLAT_RIGHT_THUMB:
            ObjectName = tr("Flat Right Thumb");
            break;
        case GBMSAPI_SO_FLAT_RIGHT_INDEX:
            ObjectName = tr("Flat Right Index");
            break;
        case GBMSAPI_SO_FLAT_RIGHT_MIDDLE:
            ObjectName = tr("Flat Right Middle");
            break;
        case GBMSAPI_SO_FLAT_RIGHT_RING:
            ObjectName = tr("Flat Right Ring");
            break;
        case GBMSAPI_SO_FLAT_RIGHT_LITTLE:
            ObjectName = tr("Flat Right Little");
            break;
        case GBMSAPI_SO_FLAT_LEFT_THUMB:
            ObjectName = tr("Flat Left Thumb");
            break;
        case GBMSAPI_SO_FLAT_LEFT_INDEX:
            ObjectName = tr("Flat Left Index");
            break;
        case GBMSAPI_SO_FLAT_LEFT_MIDDLE:
            ObjectName = tr("Flat Left Middle");
            break;
        case GBMSAPI_SO_FLAT_LEFT_RING:
            ObjectName = tr("Flat Left Ring");
            break;
        case GBMSAPI_SO_FLAT_LEFT_LITTLE:
            ObjectName = tr("Flat Left Little");
            break;
        case GBMSAPI_SO_ROLL_RIGHT_THUMB:
            ObjectName = tr("Rolled Right Thumb");
            break;
        case GBMSAPI_SO_ROLL_RIGHT_INDEX:
            ObjectName = tr("Rolled Right Index");
            break;
        case GBMSAPI_SO_ROLL_RIGHT_MIDDLE:
            ObjectName = tr("Rolled Right Middle");
            break;
        case GBMSAPI_SO_ROLL_RIGHT_RING:
        ObjectName = tr("Rolled Right Ring");
            break;
        case GBMSAPI_SO_ROLL_RIGHT_LITTLE:
            ObjectName = tr("Rolled Right Little");
            break;
        case GBMSAPI_SO_ROLL_LEFT_THUMB:
            ObjectName = tr("Rolled Left Thumb");
            break;
        case GBMSAPI_SO_ROLL_LEFT_INDEX:
            ObjectName = tr("Rolled Left Index");
            break;
        case GBMSAPI_SO_ROLL_LEFT_MIDDLE:
            ObjectName = tr("Rolled Left Middle");
            break;
        case GBMSAPI_SO_ROLL_LEFT_RING:
            ObjectName = tr("Rolled Left Ring");
            break;
        case GBMSAPI_SO_ROLL_LEFT_LITTLE:
            ObjectName = tr("Rolled Left Little");
            break;
        case GBMSAPI_SO_SLAP_2_THUMBS:
            ObjectName = tr("Flat Two Thumbs");
            break;
        case GBMSAPI_SO_SLAP_4_LEFT:
            ObjectName = tr("Flat Left Four Fingers");
            break;
        case GBMSAPI_SO_SLAP_4_RIGHT:
            ObjectName = tr("Flat Right Four Fingers");
            break;
        case GBMSAPI_SO_UPPER_HALF_PALM_LEFT:
            ObjectName = tr("Left Upper Half Palm");
            break;
        case GBMSAPI_SO_UPPER_HALF_PALM_RIGHT:
            ObjectName = tr("Right Upper Half Palm");
            break;
        case GBMSAPI_SO_LOWER_HALF_PALM_LEFT:
            ObjectName = tr("Left Lower Half Palm");
            break;
        case GBMSAPI_SO_LOWER_HALF_PALM_RIGHT:
            ObjectName = tr("Right Lower Half Palm");
            break;
        case GBMSAPI_SO_WRITER_PALM_LEFT:
            ObjectName = tr("Left Writers Palm");
            break;
        case GBMSAPI_SO_WRITER_PALM_RIGHT:
            ObjectName = tr("Right Writers Palm");
            break;
        case GBMSAPI_SO_SLAP_2_LEFT:
            ObjectName = tr("Flat Left Two Fingers");
            break;
        case GBMSAPI_SO_SLAP_2_RIGHT:
            ObjectName = tr("Flat Right Two Fingers");
            break;
        case GBMSAPI_SO_SLAP_2_INDEXES:
            ObjectName = tr("Flat Two Indexes");
            break;
        default:
            ObjectName = "";
            break;
    }

    return ObjectName;
}

// show pictogram of the currently scanned object
void AcquisitionForm::ShowScannedObjectPictogram()
{
    switch (mScannedObject->ScannedObjectID)
    {
        case GBMSAPI_SO_FLAT_RIGHT_THUMB:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Right_Thumb.bmp"));
            break;
        case GBMSAPI_SO_FLAT_RIGHT_INDEX:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Right_Index.bmp"));
            break;
        case GBMSAPI_SO_FLAT_RIGHT_MIDDLE:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Right_Middle.bmp"));
            break;
        case GBMSAPI_SO_FLAT_RIGHT_RING:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Right_Ring.bmp"));
            break;
        case GBMSAPI_SO_FLAT_RIGHT_LITTLE:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Right_Little.bmp"));
            break;
        case GBMSAPI_SO_FLAT_LEFT_THUMB:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Left_Thumb.bmp"));
            break;
        case GBMSAPI_SO_FLAT_LEFT_INDEX:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Left_Index.bmp"));
            break;
        case GBMSAPI_SO_FLAT_LEFT_MIDDLE:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Left_Middle.bmp"));
            break;
        case GBMSAPI_SO_FLAT_LEFT_RING:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Left_Ring.bmp"));
            break;
        case GBMSAPI_SO_FLAT_LEFT_LITTLE:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Left_Little.bmp"));
            break;
        case GBMSAPI_SO_ROLL_RIGHT_THUMB:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Right_Thumb.bmp"));
            break;
        case GBMSAPI_SO_ROLL_RIGHT_INDEX:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Right_Index.bmp"));
            break;
        case GBMSAPI_SO_ROLL_RIGHT_MIDDLE:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Right_Middle.bmp"));
            break;
        case GBMSAPI_SO_ROLL_RIGHT_RING:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Right_Ring.bmp"));
            break;
        case GBMSAPI_SO_ROLL_RIGHT_LITTLE:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Right_Little.bmp"));
            break;
        case GBMSAPI_SO_ROLL_LEFT_THUMB:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Left_Thumb.bmp"));
            break;
        case GBMSAPI_SO_ROLL_LEFT_INDEX:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Left_Index.bmp"));
            break;
        case GBMSAPI_SO_ROLL_LEFT_MIDDLE:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Left_Middle.bmp"));
            break;
        case GBMSAPI_SO_ROLL_LEFT_RING:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Left_Ring.bmp"));
            break;
        case GBMSAPI_SO_ROLL_LEFT_LITTLE:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Left_Little.bmp"));
            break;
        case GBMSAPI_SO_SLAP_2_THUMBS:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/2_Thumbs.bmp"));
            break;
        case GBMSAPI_SO_SLAP_4_LEFT:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Left_4_Fingers.bmp"));
            break;
        case GBMSAPI_SO_SLAP_4_RIGHT:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Right_4_Fingers.bmp"));
            break;
        case GBMSAPI_SO_UPPER_HALF_PALM_LEFT:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Left_Upper_Palm.bmp"));
            break;
        case GBMSAPI_SO_UPPER_HALF_PALM_RIGHT:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Right_Upper_Palm.bmp"));
            break;
        case GBMSAPI_SO_LOWER_HALF_PALM_LEFT:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Left_Lower_Palm.bmp"));
            break;
        case GBMSAPI_SO_LOWER_HALF_PALM_RIGHT:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Right_Lower_Palm.bmp"));
            break;
        case GBMSAPI_SO_WRITER_PALM_LEFT:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Left_Writer_Palm.bmp"));
            break;
        case GBMSAPI_SO_WRITER_PALM_RIGHT:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Right_Writer_Palm.bmp"));
            break;
        case GBMSAPI_SO_SLAP_2_LEFT:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Left_Slap_2.bmp"));
            break;
        case GBMSAPI_SO_SLAP_2_RIGHT:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/Right_Slap_2.bmp"));
            break;
        case GBMSAPI_SO_SLAP_2_INDEXES:
            ui->lblScannedObjectImg->setPixmap(QPixmap(":/Graphics/2_Indexes.bmp"));
            break;
    }
}

void AcquisitionForm::DisplayMessage(QString Message, bool IsError)
{
	// avoid not necessary update 
	if (ui->lblMessage->text() == Message)
		return;

    QColor MsgColor = palette().base().color();

    ui->lblMessage->setText(Message);

    if (IsError)
    {
        MsgColor = Qt::red;

        if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_SOUND))
        {
            if (MSGUIRef->IsSoundPresent())
                GBMSAPI_Sound(12, 1, 4);
            else
                MSGUIRef->Beep(500, 200);
        }
    }

    QString strStyle = QString("QLabel { background-color : %1;}").arg(MsgColor.name());
    ui->lblMessage->setStyleSheet(strStyle);
}

void AcquisitionForm::SetPersonID(QString Value)
{
    PersonID = Value;
    ui->lblPersonID->setText(Value);
}

void AcquisitionForm::AcquireEnd()
{
    if (MSGUIRef->IsLedPresent())
        TurnOffLeds();

	ListenerTimer->stop();
}

void AcquisitionForm::on_btnBreak_clicked()
{
    QString Msg;

    if (MSGUIRef->GetAcquisitionMode() == GBMSGUI_AM_SINGLE_ACQUISITION)
        Msg = tr("Are you sure you want to cancel the acquisition?");
    else
        Msg = tr("Are you sure you want to break the sequence?");

    // ask confirmation
    if (QMessageBox::question(this, tr("Confirm"), Msg, QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Cancel)
        return;

    ScanResult = GBMSGUI_RET_BREAK_SEQUENCE;
}

void AcquisitionForm::SetScannedObject(ScannedObject *pObject)
{
    mScannedObject = pObject;
}

void AcquisitionForm::SetForceClose()
{
    mForceClose = true;
}

int AcquisitionForm::GetScanResult()
{
    return ScanResult;
}

bool AcquisitionForm::IsAcquiring()
{
    return Acquiring;
}

void AcquisitionForm::SetAcquisitionEventsManagerRet(int Value)
{
    AcquisitionEventsManagerRet = Value;
}

int AcquisitionForm::GetAcquisitionEventsManagerRet()
{
    return AcquisitionEventsManagerRet;
}

void AcquisitionForm::DeleteDiagItem(QString DiagText)
{
	QList<QListWidgetItem*> pList = ui->lstDiag->findItems(DiagText, Qt::MatchExactly);
	if (pList.count() == 0)
		return;

	delete pList[0];
}

void AcquisitionForm::CleanDiagnostic()
{
    QColor DiagColor = palette().base().color();
    ui->lstDiag->clear();
    QString strStyle = QString("QListWidget { background-color : %1;}").arg(DiagColor.name());
    ui->lstDiag->setStyleSheet(strStyle);
}

// Display scanning diagnostic in the diagnostic list
void AcquisitionForm::DisplayDiagnostic()
{
    QListWidgetItem *DiagItem;
    QColor DiagColor = palette().base().color();
	QString DiagText;

    // save the heaviest for the LCD (can show only one) (only PostProcessing)
    unsigned char LCDPPDiag = 0;

    //ui->lstDiag->clear();

    // first, manage list

    // check firts heavy diagnostic conditions, so that they appear first in the list
    // check first if bit has changed

	// Scanning diagnostic
	if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_SCANNER_SURFACE_NOT_NORMA))
	{
		DiagText = tr("A FINGER HAS BEEN PLACED BEFORE SCANNING PROCESS START, OR THE SCANNER SURFACE IS DIRTY");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_SCANNER_SURFACE_NOT_NORMA))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, tr("The scanner surface is not in optimal state.\nThis message indicates that the scanning surface is dirty, or finger is already present on the scanner surface when the acquisition starts.\nRemove finger from the scanner or clean the scanner surface according to the cleaning instruction.\nIf the message persists, please contact the Technical Support."));
			ui->lstDiag->addItem(DiagItem);
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_SCANNER_FAILURE))
	{
		DiagText = tr("SCANNER ERROR. ACQUIRED IMAGE PARAMETERS DO NOT CONFORM TO QUALITY STANDARD");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_SCANNER_FAILURE))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, tr("This message reports that some parameters of the acquired image are not suitable.\nCheck that there is no finger placed on scanner at the beginning of the scanning and that the scanning surface is clear.\nContact the Technical Support, if this diagnostic message persists."));
			ui->lstDiag->addItem(DiagItem);
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_COMPOSITION_SLOW))
	{
		DiagText = tr("FRAME RATE TOO SLOW FOR ROLLED FINGERPRINT IMAGE COMPOSITION");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_COMPOSITION_SLOW))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, tr("Critical resources for the rolled image composition.\nThis message indicates that the Personal Computer is running low on system resources, so that the optimal image composition of the rolled fingerprint is not guaranteed.\nThe operator should carefully evaluate the acquisition result. To avoid this diagnostic message, all the unnecessary processes on the Personal Computer should be stopped, to free more resources. To complete the current acquisition operator can reduce the finger rotation speed. If the message appears frequently even when all the unnecessary processes are stopped, it could be recommended to increase the Personal Computer resources."));
			ui->lstDiag->addItem(DiagItem);
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_FLAT_FINGER_SLIDING))
	{
		// DEBUG
		qDebug() << "LastDiagnostic = " << LastDiagnostic << "PrevDiagnostic = " << PrevDiagnostic;
		DiagText = tr("FLAT FINGER(S) SLIDING IS DETECTED");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_FLAT_FINGER_SLIDING))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, tr("Flat finger(s) sliding is detected.\nThe person should put flatly his/her finger(s) on scanning surface, ensuring no-sliding of the finger skin on the glass.\nIt is recommended to repeat acquisition, if this message appears."));
			ui->lstDiag->addItem(DiagItem);
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_EXT_LIGHT_TOO_STRONG))
	{
		DiagText = tr("TOO STRONG EXTERNAL LIGHT INFLUENCE");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_EXT_LIGHT_TOO_STRONG))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, tr("Too strong external light influence is detected.\nA very powerful external light source (such as direct sun rays coming in certain angles to scanner) can deteriorate fingerprint image quality.\nIt is recommended to eliminate the external light influence and to repeat the acquisition."));
			ui->lstDiag->addItem(DiagItem);
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_VSROLL_IMPROPER_ROLL))
	{
		DiagText = tr("IMPROPER FINGER ROLL");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_VSROLL_IMPROPER_ROLL))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, tr("A finger has been rolled improperly.\nIt is important to roll a finger only in the chosen direction (to the left or to the right), avoiding even the most short rolling to the opposite direction passing from flat to roll phase.\nOtherwise, visible \"seams\" in the final composite image will appear."));
			ui->lstDiag->addItem(DiagItem);
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_VSROLL_TOO_FAST_ROLL))
	{
		DiagText = tr("TOO FAST FINGER ROLL");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_VSROLL_TOO_FAST_ROLL))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, tr("A finger has been rolled too fast.\nThe rolling phase duration should be about 1.5 seconds."));
			ui->lstDiag->addItem(DiagItem);
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_VSROLL_TOO_NARROW_ROLL))
	{
		DiagText = tr("TOO NARROW FINGER ROLL");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_VSROLL_TOO_NARROW_ROLL))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, tr("A finger has been rolled too narrowly.\nIt's important to roll a finger widely, ideally in a \"nail to nail\" manner."));
			ui->lstDiag->addItem(DiagItem);
		}
		else
			DeleteDiagItem(DiagText);
	}

    // Yellow diagnostics
	if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_LEFT))
	{
		DiagText = tr("FINGER GOES OUT ON THE LEFT BORDER");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_LEFT))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, tr("It is recommended to take finger(s) away and then retry, properly selecting the region on the scanning window where to place finger(s)."));
			ui->lstDiag->addItem(DiagItem);
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_RIGHT))
	{
		DiagText = tr("FINGER GOES OUT ON THE RIGHT BORDER");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_RIGHT))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, tr("It is recommended to take finger(s) away and then retry, properly selecting the region on the scanning window where to place finger(s)."));
			ui->lstDiag->addItem(DiagItem);
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_TOP))
	{
		DiagText = tr("FINGER GOES OUT ON THE TOP BORDER");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_TOP))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, tr("It is recommended to take finger(s) away and then retry, properly selecting the region on the scanning window where to place finger(s)."));
			ui->lstDiag->addItem(DiagItem);
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_FLAT_FINGER_DISPLACED_DOWN))
	{
		DiagText = tr("FINGER DISPLACED TO THE DOWN BORDER");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_FLAT_FINGER_DISPLACED_DOWN))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, tr("It is recommended to take finger(s) away and then retry, properly selecting the region on the scanning window where to place finger(s)."));
			ui->lstDiag->addItem(DiagItem);
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_VSROLL_OUTSIDE_BORDER_LEFT))
	{
		DiagText = tr("FINGER GOES OUT ON THE LEFT BORDER");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_VSROLL_OUTSIDE_BORDER_LEFT))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, tr("A finger significantly goes out on the left border.\nIt's important to properly select a point on the scanning window where initially the finger has to be placed, ensuring that there is enough space to perform finger rolling \"from nail to nail\"."));
			ui->lstDiag->addItem(DiagItem);
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_VSROLL_OUTSIDE_BORDER_RIGHT))
	{
		DiagText = tr("FINGER GOES OUT ON THE RIGHT BORDER");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_VSROLL_OUTSIDE_BORDER_RIGHT))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, tr("A finger significantly goes out on the right border.\nIt's important to properly select a point on the scanning window where initially the finger has to be placed, ensuring that there is enough space to perform finger rolling \"from nail to nail\"."));
			ui->lstDiag->addItem(DiagItem);
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_VSROLL_OUTSIDE_BORDER_TOP))
	{
		DiagText = tr("FINGER GOES OUT ON THE TOP BORDER");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_VSROLL_OUTSIDE_BORDER_TOP))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, tr("A finger significantly goes out on the top border.\nIt's important to properly select a point on the scanning window where initially the finger has to be placed, ensuring that the finger rolling will be performed near the center of the scanning window."));
			ui->lstDiag->addItem(DiagItem);
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_VSROLL_DISPLACED_DOWN))
	{
		DiagText = tr("FINGER DISPLACED TO THE DOWN BORDER");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_VSROLL_DISPLACED_DOWN))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, tr("A finger significantly displaced to the down border.\nIt's important to properly select a point on the scanning window where initially the finger has to be placed, ensuring that the finger rolling will be performed near the center of the scanning window."));
			ui->lstDiag->addItem(DiagItem);
		}
		else
			DeleteDiagItem(DiagText);
	}

    // 1.5.4.0 - wet/dry finger detection
    // this is not in the list, don't check PrevDiagnostic
    //if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_DRY_FINGER))
    //{
		DiagText = tr("FINGER TOO DRY / UNDERPRESSED");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_DRY_FINGER))
		{
			// 1.5.5.0 - no more message, only icon
			//DiagItem = new QListWidgetItem(DiagText);
			//DiagItem->setData(Qt::UserRole, tr("The finger being scanned is either too dry or not sufficiently pressed on the scanner plate"));
			//ui->lstDiag->addItem(DiagItem);

			// only if visible
			if (ui->panDryWet->isVisible())
			{
				ui->lblDryWet->setText(DiagText);
				ui->frmDryWet->setVisible(true);

				// 1.5.5.0 - show on LCD on stop screen
				if (MSGUIRef->IsLCDPresent() && !IsAcquiring())
					GBMSAPI_VUI_LCD_SetWetDryFingerDiagnosticIconOnStopScreen(GBMSAPI_VILCD_MSG_DRY_FINGER);
			}
		}
		else
		{
			//DeleteDiagItem(DiagText);
            //ui->frmDryWet->setVisible(false);
		}
    //}
    // this is not in the list, don't check PrevDiagnostic
    //if (CheckMask(LastDiagnostic ^ PrevDiagnostic, GBMSAPI_DM_WET_FINGER))
    //{
		DiagText = tr("FINGER TOO WET / OVERPRESSED");
		if (CheckMask(LastDiagnostic, GBMSAPI_DM_WET_FINGER))
		{
			// 1.5.5.0 - no more message, only icon
			//DiagItem = new QListWidgetItem(DiagText);
			//DiagItem->setData(Qt::UserRole, tr("The finger being scanned is either too wet or pressed too forcefully on the scanner plate"));
			//ui->lstDiag->addItem(DiagItem);

			// only if visible
			if (ui->panDryWet->isVisible())
			{
				ui->lblDryWet->setText(DiagText);
				ui->frmDryWet->setVisible(true);

				// 1.5.5.0 - show on LCD on stop screen
				if (MSGUIRef->IsLCDPresent() && !IsAcquiring())
					GBMSAPI_VUI_LCD_SetWetDryFingerDiagnosticIconOnStopScreen(GBMSAPI_VILCD_MSG_WET_FINGER);
			}
		}
		else
		{
			//DeleteDiagItem(DiagText);
            //ui->frmDryWet->setVisible(false);
		}
    //}

	if (!CheckMask(LastDiagnostic, GBMSAPI_DM_DRY_FINGER) &&
		!CheckMask(LastDiagnostic, GBMSAPI_DM_WET_FINGER))
	{
        // frmDryWet is in common, hide only if both are false
		// only if visible
		if (ui->panDryWet->isVisible())
			ui->frmDryWet->setVisible(false);
	}

	// Post-processing diagnostic
	if (CheckMask(PostProcessDiag ^ PrevPostProcessDiag, POST_PROC_DIAG_INVALID_PATTERN))
	{
		DiagText = tr("INVALID PATTERN DETECTED");
		if (CheckMask(PostProcessDiag, POST_PROC_DIAG_INVALID_PATTERN))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, DiagItem->text());
			ui->lstDiag->addItem(DiagItem);
			LCDPPDiag = GBMSAPI_VILCD_MSG_INVALID_PATTERN_DETECTED;
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(PostProcessDiag ^ PrevPostProcessDiag, POST_PROC_DIAG_INCOMPLETE_PATTERN))
	{
		DiagText = tr("INCOMPLETE PATTERN DETECTED");
		if (CheckMask(PostProcessDiag, POST_PROC_DIAG_INCOMPLETE_PATTERN))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, DiagItem->text());
			ui->lstDiag->addItem(DiagItem);
			if (LCDPPDiag == 0)
				LCDPPDiag = GBMSAPI_VILCD_MSG_INCOMPLETE_PATTERN_DETECTED;
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(PostProcessDiag ^ PrevPostProcessDiag, POST_PROC_DIAG_INCLINATION))
	{
		DiagText = tr("SIGNIFICANT FINGER INCLINATION DETECTED");
		if (CheckMask(PostProcessDiag, POST_PROC_DIAG_INCLINATION))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, DiagItem->text());
			ui->lstDiag->addItem(DiagItem);
			if (LCDPPDiag == 0)
				LCDPPDiag = GBMSAPI_VILCD_MSG_FINGER_INCLINATION_DETECTED;
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(PostProcessDiag ^ PrevPostProcessDiag, POST_PROC_DIAG_SEGM_AMBIGUOUS_SLAP))
	{
		DiagText = tr("AMBIGUOUS SLAP DETECTED");
		if (CheckMask(PostProcessDiag, POST_PROC_DIAG_SEGM_AMBIGUOUS_SLAP))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, DiagItem->text());
			ui->lstDiag->addItem(DiagItem);
			if (LCDPPDiag == 0)
				LCDPPDiag = GBMSAPI_VILCD_MSG_AMBIGUOUS_SLAP;
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(PostProcessDiag ^ PrevPostProcessDiag, POST_PROC_DIAG_SEGM_INCOMPLETE_SLAP))
	{
		DiagText = tr("INCOMPLETE SLAP DETECTED");
		if (CheckMask(PostProcessDiag, POST_PROC_DIAG_SEGM_INCOMPLETE_SLAP))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, DiagItem->text());
			ui->lstDiag->addItem(DiagItem);
			if (LCDPPDiag == 0)
				LCDPPDiag = GBMSAPI_VILCD_MSG_INCOMPLETE_SLAP;
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(PostProcessDiag ^ PrevPostProcessDiag, POST_PROC_DIAG_SEGM_POSSIBLE_INCOMPLTETE_SLAP))
	{
		DiagText = tr("POSSIBLE INCOMPLETE SLAP DETECTED");
		if (CheckMask(PostProcessDiag, POST_PROC_DIAG_SEGM_POSSIBLE_INCOMPLTETE_SLAP))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, DiagItem->text());
			ui->lstDiag->addItem(DiagItem);
			if (LCDPPDiag == 0)
				LCDPPDiag = GBMSAPI_VILCD_MSG_POSSIBLE_INCOMPLETE_SLAP;
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(PostProcessDiag ^ PrevPostProcessDiag, POST_PROC_DIAG_LEFT_RIGHT_MIXUP))
	{
		DiagText = tr("LEFT-RIGHT HAND MIX-UP DETECTED");
		if (CheckMask(PostProcessDiag, POST_PROC_DIAG_LEFT_RIGHT_MIXUP))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, DiagItem->text());
			ui->lstDiag->addItem(DiagItem);
			if (LCDPPDiag == 0)
				LCDPPDiag = GBMSAPI_VILCD_MSG_LEFT_RIGHT_MIXUP;
		}
		else
			DeleteDiagItem(DiagText);
	}

	if (CheckMask(PostProcessDiag ^ PrevPostProcessDiag, POST_PROC_DIAG_POSSIBLE_LEFT_RIGHT_MIXUP))
	{
		DiagText = tr("POSSIBLE LEFT-RIGHT HAND MIX-UP DETECTED");
		if (CheckMask(PostProcessDiag, POST_PROC_DIAG_POSSIBLE_LEFT_RIGHT_MIXUP))
		{
			DiagItem = new QListWidgetItem(DiagText);
			DiagItem->setData(Qt::UserRole, DiagItem->text());
			ui->lstDiag->addItem(DiagItem);
			if (LCDPPDiag == 0)
				LCDPPDiag = GBMSAPI_VILCD_MSG_POSSIBLE_LEFT_RIGHT_MIXUP;
		}
		else
			DeleteDiagItem(DiagText);
	}

    // then, manage color
    // Red diagnostics
	if (CheckMask(LastDiagnostic, GBMSAPI_DM_SCANNER_SURFACE_NOT_NORMA) ||
		CheckMask(LastDiagnostic, GBMSAPI_DM_SCANNER_FAILURE) ||
		CheckMask(LastDiagnostic, GBMSAPI_DM_COMPOSITION_SLOW) ||
		CheckMask(LastDiagnostic, GBMSAPI_DM_FLAT_FINGER_SLIDING) ||
		CheckMask(LastDiagnostic, GBMSAPI_DM_EXT_LIGHT_TOO_STRONG) ||
		CheckMask(LastDiagnostic, GBMSAPI_DM_VSROLL_IMPROPER_ROLL) ||
		CheckMask(LastDiagnostic, GBMSAPI_DM_VSROLL_TOO_FAST_ROLL) ||
		CheckMask(LastDiagnostic, GBMSAPI_DM_VSROLL_TOO_NARROW_ROLL) ||
		CheckMask(PostProcessDiag, POST_PROC_DIAG_SEGM_AMBIGUOUS_SLAP) ||
		CheckMask(PostProcessDiag, POST_PROC_DIAG_SEGM_INCOMPLETE_SLAP) ||
		CheckMask(PostProcessDiag, POST_PROC_DIAG_LEFT_RIGHT_MIXUP))
		DiagColor = Qt::red;
	// Yellow diagnostics
	if (CheckMask(LastDiagnostic, GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_LEFT) ||
		CheckMask(LastDiagnostic, GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_RIGHT) ||
		CheckMask(LastDiagnostic, GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_TOP) ||
		CheckMask(LastDiagnostic, GBMSAPI_DM_FLAT_FINGER_DISPLACED_DOWN) ||
		CheckMask(LastDiagnostic, GBMSAPI_DM_VSROLL_OUTSIDE_BORDER_LEFT) ||
		CheckMask(LastDiagnostic, GBMSAPI_DM_VSROLL_OUTSIDE_BORDER_RIGHT) ||
		CheckMask(LastDiagnostic, GBMSAPI_DM_VSROLL_OUTSIDE_BORDER_TOP) ||
		CheckMask(LastDiagnostic, GBMSAPI_DM_VSROLL_DISPLACED_DOWN) ||
		// 1.5.5.0 - no more message, only icon
		//CheckMask(LastDiagnostic, GBMSAPI_DM_DRY_FINGER) ||
		//CheckMask(LastDiagnostic, GBMSAPI_DM_WET_FINGER) ||
		CheckMask(PostProcessDiag, POST_PROC_DIAG_INVALID_PATTERN) ||
		CheckMask(PostProcessDiag, POST_PROC_DIAG_INCOMPLETE_PATTERN) ||
		CheckMask(PostProcessDiag, POST_PROC_DIAG_INCLINATION) ||
		CheckMask(PostProcessDiag, POST_PROC_DIAG_SEGM_POSSIBLE_INCOMPLTETE_SLAP) ||
		CheckMask(PostProcessDiag, POST_PROC_DIAG_POSSIBLE_LEFT_RIGHT_MIXUP))
		if (DiagColor != Qt::red)
			DiagColor = Qt::yellow;

    QString strStyle = QString("QListWidget { background-color : %1;}").arg(DiagColor.name());
    ui->lstDiag->setStyleSheet(strStyle);

    // show on LCD ONLY when acquisition is stopped
	if (MSGUIRef->IsLCDPresent() && !IsAcquiring())
    {
        if (LCDPPDiag != 0)
        {
            GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(LCDPPDiag, GBMSAPI_VILCD_DIAG_AREA);
        }
    }

    // save previous diagnostic
    PrevDiagnostic = LastDiagnostic;
    PrevPostProcessDiag = PostProcessDiag;
}

void AcquisitionForm::SetScanningOptions()
{
    unsigned int AcquisitionOptions = MSGUIRef->GetAcquisitionOptions();
    unsigned int SupportedOptions = MSGUIRef->GetSupportedScanOptions();

    ScanOptions = 0;

    ui->lstMessages->clear();

    // verify that requested options are available; if not, show a warning
    if (CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_FULL_RES_PREVIEW))
    {
        if (CheckMask(SupportedOptions, GBMSAPI_AO_FULL_RES_PREVIEW))
            ScanOptions |= GBMSAPI_AO_FULL_RES_PREVIEW;
        else
            AddMessageToList(tr("Full resolution preview not supported"), QIcon(":/Icons/Warning.png"));
    }

    // Autocapture only for flat (for Rolled is always active)
    if (!GBMSGUI::IsRolled(mScannedObject->ScannedObjectID))
    {
        if (CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_AUTOCAPTURE))
        {
            if (CheckMask(SupportedOptions, GBMSAPI_AO_AUTOCAPTURE))
            {
                ScanOptions |= GBMSAPI_AO_AUTOCAPTURE;
                AddMessageToList(tr("Autocapture: ON"), QIcon(":/Icons/Info.png"));
            }
            else
                AddMessageToList(tr("Autocapture: OFF"), QIcon(":/Icons/Warning.png"));
        }
        else
            AddMessageToList(tr("Autocapture: OFF"), QIcon(":/Icons/Info.png"));
    }

    // Roll preview; only for rolled
    if (GBMSGUI::IsRolled(mScannedObject->ScannedObjectID))
    {
        if (CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_NO_ROLL_PREVIEW)
            && CheckMask(SupportedOptions, GBMSAPI_AO_NO_ROLL_PREVIEW))
        {
            ScanOptions |= GBMSAPI_AO_NO_ROLL_PREVIEW;
            AddMessageToList(tr("No roll preview"), QIcon(":/Icons/Info.png"));
        }
        else
        {
            if (CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_ROLL_PREVIEW_MANUAL_STOP))
            {
                if (CheckMask(SupportedOptions, GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP))
                {
                    ScanOptions |= GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP;
                    AddMessageToList(tr("Roll preview manual stop"), QIcon(":/Icons/Info.png"));
                }
                else
                    AddMessageToList(tr("Roll preview manual stop not supported"), QIcon(":/Icons/Warning.png"));
            }
            else
            {
                // automatic roll preview modes are always supported
                if (CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_ROLL_PREVIEW_TYPE))
                {
                    ScanOptions |= GBMSAPI_AO_ROLL_PREVIEW_TYPE;
                    AddMessageToList(tr("Roll preview atomatic stop (CENTER)"), QIcon(":/Icons/Info.png"));
                }
                else
                {
                    AddMessageToList(tr("Roll preview atomatic stop (SIDE)"), QIcon(":/Icons/Info.png"));
                }
            }
        }
    }

    // Flat finger on roll scanner
    if (GBMSGUI::IsFlatSingleFinger(mScannedObject->ScannedObjectID)
        && MSGUIRef->GetCfgData()->FlatFingerOnRollArea
        && CheckMask(SupportedOptions, GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA))
        ScanOptions |= GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA;

    // High speed full res preview
    if (CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_HIGH_SPEED_PREVIEW))
    {
        if (CheckMask(SupportedOptions, GBMSAPI_AO_HIGH_SPEED_PREVIEW))
            ScanOptions |= GBMSAPI_AO_HIGH_SPEED_PREVIEW;
        else
            AddMessageToList(tr("High speed preview not supported"), QIcon(":/Icons/Warning.png"));
    }

    // show other options

    if (!GBMSGUI::IsRolled(mScannedObject->ScannedObjectID))
    {
        // Block autocapture
        if (CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_BLOCK_AUTOCAPTURE)
                && !MSGUIRef->IsBlockAutocaptureSupported())
        {
            AddMessageToList(tr("Block autocapture not supported"), QIcon(":/Icons/Warning.png"));
        }
    }

    // show only for Slap
    if (GBMSGUI::IsSlap(mScannedObject->ScannedObjectID))
    {
        if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_SEGMENTATION))
            AddMessageToList(tr("Segmentation: ON"), QIcon(":/Icons/Info.png"));
        else
            AddMessageToList(tr("Segmentation: OFF"), QIcon(":/Icons/Info.png"));
    }

    if (CheckMask(MSGUIRef->GetSessionOptions(), GBMSGUI_SESSION_OPTION_SEQUENCE_CHECK))
        AddMessageToList(tr("Sequence check: ON"), QIcon(":/Icons/Info.png"));
    else
        AddMessageToList(tr("Sequence check: OFF"), QIcon(":/Icons/Info.png"));

    // High speed full res preview
    if (GBMSGUI::IsRolled(mScannedObject->ScannedObjectID))
        if (CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_AUTO_CLEAR_OUTSIDE_ROLL))
            AddMessageToList(tr("Auto clear outside roll active"), QIcon(":/Icons/Info.png"));

    // 1.5.4.0 - dry finger enhancement
    if (MSGUIRef->IsDryFingerImageEnhancementSupported())
    {
        if (!SetDryFingerImageEnhancementOption(ui->chkDryFingerImageEnhancement->isChecked()))
            return;
    }
    else
    {
        if (CheckMask(MSGUIRef->GetSessionOptions(), GBMSGUI_SESSION_OPTION_DRY_FINGER_IMAGE_ENHANCEMENT))
        {
            AddMessageToList(tr("Dry finger image enhancement not supported"), QIcon(":/Icons/Warning.png"));
        }
    }
}

void AcquisitionForm::changeEvent ( QEvent * event )
{
    QWidget::changeEvent(event);

    switch (event->type())
    {
        case QEvent::LanguageChange:
            ui->retranslateUi(this); // was commented; but it is needed in TWAIN driver, because SetLanguage is called after StartSequence
            break;
        default:
            break;
    }
}

void AcquisitionForm::SetPreviewScale()
{
    int ret;
    bool FlatScanArea = true;

    if (GBMSGUI::IsRolled(mScannedObject->ScannedObjectID))
        FlatScanArea = false;
    else
        if (GBMSGUI::IsFlatSingleFinger(mScannedObject->ScannedObjectID)
            && MSGUIRef->GetCfgData()->FlatFingerOnRollArea
            && CheckMask(MSGUIRef->GetSupportedScanOptions(), GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA))
            FlatScanArea = false;

    ret = GBMSAPI_GetImageSize(
                GBMSAPI_GetTypeFromObject(mScannedObject->ScannedObjectID),
                FlatScanArea,
                false,  // final image size
                &ImageSizeX,
                &ImageSizeY
                );
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        MSGUIRef->SetError(MSGUIRef->GetMSAPIErrorMessage("GBMSAPI_GetImageSize", ret), ret);
        ScanResult = GBMSGUI_RET_FAILURE;
        return;
    }

    ret = GBMSAPI_GetImageSize(
                GBMSAPI_GetTypeFromObject(mScannedObject->ScannedObjectID),
                FlatScanArea,
                true,  // preview image size
                &PreviewSizeX,
                &PreviewSizeY
                );
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        MSGUIRef->SetError(MSGUIRef->GetMSAPIErrorMessage("GBMSAPI_GetImageSize", ret), ret);
        ScanResult = GBMSGUI_RET_FAILURE;
        return;
    }

    if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_FULL_RES_PREVIEW) &&
        CheckMask(MSGUIRef->GetSupportedScanOptions(), GBMSAPI_AO_FULL_RES_PREVIEW))
    {
        PreviewSizeX = ImageSizeX;
        PreviewSizeY = ImageSizeY;
    }

    PreviewScale = (float)ImageSizeX/PreviewSizeX;
}

void AcquisitionForm::ShowFrameRate(double CurrentFrameRate, double NominalFrameRate)
{
    int FrameRateLimit = 80;
    QColor lblColor = palette().base().color();

    if (CurrentFrameRate && NominalFrameRate)
    {
        ui->lblCurrentNominalFrameRate->setText(QString("%1/%2").arg(CurrentFrameRate, 0, 'f', 1).arg(NominalFrameRate, 0, 'f', 1));

        if (CurrentFrameRate < NominalFrameRate*FrameRateLimit/100)
            lblColor = Qt::red;
    }
    else
        ui->lblCurrentNominalFrameRate->setText("");

    QString strStyle = QString("QLabel { background-color : %1;}").arg(lblColor.name());
    ui->lblCurrentNominalFrameRate->setStyleSheet(strStyle);
}

void AcquisitionForm::AddMessageToList(QString Message, QIcon Icon)
{
    QListWidgetItem *ListItem;
    QList<QListWidgetItem *> items;

    // add only if not present
    items = ui->lstMessages->findItems(Message, Qt::MatchCaseSensitive);
    if (items.isEmpty())
    {
        ListItem = new QListWidgetItem(Icon, Message);
        ui->lstMessages->addItem(ListItem);
    }
}

void AcquisitionForm::RemoveMessageFromList(QString Message)
{
    QList<QListWidgetItem *> items;

    items = ui->lstMessages->findItems(Message, Qt::MatchCaseSensitive);
    if (!items.isEmpty())
    {
        delete items.at(0);
    }
}

void AcquisitionForm::GetRolledInfo()
{
    int ret;
    unsigned char *NotWipedArtefactFrame;
    unsigned int CompositeFingerprintSize;
    unsigned char CompositeFingerprintContrast;
    unsigned int FlatFingerprintSize;

    ret = GBMSAPI_ROLL_GetCompositeImageInfo(&mScannedObject->RolledArtefactSize,
        &MarkerFrame, &NotWipedArtefactFrame,
        &CompositeFingerprintSize, &CompositeFingerprintContrast, &FlatFingerprintSize);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        MSGUIRef->SetError(MSGUIRef->GetMSAPIErrorMessage("GBMSAPI_ROLL_GetCompositeImageInfo", ret), ret);
        ScanResult = GBMSGUI_RET_FAILURE;
        return;
    }

    // show CompositeFingerprintContrast
    Contrast = CompositeFingerprintContrast;
    ui->progressContrast->SetValue(Contrast);

    // update image with Marker frame
    DisplayImage();
}

void AcquisitionForm::on_btnHelpDiag_clicked()
{
    on_lstDiag_itemDoubleClicked(ui->lstDiag->currentItem());
}

void AcquisitionForm::on_lstDiag_itemClicked(QListWidgetItem *item)
{
    ui->btnHelpDiag->setEnabled(true);
}

void AcquisitionForm::on_lstDiag_itemDoubleClicked(QListWidgetItem *item)
{
    QString DiagMessage = item->data(Qt::UserRole).toString();

    if (ui->lstDiag->palette().base().color() == Qt::yellow)
        QMessageBox::warning(this, tr("Diagnostic"), DiagMessage, QMessageBox::Ok);
    else
        QMessageBox::critical(this, tr("Diagnostic"), DiagMessage, QMessageBox::Ok);
}

int AcquisitionForm::GetNFIQQuality(unsigned char *ImagePtr, int ImageSizeX, int ImageSizeY)
{
    int ret;
    int Quality;
    unsigned char *Frame500DPI = NULL;

    if (MSGUIRef->IsResolution1000Dpi())
    {
        // need to convert image to 500 dpi for passing it to GBNFIQ
		Frame500DPI = (unsigned char *)malloc(ImageSizeX*ImageSizeY/4);
		if (Frame500DPI == NULL)
		{
            MSGUIRef->SetError(tr("Insufficient memory"), GBMSGUI_ERR_INSUFFICIENT_MEMORY);
            ScanResult = GBMSGUI_RET_FAILURE;
            return 255;
		}
        ret = GBMSAPI_ConvertImageFrom1000to500DPI(ImagePtr, Frame500DPI, ImageSizeX, ImageSizeY);
        if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
        {
            MSGUIRef->SetError(MSGUIRef->GetMSAPIErrorMessage("GBMSAPI_ConvertImageFrom1000to500DPI", ret), ret);
            ScanResult = GBMSGUI_RET_FAILURE;
            return 255;
        }

	    ret = GBNFIQ_GetImageQuality(Frame500DPI, ImageSizeX/2, ImageSizeY/2, &Quality, NULL);
    }
    else
    {
	    ret = GBNFIQ_GetImageQuality(ImagePtr, ImageSizeX, ImageSizeY, &Quality, NULL);
    }

    if (Frame500DPI)
        free(Frame500DPI);

    if (ret == GBNFIQ_NO_ERROR)
        return Quality;

    return 255;     // attempt failed
}

void AcquisitionForm::SetQualityThresholds(int Threshold1, int Threshold2, int Dir)
{
    if (Dir == THRESHOLD_SMALLER_THAN)
    {
        ui->lblQualityGoodRule->setText(QString("%1%2").arg(QChar(0x2264)).arg(Threshold1));
        ui->lblQualityMediumRule->setText(QString("%1%2").arg(QChar(0x2264)).arg(Threshold2));
        ui->lblQualityBadRule->setText(QString(">%1").arg(Threshold2));
    }
    else
    {
        ui->lblQualityBadRule->setText(QString("<%1").arg(Threshold1));
        ui->lblQualityMediumRule->setText(QString("%1%2").arg(QChar(0x2265)).arg(Threshold1));
        ui->lblQualityGoodRule->setText(QString("%1%2").arg(QChar(0x2265)).arg(Threshold2));
    }

    if (Threshold1 == Threshold2)
    {
        if ((Threshold1 == 0) || (Threshold1 == -1))
        {
            // no thresholds
            ui->QualityGreenBox->setVisible(false);
            ui->lblQualityGoodRule->setVisible(false);
            ui->QualityYellowBox->setVisible(false);
            ui->lblQualityMediumRule->setVisible(false);
            ui->QualityRedBox->setVisible(false);
            ui->lblQualityBadRule->setVisible(false);
        }
        else
        {
            // one threshold
            ui->QualityYellowBox->setVisible(false);
            ui->lblQualityMediumRule->setVisible(false);
            ui->QualityRedBox->move(ui->QualityYellowBox->geometry().x(), ui->QualityYellowBox->geometry().y());
            ui->lblQualityBadRule->move(ui->lblQualityMediumRule->geometry().x(), ui->lblQualityMediumRule->geometry().y());
        }
    }
}

// Evaluate quality of scanned object, based on currently selected algorithm
bool AcquisitionForm::EvalIAIFSQuality()
{
    bool Result = false;
    QColor lblColor = palette().base().color();

    // NFIQ Algorithm
    if (MSGUIRef->GetCfgData()->QualityAlgorithm == GBMSGUI_NFIQ_ALGORITHM)
    {
        if (mScannedObject->NFIQQuality == 255)
        {
            ui->lblIAFISQuality->setText("--");
            lblColor = Qt::red;
        }
        else
        {
            ui->lblIAFISQuality->setText(QString().setNum(mScannedObject->NFIQQuality));

            if ((mScannedObject->QualityThreshold1 == 0) || (mScannedObject->QualityThreshold2 == 0))
            {
                // no thresholds, no evaluation
                Result = true;
            }
            else
            {
                if (mScannedObject->NFIQQuality <= mScannedObject->QualityThreshold1)
                {
                    lblColor = Qt::green;
                    Result = true;
                }
                else if ((mScannedObject->NFIQQuality > mScannedObject->QualityThreshold1) &&
                    (mScannedObject->NFIQQuality <= mScannedObject->QualityThreshold2))
                {
                    lblColor = Qt::yellow;
                    Result = false;
                }
                else
                {
                    lblColor = Qt::red;
                    Result = false;
                }
            }
        }

        // not yet supported in linux
#ifdef WIN32
        // show on LCD
        if (MSGUIRef->IsLCDPresent())
        {
            GBMSAPI_VUI_LCD_SetQualityLimitOnStopScreen(mScannedObject->QualityThreshold1,
                GBMSAPI_VILCD_SMALLER_THAN_LIMIT_DIRECTION);
            if (MSGUIRef->IsLCDMidLimitSupported())
                GBMSAPI_VUI_LCD_SetQualityIntermediateLimitOnStopScreen(mScannedObject->QualityThreshold2);
            GBMSAPI_VUI_LCD_SetQualityOnStopScreen(mScannedObject->NFIQQuality);
        }
#endif
    }
    else
    {
        // GB Algorithm
        ui->lblIAFISQuality->setText(QString().setNum(mScannedObject->GBQuality));

        if ((mScannedObject->QualityThreshold1 == 0) || (mScannedObject->QualityThreshold2 == 0))
        {
            // no thresholds, no evaluation
            Result = true;
        }
        else
        {
            if (mScannedObject->GBQuality >= mScannedObject->QualityThreshold2)
            {
                lblColor = Qt::green;
                Result = true;
            }
            else if ((mScannedObject->GBQuality < mScannedObject->QualityThreshold2) &&
                (mScannedObject->GBQuality >= mScannedObject->QualityThreshold1))
            {
                lblColor = Qt::yellow;
                Result = false;
            }
            else
            {
                lblColor = Qt::red;
                Result = false;
            }
        }

        // not yet supported in linux
#ifdef WIN32
        // show on LCD
        if (MSGUIRef->IsLCDPresent())
        {
            GBMSAPI_VUI_LCD_SetQualityLimitOnStopScreen(mScannedObject->QualityThreshold2,
                GBMSAPI_VILCD_GREATER_THAN_LIMIT_DIRECTION);
            if (MSGUIRef->IsLCDMidLimitSupported())
                GBMSAPI_VUI_LCD_SetQualityIntermediateLimitOnStopScreen(mScannedObject->QualityThreshold1);
            GBMSAPI_VUI_LCD_SetQualityOnStopScreen(mScannedObject->GBQuality);
        }
#endif
    }

    QString strStyle = QString("QLabel { background-color : %1;}").arg(lblColor.name());
    ui->lblIAFISQuality->setStyleSheet(strStyle);

    return Result;
}

void AcquisitionForm::ResetIAFISQuality()
{
    QColor lblColor = palette().base().color();

    ui->lblIAFISQuality->setText("");
    QString strStyle = QString("QLabel { background-color : %1;}").arg(lblColor.name());
    ui->lblIAFISQuality->setStyleSheet(strStyle);
}

void AcquisitionForm::SetArtefactsThresholds(int Threshold1, int Threshold2, int Dir)
{
    if (Dir == THRESHOLD_SMALLER_THAN)
    {
        ui->lblArtefactsGoodRule->setText(QString("%1%2").arg(QChar(0x2264)).arg(Threshold1));
        ui->lblArtefactsMediumRule->setText(QString("%1%2").arg(QChar(0x2264)).arg(Threshold2));
        ui->lblArtefactsBadRule->setText(QString(">%1").arg(Threshold2));
    }
    else
    {
        ui->lblArtefactsBadRule->setText(QString("<%1").arg(Threshold1));
        ui->lblArtefactsMediumRule->setText(QString("%1%2").arg(QChar(0x2265)).arg(Threshold1));
        ui->lblArtefactsGoodRule->setText(QString("%1%2").arg(QChar(0x2265)).arg(Threshold2));
    }

    if (Threshold1 == Threshold2)
    {
        if ((Threshold1 == 0) || (Threshold1 == -1))
        {
            // no thresholds
            ui->ArtefactsGreenBox->setVisible(false);
            ui->lblArtefactsGoodRule->setVisible(false);
            ui->ArtefactsYellowBox->setVisible(false);
            ui->lblArtefactsMediumRule->setVisible(false);
            ui->ArtefactsRedBox->setVisible(false);
            ui->lblArtefactsBadRule->setVisible(false);
        }
        else
        {
            // one threshold
            ui->ArtefactsYellowBox->setVisible(false);
            ui->lblArtefactsMediumRule->setVisible(false);
            ui->ArtefactsRedBox->move(ui->ArtefactsYellowBox->geometry().x(), ui->ArtefactsYellowBox->geometry().y());
            ui->lblArtefactsBadRule->move(ui->lblArtefactsMediumRule->geometry().x(), ui->lblArtefactsMediumRule->geometry().y());
        }
    }
}

bool AcquisitionForm::EvalRolledArtefactsSize()
{
    bool Result = false;
    QColor lblColor = palette().base().color();

    // show artefacts size
    ui->lblArtefacsSize->setText(QString("%1").arg(mScannedObject->RolledArtefactSize));

    if ((MSGUIRef->GetCfgData()->ArtefactsThreshold1 == -1) || (MSGUIRef->GetCfgData()->ArtefactsThreshold2 == -1))
    {
        // no thresholds, no evaluation
        Result = true;
    }
    else
    {
        if (mScannedObject->RolledArtefactSize <= MSGUIRef->GetCfgData()->ArtefactsThreshold1)
        {
            lblColor = Qt::green;
            Result = true;
        }
        if ((mScannedObject->RolledArtefactSize > MSGUIRef->GetCfgData()->ArtefactsThreshold1) &&
            (mScannedObject->RolledArtefactSize <= MSGUIRef->GetCfgData()->ArtefactsThreshold2))
        {
            lblColor = Qt::yellow;
            Result = false;
        }
        if (mScannedObject->RolledArtefactSize > MSGUIRef->GetCfgData()->ArtefactsThreshold2)
        {
            lblColor = Qt::red;
            Result = false;
        }
    }

    QString strStyle = QString("QLabel { background-color : %1;}").arg(lblColor.name());
    ui->lblArtefacsSize->setStyleSheet(strStyle);

    // not yet supported in linux
#ifdef WIN32
    // show on LCD
    if (MSGUIRef->IsLCDPresent())
    {
        GBMSAPI_VUI_LCD_SetArtefactLimitOnStopScreen(MSGUIRef->GetCfgData()->ArtefactsThreshold1,
            GBMSAPI_VILCD_SMALLER_THAN_LIMIT_DIRECTION);
        if (MSGUIRef->IsLCDMidLimitSupported())
            GBMSAPI_VUI_LCD_SetArtefactIntermediateLimitOnStopScreen(MSGUIRef->GetCfgData()->ArtefactsThreshold2);
        GBMSAPI_VUI_LCD_SetArtefactOnStopScreen(mScannedObject->RolledArtefactSize);
    }
#endif

    return Result;
}

void AcquisitionForm::ResetArtefactsSize()
{
    QColor lblColor = palette().base().color();

    ui->lblArtefacsSize->setText("");
    QString strStyle = QString("QLabel { background-color : %1;}").arg(lblColor.name());
    ui->lblArtefacsSize->setStyleSheet(strStyle);
}

// return a mask corresponding to scanned object id, to use to set indicators
unsigned int AcquisitionForm::SetIndicatorMask(unsigned int ObjectID)
{
    unsigned int IndicatorMask = 0;

    if ((ObjectID == GBMSAPI_SO_FLAT_LEFT_INDEX) ||
        (ObjectID == GBMSAPI_SO_ROLL_LEFT_INDEX))
            IndicatorMask |= GBMSAPI_VIL_INDEX_LEFT;

    else if ((ObjectID == GBMSAPI_SO_FLAT_LEFT_LITTLE) ||
        (ObjectID == GBMSAPI_SO_ROLL_LEFT_LITTLE))
            IndicatorMask |= GBMSAPI_VIL_LITTLE_LEFT;

    else if ((ObjectID == GBMSAPI_SO_FLAT_LEFT_MIDDLE) ||
        (ObjectID == GBMSAPI_SO_ROLL_LEFT_MIDDLE))
            IndicatorMask |= GBMSAPI_VIL_MIDDLE_LEFT;

    else if ((ObjectID == GBMSAPI_SO_FLAT_LEFT_RING) ||
        (ObjectID == GBMSAPI_SO_ROLL_LEFT_RING))
            IndicatorMask |= GBMSAPI_VIL_RING_LEFT;

    else if ((ObjectID == GBMSAPI_SO_FLAT_LEFT_THUMB) ||
        (ObjectID == GBMSAPI_SO_ROLL_LEFT_THUMB))
            IndicatorMask |= GBMSAPI_VIL_THUMB_LEFT;

    else if ((ObjectID == GBMSAPI_SO_FLAT_RIGHT_INDEX) ||
        (ObjectID == GBMSAPI_SO_ROLL_RIGHT_INDEX))
            IndicatorMask |= GBMSAPI_VIL_INDEX_RIGHT;

    else if ((ObjectID == GBMSAPI_SO_FLAT_RIGHT_LITTLE) ||
        (ObjectID == GBMSAPI_SO_ROLL_RIGHT_LITTLE))
            IndicatorMask |= GBMSAPI_VIL_LITTLE_RIGHT;

    else if ((ObjectID == GBMSAPI_SO_FLAT_RIGHT_MIDDLE) ||
        (ObjectID == GBMSAPI_SO_ROLL_RIGHT_MIDDLE))
            IndicatorMask |= GBMSAPI_VIL_MIDDLE_RIGHT;

    else if ((ObjectID == GBMSAPI_SO_FLAT_RIGHT_RING) ||
        (ObjectID == GBMSAPI_SO_ROLL_RIGHT_RING))
            IndicatorMask |= GBMSAPI_VIL_RING_RIGHT;

    else if ((ObjectID == GBMSAPI_SO_FLAT_RIGHT_THUMB) ||
        (ObjectID == GBMSAPI_SO_ROLL_RIGHT_THUMB))
            IndicatorMask |= GBMSAPI_VIL_THUMB_RIGHT;

    return IndicatorMask;
}

// turns off all indicators
void AcquisitionForm::TurnOffLeds()
{
    unsigned int IndicatorMask = GBMSAPI_VIL_INDEX_LEFT |
        GBMSAPI_VIL_INDEX_RIGHT |
        GBMSAPI_VIL_LITTLE_LEFT |
        GBMSAPI_VIL_LITTLE_RIGHT |
        GBMSAPI_VIL_MIDDLE_LEFT |
        GBMSAPI_VIL_MIDDLE_RIGHT |
        GBMSAPI_VIL_RING_LEFT |
        GBMSAPI_VIL_RING_RIGHT |
        GBMSAPI_VIL_THUMB_LEFT |
        GBMSAPI_VIL_THUMB_RIGHT;

    GBMSAPI_SetFingerIndicatorVUIState(IndicatorMask, GBMSAPI_VIL_COLOR_OFF, false);
}

void AcquisitionForm::on_btnSkip_clicked()
{
    if (CheckMask(MSGUIRef->GetSessionOptions(), GBMSGUI_SESSION_OPTION_ASK_UNAVAILABILITY_REASON))
    {
        // if skip an entire slap, first reset segments
        if (GBMSGUI::IsSlap(mScannedObject->ScannedObjectID))
        {
            ResetSegments();
        }

        // prompt operator for unavailability reason
        if (!AskUnavailabilityReason())
            return;
    }
    else
    {
        // ask confirmation
        if (QMessageBox::question(this, tr("Confirm"), tr("Are you sure you want to skip this object?"),
                                  QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Cancel)
        {
            return;
        }
    }

    ScanResult = GBMSGUI_RET_SKIP_IMAGE;
}

void AcquisitionForm::on_btnBack_clicked()
{
    // ask confirmation
    if (QMessageBox::question(this, tr("Confirm"), tr("Are you sure you want to go back to previuos object?"),
                              QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Cancel)
    {
        return;
    }

    ScanResult = GBMSGUI_RET_GO_BACK;
}

void AcquisitionForm::on_btnAdvancedView_toggled(bool checked)
{
    // show/hide frame rate
    ui->grpScanningInfo->setVisible(checked);
}

bool AcquisitionForm::DoSegmentation(bool ManualMode, bool IncompleteSlap)
{
    int ret;
    int ImageType;
    int i;
    GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO  *EIL = NULL;
    bool Failed = false;
    unsigned int ImageProcessingOption = 0;
    unsigned int DetectionFailureDiagnostic;
    unsigned int ExtNumSegments = 0;
	bool SingleObjectFlag = false;

	// show message on screen and LCD, because is a long operation
	DisplayMessage(tr("Processing image..."), false);
	QApplication::processEvents();
	if (MSGUIRef->IsLCDPresent())
		GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_PROCESSING, GBMSAPI_VILCD_MSG_AREA);

    if (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_4_LEFT)
        ImageType = GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4;
    if (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_4_RIGHT)
        ImageType = GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4;
    if (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_2_LEFT)
        ImageType = GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_2;
    if (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_2_RIGHT)
        ImageType = GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_2;
    if ((mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_2_THUMBS) || (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_2_INDEXES))
        ImageType = GBFINIMG_INPUT_IMAGE_TYPE_THUMBS_2;

    if (ManualMode)
    {
        EIL = BuildExtInfo(&ExtNumSegments);
        // do not pass an empty external info
        if (ExtNumSegments == 0)
        {
            ResetSegments();
            return false;
        }
    }

	// missing fingers
	if ((ImageType == GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4) || (ImageType == GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4))
	{
		if (CheckUnavailableFingers())
		{
			unsigned char MissingFingerMask = 0;

			if (ImageType == GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4)
			{
				if (MSGUIRef->GetSegmCfgData(0)->UnavailabilityReason != -1)
					MissingFingerMask |= GBFINIMG_MISSING_FINGER_SLAP4_INDEX;
				if (MSGUIRef->GetSegmCfgData(1)->UnavailabilityReason != -1)
					MissingFingerMask |= GBFINIMG_MISSING_FINGER_SLAP4_MIDDLE;
				if (MSGUIRef->GetSegmCfgData(2)->UnavailabilityReason != -1)
					MissingFingerMask |= GBFINIMG_MISSING_FINGER_SLAP4_RING;
				if (MSGUIRef->GetSegmCfgData(3)->UnavailabilityReason != -1)
					MissingFingerMask |= GBFINIMG_MISSING_FINGER_SLAP4_LITTLE;
			}
			else	// GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4
			{
				if (MSGUIRef->GetSegmCfgData(3)->UnavailabilityReason != -1)
					MissingFingerMask |= GBFINIMG_MISSING_FINGER_SLAP4_INDEX;
				if (MSGUIRef->GetSegmCfgData(2)->UnavailabilityReason != -1)
					MissingFingerMask |= GBFINIMG_MISSING_FINGER_SLAP4_MIDDLE;
				if (MSGUIRef->GetSegmCfgData(1)->UnavailabilityReason != -1)
					MissingFingerMask |= GBFINIMG_MISSING_FINGER_SLAP4_RING;
				if (MSGUIRef->GetSegmCfgData(0)->UnavailabilityReason != -1)
					MissingFingerMask |= GBFINIMG_MISSING_FINGER_SLAP4_LITTLE;
			}

			GBFINIMG_Slap4_MissingFingers(MissingFingerMask);
		}
	}
    else    // Slap 2
    {
        if (CheckUnavailableFingers())
        {
            // if both are set as unavailable, fail
            if ((MSGUIRef->GetSegmCfgData(0)->UnavailabilityReason != -1) &&
				(MSGUIRef->GetSegmCfgData(1)->UnavailabilityReason != -1))
			{
				ResetSegments();
				return false;
			}
            ImageType = GBFINIMG_INPUT_IMAGE_TYPE_SINGLE_FINGER;
            SingleObjectFlag = true;
        }
    }

    PostProcessDiag = 0;

    if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_REMOVE_HALO_LATENT))
	{
		if (ToDoFinalization)
			// 1.5.6.0 - lighter processing if after ImageFinalization will be called
			ImageProcessingOption |= GBFINIMG_HALO_LATENT_ELIMINATION;
		else
			// 1.5.4.0 - new option substitute halo latent
			ImageProcessingOption = GBFINIMG_REFINE_FINGERPRINT_CONTRAST_AND_CLEAN_BACKGROUND;
	}

    if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_ROTATE_FINGER))
        ImageProcessingOption |= GBFINIMG_SEGMENT_VERTICAL_ROTATION;

    if (MSGUIRef->IsResolution1000Dpi())
        ImageProcessingOption |= GBFINIMG_1000DPI_PRINT_IMAGE_PROCESSING;

    // not yet supported in linux
#ifdef WIN32
    if (IncompleteSlap)
		ImageProcessingOption |= GBFINIMG_ALLOW_INCOMPLETE_SLAP_SEGMENTS_DATA_GENERATION |
			GBFINIMG_DISABLE_LEFT_RIGHT_HAND_MIXUP_CHECK;
#endif

	setCursor(Qt::WaitCursor);
    ret = GBFINIMG_ProcessImage(FramePtr, FrameSizeX, FrameSizeY,
            ImageType, ImageProcessingOption, SegmImgDescr, SegmSizeX, SegmSizeY,
            &NumSegments, &DetectionFailureDiagnostic, EIL, ExtNumSegments, SegmTempBuff,
            GBFINIMG_MEMORY_BUFFER_SIZE);
	unsetCursor();

    if (EIL)
        free (EIL);

    if (ret != GBFINIMG_NO_ERROR)
    {
        DisplayGBFinImgError(ret);
        return false;
    }

    // if remove halo active, redraw image
    if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_REMOVE_HALO_LATENT))
    {
        DisplayImage();
    }

    if (DetectionFailureDiagnostic)
    {
	    // not yet supported in linux
#ifdef WIN32
		// NEW - if incomplete or ambiguous slap, and option active, try automatic mode
		if ((CheckMask(DetectionFailureDiagnostic, GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP) ||
			CheckMask(DetectionFailureDiagnostic, GBFINIMG_SEGMENTATION_FAIL_INCOMPLETE_SLAP)) &&
			CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_AUTO_INCOMPLETE_SLAP_SEGM)
			&& !IncompleteSlap)	// avoid to call recursively...
		{
			return DoSegmentation(false, true);
		}
#endif

        // add diagnostic messages
        if (CheckMask(DetectionFailureDiagnostic, GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP))
        {
            PostProcessDiag |= POST_PROC_DIAG_SEGM_AMBIGUOUS_SLAP;
        }
        if (CheckMask(DetectionFailureDiagnostic, GBFINIMG_SEGMENTATION_FAIL_INCOMPLETE_SLAP))
        {
            // different message depending if segments are found or not
            //if (Failed)
            if (NumSegments == 0)
                PostProcessDiag |= POST_PROC_DIAG_SEGM_INCOMPLETE_SLAP;
            else
                PostProcessDiag |= POST_PROC_DIAG_SEGM_POSSIBLE_INCOMPLTETE_SLAP;
        }
        if (CheckMask(DetectionFailureDiagnostic, GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX))
        {
            // different message depending if segments are found or not
            //if (Failed)
            if (NumSegments == 0)
                PostProcessDiag |= POST_PROC_DIAG_LEFT_RIGHT_MIXUP;
            else
                PostProcessDiag |= POST_PROC_DIAG_POSSIBLE_LEFT_RIGHT_MIXUP;
        }

		if (IncompleteSlap)
			Failed = false;
		else
			Failed = true;
    }

    if (!ManualMode)
        if (NumSegments == 0)
            Failed = true;

    // check if segments found are identified
    // don't check in case of SingleObjectFlag
    if (!Failed && !SingleObjectFlag)
    {
        for (i=0; i<NumSegments; i++)
        {
            if ((SegmImgDescr[i].Finger == GBFINIMG_FINGER_UNDEFINED)
                || (SegmImgDescr[i].Hand == GBFINIMG_HAND_UNDEFINED))
            {
                // needed manual segmentation
                Failed = true;
            }
        }
    }

    if (Failed)
    {
        ResetSegments();
        return false;
    }

    // read results
    for (i=0; i<NumSegments; i++)
    {
		// PATCH - if object is GBMSAPI_SO_SLAP_2_INDEXES, replace FingerID (for GBFINIMG are thumbs...)
		if (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_2_INDEXES)
			SegmImgDescr[i].Finger = GBFINIMG_FINGER_INDEX;

		int FngIndex;
        // note that elements in list can be in different order; check FingerID
        if (SingleObjectFlag)
        {
            if (MSGUIRef->GetSegmCfgData(0)->UnavailabilityReason == -1)
                FngIndex = 0;
            else if (MSGUIRef->GetSegmCfgData(1)->UnavailabilityReason == -1)
                FngIndex = 1;
            else
				continue;
        }
		else
		{
			FngIndex = SegmDataIndex(SegmImgDescr[i].Finger, SegmImgDescr[i].Hand);
			if (FngIndex == -1)
				continue;
		}

        SegmData[FngIndex].Found = true;
        SegmData[FngIndex].FingerCenterX = SegmImgDescr[i].FingerCenterX;
        SegmData[FngIndex].FingerCenterY = SegmImgDescr[i].FingerCenterY;
        SegmData[FngIndex].RealBoundingRect.setCoords(SegmImgDescr[i].BoundingBoxL, SegmImgDescr[i].BoundingBoxT,
                                                      SegmImgDescr[i].BoundingBoxR, SegmImgDescr[i].BoundingBoxB);
        SegmData[FngIndex].GBQuality = SegmImgDescr[i].Quality;
		SegmData[FngIndex].NFIQQuality = GetNFIQQuality(SegmImgDescr[i].SegmentImagePtr, SegmSizeX, SegmSizeY);

		// add minutiae for sequence check
		MINUTIAE *MinutiaePtr;
		ret = get_minutiae(&MinutiaePtr, SegmImgDescr[i].SegmentImagePtr,
			SegmSizeX, SegmSizeY, IMAGE_BIT_DEPTH, IMAGE_PIXEL_PER_MM);
		if (ret < 0)
			QMessageBox::critical(this, tr("Error"), "get_minutiae error!", QMessageBox::Ok);
		else
			MSGUIRef->AddSequenceCheckItem(GBSegmId2ObjectID(SegmImgDescr[i].Finger, SegmImgDescr[i].Hand), MinutiaePtr);

		// check diagnostic on segment (pattern, inclination)
		if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_DETECT_INCOMPLETE_PATTERN) &&
				(SegmImgDescr[i].PatternCompleteness < MSGUIRef->GetCfgData()->PatternCompletenessThreshold))
		{
			SegmData[FngIndex].IncompletePatternDetected = true;
			PostProcessDiag |= POST_PROC_DIAG_INCOMPLETE_PATTERN;
		}
		if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_DETECT_INVALID_PATTERN) &&
				(SegmImgDescr[i].PatternValidity < MSGUIRef->GetCfgData()->PatternValidityThreshold))
		{
			SegmData[FngIndex].InvalidPatternDetected = true;
			PostProcessDiag |= POST_PROC_DIAG_INVALID_PATTERN;
		}
		if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_DETECT_INCLINATION) &&
				(abs(SegmImgDescr[i].Inclination) > MSGUIRef->GetCfgData()->InclinationThreshold))
		{
			SegmData[FngIndex].InclinationDetected = true;
			PostProcessDiag |= POST_PROC_DIAG_INCLINATION;
		}

		// show diagnostic
		if (SegmData[FngIndex].IncompletePatternDetected || SegmData[FngIndex].InvalidPatternDetected)
			SegmData[FngIndex].BoundingRectItem->SetDiagIcon(QPixmap(":/Graphics/Pattern.bmp"));
		else if (SegmData[FngIndex].InclinationDetected)
			SegmData[FngIndex].BoundingRectItem->SetDiagIcon(QPixmap(":/Graphics/Inclination.bmp"));
    }

    ShowBoundingBoxes();

    return true;
}

GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO* AcquisitionForm::BuildExtInfo(unsigned int *Num)
{
    GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO* pInfo;
    int i, n;

    // count the segments
    n = 0;
    for (i=0; i<SegmData.count(); i++)
        if (SegmData[i].FingerCenterX != -1)
            n++;

    *Num = n;

    if (n == 0)
        return NULL;

    pInfo = (GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO*)malloc(n*sizeof(GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO));
    if (pInfo)
    {
        memset(pInfo, 0, sizeof(GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO)*n);

        n = 0;
        for (i=0; i<SegmData.count(); i++)
        {
            if (SegmData[i].FingerCenterX != -1)
            {
                pInfo[n].FingerCenterX = SegmData[i].FingerCenterX;
                pInfo[n].FingerCenterY = SegmData[i].FingerCenterY;
                pInfo[n].Finger = SegmData[i].Finger;
                pInfo[n].Hand = SegmData[i].Hand;

                n++;
            }
        }
    }

    return pInfo;
}


// return the maximun size requested for segments, used for segmentation
void AcquisitionForm::GetMaxSegmentsSize(int *SizeX, int *SizeY)
{
    int SegmNum;
    if ((mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_4_LEFT) ||
            (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_4_RIGHT))
        SegmNum = 4;
    else
        SegmNum = 2;

    *SizeX = *SizeY = 0;
    for (int i = 0; i < SegmNum; i++)
    {
        if (MSGUIRef->GetSegmCfgData(i)->UnavailabilityReason == -1)
        {
            if (MSGUIRef->GetSegmCfgData(i)->ImageSizeX > *SizeX)
                *SizeX = MSGUIRef->GetSegmCfgData(i)->ImageSizeX;
            if (MSGUIRef->GetSegmCfgData(i)->ImageSizeY > *SizeY)
                *SizeY = MSGUIRef->GetSegmCfgData(i)->ImageSizeY;
        }
    }
}

void AcquisitionForm::ResetSegments()
{
    int i;

    NumSegments = 0;
    for (i = 0; i < SegmData.count(); i++)
    {
        SegmData[i].Found = false;
        SegmData[i].FingerCenterX = -1;
        SegmData[i].FingerCenterY = -1;
        SegmData[i].RealBoundingRect.setCoords(0, 0, 0, 0);
        SegmData[i].BoundingRectItem->setVisible(false);
        SegmData[i].BoundingRectItem->SetDiagIcon(NULL);
        SegmData[i].GBQuality = 0;
        SegmData[i].NFIQQuality = 0;
        SegmData[i].IncompletePatternDetected = false;
        SegmData[i].InvalidPatternDetected = false;
        SegmData[i].InclinationDetected = false;
        SegmData[i].UnavailabilityReason = -1;
    }
}

bool AcquisitionForm::InitSegmImgDescr()
{
    int i;

    // get maximum size requested for segments
    GetMaxSegmentsSize(&SegmSizeX, &SegmSizeY);

    // alloc memory for segments
    for (i=0; i< SEGM_IMG_MAX_NUM; i++)
    {
        if (SegmImgDescr[i].SegmentImagePtr)
            free (SegmImgDescr[i].SegmentImagePtr);

        SegmImgDescr[i].SegmentImagePtr = (unsigned char *)calloc(SegmSizeX*SegmSizeY, sizeof(unsigned char));
        if (!(SegmImgDescr[i].SegmentImagePtr))
        {
            MSGUIRef->SetError(tr("Insufficient memory"), GBMSGUI_ERR_INSUFFICIENT_MEMORY);
            return false;
        }
    }

    return true;
}


void AcquisitionForm::FreeSegmImgDescr()
{
    int i;

    for (i=0; i< SEGM_IMG_MAX_NUM; i++)
    {
        if ((SegmImgDescr[i].SegmentImagePtr))
            free((SegmImgDescr[i].SegmentImagePtr));
    }
}

void AcquisitionForm::SetSegments()
{
    int i;
    SegmentData *Item;

    for (i=0; i<SegmData.count(); i++)
    {
        if (SegmData[i].BoundingRectItem)
            delete SegmData[i].BoundingRectItem;
    }
    SegmData.clear();

    // according to ScannedObjectID, set data for segments
    if (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_2_THUMBS)
    {
        Item = new SegmentData;
        Item->Finger = GBFINIMG_FINGER_THUMB;
        Item->Hand = GBFINIMG_HAND_LEFT;
		Item->FingerText = tr("Left");
        SegmData.append(*Item);

        Item = new SegmentData;
        Item->Finger = GBFINIMG_FINGER_THUMB;
        Item->Hand = GBFINIMG_HAND_RIGHT;
		Item->FingerText = tr("Right");
        SegmData.append(*Item);
    }
    else if (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_4_LEFT)
    {
        Item = new SegmentData;
        Item->Finger = GBFINIMG_FINGER_LITTLE;
        Item->Hand = GBFINIMG_HAND_LEFT;
		Item->FingerText = tr("Little");
        SegmData.append(*Item);

        Item = new SegmentData;
        Item->Finger = GBFINIMG_FINGER_RING;
        Item->Hand = GBFINIMG_HAND_LEFT;
		Item->FingerText = tr("Ring");
        SegmData.append(*Item);

        Item = new SegmentData;
        Item->Finger = GBFINIMG_FINGER_MIDDLE;
        Item->Hand = GBFINIMG_HAND_LEFT;
		Item->FingerText = tr("Middle");
        SegmData.append(*Item);

        Item = new SegmentData;
        Item->Finger = GBFINIMG_FINGER_INDEX;
        Item->Hand = GBFINIMG_HAND_LEFT;
		Item->FingerText = tr("Index");
        SegmData.append(*Item);
    }
    else if (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_4_RIGHT)
    {
        Item = new SegmentData;
        Item->Finger = GBFINIMG_FINGER_INDEX;
        Item->Hand = GBFINIMG_HAND_RIGHT;
		Item->FingerText = tr("Index");
        SegmData.append(*Item);

        Item = new SegmentData;
        Item->Finger = GBFINIMG_FINGER_MIDDLE;
        Item->Hand = GBFINIMG_HAND_RIGHT;
		Item->FingerText = tr("Middle");
        SegmData.append(*Item);

        Item = new SegmentData;
        Item->Finger = GBFINIMG_FINGER_RING;
        Item->Hand = GBFINIMG_HAND_RIGHT;
		Item->FingerText = tr("Ring");
        SegmData.append(*Item);

        Item = new SegmentData;
        Item->Finger = GBFINIMG_FINGER_LITTLE;
        Item->Hand = GBFINIMG_HAND_RIGHT;
		Item->FingerText = tr("Little");
        SegmData.append(*Item);
    }
    else if (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_2_LEFT)
    {
        Item = new SegmentData;
        Item->Finger = GBFINIMG_FINGER_MIDDLE;
        Item->Hand = GBFINIMG_HAND_LEFT;
		Item->FingerText = tr("Middle");
        SegmData.append(*Item);

        Item = new SegmentData;
        Item->Finger = GBFINIMG_FINGER_INDEX;
        Item->Hand = GBFINIMG_HAND_LEFT;
		Item->FingerText = tr("Index");
        SegmData.append(*Item);
    }
    else if (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_2_RIGHT)
    {
        Item = new SegmentData;
        Item->Finger = GBFINIMG_FINGER_INDEX;
        Item->Hand = GBFINIMG_HAND_RIGHT;
		Item->FingerText = tr("Index");
        SegmData.append(*Item);

        Item = new SegmentData;
        Item->Finger = GBFINIMG_FINGER_MIDDLE;
        Item->Hand = GBFINIMG_HAND_RIGHT;
		Item->FingerText = tr("Middle");
        SegmData.append(*Item);
    }
    else if (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_2_INDEXES)
    {
        Item = new SegmentData;
        Item->Finger = GBFINIMG_FINGER_INDEX;
        Item->Hand = GBFINIMG_HAND_LEFT;
		Item->FingerText = tr("Left");
        SegmData.append(*Item);

        Item = new SegmentData;
        Item->Finger = GBFINIMG_FINGER_INDEX;
        Item->Hand = GBFINIMG_HAND_RIGHT;
		Item->FingerText = tr("Right");
        SegmData.append(*Item);
    }

    for (i=0; i<SegmData.count(); i++)
    {
        SegmData[i].RealBoundingRect.setCoords(0, 0, 0, 0);

        SegmData[i].BoundingRectItem = new QClipRectItem(Qt::magenta, 1);
        SegmData[i].BoundingRectItem->SetSizeOption(AllowResize);
        scene.addItem(SegmData[i].BoundingRectItem);
        SegmData[i].BoundingRectItem->setVisible(false);
        SegmData[i].BoundingRectItem->SetText(SegmData[i].FingerText);

        SegmData[i].FingerCenterX = -1;
        SegmData[i].FingerCenterY = -1;
        SegmData[i].NFIQQuality = -1;
        SegmData[i].GBQuality = -1;
        SegmData[i].QualityColor = Qt::black;
        SegmData[i].IncompletePatternDetected = false;
        SegmData[i].InvalidPatternDetected = false;
        SegmData[i].InclinationDetected = false;
        SegmData[i].UnavailabilityReason = -1;
        SegmData[i].Found = false;
    }
}

int AcquisitionForm::SegmDataIndex(int FingID, int HandID)
{
    int i;

    for (i=0; i< SegmData.count(); i++)
    {
        if ((SegmData[i].Finger == FingID) && (SegmData[i].Hand == HandID))
            return i;
    }

    return -1;
}

int AcquisitionForm::SegmImgDescrIndex(int FingID, int HandID)
{
    int i;

    for (i=0; i< NumSegments; i++)
    {
        if ((SegmImgDescr[i].Finger == FingID) && (SegmImgDescr[i].Hand == HandID))
            return i;
    }

    return -1;
}

int AcquisitionForm::GBSegmId2ObjectID(int Finger, int Hand)
{
    if ((Hand == GBFINIMG_HAND_LEFT) && (Finger == GBFINIMG_FINGER_THUMB))
        return GBMSAPI_SO_FLAT_LEFT_THUMB;

    if ((Hand == GBFINIMG_HAND_LEFT) && (Finger == GBFINIMG_FINGER_INDEX))
        return GBMSAPI_SO_FLAT_LEFT_INDEX;

    if ((Hand == GBFINIMG_HAND_LEFT) && (Finger == GBFINIMG_FINGER_MIDDLE))
        return GBMSAPI_SO_FLAT_LEFT_MIDDLE;

    if ((Hand == GBFINIMG_HAND_LEFT) && (Finger == GBFINIMG_FINGER_RING))
        return GBMSAPI_SO_FLAT_LEFT_RING;

    if ((Hand == GBFINIMG_HAND_LEFT) && (Finger == GBFINIMG_FINGER_LITTLE))
        return GBMSAPI_SO_FLAT_LEFT_LITTLE;

    if ((Hand == GBFINIMG_HAND_RIGHT) && (Finger == GBFINIMG_FINGER_THUMB))
        return GBMSAPI_SO_FLAT_RIGHT_THUMB;

    if ((Hand == GBFINIMG_HAND_RIGHT) && (Finger == GBFINIMG_FINGER_INDEX))
        return GBMSAPI_SO_FLAT_RIGHT_INDEX;

    if ((Hand == GBFINIMG_HAND_RIGHT) && (Finger == GBFINIMG_FINGER_MIDDLE))
        return GBMSAPI_SO_FLAT_RIGHT_MIDDLE;

    if ((Hand == GBFINIMG_HAND_RIGHT) && (Finger == GBFINIMG_FINGER_RING))
        return GBMSAPI_SO_FLAT_RIGHT_RING;

    if ((Hand == GBFINIMG_HAND_RIGHT) && (Finger == GBFINIMG_FINGER_LITTLE))
        return GBMSAPI_SO_FLAT_RIGHT_LITTLE;

    return 0;
}

void AcquisitionForm::ShowBoundingBoxes()
{
    QRect BoundingBoxRect;

    // calculate scale factor
    double ScaleFactor = (double)scene.width()/(double)ImageSizeX;

    for (int i=0; i<SegmData.count(); i++)
    {
        if (!SegmData[i].Found)
            continue;
        BoundingBoxRect.setLeft(floor((double)SegmData[i].RealBoundingRect.left() *ScaleFactor + 0.5));
        BoundingBoxRect.setTop(floor((double)SegmData[i].RealBoundingRect.top()*ScaleFactor + 0.5));
        BoundingBoxRect.setWidth(floor((double)SegmData[i].RealBoundingRect.width()*ScaleFactor + 0.5));
        BoundingBoxRect.setHeight(floor((double)SegmData[i].RealBoundingRect.height()*ScaleFactor + 0.5));

        SegmData[i].BoundingRectItem->setRect(0, 0, BoundingBoxRect.width(), BoundingBoxRect.height());
        SegmData[i].BoundingRectItem->setPos(BoundingBoxRect.x(), BoundingBoxRect.y());
        SegmData[i].BoundingRectItem->setVisible(true);
        SegmData[i].BoundingRectItem->SetEnabled(true);

        // notify changes
        SegmBoxChangeSignalMapper->setMapping(SegmData[i].BoundingRectItem, i);
        connect(SegmData[i].BoundingRectItem, SIGNAL(Changed()), SegmBoxChangeSignalMapper, SLOT(map()));
    }
}

// Evaluate quality of a segment, based on currently selected algorithm
bool AcquisitionForm::EvalSegmentIAFISQuality(int Index)
{
    bool Result = false;
    int Threshold1, Threshold2;

    // read thresholsd for the segment from configuration
    Threshold1 = MSGUIRef->GetSegmCfgData(Index)->QualityThreshold1;
    Threshold2 = MSGUIRef->GetSegmCfgData(Index)->QualityThreshold2;

    if ((Threshold1 == 0) || (Threshold2 == 0))
    {
        // no tresholds, no evaluation
        SegmData[Index].QualityColor = Qt::lightGray;
        Result = true;
    }
    else
    {
        // NFIQ quality
        if (MSGUIRef->GetCfgData()->QualityAlgorithm == GBMSGUI_NFIQ_ALGORITHM)
        {
            if (SegmData[Index].NFIQQuality <= Threshold1)
            {
                SegmData[Index].QualityColor = Qt::green;
                Result = true;
            }
            else if ((SegmData[Index].NFIQQuality > Threshold1) &&
                (SegmData[Index].NFIQQuality <= Threshold2))
            {
                SegmData[Index].QualityColor = Qt::yellow;
                Result = false;
            }
            else
            {
                SegmData[Index].QualityColor = Qt::red;
                Result = false;
            }
        }
        else
        {
            // GB Quality
            if (SegmData[Index].GBQuality >= Threshold2)
            {
                SegmData[Index].QualityColor = Qt::green;
                Result = true;
            }
            else if ((SegmData[Index].GBQuality < Threshold2) &&
                (SegmData[Index].GBQuality >= Threshold1))
            {
                SegmData[Index].QualityColor = Qt::yellow;
                Result = false;
            }
            else
            {
                SegmData[Index].QualityColor = Qt::red;
                Result = false;
            }
        }
    }

    // display value
    if (MSGUIRef->GetCfgData()->QualityAlgorithm == GBMSGUI_NFIQ_ALGORITHM)
        SegmData[Index].BoundingRectItem->SetQuality(SegmData[Index].NFIQQuality, SegmData[Index].QualityColor);
    else
        SegmData[Index].BoundingRectItem->SetQuality(SegmData[Index].GBQuality, SegmData[Index].QualityColor);

    return Result;
}

void AcquisitionForm::PopUpSegmMenu(const QPoint& pos) // this is a slot
{
    if (Acquiring)
    //if (Acquiring || !GBMSGUI::IsSlap(mScannedObject->ScannedObjectID) ||
    //    !CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_SEGMENTATION))
        return;

    // save menu point (relative to scene)
    SegmMenuPoint = ui->graphicsView->mapToScene(pos);
    // check that point is inside image
    if (!scene.sceneRect().contains(SegmMenuPoint))
        return;

    // for most widgets
    QPoint globalPos = ui->graphicsView->mapToGlobal(pos);

    int i;
    QAction *action;

    // check if cursor is over a segment
    for (i = 0; i < SegmData.count(); i++)
    {
        if (SegmData[i].Found &&
                SegmData[i].BoundingRectItem->isVisible() &&
                SegmData[i].BoundingRectItem->contains(SegmData[i].BoundingRectItem->mapFromScene(SegmMenuPoint)))
        {
            // popup menu for this segment
            QMenu EditSegmMenu;

            if (GBMSGUI::IsSlap(mScannedObject->ScannedObjectID))
            {
                action = EditSegmMenu.addAction(tr("View segment"));
                ViewSegmentSignalMapper->setMapping(action, i);
                connect(action, SIGNAL(triggered()), ViewSegmentSignalMapper, SLOT(map()));

                action = EditSegmMenu.addAction(tr("Remove segment"));
                RemoveSegmentSignalMapper->setMapping(action, i);
                connect(action, SIGNAL(triggered()), RemoveSegmentSignalMapper, SLOT(map()));
            }
            else if (GBMSGUI::IsSingleFinger(mScannedObject->ScannedObjectID))
            {
                // menu for clear outside
                action = EditSegmMenu.addAction(tr("Clear outside bounding box"));
                connect(action, SIGNAL(triggered()), this, SLOT(OnClearOutside()));
            }

            EditSegmMenu.exec(globalPos);
            return;
        }
    }

    // add menu for segments not yet found
    QMenu FindSegmMenu;
    for (i = 0; i < SegmData.count(); i++)
    {
        if (!SegmData[i].Found && (MSGUIRef->GetSegmCfgData(i)->UnavailabilityReason == -1))
        {
            action = FindSegmMenu.addAction(SegmData[i].FingerText);
            FindSegmentSignalMapper->setMapping(action, i);
            connect(action, SIGNAL(triggered()), FindSegmentSignalMapper, SLOT(map()));
        }
    }

	// not yet supported in linux
#ifdef WIN32
	// if incomplete slap, add possibility to try automatic
	if (CheckMask(PostProcessDiag, POST_PROC_DIAG_SEGM_INCOMPLETE_SLAP) ||
		CheckMask(PostProcessDiag, POST_PROC_DIAG_SEGM_POSSIBLE_INCOMPLTETE_SLAP) ||
		CheckMask(PostProcessDiag, POST_PROC_DIAG_SEGM_AMBIGUOUS_SLAP))
	{
		FindSegmMenu.addSeparator();
		action = FindSegmMenu.addAction(tr("Incomplete slap segmentation"));
        connect(action, SIGNAL(triggered()), this, SLOT(OnIncompleteSlap()));
	}
#endif

    FindSegmMenu.exec(globalPos);
}

void AcquisitionForm::OnSegmentSelected(int Index) // this is a slot
{
    // convert menu point to real image coordinate
    double ScaleFactor = (double)ImageSizeX/(double)scene.width();
    QPoint RealPoint(floor(SegmMenuPoint.x()*ScaleFactor + 0.5), floor(SegmMenuPoint.y()*ScaleFactor + 0.5));

    SegmData[Index].FingerCenterX = RealPoint.x();
    SegmData[Index].FingerCenterY = RealPoint.y();

    // perform segmentation with manually added info
    SegmentationResult = DoSegmentation(true, false);
    CheckResult();
}

void AcquisitionForm::OnViewSegment(int Index) // this is a slot
{
    int i = SegmImgDescrIndex(SegmData[Index].Finger, SegmData[Index].Hand);
    if (i == -1)
        return;

    // create bitmap for segment
    QImage ImgSegment(SegmSizeX, SegmSizeY, QImage::Format_Indexed8);
	unsigned char *pImage = SegmImgDescr[i].SegmentImagePtr;
	for (int y=0; y<SegmSizeY; y++, pImage += SegmSizeX)
		memcpy(ImgSegment.scanLine(y), pImage, SegmSizeX);
    ImgSegment.setColorTable(GrayTable);

    ViewSegmForm Form;
    Form.SetImage(ImgSegment, SegmData[Index].FingerText);
    Form.exec();
}

void AcquisitionForm::OnRemoveSegment(int Index) // this is a slot
{
    // remove segment
    SegmData[Index].Found = false;
    SegmData[Index].FingerCenterX = -1;
    SegmData[Index].FingerCenterY = -1;
    SegmData[Index].BoundingRectItem->setVisible(false);

    // perform segmentation again
    SegmentationResult = DoSegmentation(true, false);
    CheckResult();
}


int AcquisitionForm::GetSegmentationResult(int Index, int QualityAlgorithmID,
    bool *SegmentAvailable, int *Left, int *Top, int *Right, int *Bottom,
    int *UnavailabilityReason, int *FingerprintQuality,
    unsigned char *ImagePtr, int *ImageSizeX, int *ImageSizeY)
{
    int SegmentIndex;
    // in case of left slap, reverse index
    if ((mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_4_LEFT)
        || (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_2_LEFT))
        SegmentIndex = (SegmData.count()-1)  - Index;
    else
        SegmentIndex = Index;

	// if segmentation is not active, SegmData is empty...
	if (SegmentIndex >= SegmData.count())
	{
        *SegmentAvailable = false;
        *Left = -1;
        *Top = -1;
        *Right = -1;
        *Bottom = -1;
        *FingerprintQuality = -1;
        *UnavailabilityReason = -1;
        *ImageSizeX = -1;
        *ImageSizeY = -1;

        return GBMSGUI_RET_SUCCESS;
	}

    if (SegmData[SegmentIndex].Found)
    {
        *SegmentAvailable = true;
        *Left = SegmData[SegmentIndex].RealBoundingRect.left();
        *Top = SegmData[SegmentIndex].RealBoundingRect.top();
        *Right = SegmData[SegmentIndex].RealBoundingRect.right();
        *Bottom = SegmData[SegmentIndex].RealBoundingRect.bottom();

        if (QualityAlgorithmID == GBMSGUI_NFIQ_ALGORITHM)
            *FingerprintQuality = SegmData[SegmentIndex].NFIQQuality;
        else
            *FingerprintQuality = SegmData[SegmentIndex].GBQuality;
        *UnavailabilityReason = -1;
        if (ImagePtr != NULL)
        {
            // get the corresponding index of SegmImgDescr
            int i = SegmImgDescrIndex(SegmData[SegmentIndex].Finger, SegmData[SegmentIndex].Hand);

            int ReqSegmSizeX = MSGUIRef->GetSegmCfgData(SegmentIndex)->ImageSizeX;
            int ReqSegmSizeY = MSGUIRef->GetSegmCfgData(SegmentIndex)->ImageSizeY;
            // clip the segment image if necessary
            if ((ReqSegmSizeX < SegmSizeX) || (ReqSegmSizeY < SegmSizeY))
            {
                // align to top, center horizontally
                QRect Rect;
                Rect.setTop(0);
                Rect.setLeft(SegmSizeX/2 - ReqSegmSizeX/2);
                Rect.setWidth(ReqSegmSizeX);
                Rect.setHeight(ReqSegmSizeY);
                // clip
                CutImage(SegmImgDescr[i].SegmentImagePtr, SegmSizeX, SegmSizeY,
                         Rect.left(), Rect.top(), ImagePtr, ReqSegmSizeX, ReqSegmSizeY);
                *ImageSizeX = ReqSegmSizeX;
                *ImageSizeY = ReqSegmSizeY;
            }
            else
            {
                *ImageSizeX = SegmSizeX;
                *ImageSizeY = SegmSizeY;
                memcpy(ImagePtr, SegmImgDescr[i].SegmentImagePtr, SegmSizeX*SegmSizeY);
            }
        }
        else
        {
            *ImageSizeX = -1;
            *ImageSizeY = -1;
        }
    }
    else
    {
        *SegmentAvailable = false;
        *Left = -1;
        *Top = -1;
        *Right = -1;
        *Bottom = -1;
        *FingerprintQuality = -1;
        *UnavailabilityReason = SegmData[SegmentIndex].UnavailabilityReason;
        *ImageSizeX = -1;
        *ImageSizeY = -1;
    }

    return GBMSGUI_RET_SUCCESS;

}

void AcquisitionForm::DisplayGBFinImgError(int ErrCode)
{
    QString Msg;

    switch (ErrCode)
    {
        case GBFINIMG_ERROR_PARAMETER:
            Msg = "GBFINIMG_ProcessImage: parameter error";
            break;

        case GBFINIMG_ERROR_UNAVAILABLE_OPTION:
            Msg = "GBFINIMG_ProcessImage: unavailable option";
            break;

        case GBFINIMG_ERROR_NO_MEMORY:
            Msg = "GBFINIMG_ProcessImage: insufficient memory";
            break;

		case GBFINIMG_ERROR_LICENSE_NOT_GRANTED:	// 1.5.7.0
            Msg = "GBFINIMG_ProcessImage: license not granted";
            break;
    }

    QMessageBox::critical(this, "GBFINIMG", Msg, QMessageBox::Ok);
}

bool AcquisitionForm::CheckSequence()
{
    int Index = MSGUIRef->FindSequenceCheckSameFinger(mScannedObject->ScannedObjectID);
    if (Index == -1)
    {
        // item not found, wrong sequence
        AddMessageToList(tr("Acquisition order does not allow sequence check"), QIcon(":/Icons/Warning.png"));
        if (MSGUIRef->IsLCDPresent())
            GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_SEQUENCE_CHECK_FAILED,
                GBMSAPI_VILCD_MSG_AREA);
        // can't compare, return true
        return true;
    }

    MINUTIAE *RefMinutiae = MSGUIRef->GetItemMinutiae(Index);

    // get minutiae of current object
    MINUTIAE *Minutiae;
    int ret = get_minutiae(&Minutiae, FramePtr, FrameSizeX, FrameSizeY, IMAGE_BIT_DEPTH, IMAGE_PIXEL_PER_MM);
    if (ret < 0)
    {
        QMessageBox::critical(this, tr("Error"), "get_minutiae error!", QMessageBox::Ok);
        return false;
    }

    // compare minutiae
    int Score = boz_direct_call(200, Minutiae, RefMinutiae);

    free_minutiae(Minutiae);

    // minimum match score set to 24
    if (Score < 24)
    {
        DisplayMessage(tr("Sequence check: low matching score.\nPlease, check the requested object."), true);
        if (MSGUIRef->IsLCDPresent())
            GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_SEQUENCE_CHECK_FAILED,
                GBMSAPI_VILCD_MSG_AREA);
        return false;
    }

    return true;
}

void AcquisitionForm::CutImage(unsigned char* SrcImg, unsigned int SrcSizeX, unsigned int SrcSizeY,
                          unsigned int CutStartX, unsigned int CutStartY,
                          unsigned char* DstImg, unsigned int DstSizeX, unsigned int DstSizeY)
{
    int i;
    BYTE *ptrSrc, *ptrDst;

    ptrSrc = SrcImg+CutStartY*SrcSizeX+CutStartX;
    ptrDst = DstImg;

    for(i=0; i < DstSizeY; i++)
    {
        memcpy(ptrDst, ptrSrc, DstSizeX);
        ptrSrc += SrcSizeX;
        ptrDst += DstSizeX;
    }
}

// check if some fingers are set as unavailable from the application
bool AcquisitionForm::CheckUnavailableFingers()
{
    bool Result = false;
    int SegmNum;
    if (GBMSAPI_GetTypeFromObject(mScannedObject->ScannedObjectID) == GBMSAPI_SBT_FLAT_SLAP_4)
        SegmNum = 4;
    else
        SegmNum = 2;

    for (int i = 0; i < SegmNum; i++)
    {
        if (MSGUIRef->GetSegmCfgData(i)->UnavailabilityReason != -1)
        {
            SegmData[i].UnavailabilityReason = MSGUIRef->GetSegmCfgData(i)->UnavailabilityReason;
            Result = true;
        }
    }

    return Result;
}

bool AcquisitionForm::ProcessImage()
{
    int ret;
    int ImageType;
    bool Failed = false;
    GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR ImgDescr[1];
    unsigned int Num = 0;
    unsigned int ImageProcessingOption = 0;
    unsigned int DetectionFailureDiagnostic;

    // show message on screen and LCD, because is a long operation
    DisplayMessage(tr("Processing image..."), false);
    QApplication::processEvents();
    if (MSGUIRef->IsLCDPresent())
        GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_PROCESSING, GBMSAPI_VILCD_MSG_AREA);

    memset(&(ImgDescr[0]), 0, sizeof(GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR));

    // prepare input data
    if (GBMSGUI::IsSingleFinger(mScannedObject->ScannedObjectID))
        ImageType = GBFINIMG_INPUT_IMAGE_TYPE_SINGLE_FINGER;
    else if (mScannedObject->ScannedObjectID == GBMSAPI_SO_LOWER_HALF_PALM_RIGHT)
        ImageType = GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_HALF_PALM;
    else if (mScannedObject->ScannedObjectID == GBMSAPI_SO_LOWER_HALF_PALM_LEFT)
        ImageType = GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_HALF_PALM;
    else if (mScannedObject->ScannedObjectID == GBMSAPI_SO_WRITER_PALM_LEFT)
        ImageType = GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_WRITER_PALM;
    else if (mScannedObject->ScannedObjectID == GBMSAPI_SO_WRITER_PALM_RIGHT)
        ImageType = GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_WRITER_PALM;

    PostProcessDiag = 0;
    DetectionFailureDiagnostic = 0;

    if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_REMOVE_HALO_LATENT))
	{
		if (ToDoFinalization)
		{
			// 1.5.6.0 - lighter processing if after ImageFinalization will be called
			ImageProcessingOption |= GBFINIMG_HALO_LATENT_ELIMINATION;
			if (GBMSGUI::IsPalm(mScannedObject->ScannedObjectID) && !GBMSGUI::IsUpperPalm(mScannedObject->ScannedObjectID))
				ImageProcessingOption |= GBFINIMG_PALM_PRINT_IMAGE_DELETE_FINGER_SEGMENTS;
		}
		else
			// 1.5.4.0 - new option substitute halo latent
			ImageProcessingOption = GBFINIMG_REFINE_FINGERPRINT_CONTRAST_AND_CLEAN_BACKGROUND;
	}
    if (MSGUIRef->IsResolution1000Dpi())
        ImageProcessingOption |= GBFINIMG_1000DPI_PRINT_IMAGE_PROCESSING;
    if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_PALM_PRINT_QUALITY_CALCULATION))
        ImageProcessingOption |= GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION;

    setCursor(Qt::WaitCursor);
    ret = GBFINIMG_ProcessImage(FramePtr, FrameSizeX, FrameSizeY,
            ImageType, ImageProcessingOption, ImgDescr, FrameSizeX, FrameSizeY,
            &Num, &DetectionFailureDiagnostic, NULL, 0,
            SegmTempBuff, GBFINIMG_MEMORY_BUFFER_SIZE);
    unsetCursor();

    if (ret != GBFINIMG_NO_ERROR)
    {
        DisplayGBFinImgError(ret);
        return false;
    }

    if (Num != 1)
    {
        QMessageBox::critical(this, "Error", "GBFINIMG_ProcessImage: no image found", QMessageBox::Ok);
        return false;
    }

    // if remove halo active, redraw image
    if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_REMOVE_HALO_LATENT))
    {
        DisplayImage();
    }

    // check diagnostic
    if (DetectionFailureDiagnostic)
    {
        if (CheckMask(DetectionFailureDiagnostic, GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX))
            PostProcessDiag |= POST_PROC_DIAG_LEFT_RIGHT_MIXUP;

        // add diagnostic messages
        if (CheckMask(DetectionFailureDiagnostic, GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP))
        {
            PostProcessDiag |= POST_PROC_DIAG_SEGM_AMBIGUOUS_SLAP;
        }
    }

    // in case of ambiguous slap, show the segment bounding box
    if (GBMSGUI::IsSingleFinger(mScannedObject->ScannedObjectID) &&
        CheckMask(DetectionFailureDiagnostic, GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP))
    {
        SegmData.clear();
        SegmentData Segm;
        //Segm.FingerID = GBMSGUI.ScannedObjectIDToAN2KFingerID(ScannedObject.ScannedObjectID);
        SegmData.append(Segm);
        SegmData[0].Found = true;
        SegmData[0].FingerCenterX = ImgDescr[0].FingerCenterX;
        SegmData[0].FingerCenterY = ImgDescr[0].FingerCenterY;
        SegmData[0].BoundingRectItem = new QClipRectItem(Qt::magenta, 1);
        SegmData[0].BoundingRectItem->SetSizeOption(AllowResize);
        scene.addItem(SegmData[0].BoundingRectItem);
        SegmData[0].BoundingRectItem->setVisible(false);
        SegmData[0].RealBoundingRect.setCoords(ImgDescr[0].BoundingBoxL, ImgDescr[0].BoundingBoxT,
                                                      ImgDescr[0].BoundingBoxR, ImgDescr[0].BoundingBoxB);
        ShowBoundingBoxes();

        // only for rolled, if option active, clear outside automatically
        if (GBMSGUI::IsRolled(mScannedObject->ScannedObjectID) &&
            CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_AUTO_CLEAR_OUTSIDE_ROLL))
        {
            ClearOutsideBoundingBox();
            // remove diagnostic
            PostProcessDiag &= ~POST_PROC_DIAG_SEGM_AMBIGUOUS_SLAP;
        }
        else
        {
            // show message
            AddMessageToList(tr("Right click on bounding box for context menu"), QIcon(":/Icons/Info.png"));
        }
    }

    mScannedObject->GBQuality = ImgDescr[0].Quality;
    // for palm objects, is valid only if option is active
    if (GBMSGUI::IsPalm(mScannedObject->ScannedObjectID))
    {
        if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_PALM_PRINT_QUALITY_CALCULATION))
            // convert also to NFIQ quality
            mScannedObject->NFIQQuality = GBQualityToNFIQ(mScannedObject->GBQuality);
        else
            mScannedObject->GBQuality = -1;
    }

    // read results
    // check diagnostic (pattern, inclination)
    if (GBMSGUI::IsLowerPalm(mScannedObject->ScannedObjectID) && MSGUIRef->IsPalmCompletenessSupported())
    {
        if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_DETECT_INCOMPLETE_PATTERN) &&
            PalmCompleteness < MSGUIRef->GetCfgData()->LowerPalmCompletenessThreshold2)
        {
            IncompletePatternDetected = true;
            PostProcessDiag |= POST_PROC_DIAG_INCOMPLETE_PATTERN;
        }
    }
    else
    {
        if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_DETECT_INCOMPLETE_PATTERN) &&
                (ImgDescr[0].PatternCompleteness < MSGUIRef->GetCfgData()->PatternCompletenessThreshold))
        {
            IncompletePatternDetected = true;
            PostProcessDiag |= POST_PROC_DIAG_INCOMPLETE_PATTERN;
        }
    }

    // for Palm, PatternValidity is present only if NW_GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION
    // option is activated;
    // for writer palm, no PatternValidity evaluation if left-right mix-up diagnostic is present
    if (!(
        (GBMSGUI::IsPalm(mScannedObject->ScannedObjectID) && !CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_PALM_PRINT_QUALITY_CALCULATION))
        || (GBMSGUI::IsWritersPalm(mScannedObject->ScannedObjectID) && (CheckMask(DetectionFailureDiagnostic, GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX)
        || CheckMask(DetectionFailureDiagnostic, GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP)))))
    {
        if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_DETECT_INVALID_PATTERN) &&
                (ImgDescr[0].PatternValidity < MSGUIRef->GetCfgData()->PatternValidityThreshold))
        {
            InvalidPatternDetected = true;
            PostProcessDiag |= POST_PROC_DIAG_INVALID_PATTERN;
        }
    }

    if (CheckMask(MSGUIRef->GetAcquisitionOptions(), GBMSGUI_ACQUISITION_OPTION_DETECT_INCLINATION) &&
            (abs(ImgDescr[0].Inclination) > MSGUIRef->GetCfgData()->InclinationThreshold))
    {
        InclinationDetected = true;
        PostProcessDiag |= POST_PROC_DIAG_INCLINATION;
    }

    return true;
}

// try to center clipping rectangle with bounding boxes
void AcquisitionForm::CenterClipWithBoxes()
{
    int i;
    int extTop, extLeft, extRight, extBottom;   // extremes

    extTop = (int)FrameSizeX + 1;
    extLeft = (int)FrameSizeY + 1;
    extRight = -1;
    extBottom = -1;

    // find bounding boxes extremes
    for (i = 0; i < SegmData.count(); i++)
    {
        if (!SegmData[i].Found)
            continue;

        // top
        if (SegmData[i].RealBoundingRect.top() < extTop)
            extTop = SegmData[i].RealBoundingRect.top();
        // left
        if (SegmData[i].RealBoundingRect.left() < extLeft)
            extLeft = SegmData[i].RealBoundingRect.left();
        // right
        if (SegmData[i].RealBoundingRect.right() > extRight)
            extRight = SegmData[i].RealBoundingRect.right();
        // bottom
        if (SegmData[i].RealBoundingRect.bottom() > extBottom)
            extBottom = SegmData[i].RealBoundingRect.bottom();
    }

    if (extTop < 0)
        extTop = 0;
    if (extLeft < 0)
        extLeft = 0;

    // center clipping, with the same top
    RealClipRect.moveTop(extTop);
    RealClipRect.moveLeft((extRight + extLeft) / 2 - RealClipRect.width() / 2);

    if (RealClipRect.right() > FrameSizeX)
        RealClipRect.moveLeft(FrameSizeX - RealClipRect.width());

    if (RealClipRect.bottom() > FrameSizeY)
        RealClipRect.moveTop(FrameSizeY - RealClipRect.height());

    // update clipping in view
    ShowClipRectangle();
}

bool AcquisitionForm::CheckSegmentBoundingBoxPositions()
{
    int i;
    bool result = true;
    int Amount;
    int TestSize;
    int Limit = 10; // tolerance
    bool Unacceptable = false;

    // check that bounding boxes do not go out of clipping rectangle, or image,
    // more than 30% of its size
    for (i = 0; i < SegmData.count(); i++)
    {
        if (!SegmData[i].Found)
            continue;

        if (ClipRectItem->isVisible())
        {
            if (SegmData[i].RealBoundingRect.left() < RealClipRect.left())
            {
                result = false;
                Amount = RealClipRect.left() - SegmData[i].RealBoundingRect.left();
                TestSize = SegmData[i].RealBoundingRect.width();
                if (Amount > TestSize * Limit / 100)
                    Unacceptable = true;
            }
            if (SegmData[i].RealBoundingRect.top() < RealClipRect.top())
            {
                result = false;
                Amount = RealClipRect.top() - SegmData[i].RealBoundingRect.top();
                TestSize = SegmData[i].RealBoundingRect.height();
                if (Amount > TestSize * Limit / 100)
                    Unacceptable = true;
            }
            if (SegmData[i].RealBoundingRect.right() > RealClipRect.right())
            {
                result = false;
                Amount = SegmData[i].RealBoundingRect.right() - RealClipRect.right();
                TestSize = SegmData[i].RealBoundingRect.width();
                if (Amount > TestSize * Limit / 100)
                    Unacceptable = true;
            }
            if (SegmData[i].RealBoundingRect.bottom() > RealClipRect.bottom())
            {
                result = false;
                Amount = SegmData[i].RealBoundingRect.bottom() - RealClipRect.bottom();
                TestSize = SegmData[i].RealBoundingRect.height();
                if (Amount > TestSize * Limit / 100)
                    Unacceptable = true;
            }
        }
        else
        {
            if (SegmData[i].RealBoundingRect.left() < 0)
            {
                result = false;
                Amount = 0 - SegmData[i].RealBoundingRect.left();
                TestSize = SegmData[i].RealBoundingRect.width();
                if (Amount > TestSize * Limit / 100)
                    Unacceptable = true;
            }
            if (SegmData[i].RealBoundingRect.top() < 0)
            {
                result = false;
                Amount = 0 - SegmData[i].RealBoundingRect.top();
                TestSize = SegmData[i].RealBoundingRect.height();
                if (Amount > TestSize * Limit / 100)
                    Unacceptable = true;
            }
            if (SegmData[i].RealBoundingRect.right() > FrameSizeX)
            {
                result = false;
                Amount = SegmData[i].RealBoundingRect.right() - FrameSizeX;
                TestSize = SegmData[i].RealBoundingRect.width();
                if (Amount > TestSize * Limit / 100)
                    Unacceptable = true;
            }
            if (SegmData[i].RealBoundingRect.bottom() > FrameSizeY)
            {
                result = false;
                Amount = SegmData[i].RealBoundingRect.bottom() - FrameSizeY;
                TestSize = SegmData[i].RealBoundingRect.height();
                if (Amount > TestSize * Limit / 100)
                    Unacceptable = true;
            }
        }
    }

    if (result == false)
    {
        if (Unacceptable)
            DisplayMessage(tr("Segment bounding boxes go significantly outside the image.\nPlease, repeat the acquisition repositioning the fingers"), true);
        else
            DisplayMessage(tr("Segment bounding boxes go slightly outside the image.\nBounding boxes will be cropped"), true);

        if (MSGUIRef->IsLCDPresent())
            GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_SEGMENT_BOUNDING_PROBLEM,
                GBMSAPI_VILCD_MSG_AREA);
    }

    return result;
}

// check that bounding boxex don't overlap more than 15% of their size
bool AcquisitionForm::CheckSegmentBoundingBoxOverlap()
{
    int i,j;
    bool result = true;
    int Idx1;
    int Idx2;

    Idx1 = -1;
    for (i = 0; i < SegmData.count() - 1; i++)
    {
        if (!SegmData[i].Found)
            continue;

        Idx1 = i;

        // search second segment
        Idx2 = -1;
        for (j = Idx1 + 1; j < SegmData.count(); j++)
        {
            if (SegmData[j].Found)
            {
                Idx2 = j;
                break;
            }
        }

        if (Idx2 == -1)     // no neighbour found
            continue;

        if (AreRectOverlapping(SegmData[Idx1].RealBoundingRect, SegmData[Idx2].RealBoundingRect, 15))
        {
            result = false;
            break;
        }
        else
            continue;
    }

    if (Idx1 == -1)     // no segments assigned
        return true;

    if (!result)
    {
        DisplayMessage(tr("Segment bounding boxes overlap significantly\nPlease, repeat the acquisition repositioning the fingers"), true);

        if (MSGUIRef->IsLCDPresent())
            GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_SEGMENT_BOUNDING_PROBLEM,
                GBMSAPI_VILCD_MSG_AREA);

        return false;
    }

    return true;
}

bool AcquisitionForm::AreRectOverlapping(QRect r1, QRect r2, int Limit)
{
    int HorzOverlap, VertOverlap;

    // check which starts first from left
    if (r1.left() < r2.left())
        HorzOverlap = r2.left() - r1.right();
    else
        HorzOverlap = r1.left() - r2.right();

    // determine which one starts first from top
    if (r1.top() < r2.top())
        VertOverlap = r2.top() - r1.bottom();
    else
        VertOverlap = r1.top() - r2.bottom();

    if ((HorzOverlap > 0) || (VertOverlap > 0))
    {
        // not overlapping
        return false;
    }

    // see if overlap more than Limit
    if ((HorzOverlap * VertOverlap) > (qMin(r1.width() * r1.height(), r2.width() * r2.height()) * Limit / 100))
    {
        return true;
    }

    return false;
}

void AcquisitionForm::CutBoundingBoxPositions()
{
    int i;
    int TestWidth, TestHeight;

    // cut bounding box coordinates if they go out of clipping rectangle or image
    for (i = 0; i < SegmData.count(); i++)
    {
        if (!SegmData[i].Found)
            continue;

        if (ClipRectItem->isVisible())
        {
            // make coordinates relative to clipped image
            SegmData[i].RealBoundingRect.setX(SegmData[i].RealBoundingRect.x() - RealClipRect.x());
            SegmData[i].RealBoundingRect.setY(SegmData[i].RealBoundingRect.y() - RealClipRect.y());

            TestWidth = RealClipRect.width();
            TestHeight = RealClipRect.height();
        }
        else
        {
            TestWidth = FrameSizeX;
            TestHeight = FrameSizeY;
        }

        if (SegmData[i].RealBoundingRect.x() < 0)
            SegmData[i].RealBoundingRect.setX(0);

        if (SegmData[i].RealBoundingRect.y() < 0)
            SegmData[i].RealBoundingRect.setY(0);

        if (SegmData[i].RealBoundingRect.right() > TestWidth)
            SegmData[i].RealBoundingRect.setRight(TestWidth);

        if (SegmData[i].RealBoundingRect.bottom() > TestHeight)
            SegmData[i].RealBoundingRect.setBottom(TestHeight);
    }
}

bool AcquisitionForm::AreSegmentsAllFound()
{
    // check if segments are found or unavailability specified
    for (int i = 0; i < SegmData.count(); i++)
    {
        if (!SegmData[i].Found &&
            (CheckMask(MSGUIRef->GetSessionOptions(), GBMSGUI_SESSION_OPTION_ASK_UNAVAILABILITY_REASON)
                && (SegmData[i].UnavailabilityReason == -1)))
            return false;
    }

    return true;
}

bool AcquisitionForm::AskUnavailabilityReason()
{
    UnavailabilityReasonForm AskForm;
    int i;

    // add items to the grid
    if (!GBMSGUI::IsSlap(mScannedObject->ScannedObjectID))
    {
        AskForm.AddItem(GetScannedObjectName());
    }
    else
    {
        // add missing segments
        for (i = 0; i < SegmData.count(); i++)
        {
            if (!SegmData[i].Found)
            {
                AskForm.AddItem(SegmData[i].FingerText);
            }
        }
    }

    // show dialog
    if (AskForm.exec() == QDialog::Rejected)
        return false;

    // read result
    if (!GBMSGUI::IsSlap(mScannedObject->ScannedObjectID))
    {
        mScannedObject->UnavailabilityReason = AskForm.GetReason(0);
    }
    else
    {
        int j = 0;
        for (i = 0; i < SegmData.count(); i++)
        {
            if (!SegmData[i].Found)
            {
                SegmData[i].UnavailabilityReason = AskForm.GetReason(j);
                j++;
            }
        }
    }

    return true;
}

// fill image with FillColor, excluding Excluded area
void AcquisitionForm::FillImage(unsigned char *ImgBuff, int SizeX, int SizeY,
    int ExcludedLeft, int ExcludedTop, int ExcludedRight, int ExcludedBottom,
    unsigned char FillColor)
{
    int x, y;
    unsigned char *pByte = ImgBuff;

    for (y = 0; y < SizeY; y++)
    {
        for (x = 0; x < SizeX; x++)
        {
            // if outside Excluded area, set to FillColor
            if ((y < ExcludedTop) || (y >= ExcludedBottom) ||
                (x < ExcludedLeft) || (x >= ExcludedRight))
                *pByte = FillColor;

            pByte++;
        }
    }
}

void AcquisitionForm::ClearOutsideBoundingBox()
{
    FillImage(FramePtr, FrameSizeX, FrameSizeY,
        SegmData[0].RealBoundingRect.left(), SegmData[0].RealBoundingRect.top(),
        SegmData[0].RealBoundingRect.right(), SegmData[0].RealBoundingRect.bottom(),
        255);

    DisplayImage();
    CheckResult();
}

void AcquisitionForm::OnClearOutside()  // this is a slot
{
    ClearOutsideBoundingBox();
}

void AcquisitionForm::done(int r)  // this is a slot
{
    // this is because the ESC key closes the dialog BUT does not trigger the CloseEvent.....
    close();
}

void AcquisitionForm::OnSegmBoxChanged(int Index)  // this is a slot
{
    // update RealBoundingRect
    // calculate scale factor
    double ScaleFactor = (double)ImageSizeX/(double)scene.width();

    int Left = SegmData[Index].BoundingRectItem->scenePos().x();
    int Top = SegmData[Index].BoundingRectItem->scenePos().y();
    int Right = Left + SegmData[Index].BoundingRectItem->rect().width();
    int Bottom = Top + SegmData[Index].BoundingRectItem->rect().height();

    SegmData[Index].RealBoundingRect.setCoords(
                floor(Left*ScaleFactor + 0.5),
                floor(Top*ScaleFactor + 0.5),
                floor(Right*ScaleFactor + 0.5),
                floor(Bottom*ScaleFactor + 0.5)
                );

    CheckResult();
}

void AcquisitionForm::OnClipRectChanged()  // this is a slot
{
    // update RealClipRectangle position (leave size fixed)
    // calculate scale factor
    double ScaleFactor = (double)ImageSizeX/(double)scene.width();
    QPointF ClipPos = ClipRectItem->pos();
    RealClipRect.moveLeft(floor(ClipPos.x()*ScaleFactor + 0.5));
    RealClipRect.moveTop(floor(ClipPos.y()*ScaleFactor + 0.5));
}

// converts a Green Bit quality value lto NFIQ value
int AcquisitionForm::GBQualityToNFIQ(int GBQuality)
{
    int NFIQQuality;

    if (GBQuality == 100)
        NFIQQuality = 1;
    else
        NFIQQuality = 5 - GBQuality / 20;

    return NFIQQuality;
}

void AcquisitionForm::StartListenPedal(int Condition)
{
	memset(&PedalInfo, 0, sizeof(PedalInformation));
    //PedalInfo.PressedTime = 0;
    //PedalInfo.ReleasedTime = 0;

    // call once to update the state
    GBMSAPI_CheckPedalState(&PedalInfo.PedalPressed);

    // get start time
	PedalInfo.tStart.start();

    // set condition to check
    PedalInfo.Condition = Condition;
    
    // enable timer
    ListenerTimer->start();
}

void AcquisitionForm::OnListenerTimer()	// this is a slot
{
    bool Continue = true;

    ListenerTimer->stop();

    if (MSGUIRef->IsPedalPresent())
        Continue = CheckPedal();

    if (Continue && MSGUIRef->IsLCDPresent())
        Continue = CheckLCDButtons();

    if (Continue)
	    ListenerTimer->start();
}

bool AcquisitionForm::CheckPedal()
{
    unsigned char prevPedalState;
    int ret;

    // save last state
    prevPedalState = PedalInfo.PedalPressed;

    // get current state
    ret = GBMSAPI_CheckPedalState(&PedalInfo.PedalPressed);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        MSGUIRef->SetError(MSGUIRef->GetMSAPIErrorMessage("GBMSAPI_CheckPedalState", ret), ret);
        ScanResult = GBMSGUI_RET_FAILURE;
        return false;
    }

    if ((PedalInfo.Condition == ReleasePedal) && !PedalInfo.PedalPressed)
    {
        // inform about pedal released
        OnPedalReleased();
        return false;
    }

    // if state is not changed
    if (prevPedalState == PedalInfo.PedalPressed)
    {
        if (PedalInfo.PedalPressed)
            PedalInfo.PressedTime = PedalInfo.tStart.elapsed();
        else
            PedalInfo.ReleasedTime = PedalInfo.tStart.elapsed();

        if (PedalInfo.Condition == ReleasePedal)
        {
            // if pressed for more than 2 seconds
            if (PedalInfo.PressedTime >= 2000)
            {
                // Send Release pedal message
                OnReleasePedal();
            }
        }

        if ((PedalInfo.Condition == PedalPressing) && PedalInfo.PedalPressed)
        {
            if (PedalInfo.PressedTime >= 2000)
            {
                // inform about long pressing
                if (OnLongPedalPressing() == true)
                    // exit from timer only if handled
                    return false;
            }
        }
    }
    else
    {
        // pedal state has changed
        // if released, check for how much time has been pressed
        if ((PedalInfo.Condition == PedalPressing) && !PedalInfo.PedalPressed)
        {
            if (PedalInfo.PressedTime >= 100)
            {
                // inform about short pressing
                if (OnShortPedalPressing() == true)
                    // exit from timer only if handled
                    return false;
            }
        }

        // start counting
        PedalInfo.tStart.restart();
        PedalInfo.PressedTime = 0;
        PedalInfo.ReleasedTime = 0;
    }

    // continue checking
    return true;
}

void AcquisitionForm::OnPedalReleased()
{
    DisplayMessage("", false);
}

bool AcquisitionForm::OnShortPedalPressing()
{
	if (ui->btnAccept->isEnabled())
    {
        on_btnAccept_clicked();
        return true;
    }

    return false;
}

bool AcquisitionForm::OnLongPedalPressing()
{
	on_btnRepeat_clicked();
    return true;
}

void AcquisitionForm::OnReleasePedal()
{
    DisplayMessage(tr("Release pedal!"), true);
    if (MSGUIRef->IsLCDPresent())
        GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(GBMSAPI_VILCD_MSG_RELEASE_PEDAL,
            GBMSAPI_VILCD_MSG_AREA);
}

bool AcquisitionForm::CheckLCDButtons()
{
    unsigned char PressedButton;
    int ret;

    // get LCD touchscreen pressed button
    ret = GBMSAPI_VUI_LCD_GetPressedButton(&PressedButton);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        MSGUIRef->SetError(MSGUIRef->GetMSAPIErrorMessage("GBMSAPI_VUI_LCD_GetPressedButton", ret), ret);
        ScanResult = GBMSGUI_RET_FAILURE;
        return false;
    }

    if (PressedButton == GBMSAPI_VILCD_TOUCHSCREEN_OK_BUTTON)
    {
        on_btnAccept_clicked();
        return false;
    }
    else if (PressedButton == GBMSAPI_VILCD_TOUCHSCREEN_REPEAT_BUTTON)
    {
		on_btnRepeat_clicked();
        return false;
    }

    // continue checking
    return true;
}

// DEBUG
void AcquisitionForm::ReadIni()
{
    QSettings Configuration("./GBMSGUI.ini", QSettings::IniFormat);

	DontDrawFrame = Configuration.value("Debug/DontDrawFrame", false).toBool();
	DrawScaled = Configuration.value("Debug/DrawScaled", false).toBool();
}

bool AcquisitionForm::LiveSegmentation()
{
    int ret;
    int ImageType;
    int i;
    bool Failed = false;
    unsigned int ImageProcessingOption = 0;
    unsigned int DetectionFailureDiagnostic;
	bool SingleObjectFlag = false;
	bool QualityCheck = true;

    if (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_4_LEFT)
        ImageType = GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4;
    if (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_4_RIGHT)
        ImageType = GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4;
    if (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_2_LEFT)
        ImageType = GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_2;
    if (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_2_RIGHT)
        ImageType = GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_2;
    if ((mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_2_THUMBS) || (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_2_INDEXES))
        ImageType = GBFINIMG_INPUT_IMAGE_TYPE_THUMBS_2;

	// missing fingers
	if ((ImageType == GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4) || (ImageType == GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4))
	{
		if (CheckUnavailableFingers())
		{
			unsigned char MissingFingerMask = 0;

			if (ImageType == GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4)
			{
				if (MSGUIRef->GetSegmCfgData(0)->UnavailabilityReason != -1)
					MissingFingerMask |= GBFINIMG_MISSING_FINGER_SLAP4_INDEX;
				if (MSGUIRef->GetSegmCfgData(1)->UnavailabilityReason != -1)
					MissingFingerMask |= GBFINIMG_MISSING_FINGER_SLAP4_MIDDLE;
				if (MSGUIRef->GetSegmCfgData(2)->UnavailabilityReason != -1)
					MissingFingerMask |= GBFINIMG_MISSING_FINGER_SLAP4_RING;
				if (MSGUIRef->GetSegmCfgData(3)->UnavailabilityReason != -1)
					MissingFingerMask |= GBFINIMG_MISSING_FINGER_SLAP4_LITTLE;
			}
			else	// GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4
			{
				if (MSGUIRef->GetSegmCfgData(3)->UnavailabilityReason != -1)
					MissingFingerMask |= GBFINIMG_MISSING_FINGER_SLAP4_INDEX;
				if (MSGUIRef->GetSegmCfgData(2)->UnavailabilityReason != -1)
					MissingFingerMask |= GBFINIMG_MISSING_FINGER_SLAP4_MIDDLE;
				if (MSGUIRef->GetSegmCfgData(1)->UnavailabilityReason != -1)
					MissingFingerMask |= GBFINIMG_MISSING_FINGER_SLAP4_RING;
				if (MSGUIRef->GetSegmCfgData(0)->UnavailabilityReason != -1)
					MissingFingerMask |= GBFINIMG_MISSING_FINGER_SLAP4_LITTLE;
			}

			GBFINIMG_Slap4_MissingFingers(MissingFingerMask);
		}
	}
    else    // Slap 2
    {
        if (CheckUnavailableFingers())
        {
            // if both are set as unavailable, fail
            if ((MSGUIRef->GetSegmCfgData(0)->UnavailabilityReason != -1) &&
				(MSGUIRef->GetSegmCfgData(1)->UnavailabilityReason != -1))
			{
				ResetSegments();
				return false;
			}
            ImageType = GBFINIMG_INPUT_IMAGE_TYPE_SINGLE_FINGER;
            SingleObjectFlag = true;
        }
    }

	PostProcessDiag = 0;

    if (MSGUIRef->IsResolution1000Dpi())
        ImageProcessingOption |= GBFINIMG_1000DPI_PRINT_IMAGE_PROCESSING;

	// segmentation during scanning
	ImageProcessingOption |= GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA;

	// adjust segments size to preview image size
	int SegmX = SegmSizeX / PreviewScale;
	int SegmY = SegmSizeY / PreviewScale;

    ret = GBFINIMG_ProcessImage(FramePtr, FrameSizeX, FrameSizeY,
            ImageType, ImageProcessingOption, SegmImgDescr, SegmX, SegmY,
            &NumSegments, &DetectionFailureDiagnostic, NULL, 0, SegmTempBuff,
            GBFINIMG_MEMORY_BUFFER_SIZE);

    if (ret != GBFINIMG_NO_ERROR)
    {
        //DisplayGBFinImgError(ret);
        return false;
    }

    if (DetectionFailureDiagnostic)
    {
        // add diagnostic messages
        if (CheckMask(DetectionFailureDiagnostic, GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP))
        {
            PostProcessDiag |= POST_PROC_DIAG_SEGM_AMBIGUOUS_SLAP;
        }
        if (CheckMask(DetectionFailureDiagnostic, GBFINIMG_SEGMENTATION_FAIL_INCOMPLETE_SLAP))
        {
            // different message depending if segments are found or not
            if (NumSegments == 0)
                PostProcessDiag |= POST_PROC_DIAG_SEGM_INCOMPLETE_SLAP;
            else
                PostProcessDiag |= POST_PROC_DIAG_SEGM_POSSIBLE_INCOMPLTETE_SLAP;
        }
        if (CheckMask(DetectionFailureDiagnostic, GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX))
        {
            // different message depending if segments are found or not
            if (NumSegments == 0)
                PostProcessDiag |= POST_PROC_DIAG_LEFT_RIGHT_MIXUP;
            else
                PostProcessDiag |= POST_PROC_DIAG_POSSIBLE_LEFT_RIGHT_MIXUP;
        }
        Failed = true;
    }

    // check if segments found are identified
    // don't check in case of SingleObjectFlag
    if (!Failed && !SingleObjectFlag)
	{
		for (i=0; i<NumSegments; i++)
		{
			if ((SegmImgDescr[i].Finger == GBFINIMG_FINGER_UNDEFINED)
				|| (SegmImgDescr[i].Hand == GBFINIMG_HAND_UNDEFINED))
			{
				// needed manual segmentation
				Failed = true;
			}
		}
	}

    if (Failed)
    {
        ResetSegments();
        return false;
    }

    // read results
    for (i=0; i<NumSegments; i++)
    {
		// PATCH - if object is GBMSAPI_SO_SLAP_2_INDEXES, replace FingerID (for GBFINIMG are thumbs...)
		if (mScannedObject->ScannedObjectID == GBMSAPI_SO_SLAP_2_INDEXES)
			SegmImgDescr[i].Finger = GBFINIMG_FINGER_INDEX;

		int FngIndex;
        // note that elements in list can be in different order; check FingerID
        if (SingleObjectFlag)
        {
            if (MSGUIRef->GetSegmCfgData(0)->UnavailabilityReason == -1)
                FngIndex = 0;
            else if (MSGUIRef->GetSegmCfgData(1)->UnavailabilityReason == -1)
                FngIndex = 1;
            else
				continue;
        }
		else
		{
			FngIndex = SegmDataIndex(SegmImgDescr[i].Finger, SegmImgDescr[i].Hand);
			if (FngIndex == -1)
				continue;
		}

        SegmData[FngIndex].Found = true;
        SegmData[FngIndex].FingerCenterX = SegmImgDescr[i].FingerCenterX;
        SegmData[FngIndex].FingerCenterY = SegmImgDescr[i].FingerCenterY;
        SegmData[FngIndex].RealBoundingRect.setCoords(SegmImgDescr[i].BoundingBoxL, SegmImgDescr[i].BoundingBoxT,
                                                      SegmImgDescr[i].BoundingBoxR, SegmImgDescr[i].BoundingBoxB);
        SegmData[FngIndex].GBQuality = SegmImgDescr[i].Quality;
		// can't be calculated from image, convert it...
		SegmData[FngIndex].NFIQQuality = GBQualityToNFIQ(SegmImgDescr[i].Quality);
		
		if (!EvalSegmentIAFISQuality(FngIndex))
			QualityCheck = false;
    }

    ShowBoundingBoxes();

	// check quality and num segments
	if (!QualityCheck)
		return false;

    return true;
}

void AcquisitionForm::OnIncompleteSlap()	// this is a SLOT
{
	SegmentationResult = DoSegmentation(false, true);
    CheckResult();
}

void AcquisitionForm::on_chkDryFingerImageEnhancement_toggled(bool checked)
{
	SetDryFingerImageEnhancementOption(checked);
}

bool AcquisitionForm::SetDryFingerImageEnhancementOption(bool Option)
{
    int ret = GBMSAPI_EnableDrySkinImgEnhance(Option);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        MSGUIRef->SetError(MSGUIRef->GetMSAPIErrorMessage("GBMSAPI_EnableDrySkinImgEnhance", ret), ret);
        ScanResult = GBMSGUI_RET_FAILURE;
        return false;
    }

    return true;
}
