﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

using LFS_NET_WRAPPER;
using Lfs_Net_Wrapper_Utilities;

namespace Lfs_Net_Wrapper_Utilities
{
    /************************************************************
     ************************************************************
     *
     * Structures definition: PACK 1 for compatibility with
     * C structures
     * 
     ************************************************************
    ************************************************************/
    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct MINUTIA
    {
        public int x;
        public int y;
        public int ex;
        public int ey;
        public int direction;
        public double reliability;
        public int type;
        public int appearing;
        public int feature_id;
        public IntPtr nbrs; // int*
        public IntPtr ridge_counts; // int*
        public int num_nbrs;
    }

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct MINUTIAE
    {
        public int alloc;
        public int num;
        public IntPtr list; // MINUTIA**
    }

    /************************************************************
     ************************************************************
     *
     * Direct dll wrapper
     * 
     ************************************************************
    ************************************************************/
    public class Lfsdll_NetWrapper
    {
        [DllImport("lfs.dll", EntryPoint = "get_minutiae")]
        public static extern int get_minutiae (
            IntPtr ominutiae, // MINUTIAE **
            IntPtr idata, // Byte *
            int iw,
            int ih,
            int id, 
            double ppmm
        );

        [DllImport("lfs.dll", EntryPoint = "free_minutiae", CharSet = CharSet.Ansi)]
        public static extern void free_minutiae(
            IntPtr minutiae // MINUTIAE *
        );
    }
}

namespace LFS_NET_WRAPPER
{
    /**********************************************************************/
    /**********************************************************************/
    //**********			SECTION 1: CONSTANTS		******************//
    /**********************************************************************/
    /**********************************************************************/
    /// <summary>
    /// Values for minutia types: there are no equivalents in the C version of the library
    /// </summary>
    public class NW_LFS_MINUTIA_TYPES
	{
        public static Int32 NW_BIFURCATION	= 0;
        public static Int32 NW_RIDGEENDING = 1;
	};

	/**********************************************************************/
	/**********************************************************************/
	//**********			SECTION 2: CLASSES		**********************//
	/**********************************************************************/
	/**********************************************************************/

	/// <summary>
	/// This is the equivalent of the "MINUTIA" structure in the C version of the LFS library.
	/// Here the information about a single minutia are collected.
	/// </summary>
	public class NW_LFS_MINUTIA
	{
	
		private Int32 x_coord;
		private Int32 y_coord;
		private Int32 dir;
		private Double rel;
		private Int32 min_type;
	
		/// <summary>
		/// Constructor overload: angle in degrees
		/// <para><see cref = "NW_LFS_MINUTIA.Direction_Degrees"/></para>
		/// </summary>
		public NW_LFS_MINUTIA(Int32 xin, Int32 yin,Double angle,Double quality,Int32 mintype)
        {
            this.x = xin;
            this.y = yin;
            this.Direction_Degrees = angle;
            this.reliability = quality;
            this.type = mintype;
        }
		/// <summary>
		/// Constructor overload: direction instead of angle
		/// <para><see cref = "NW_LFS_MINUTIA.Direction_Degrees"/></para>
		/// </summary>
		/// <remarks>
		/// <para> Direction is given by [(angle_in_degrees) / 11.25] </para>
		/// </remarks>
		public NW_LFS_MINUTIA(Int32 xin, Int32 yin,Int32 IntDirection,Double quality,Int32 mintype)
        {
            this.x = xin;
            this.y = yin;
            this.dir = IntDirection;
            this.reliability = quality;
            this.type = mintype;
        }

		/// <summary>
		/// <para>X coordinate of minutia</para>
		/// <para>get and set allowed</para>
		/// </summary>
		public  Int32 x 
        {
			get
            {
                return this.x_coord;
            }
			set
            {
                this.x_coord = value;
            }
		}

		/// <summary>
		/// <para>Y coordinate of minutia</para>
		/// <para>get and set allowed</para>
		/// </summary>
		public  Int32 y
        {
            get
            {
                return this.y_coord;
            }
            set
            {
                this.y_coord = value;
            }
		}

		/// <summary>
		/// <para>minutia direction in degrees</para>
		/// <para>reference system:</para>
		/// <para>- 0 degrees represents the positive direction of y axis,</para>
		/// <para>- angle values increase clockwise</para>
		/// </summary>
		/// <remarks>
		/// <para>Whenever set, this variable is quantized by multiple values of 11.25 degrees </para>
		/// <para>This  has no equivalents in the C version, but is an alternative to the
		/// "direction" member of the "MINUTIA" structure, where angles are expressed by integer values
		/// representing a conventional direction</para>
		/// </remarks>
		public  Double Direction_Degrees 
        {
			get
            {
                return ((this.dir) * 11.25);
            }
			set
            {
                this.dir = (int)(value / 11.25);
            }
		}

		/// <summary>
		/// <para>minutia quality in range [0-1]</para>
		/// </summary>
		public  Double reliability 
        {
			get
            {
                return this.rel;
            }
			set
            {
                this.rel = value;
            }
		}

		/// <summary>
		/// <para>minutia type:</para>
		/// <para><see cref = "NW_LFS_MINUTIA_TYPES"/></para>
		/// </summary>
        public int type
        {
            get
            {
                return this.min_type;
            }
            set
            {
                this.min_type = value;
            }
		}
	};

	/**********************************************************************/
	/**********************************************************************/
	//**********			SECTION 3: METHODS		**********************//
	/**********************************************************************/
	/**********************************************************************/

	/// <summary>
	/// <para>Input data for the LFS_NET_WRAPPER.NW_LFS_get_minutiae" class</para>
	/// </summary>
	/// <remarks> The members of this class are equivalent to the input data for the
	/// "get_minutiae" function of the C version of the library
	/// </remarks>
	public class NW_LFS_get_minutiae_InputData
	{	
		/// <summary>
		/// <para>Input image frame</para>
		/// </summary>
		/// <remarks> corresponds to the "idata" parameter of the "get_minutiae" function
		/// </remarks>
		public Byte [] Frame;
		/// <summary>
		/// <para>Input image width</para>
		/// </summary>
		/// <remarks> corresponds to the "iw" parameter of the "get_minutiae" function
		/// </remarks>
        public Int32 Width;
		/// <summary>
		/// <para>Input image height</para>
		/// </summary>
		/// <remarks> corresponds to the "ih" parameter of the "get_minutiae" function
		/// </remarks>
        public Int32 Height;
		/// <summary>
		/// <para>Input image bits per pixel number</para>
		/// </summary>
		/// <remarks> corresponds to the "id" parameter of the "get_minutiae" function
		/// </remarks>
        public Int32 BitsPerPixel;
		/// <summary>
		/// <para>Input image resolution in pixel per mm (500 dpi corresponds to 19.69 ppmm)</para>
		/// </summary>
		/// <remarks> corresponds to the "ppmm" parameter of the "get_minutiae" function
		/// </remarks>
        public Double PixelPermm;
	};

	/// <summary>
	/// <para>In this class the minutiae extraction is purchased</para>
	/// </summary>
	public class NW_LFS_get_minutiae
	{
		private NW_LFS_get_minutiae_InputData data;
	
		/// <summary>
		/// <para>Constructot Overload</para>
		/// </summary>
		/// <exception cref="System.Exception">Thrown when <paramref name="inp"/> is null.</exception>
		public NW_LFS_get_minutiae(NW_LFS_get_minutiae_InputData inp)
        {
            if (inp == null)
            {
                Exception ex = new Exception("NW_LFS_get_minutiae constructor: parameter inp cannot be null");
                throw (ex);
            }
            this.data = inp;
        }
		
		/// <summary>
		/// <para>Minutiae extraction</para>
		/// </summary>
		/// <exception cref="System.Exception">Thrown when an error occurs.
		/// <para>Read the <see cref ="System.Exception.Message"/>  </para>
		/// </exception>
		/// <remarks>
		/// Getting this  is the equivalent of calling the "get_minutiae" function in the C version
		/// of the library.
		/// This  is the equivalent of the "ominutiae" output parameter of "get_minutiae".
		/// </remarks>
		public List<NW_LFS_MINUTIA >  minutiae 
        {
            get
            {
                if (this.data == null || this.data.Frame == null)
                {
                    Exception ex = new Exception("NW_LFS_get_minutiae GetMinutiae: input data is null, try setting it again");
                    throw (ex);
                }
                
                ////////////////////////////////
                // Alloc memory for image
                ////////////////////////////////
                IntPtr InpImg = Marshal.AllocHGlobal(this.data.Frame.Length);
                if (InpImg == IntPtr.Zero)
                {
                    Exception ex = new Exception("NW_LFS_get_minutiae GetMinutiae: not enough memory");
                    throw (ex);
                }
                Marshal.Copy(this.data.Frame,0,InpImg,this.data.Frame.Length);

                /////////////////////////////////
                // Prepare pointer for minutiae
                // list
                /////////////////////////////////
                IntPtr pMminList = Marshal.AllocHGlobal(IntPtr.Size);
                
                /////////////////////////////////
                // Call low level function
                /////////////////////////////////
                int RetVal = Lfsdll_NetWrapper.get_minutiae(
                    pMminList,
                    InpImg,
                    this.data.Width, 
                    this.data.Height,
                    this.data.BitsPerPixel,
                    this.data.PixelPermm
                );

                ///////////////////////////
                // free memory for image
                ///////////////////////////
                Marshal.FreeHGlobal(InpImg);

                ///////////////////////////
                // get pointer to the
                // minutiae list
                ///////////////////////////
                IntPtr minlist = Marshal.ReadIntPtr(pMminList);                
                if (RetVal != 0 || minlist == IntPtr.Zero)
                {
                    
                    if (minlist != IntPtr.Zero)
                    {
                        Lfsdll_NetWrapper.free_minutiae(minlist);
                    }
                    Exception ex = new Exception("NW_LFS_get_minutiae GetMinutiae: low level function get_minutiae(...) failed");
                    throw (ex);
                }
                
                //////////////////////////////
                // Now get minutiae list
                //////////////////////////////
                MINUTIAE LowLevelList = (MINUTIAE)(Marshal.PtrToStructure(minlist, typeof(MINUTIAE)));

                ///////////////////////////////
                // Copy into high level list
                ///////////////////////////////
                List<NW_LFS_MINUTIA > ObjToReturn = new List<NW_LFS_MINUTIA >();
                if (ObjToReturn == null)
                {
                    Lfsdll_NetWrapper.free_minutiae(minlist);
                    Exception ex = new Exception("NW_LFS_get_minutiae GetMinutiae: not enough memory");
                    throw (ex);
                }

                for (int i = 0; i < LowLevelList.num; i++)
                {
                    // get pointer to lowlevel item
                    IntPtr pLLItem = Marshal.ReadIntPtr(LowLevelList.list, (i * IntPtr.Size));
                    MINUTIA LLItem = (MINUTIA)(Marshal.PtrToStructure(pLLItem, typeof(MINUTIA)));
                    NW_LFS_MINUTIA Item = new NW_LFS_MINUTIA(LLItem.x,
                        LLItem.y,
                        LLItem.direction,
                        LLItem.reliability,
                        LLItem.type
			        );
                    if (Item == null)
                    {
                        Lfsdll_NetWrapper.free_minutiae(minlist);
                        Exception ex = new Exception("NW_LFS_get_minutiae GetMinutiae: not enough memory");
                        throw (ex);
                    }
                    ObjToReturn.Add(Item);
                }
                
                // free memory
                Lfsdll_NetWrapper.free_minutiae(minlist);
                // return normally
                return ObjToReturn;
            }
		}

		/// <summary>
		/// <para>Set Input data</para>
		/// </summary>
		/// <exception cref="System.Exception">Thrown when <paramref name="inp"/> is null.</exception>
        public NW_LFS_get_minutiae_InputData NW_get_minutiae_Input
        {
            set
            {
                if (value == null)
                {
                    Exception ex = new Exception("NW_get_minutiae_Input property: parameter inp cannot be null");
                    throw (ex);
                }
                this.data = value;
            }
		}
	};
}
