#ifndef GBFINIMG_H_
#define GBFINIMG_H_
#include <stdint.h>

#pragma pack(1)
#define FAR
#define PASCAL
#define _stdcall

//	VER3000 palm-prints support added:
//			- LOWER HALF PALM prints 
//			- WRITER'S PALM prints 
//	VER3000 1000dpi prints support added

//----------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------
//											 SECTION 1
//										Structure definitions
//----------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------
// This structure is used to describe segmented image of finger/palm print(s) generated by 
//    GBFINIMG_ProcessImage() as a result of multi-or-single-print input image processing. 
//    For fingerprints: all information refers to the top phalange.
//----------------------------------------------------------------------------------------------------------


typedef struct 
{
	unsigned char  *SegmentImagePtr; // pointer to buffer with size <SegmentImageSizeX>*<SegmentImageSizeY>
									 //   allocated by application;
									 // this buffer is used to return to application segmented image
									 //   of a fingerprint (only top phalange, lower phalanges are eliminated) 
									 // possible halo-effect and latent-prints around prints are cleared;
									 // if option GBFINIMG_SEGMENT_VERTICAL_ROTATION is set, the print
									 //    is rotated to the vertical position. 
									 // if pointer is NULL, segmented image is not returned
								     // VER3000: palm-prints are supported  

    unsigned int   FingerCenterX; // X-coord of the print segment upper-part center within input image

    unsigned int   FingerCenterY; // Y-coord of the print segment upper-part center within input image

    unsigned int   Quality;		  // fingerprint image quality: 0-100
								  //   it is a quantitative expression of the predicted matching performance 
								  //   of the given fingerprint image;
								  // higher values indicate better quality (corresponding to 
								  //   ANSI/NIST-ITL 1-2007);
								  // proprietary quality evaluation algorithm is used;
								  // recommended thresholds are:  
								  //     1. [70-100] - high quality
								  //     2. [50-70] - medium quality
								  //     3. [0-50] - low quality
								  // VER3000: is available for palm-prints image only if option
								  //		    GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION is set;
								  //		  NOTE (for writer's palm only): print quality estimation is 
								  //			excluded for the edge region where regular 
								  //            ridge/value relief structure does not exist

    unsigned int   Hand;		  // left or right hand, see definitions for <Hand>

    unsigned int   Finger;		  // finger name (index, middle, etc.), see definitions for <Finger>
								  // VER3000: is not defined for palm-print images
	
	// bounding box data is generated in correspondence with 
	//   EBTS - ELECTRONIC FINGERPRINT TRANSMISSION SPECIFICATION (May 2, 2005) Appendix N.
    int  BoundingBoxL; // X-coord (in pixels) of the left bound of the print segment within input image
    int  BoundingBoxR; // X-coord (in pixels) of the right bound of the print segment within input image
    int  BoundingBoxT; // Y-coord (in pixels) of the top bound of the print segment within input image
    int  BoundingBoxB; // Y-coord (in pixels) of the bottom bound of the print segment within input image

    unsigned int  PatternValidity;	  // print pattern validity, in percents: 0-100;
									  // it is used to distinguish a real human finger/palm patterns from 
									  //   other surfaces with relief structure (e.g. gloves, tissue)
									  // higher value indicates higher validity;
									  // recommended threshold is 75%;
									  // VER3000: is available for palm-prints image only if option
									  //		    GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION is set;
									  //		  NOTES (for writer's palm only): 
									  //			- pattern validity estimation is excluded for the edge  
									  //              region where regular ridge/value relief structure does 
								      //			  not exist;
									  //			- pattern validity value should not be checked when 
									  //			  one of the diagnostic messages appears:
									  //			   GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX or
									  //			   GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP.

    unsigned int  PatternCompleteness;// print pattern completeness, in percents 0-100;
									  // it is used to detect improper finger allocation (finger tip, 
									  //   finger side, etc.) by checking presence of main fingerprint 
									  //   flow components (finger core, basic/distal flows, etc.)
									  // higher value indicates higher completeness;
									  // recommended threshold is 85%;
									  // VER3000: -for half-palm image it specifically refers to dimensions of 
									  //			 palm central "empty" zone that appears due to 
									  //			 the fact that concave palm surface is scanned on flat 
									  //			 scanner surface; recommended threshold is 90%.
									  //		  -for writer's-palm image it specifically refers to possibly 
									  //			 shorten print height due to the fact that convex writer's 
									  //			 palm surface is scanned on flat scanner surface; 
									  //			 recommended threshold is 90%.


    unsigned int  Size;				  // print area size, in mm2

    int  Inclination;				  // print inclination expressed as deviation from vertical position,
									  // in degrees: [-55(CCW);55(CW)]
									  // VER3200 NOTE: for image types GBFINIMG_INPUT_IMAGE_TYPE_THUMBS_2 and
									  //    GBFINIMG_INPUT_IMAGE_TYPE_SINGLE_FINGER_FLAT inclination range is
									  //    [-90(CCW);90(CW)] degrees
}
	GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR;
//----------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------
// This structure is used to pass externally specified fingerprint segment position and name to run 
//   GBFINIMG_ProcessImage() in SEMI-AUTOMATIC mode.
//   This position should refer to the top phalange of a fingerprint.
// Application is responsible to provide correct fingerprint segment position and naming
//   (normally, an  operator points to the certain segment inside image using some graphical pointing tool).
//----------------------------------------------------------------------------------------------------------
typedef struct 
{
    unsigned int FingerCenterX;// approximate X-coord of the center of the finger segment within input image, 
							   //    specified externally
    unsigned int FingerCenterY;// approximate Y-coord of the center of the finger segment within input image, 
							   //    specified externally
    unsigned int Hand;		   // hand, specified externally, see definitions for <Hand>
    unsigned int Finger;       // the name of the finger, specified externally, see definitions for <Finger>
}
	GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO;
//----------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------
// DEFINITIONS FOR <Hand>:
//----------------------------------------------------------------------------------------------------------
#define GBFINIMG_HAND_UNDEFINED						0
#define GBFINIMG_HAND_LEFT							1
#define GBFINIMG_HAND_RIGHT							2

//----------------------------------------------------------------------------------------------------------
// DEFINITIONS FOR <Finger>:
//----------------------------------------------------------------------------------------------------------
#define GBFINIMG_FINGER_UNDEFINED					0
#define GBFINIMG_FINGER_THUMB						1
#define GBFINIMG_FINGER_INDEX						2
#define GBFINIMG_FINGER_MIDDLE						3
#define GBFINIMG_FINGER_RING						4
#define GBFINIMG_FINGER_LITTLE						5


//----------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------
//											 SECTION 2
//										Function definition
//----------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------


//VER3000-1000DPI
//----------------------------------------------------------------------------------------------------------
extern "C" void FAR PASCAL GBFINIMG_SetRotationBuffer1000DPI(unsigned char	*RotationBuffer,
															 unsigned int	RotationBufferSize);
//----------------------------------------------------------------------------------------------------------
// The function stores internally a pointer to an auxiliary buffer <*RotationBuffer> that will be 
//   used by GBFINIMG_ProcessImage() to perform rotation operation for 1000-dpi images when  
//	 both the options GBFINIMG_SEGMENT_VERTICAL_ROTATION and GBFINIMG_1000DPI_PRINT_IMAGE_PROCESSING  
//	 are activated.
// Call this function before GBFINIMG_ProcessImage(). Enough to call once after loading DLL providing
//	 that <RotationBufferSize> is calculated on the basis of maximum input image size used in an application.
//----------------------------------------------------------------------------------------------------------
// Return value: none
//----------------------------------------------------------------------------------------------------------
// Parameters:
// <*RotationBuffer>  - pointer to a auxiliary buffer allocated by application with size <RotationBufferSize>
// <RotationBufferSize>  - specifies the size of the auxiliary buffer, should be calclulated as follows:
//							<RotationBufferSize>=<InputSizeX>*<InputSizeY>*GBFINIMG_ROTATION_BUFFER_FACTOR; 
//							where <InputSizeX> and <InputSizeY> are input parameters of 
//							GBFINIMG_ProcessImage() that define processed image dimensions.
//						 NOTE: if <RotationBufferSize> will be insufficient to perform rotation operation, 
//							function GBFINIMG_ProcessImage() will return error code GBFINIMG_ERROR_NO_MEMORY
#define  GBFINIMG_ROTATION_BUFFER_FACTOR (3.3) 
//----------------------------------------------------------------------------------------------------------


//VER3100
//----------------------------------------------------------------------------------------------------------
extern "C" void FAR PASCAL GBFINIMG_Slap4_MissingFingers(unsigned char MissingFingerMask);
//----------------------------------------------------------------------------------------------------------
// The function instructs GBFINIMG_ProcessImage to elaborate incomplete 4-finger slap (with some finger(s) 
//  missing) in FULLY-AUTOMATIC mode only for the given <InputImageType>: 
//     - GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4  
//     - GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4
//  If this function is called before, GBFINIMG_ProcessImage will try to identify separate finger segments 
//  using information about missing fingers supplied by this function. 
//  NOTE: if this function is called, GBFINIMG_ProcessImage will not perform left/right hand mix check and 
//        will omit some types of ambiguos scanned object check; so, it is not ensured that appropritae 
//		  diagnostic messages will be always properly generated and returned by <DetectionFailureDiagnostic>
//	      output parameter. It is operator's/application's responsibility  in such cases to ensure the 
//		  submission of appropriate slap to be scanned that really corresponds to the <InputImageType> 
//	      parameter value of GBFINIMG_ProcessImage function.
// Call this function before GBFINIMG_ProcessImage() every time when necessary to elaborate incomplete slap.
//----------------------------------------------------------------------------------------------------------
// Return value: none
//----------------------------------------------------------------------------------------------------------
// Parameters:
// <MissingFingerMask>  - bit masks that specify missing fingers on a 4-finger slap 
#define GBFINIMG_MISSING_FINGER_SLAP4_INDEX		0x01 // index finger is missing
#define GBFINIMG_MISSING_FINGER_SLAP4_MIDDLE	0x02 // middle finger is missing
#define GBFINIMG_MISSING_FINGER_SLAP4_RING		0x04 // ring finger is missing
#define GBFINIMG_MISSING_FINGER_SLAP4_LITTLE	0x08 // little finger is missing
//----------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------
extern "C" int FAR PASCAL GBFINIMG_ProcessImage(unsigned char	*InputImage,
												unsigned int	InputSizeX,		
												unsigned int	InputSizeY,		
												unsigned int	InputImageType,	

												unsigned int    ImageProcessingOption, 

												GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR *SegmentImageList, 
												unsigned int	SegmentImageSizeX, 
												unsigned int	SegmentImageSizeY, 

												unsigned int	*DetectedSegmentImageNumber, 
												unsigned int	*DetectionFailureDiagnostic, 

												GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO  *ExternalInfoList,
												unsigned int						  ExternalInfoListSize, 

												unsigned char	*TempBuffer,    
												unsigned int	TempBufferSize);
//----------------------------------------------------------------------------------------------------------
// The function performs processing of any type of slap and single flat/rolled fingerprint and palm-print 
//	 (see definitions for <InputImageType>).
// For multi-segment images (4/2-fingers slap, 2-thumbs) the function performs fingerprint segment separation
//   (segmenation) with accurate segment naming (index-middle-ring-little naming for slap, thumb of left or 
//	 right hand for 2-thumbs slap). 
// The function performs left-right hand mixing check for all finger/palm prints except single finger.
// For single print images and for separated segments of multi-segment images the additional processing 
//   and information gathering is performed - see GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR structure.
//
// The function can operate in two modes: FULLY-AUTOMATIC and SEMI-AUTOMATIC. 
// In FULLY-AUTOMATIC mode (parameter <ExternalInfoList> is NULL) the function performs all the necessary 
//   processing automatically and does not require any intervention from application(operator) part. 
//	 Trustworthy positive result is indicated by non-zero <DetectedSegmentImageNumber> and absence of 
//	 diagnostic messages (<DetectionFailureDiagnostic>==0).
// SEMI-AUTOMATIC mode (<ExternalInfoList> is not NULL) can be used in a case, when results of FULLY-AUTOMATIC 
//   mode are negative (it is indicated by the condition <DetectedSegmentImageNumber> == 0: in this case an 
//   eventual diagnostic message in <DetectionFailureDiagnostic> clarifies the failure reason). In such a case 
//   an intervention from the application (operator) is required in order to provide a correct fingerprint 
//   segment naming and positioning (segment pointing)  through the <ExternalInfoList> input parameter.
// Sometimes, FULLY-AUTOMATIC mode can provide positive but "doubtful" results. It is indicated by 
//   non-zero <DetectedSegmentImageNumber> accompanied by the presence of diagnostic messages 
//	 (non-zero <DetectionFailureDiagnostic>). In such a case application(operator) confirmation is  
//	 recommended to assure that the obtained results are correct. 
// For applications, where intervention of operator is not possible or is not desired, it is recommended 
//   to consider this result as negative. 
//		NOTE VER3000: SEMI-AUTOMATIC mode is not supported for palm prints. In case of erroneous or doubtful
//					  result an application provides tools for left/right hand naming 
//					  for the biggest segment (palm print) detected automatically.
// 
// It is important to note that there are two category of cases of negative or doubtful results.
//		1. cases RESOLVABLE by repeat -  incorrect user behaviour can lead to ambiguous situation with segment 
//				 naming, and the function refuses segment auto-naming:
//			1.1. the declared <InputImageType> does not corresponds to the actual scanned image (ex.:left-
//				 right hand mix, single finger or 2-thumbs instead of 4-finger slap or vice-versa, etc.) 
//			1.2. not natural slap allocation (slap partially-truncated by border, inclined more than 45 
//				 degrees, intentionally distorted, etc.)
//		2. cases IRRESOLVABLE by repeat - incomplete slap (lack of one or more fingers - amputated/bandaged) 
//		   leads to ambiguous situation with segment naming, and the function refuses segment auto-naming.
//		   NOTE VER3100: for such cases refer to GBFINIMG_Slap4_MissingFingers function description.	
//
// The function does not allocate any memory buffer internally.
// An application is responsible to allocate all the image buffers and working memory buffer 
//	  necessary for processing.
//
// IMPORTANT NOTE:
//  The current implementation of GBFINIMG is intentionally limited for usage only with fingerprint images 
//   produced by Green Bit live-scanners. 
//	Important to know that GBFINIMG is able to process successfully or entire original images taken from 
//		Green Bit scanner or sub-images obtained with use of built-in clipping functions - 
//		SetClippingRegionSize() and GetClippingRegionPosition(). It is not allowed to use other techniques 
//		to cut sub-images (cutting sub-image from fixed position inside original image, for example).
//----------------------------------------------------------------------------------------------------------
// Return value:
//
#define GBFINIMG_NO_ERROR						0
#define GBFINIMG_ERROR_PARAMETER				1
#define GBFINIMG_ERROR_UNAVAILABLE_OPTION		2 // if "GBImgTran.dll" not found when 
												  //	GBFINIMG_SEGMENT_VERTICAL_ROTATION is specified
#define GBFINIMG_ERROR_NO_MEMORY				3 // VER3000-1000DPI, not enough memory to perform operation,
												  //    see GBFINIMG_SetRotationBuffer1000DPI() function 
												  //    description
#define GBFINIMG_ERROR_LICENSE_NOT_GRANTED		4 // VER3510 - see GBFINIMG_GetLicenseInfo()
//----------------------------------------------------------------------------------------------------------
// Parameters:
//
// <*InputImage>  - pointer to input image buffer, size <InputSizeX>*<InputSizeY>, allocated by application;
//					expected fingerprint image characteristics are:				
//						- 8-bit grayscale, 
//						- ridges are dark (lower grayscales) and valleys are light (higher grayscale values)
//						- background grayscale level - near to 255 
//						- 500 dpi resolution or near
//						- fingerprint(s) orientation - finger tip to image top
//						- fingerprint(s) deviation from vertical position not more than 45 degrees
//
//	<InputSizeX>  - input image width, in pixels; must be <= GBFINIMG_MAX_INPUT_SIZE_X 
#define GBFINIMG_MAX_INPUT_SIZE_X			2520 // VER3000, > 5.5" (EBTS half-palm) at 500dpi 
#define GBFINIMG_MAX_INPUT_SIZE_X_1000DPI	(GBFINIMG_MAX_INPUT_SIZE_X*2) // VER3000-1000DPI
//
//	<InputSizeY>  - input image height, in pixels; must be <= GBFINIMG_MAX_INPUT_SIZE_Y
#define GBFINIMG_MAX_INPUT_SIZE_Y		2520 // VER3000, > 5.5" (EBTS half-palm) at 500dpi 
#define GBFINIMG_MAX_INPUT_SIZE_Y_1000DPI	(GBFINIMG_MAX_INPUT_SIZE_Y*2) // VER3000-1000DPI
//
//  <InputImageType> -	specifies type of input image:
#define GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4				1 // LEFT HAND SLAP 4:  INDEX+MIDDLE+RING+LITTLE 
#define GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4				2 // RIGHT HAND SLAP 4: INDEX+MIDDLE+RING+LITTLE 
#define GBFINIMG_INPUT_IMAGE_TYPE_THUMBS_2					3 // THUMB LEFT + THUMB RIGHT
#define GBFINIMG_INPUT_IMAGE_TYPE_SINGLE_FINGER				4 // FLAT OR ROLLED
#define GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_2				5 // LEFT HAND SLAP 2:  INDEX+MIDDLE 
#define GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_2				6 // RIGHT HAND SLAP 2: INDEX+MIDDLE 
#define GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_HALF_PALM		7 // VER3000, LEFT LOWER HALF PALM
#define GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_HALF_PALM		8 // VER3000, RIGHT LOWER HALF PALM
#define GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_WRITER_PALM		9 // VER3000, LEFT WRITER'S PALM
#define GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_WRITER_PALM	10// VER3000, RIGHT WRITER'S PALM
#define GBFINIMG_INPUT_IMAGE_TYPE_SINGLE_FINGER_FLAT		11// VER3200, use only for FLAT single fingers to obtain correct
															  // inclination value in wider range
															  // (see GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR.Inclination description)
															  // DO NOT APPLY THIS TYPE FOR ROLLS, for rolls use image type
															  // GBFINIMG_INPUT_IMAGE_TYPE_SINGLE_FINGER as before
//
//	<ImageProcessingOption>  - optional bit masks that specify additional image processing options: 
#define GBFINIMG_HALO_LATENT_ELIMINATION	0x00000001 
			// Wipes halo-effect/latent-prints in <InputImage> buffer around fingerprint segments.
#define GBFINIMG_SEGMENT_VERTICAL_ROTATION	0x00000002 
			//Rotates segmented images in <SegmentImageList[].SegmentImagePtr> to the vertical position.
			//NOTE: this operation requires "GBImgTran.dll" to be allocated
			//      in the same directory with "GBFINIMG.DLL".
			//VER3000: for half-palm images the rotation will not be applied 
			//		   in a case when the system is unable detect if a hand is left or right:
			//         (GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR.Hand == GBFINIMG_HAND_UNDEFINED)
#define GBFINIMG_PALM_PRINT_IMAGE_DELETE_FINGER_SEGMENTS 0x00000004 //VER3000
			// Deletes eventual fingerprint segments found around palm-prints in <InputImage> buffer. 
			// Recommended to be used.
			// This option is effective just for palm-print image types and only together with
			//  option GBFINIMG_HALO_LATENT_ELIMINATION
#define GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION	0x00000008 //VER3000
			// Enables complete palm-print image processing that provides parameters of quality of
			//	 palm-print image (GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR.PatternValidity and  
			//	 GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR.Quality)
			// ATTENTION: due to big size of palm-prints, activating this option will result in 
			//			  significantly longer processing time.
#define GBFINIMG_1000DPI_PRINT_IMAGE_PROCESSING			0x00000010 //VER3000-1000DPI
			// Allows to process 1000-dpi print images. If this option is set:  
			//	- the resolution of <InputImage> will be considered as 1000 dpi instead of 500 dpi
			//	- <InputSizeX> is limited by GBFINIMG_MAX_INPUT_SIZE_X_1000DPI 
			//	- <InputSizeY> is limited by GBFINIMG_MAX_INPUT_SIZE_Y_1000DPI 
			//  If this option is set together with GBFINIMG_SEGMENT_VERTICAL_ROTATION, an auxiliary 
			//	   buffer will be required for rotation operation, see GBFINIMG_SetRotationBuffer1000DPI()
			//	   function description.
#define GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY			0x00000020 //VER3002
			// Allows to process fingerprint images in a faster way omitting some processing steps.
		    // Attention: if this option is set: 
			// 1. application can use only  <*DetectedSegmentImageNumber> and <*DetectionFailureDiagnostic> 
			//    output parameters (with one exception - <*DetectionFailureDiagnostic> output parameters 
			//    in this case CAN NOT be used for writer's palm).
			// 2. Output parameter <SegmentImageList> CAN NOT BE USED for any kind of <InputImageType>!!!!
			// 3. The following other options will be ignored:
			//		- GBFINIMG_HALO_LATENT_ELIMINATION
			//		- GBFINIMG_SEGMENT_VERTICAL_ROTATION
			//		- GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION
			//		- GBFINIMG_REFINE_DRY_FINGERPRINT_IMAGE	//VER3500
			//		- GBFINIMG_REFINE_WET_FINGERPRINT_IMAGE //VER3500
			//		- GBFINIMG_REFINE_FINGERPRINT_CONTRAST_AND_CLEAN_BACKGROUND //VER3500
#define GBFINIMG_DISABLE_LEFT_RIGHT_HAND_MIXUP_CHECK			0x00000040 //VER3002
			// Can be useful for the cases when application/operator itself ensures that applied hand 
		    //  corresponds to the requested one (there is no surely left-right hand mix-up).
			// If this option is set, GBFINIMG_ProcessImage() function does not try to verify the hipotesis
			//	of hand mix-up and this fact improves the performnace of GBFINIMG_GetFingerprintCounter()
			//  function (decreases error rate) especially in cases of non-complete slaps (when one or more 
			//	fingers of a slap are absent/bandaged). 
			//  This option is supported only for the following <InputImageType>:
			//			- GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4 
			//			- GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4 
#define GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA		0x00000080 //VER3100
			// Allows to process fingerprint images in a faster way omitting some processing steps (similar to 
		    //   GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY option).
		    // Attention: if this option is set: 
			// 1. application can use <*DetectedSegmentImageNumber> and <*DetectionFailureDiagnostic> 
			//    output parameters (with one exception - <*DetectionFailureDiagnostic> output parameters 
			//    in this case CAN NOT be used for writer's palm) and output parameter <SegmentImageList>.
			//   In difference from GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY, if this option is set,
			//    <SegmentImageList> data will be generated and it will contain valid information 
			//    for all fields except for these: 
			//			SegmentImageList[].SegmentImagePtr			- separate segment image is NOT AVAILABLE
			//			SegmentImageList[].PatternValidity			- NOT AVAILABLE
			//			SegmentImageList[].PatternCompleteness		- NOT AVAILABLE
			//	  Note: all available fields in SegmentImageList are results of "fast" processing, so, in some
			//			cases "fast" results can differ from "complete" processing results.
			// 2. The following other options will be ignored:
			//      - GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY
			//		- GBFINIMG_HALO_LATENT_ELIMINATION
			//		- GBFINIMG_SEGMENT_VERTICAL_ROTATION
			//		- GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION
			//		- GBFINIMG_REFINE_DRY_FINGERPRINT_IMAGE	//VER3500
			//		- GBFINIMG_REFINE_WET_FINGERPRINT_IMAGE //VER3500
			//		- GBFINIMG_REFINE_FINGERPRINT_CONTRAST_AND_CLEAN_BACKGROUND //VER3500
#define GBFINIMG_ALLOW_INCOMPLETE_SLAP_SEGMENTS_DATA_GENERATION		0x00000100 //VER3400
			// This option allows to activate automatic fingerprint segment naming even in case of incomplete slap.
			// It is recommended to use this option only in applications where incomplete slaps processing is required but
			//  by some reason application is not able to inform the GBFINIMG system in advance by GBFINIMG_Slap4_MissingFingers()
			//  function which finger(s) of a slap is missing.
			// If this option is NOT set the default behaviour of GBFINIMG_ProcessImage() function will be the following -
			//   in case of missing finger(s) in slap/thumbs and with no external info available about which fingers are missing
			//	 (GBFINIMG_Slap4_MissingFingers() not called) in FULLY-AUTOMATIC mode output parameter <DetectedSegmentImageNumber>
			//   of GBFINIMG_ProcessImage() returns 0 (segmentation result is negative).
			// Instead, if this option is set, GBFINIMG_ProcessImage() function will automatically try to perform detected fingerprint
		    //   segment naming, that is, to determine to which finger a concrete fingeprint segment belongs to. In this case output
			//	 parameter <DetectedSegmentImageNumber> returns actual detected fingerprint segment number and <SegmentImageList>
			//   contains fingerprint segments information with finger names associated to the segments by special automatic naming
			//   procedure that analyses mutual finger positions and position on scanning window for better results.
			// Attention: when system has a high-level of confidence about automatic segmentation result of non-complete slap, diagnostic
			//			  message GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP will not be set; otherwise, when confidence level
			//			  is not high enough, diagnostic message GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP
			//			  will be present and in this case it is strongly recommended to check the results by human operator
			//            to avoid fingerprint naming errors that can lead in future to wrong finger identification.
			//  This option is supported only for the following <InputImageType>:
			//			- GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4
			//			- GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4
			//			- GBFINIMG_INPUT_IMAGE_TYPE_THUMBS_2
			// This option must be obligatory used together with previously defined
		    //   GBFINIMG_DISABLE_LEFT_RIGHT_HAND_MIXUP_CHECK option (see above), otherwise, it will be ignored.
			//   That means that in this case system will not check left-right hand mix and will rely completely
			//     on the information about hand passed by application. So, application/human operator should ensure
			//	   appropriate hand applying for correct result.
			//  This option will be ignored if at least one of the previously defined options is present:
			//			- GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY
			//			- GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA
#define GBFINIMG_REFINE_DRY_FINGERPRINT_IMAGE	0x00000200 //VER3500
			// IMPORTANT NOTE: DO NOT USE THIS OPTION FOR APPLICATIONS THAT REQUIRE FBI IQS APPENDIX F CERTIFICATION
			//                 (like fingerprint/palmprint forensic or civil-id enrollment)
			// Enables high-quality specially-designed software algorithm to restore/refine fingerprint ridges
			//  especially of dry/underpressed fingerprint image containing in <InputImage> buffer to improve
			//  fingerprint quality.
			// This option will be ignored if at least one of the previously defined options is present:
			//			- GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY
			//			- GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA
			//			- GBFINIMG_1000DPI_PRINT_IMAGE_PROCESSING
			// Attention: this is CPU-resource consuming option.
			// Note: if this option set for <InputImageType> == GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_HALF_PALM or
			//  GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_HALF_PALM, automatically internally also
			//	GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION option will be set to allow palm image enhancement.
#define GBFINIMG_REFINE_WET_FINGERPRINT_IMAGE	0x00000400 //VER3500
			// IMPORTANT NOTE: DO NOT USE THIS OPTION FOR APPLICATIONS THAT REQUIRE FBI IQS APPENDIX F CERTIFICATION
			//                 (like fingerprint/palmprint forensic or civil-id enrollment)
			// Enables high-quality specially-designed software algorithm to restore/refine fingerprint ridges
			//  especially of wet/overpressed fingerprint image containing in <InputImage> buffer to improve
			//  fingerprint quality.
			//  This option will be ignored if at least one of the previously defined options is present:
			//			- GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY
			//			- GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA
			//			- GBFINIMG_1000DPI_PRINT_IMAGE_PROCESSING
			// Attention: this is CPU-resource consuming option.
			// Note: if this option set for <InputImageType> == GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_HALF_PALM or
			//  GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_HALF_PALM, automatically internally also
			//	GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION option will be set to allow palm image enhancement.
#define GBFINIMG_REFINE_FINGERPRINT_CONTRAST_AND_CLEAN_BACKGROUND	0x00000800 //VER3500
			// Enables high-quality specially-designed software algorithm to improve overall contrast by
			//  expanding dynamic range of a fingerprint image containing in <InputImage> buffer to improve
			//  fingerprint quality and also cleans image areas around fingerprint(s) to delete eventual residual
			//  latent prints and halo-effects.
			//  This option will be ignored if at least one of the previously defined options is present:
			//			- GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY
			//			- GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA
			// Note: this option is similar to GBFINIMG_HALO_LATENT_ELIMINATION, but additionally
			//  it improves fingerprint image contrast and provides white valleys within fingerprint area
			//  for better seamless conjuctiuon with white background around fingerprint.
#define GBFINIMG_REFINE_DRY_FINGERPRINT_IMAGE_AGE_CREASES_UNTOUCHED	0x00001000 //VER3510
			// This option has to be used in conjuction with GBFINIMG_REFINE_DRY_FINGERPRINT_IMAGE option,
			//	otherwise, it will be ignored. If this option is present, age creases on dry-skin fingerprint image
			//  will not be corrected - ridges disconnected by age creases will not be reconnected.
//
//	<SegmentImageList> - pointer to array of structures, allocated by application; the lenght of array 
//						 must correspond to <InputImageType> (must be not less than the number of prints 
//						 presented in <InputImage>). Through this output parameter the 
//						 function returns segmented image data to application. The number of detected 
//						 segments is returned in <DetectedSegmentImageNumber> output parameter.
//  <SegmentImageSizeX> - specifies the width of segmented image buffer <SegmentImageList[].SegmentImagePtr>,
//						  must be <= InputSizeX 												
//  <SegmentImageSizeY> - specifies the height of segmented image buffer <SegmentImageList[].SegmentImagePtr>,
//						  must be <= InputSizeY 											
//  <*DetectedSegmentImageNumber> - returns a number of detected fingerprint segments stored in 
//									  <SegmentImageList>.
//								    In FULLY-AUTOMATIC mode if <DetectedSegmentImageNumber>==0, segmentation 
//								      result is negative. Otherwise, <DetectedSegmentImageNumber> always is 
//								      in correspondence to <InputImageType> (positive result).
//								    In SEMI-AUTOMATIC mode application must check that 
//								      <DetectedSegmentImageNumber> is equal to <ExternalInfoListSize>; if not 
//								      equal, not all segments externally specified in <ExternalInfoList> are 
//									  successfully detected in <InputImage> - wrong segment data.
//			
//  <*DetectionFailureDiagnostic> -  32-bit diagnostic word: bit value '1' - diagnostic message is present, 
//															 bit value '0' - diagnostic message is absent
#define GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX 0x00000001 //possible left<->right hand mix
#define GBFINIMG_SEGMENTATION_FAIL_INCOMPLETE_SLAP	   0x00000002 //some finger(s) of slap are not detected
#define GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP	   0x00000004 //ambiguous segment naming; probably, scanned 
																  // object does not correspond to the requested one
//									 Interpretation of present message depends on <DetectedSegmentImageNumber> value:
//									   1. if <DetectedSegmentImageNumber>==0, result of segment naming in 
//										  FULLY-AUTOMATIC mode is negative; diagnotsic message informs 
//										  about the most probable reason of fingerprint segment naming failure;
//									   2. if <DetectedSegmentImageNumber> is non-zero, result of segment naming 
//										  in FULLY-AUTOMATIC mode could be considered as positive but "doubtful".
//										  In this case diagnotsic message informs about the reason of doubt.
//										  Application(operator) confirmation is recommended to assure that the 
//										  obtained result is correct. If application(operator) intervention is not 
//										  possible or is not desired, it is recommended to consider this result 
//										  as negative.
//
//	<*ExternalInfoList>  - pointer to array of structures allocated by application; the lenght of array 
//						     is defined by application through <ExternalInfoListSize>. 
//						   If <ExternalInfoList> is NULL, the function operates in FULLY-AUTOMATIC mode, 
//							  where there is no need in externally specified information.
//						   If <ExternalInfoList> is not NULL, the function operates in SEMI-AUTOMATIC mode. 
//							  Through this input parameter an application specifies fingerprint segment name and 
//						      position information for one or more fingerprint segments. 
//						      VER3000: ignored for palm-print images.
//
//  <ExternalInfoListSize> - specifies a number of externally described fingerprint segments in 
//							   <*ExternalInfoList>; is interpreted only if <*ExternalInfoList> is not NULL. 
//						      VER3000: ignored for palm-print images.
//
//  <*TempBuffer>  - working memory buffer of size <TempBufferSize>, allocated by application
//
//	<TempBufferSize>  - specifies the size of <*TempBuffer>, should be at least GBFINIMG_MEMORY_BUFFER_SIZE
#define GBFINIMG_MEMORY_BUFFER_SIZE		21708951 //VER3510
//----------------------------------------------------------------------------------------------------------

//VER3002
//----------------------------------------------------------------------------------------------------------
extern "C" int FAR PASCAL GBFINIMG_GetFingerprintCounter();
//----------------------------------------------------------------------------------------------------------
// The function returns the number of fingerprints found  in image by the previous  
//   call of GBFINIMG_ProcessImage() function. Call it only after GBFINIMG_ProcessImage(). 
// This function is particularly usefull in the cases when GBFINIMG_ProcessImage() returns negative or 
//   ambiguos results. Call this function to understand better if a possible reason of processing failure or 
//   ambiguity is a lack of finger(s) on a hand.
// If GBFINIMG_ProcessImage() returns success, the value returned by this function usually corresponds to 
//   output parameter <*DetectedSegmentImageNumber> of GBFINIMG_ProcessImage(). But, in some cases, 
//   these values could be different. So, for applications, where fingerprint number check is needed,
//   it is recommended to call GBFINIMG_GetFingerprintCounter() in spite of the results obtained by 
//   GBFINIMG_ProcessImage().
// This function counts as fingerprints only higher finger plalanges (finger-tip segments) and ignores 
//  lower phalanges (middle and bottom) allocated in image below the fingertips.
//----------------------------------------------------------------------------------------------------------
// Return value:
//		-1:  this functionality (fingerprint counter) is not supported for the given <InputImageType> 
//				used in the previous GBFINIMG_ProcessImage() call. This functionality is supported 
//				only for the following <InputImageType>:
//						- GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4 
//						- GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4 
//						- GBFINIMG_INPUT_IMAGE_TYPE_THUMBS_2 
//     0-N:  returns number of fingerprints actually found in image;
//				could be even more than 4 in cases when excessive fingerprints found in image.
// NOTE: in cases when diagnostic message GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX is reported by 
//		 GBFINIMG_ProcessImage() function, the returned fingerprint counter could sometimes not correspond 
//		 to the actual finger number especially in cases of non-complete slaps (when one or more fingers 
//		 of a slap are absent/bandaged). To improve performanve of GBFINIMG_GetFingerprintCounter() - that is, 
//		 to decrease error rate - when possible use GBFINIMG_DISABLE_LEFT_RIGHT_HAND_MIXUP_CHECK - see the 
//		 description of <ImageProcessingOption> parameter in GBFINIMG_ProcessImage().
//----------------------------------------------------------------------------------------------------------


//VER3300
//----------------------------------------------------------------------------------------------------------
extern "C" void FAR PASCAL GBFINIMG_CheckFingerRotation180Degree(int		   *Probability,
																 unsigned char *PatternDiagnostic);
//----------------------------------------------------------------------------------------------------------
// The function calculates probability value that finger was scanned in opposite orientation
//   (finger tip to scan window bottom). This value can be checked together with
//   <GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR.Inclination> in some applications where orientation limitations are
//   applied during fingerprint acqusition.
// Call this function only after GBFINIMG_ProcessImage().
// The rotation probability value will be calculated only for <InputImageType>:
//	  - GBFINIMG_INPUT_IMAGE_TYPE_SINGLE_FINGER
//    - GBFINIMG_INPUT_IMAGE_TYPE_SINGLE_FINGER_FLAT
//    - GBFINIMG_INPUT_IMAGE_TYPE_THUMBS_2
//	and only if noone of fast processing options was used:
//	  -GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY
//	  -GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA
//  For <InputImageType> GBFINIMG_INPUT_IMAGE_TYPE_THUMBS_2 where two independent fingers there are in the
//   same image frame, the returned probability value refers to the finger with highest probablity of
//   180-degree rotation.
// NOTE: finger orientation detection relies on fingerprint pattern evaluation and heavily depends on pattern
//   type and completeness. The most difficulty with automatic orientation detection is for partially-scanned
//   whorl patterns (concentric round or ellipsoid circles) with no bottom delta fingerprint zone, for such
//	 cases it is not always possible to reveal pattern orientation even for human experts, not only for
//	 automatic algorithms. That's why orientation detection is expressed by a probabilistic value.

//----------------------------------------------------------------------------------------------------------
// Return value:    none
//----------------------------------------------------------------------------------------------------------
// Parameters:
//
// <*Probability>  	-1:  this functionality is not supported for the given <InputImageType> or
//						 one of the fast processing options was used
//					0-100:  returns probability value that finger was scanned in opposite orientation
//							(finger tip to window bottom);
//							Value 0 reflects the minimim probablity that fingerprint was 180-degree rotated and
//							value 100 reflects the maximum probablity that fingerprint was 180-degree rotated.
//							Value 50 means that there is equal probability and there is no possibility for automatic
//							system to distinguish the correct orientation.
//							For most pratical applications it is recommended for better FAR/FRR performance to extend
//							undefined zone from single value 50 till range [40;60] and to request for acqusition repeat
//						    if possible.
// <*PatternDiagnostic> -  due to the fact that orientation check relies on fingerprint pattern evaluation, orientation
//						   probablity value could be inappropriate if finger surface is scanned only partially (not whole
//						   fingerprint is present in the image frame that could often happen with flat acqusitions), or
//						   when significant fingerprint zone is "cutted" by image border. There are the following pattern
//						   diagnostic bit masks:
//								0x01 - partial finger scan (not whole fingerprint zone is present in image)
//								0x02 - significant fingerprint zone is "cutted" by image border ( finger is placed
//									    too near to scanning window border).
//						   It is strongly recommended to check that there is no any pattern diagnostic bit mask set
//						   to be sure that calculated probability value is appropriate and adequate. Otherwise, repeat
//						   fingerprit acqusition if possible.
//						   NOTE: it is recommeneded also to check GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR.PatternCompleteness value
//								 together with <PatternDiagnostic>


//VER3500
//----------------------------------------------------------------------------------------------------------
extern "C" void FAR PASCAL GBFINIMG_SetInternalParameterValue(int ParameterType,
															  int ParameterValue);
//----------------------------------------------------------------------------------------------------------
// The function allows to modify some internal parameter values in cases when dafault behaviour of
//   GBFINIMG_ProcessImage() function does not satisfy some specific application request regarding
//   the following options functionality:
//   - GBFINIMG_REFINE_DRY_FINGERPRINT_IMAGE
//   - GBFINIMG_REFINE_WET_FINGERPRINT_IMAGE
//	 - GBFINIMG_REFINE_FINGERPRINT_CONTRAST_AND_CLEAN_BACKGROUND
//   - GBFINIMG_HALO_LATENT_ELIMINATION //VER3510
// In this case an application developer should contact GREEN BIT technical support specifying his concrete
//   requirements to get required values of <ParameterType> and <ParameterValue>.
// Call this function before GBFINIMG_ProcessImage(). Enough to call once after loading DLL.
//----------------------------------------------------------------------------------------------------------
// Return value: none
//----------------------------------------------------------------------------------------------------------
// Parameters:
//  <ParameterType>    : parameter type to be modified (ask GREEN BIT technical support)
//  <ParameterValue>   : new value specified (ask GREEN BIT technical support)
//----------------------------------------------------------------------------------------------------------


//VER3510
//**********************************************************************************************************
//**********************************************************************************************************
//										GBFINIMG LICENSE CHECK
//**********************************************************************************************************
//**********************************************************************************************************
//	 Attention: Starting from GBFINIMG.DLL VER3510 the usage of library is licensed.
//
//   GBFINIMG license is supported by the following Green Bit fingerprint scanner models:
//		- DactyScan26(i)
//		- DactyScan40i
//		- DactyScan84(n)
//		- MultiScan500/MC500/MSC500
//		- MultiScan1000
//		- DactyScan84C
//		- MC517/MSC517
//   GBFINIMG license will be granted if at least one Green Bit scanner of supported model with activated
//		GBFINIMG license will be found connected on USB bus of PC.
//	 GBFINIMG license is controlled by the third position of scanner serial number string:
//		- if an alphabetical character "R" or "F" is present in the third position, GBFINIMG license
//		  will be granted
//	    - otherwise, GBFINIMG license will NOT be granted
//   Serial number examples: <D8RARP00035> - GBFINIMG license will be granted
//						     <D8BARP00035> - GBFINIMG license will NOT be granted
//	 Practically, in this case no additional licensing procedures (like PC's HostID generation,
//     license file transmission and so on...) are required from user part.
//----------------------------------------------------------------------------------------------------------
typedef struct
{
	int			  Result;
	unsigned int  UsbDriverErrorCodeDetail; // to be interpreted if Result == GBSCAN_ERROR_USB_DRIVER
}
GBFINIMG_LICENSE_USB_SCANNER_CHECK;
//GBFINIMG_LICENSE_USB_SCANNER.Result:
#define	GBSCAN_NO_ERROR						0  // GBFINIMG license is granted
#define	GBSCAN_ERROR_USB_DRIVER				1  // An USB driver error happend during license check,
											   //    see UsbDriverErrorCodeDetail for more detail
#define	GBSCAN_ERROR_USB_DEVICE_NOT_FOUND	2  // GBFINIMG license is NOT granted

//----------------------------------------------------------------------------------------------------------
extern "C" void FAR PASCAL GBFINIMG_GetLicenseInfo(GBFINIMG_LICENSE_USB_SCANNER_CHECK *ScannerBasedLicense);
//----------------------------------------------------------------------------------------------------------
// The function returns information about license granted (check GBFINIMG_LICENSE_USB_SCANNER.Result) and
// describes the failure reasons in case of license check failure.
// NOTE 1: call of this function is duplicated internally from GBFINIMG_ProcessImage() function, so, it is not
//      always necessary to call this function explicitly if there are no USB-bus conflicts with other
//	    software components trying to get USB-bus access to the connected fingerprint scanner.
// NOTE 2: if there is a conflict with other software components trying to get USB-bus access to connected
//   fingerprint scanner (GBFINIMG_LICENSE_USB_SCANNER.Result == GBSCAN_ERROR_USB_DRIVER) it is recommended
//   to call this function by application outside an image acquisition process in order to avoid
//	 conflicts with other software components trying to get USB-bus access to connected fingerprint scanner.
//   Call this function before GBFINIMG_ProcessImage(). Enough to call once after loading DLL.
//----------------------------------------------------------------------------------------------------------
// Return value: none
//----------------------------------------------------------------------------------------------------------
// Parameters:
// <*ScannerBasedLicense>	- if not NULL, returns more detailed scanner-based license check failure reason.
//----------------------------------------------------------------------------------------------------------


#endif 
